Package: bayesqm
Type: Package
Title: Bayesian Q Methodology: Probabilistic Factor Analysis
Version: 0.1.0
Date: 2026-05-18
Authors@R: c(
    person(given = "Raymond Dacosta", family = "Azadda",
           email = "rdazadda@alaska.edu",
           role = c("aut", "cre")),
    person(family = "AK-ACE Team", role = "aut"),
    person("Karsten", "Hueffer", role = "aut"),
    person("Taa'aii", "Peter", role = "aut"),
    person("Stacy", "Rasmus", role = "aut"))
Description: A Bayesian factor-analytic framework for Q methodology. Fits a
    low-rank factor model to Q-sort data with a Student-t likelihood and a
    hierarchical normal prior on loadings, samples the posterior with Stan,
    resolves rotational ambiguity via the MatchAlign post-processing of
    Poworoznek et al. (2025) <doi:10.1214/25-BA1544>, and returns posterior
    summaries including credible intervals for loadings and factor scores,
    probabilistic dominant-factor membership, distinguishing and consensus
    statements, and PSIS-LOO-based factor enumeration following Vehtari et
    al. (2017) <doi:10.1007/s11222-016-9696-4> with the Sivula et al. (2025)
    <doi:10.1214/25-BA1569> parsimony rule.
License: GPL (>= 3)
URL: https://github.com/rdazadda/bayesqm,
        https://rdazadda.github.io/bayesqm/
BugReports: https://github.com/rdazadda/bayesqm/issues
Encoding: UTF-8
Language: en-US
Config/testthat/edition: 3
Depends: R (>= 4.1.0)
Imports: stats, utils, tools, parallel, rstantools (>= 2.3.0)
Suggests: cmdstanr (>= 0.8.0), rstan (>= 2.32.0), loo (>= 2.7.0),
        GPArotation (>= 2024.3-1), lpSolve, readxl (>= 1.4.0), jsonlite
        (>= 1.8.0), posterior (>= 1.5.0), ggplot2 (>= 3.4.0), ggdist
        (>= 3.3.0), ggridges (>= 0.5.0), scales (>= 1.2.0), knitr (>=
        1.40), rmarkdown (>= 2.20), testthat (>= 3.0.0), vdiffr (>=
        1.0.0)
VignetteBuilder: knitr
Additional_repositories: https://stan-dev.r-universe.dev
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-10 02:15:55 UTC; rdazadda
Author: Raymond Dacosta Azadda [aut, cre],
  AK-ACE Team [aut],
  Karsten Hueffer [aut],
  Taa'aii Peter [aut],
  Stacy Rasmus [aut]
Maintainer: Raymond Dacosta Azadda <rdazadda@alaska.edu>
Repository: CRAN
Date/Publication: 2026-06-17 17:40:12 UTC
Built: R 4.5.2; ; 2026-06-17 19:45:08 UTC; unix

Package: bayesqm
Type: Package
Title: Bayesian Q Methodology: Exact Rank-Order Likelihood for Forced Q
        Sorts
Version: 0.2.0
Date: 2026-08-19
Authors@R: c(
    person(given = "Raymond Dacosta", family = "Azadda",
           email = "rdazadda@alaska.edu",
           role = c("aut", "cre"),
           comment = c(ORCID = "0009-0002-4384-4556")),
    person("Henry Ofoe", "Agbi-Kaiser", role = "aut",
           comment = c(ORCID = "0009-0008-6127-9136")),
    person("Hannah D.", "Robinson", role = "aut",
           comment = c(ORCID = "0009-0000-5159-1363")),
    person(family = "AK-ACE Team", role = "aut"),
    person("Karsten", "Hueffer", role = "aut"),
    person("Taa'aii", "Peter", role = "aut"),
    person("Stacy", "Rasmus", role = "aut"))
Description: A Bayesian analysis for Q methodology, alongside the classical
    one. Models the forced Q sort as an ordered partition of the statements
    through an exact rank-order likelihood (the design quotas fix the
    partition margins, so the likelihood of the observed sorting event is
    exact), fits it by a parameter-expanded Gibbs sampler in R with no
    compiled code and a
    convergence gate on rotation-invariant functionals, resolves
    rotational ambiguity via the MatchAlign post-processing of Poworoznek
    et al. (2025) <doi:10.1214/25-BA1544>, and returns the familiar Q
    tables as posterior summaries: credible intervals for bounded
    participant loadings, flag probabilities with an explicit unclassified
    state, quota-respecting factor arrays, distinguishing and consensus
    statements judged against a posterior critical difference and a
    grid-width equivalence region, one posterior false-discovery rule for
    all published claims, and a two-signal posterior-predictive workflow
    for the number of factors.
License: GPL (>= 3)
URL: https://github.com/rdazadda/bayesqm,
        https://rdazadda.github.io/bayesqm/
BugReports: https://github.com/rdazadda/bayesqm/issues
Encoding: UTF-8
Language: en-US
LazyData: true
Config/testthat/edition: 3
Depends: R (>= 4.1.0)
Imports: stats, utils, tools, clue, posterior (>= 1.5.0)
Suggests: loo (>= 2.7.0), readxl (>= 1.4.0), jsonlite (>= 1.8.0),
        ggplot2 (>= 3.4.0), knitr (>= 1.40), rmarkdown (>= 2.20),
        testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/Needs/website: qmethod
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-08-20 01:22:44 UTC; rdazadda
Author: Raymond Dacosta Azadda [aut, cre] (ORCID:
    <https://orcid.org/0009-0002-4384-4556>),
  Henry Ofoe Agbi-Kaiser [aut] (ORCID:
    <https://orcid.org/0009-0008-6127-9136>),
  Hannah D. Robinson [aut] (ORCID:
    <https://orcid.org/0009-0000-5159-1363>),
  AK-ACE Team [aut],
  Karsten Hueffer [aut],
  Taa'aii Peter [aut],
  Stacy Rasmus [aut]
Maintainer: Raymond Dacosta Azadda <rdazadda@alaska.edu>
Repository: CRAN
Date/Publication: 2026-08-20 10:52:15 UTC

export(fit_bayesian)
export(extend)
export(matchalign)
export(delta_grid)
export(qsort_data)
export(validate_qsort)
export(check_distribution)
export(infer_distribution)
export(parse_distribution)
export(read_qsort)
export(read_qsort_csv)
export(read_qsort_excel)
export(read_pqmethod)
export(read_kenq)
export(read_kenq_excel)
export(read_kade_zip)
export(read_easyhtml_firebase)
export(read_statements)
export(import.pqmethod)
export(import.htmlq)
export(import.kenq)
export(import.easyhtmlq)
export(generate_data)
export(generate_loadings)
export(generate_noise)
export(discretize_to_grid)
export(get_distribution)
export(assess_recovery)
export(assess_classification)
export(tucker_congruence)
export(procrustes_rotation)
export(demo_fit)
export(compute_loadings)
export(compute_flags)
export(compute_zscores)
export(compute_factor_array)
export(compute_qdc)
export(factor_characteristics)
export(crib_sheet)
export(claims)
export(check_fit)
export(check_persons)
export(fit_ladder)
export(select_k)
export(loo_ladder)
export(loglik_person)
export(plot_sorts)
export(plot_zscores)
export(plot_statement)
export(plot_zscore_posterior)
export(plot_loading_posterior)
export(plot_flags)
export(plot_factor_array)
export(plot_contrasts)
export(plot_choice_k)
export(plot_person_check)
export(plot_convergence)
export(plot_ppc)
export(plot_membership)
export(plot_dist_cons)
export(plot_hyper)
export(caption_bayesqm)
export(plot_tucker)
export(plot_elpd)
export(make_elpd_diff)
export(make_ppc_ridge)
export(make_dominant_panel)
export(rotate_factors)
export(flip_factor)
export(rename_factors)
export(bayesqm_colors)
export(bayesqm_set_colors)
export(save_bayesqm_plot)
export(posterior_interval)
export(prior_summary)
export(run_bayes)
export(select_k_peak)
export(select_k_sivula)
export(compute_threshold_prob)
export(compute_dominant_prob)
export(compute_dominant_sign)
export(compute_divergence)
export(classify_membership)
export(suggest_delta)
export(critical_delta)
export(compute_posterior_scalars)
export(demo_run)

S3method(print, qsort_data)
S3method(summary, qsort_data)
S3method(as.matrix, qsort_data)
S3method(print, bayesqm_fit)
S3method(summary, bayesqm_fit)
S3method(coef, bayesqm_fit)
S3method(fitted, bayesqm_fit)
S3method(residuals, bayesqm_fit)
S3method(nobs, bayesqm_fit)
S3method(sigma, bayesqm_fit)
S3method(family, bayesqm_fit)
S3method(print, bayesqm_family)
S3method(as.matrix, bayesqm_fit)
S3method(as.array, bayesqm_fit)
S3method(as.data.frame, bayesqm_fit)
S3method(update, bayesqm_fit)
S3method(posterior_interval, bayesqm_fit)
S3method(prior_summary, bayesqm_fit)
S3method(print, bayesqm_prior)
S3method(print, bayesqm_claims)
S3method(print, bayesqm_checks)
S3method(print, bayesqm_persons)
S3method(print, bayesqm_selection)
S3method(plot, bayesqm_fit)
S3method(plot, qsort_data)

importFrom(stats, coef, cor, density, family, fitted,
           median, nobs, pnorm, qnorm, quantile, rbinom, rexp,
           rgamma, rnorm, rt, runif, residuals, sd, setNames, sigma,
           update)
importFrom(utils, combn, head, read.csv, unzip)
importFrom(graphics, abline, arrows, axis, grconvertX, grconvertY, hist, legend, mtext, par, points, polygon, rect, segments, text)
importFrom(grDevices, adjustcolor, colorRampPalette)

<!-- README.md is generated from README.Rmd. Edit README.Rmd, then knit. -->

# bayesqm <img src="man/figures/logo.png" align="right" height="139" alt="bayesqm logo" />

<!-- badges: start -->

[![R-CMD-check](https://github.com/rdazadda/bayesqm/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/rdazadda/bayesqm/actions/workflows/R-CMD-check.yaml)
[![License: GPL
v3](https://img.shields.io/badge/License-GPLv3-blue.svg)](https://cran.r-project.org/web/licenses/GPL-3)
<!-- badges: end -->

**bayesqm turns forced Q sorts into the full set of Q-methodology
reports, each carrying its posterior uncertainty.** It reads the common
study formats, PQMethod `.DAT`, Ken-Q, KADE, HTMLQ and FlashQ exports,
CSV and Excel grids, and models the sort itself: each participant places
every statement into a fixed grid, the quotas make that placement an
ordered partition of the statement set, and the package computes the
probability of the observed partition exactly. From one panel it derives
what a Q study reports, bounded loadings, flags, factor arrays,
statement scores, distinguishing and consensus statements, the number of
factors, and the checks behind them. Every analysis returns an object
that prints its own summary and draws its own plots.

Every decision states its rule. One posterior false-discovery rule
selects everything the analysis reports, flags, distinguishing listings,
consensus statements, and pairwise stars, and gives the expected number
of false claims alongside. The number of factors is decided by two
checks read together, whether K factors account for the panel and
whether every factor earns its place. Quantities classical practice
fixes by convention, above all the critical difference behind
distinguishing statements, are estimated from the posterior instead, and
the alignment of the posterior draws follows the MatchAlign procedure of
Poworoznek et al. (2025). The implementation is checked two ways,
against planted truth in simulation and against the `qmethod` reference
implementation wherever both compute the same object, and those checks
run live in the validation article on the package website.

You can usually see the answer before you compute it. Plot the sorts and
each completed grid becomes its own pyramid, every statement on its
tile, the color giving the column it was placed in. Two like-minded
sorters put the same statements under the same colors, and a panel that
shares one viewpoint repeats itself pyramid after pyramid. The numbers
that follow put a figure to what the picture shows.

## Installation

``` r
# install.packages("remotes")
remotes::install_github("rdazadda/bayesqm")
```

CRAN still carries 0.1.0, an earlier model fitted through Stan. This
version is a different model with its own sampler. Install from GitHub
until 0.2.0 reaches CRAN.

## A first analysis

One real panel ships with the package, the childhood obesity study of
Akhtar-Danesh (2023), so this runs without any data of your own. 33
participants sorted 42 statements about childhood obesity onto a
nine-column grid.

``` r
library(bayesqm)

obesity_sorts
#> Q-sort data
#>   statements  : 42   participants : 33 
#>   distribution: 2 4 5 6 8 6 5 4 2   (sum = 42 )
#>   value range : [-4, 4]
#>   source      : excel:Childhood obesity dataset.xlsx
plot(obesity_sorts, participants = 1:12)
```

<img src="man/figures/README-obesity-sorts-1.png" alt="" width="100%" />

How many viewpoints does the panel hold? Fit two, three, and four
factors and let two checks read each candidate. One asks whether K
factors account for the shared structure. The other asks whether every
factor earns its place, meaning at least two participants define it and
at least one statement sets it apart from the rest.

``` r
sel <- select_k(fit_ladder(obesity_sorts, K_min = 2, K_max = 4,
                           seed = 11, quiet = TRUE))
plot_choice_k(sel)
```

<img src="man/figures/README-obesity-choicek-1.png" alt="" width="100%" />

The classical analysis of this panel reports three factors. Here every K
accounts for the panel and no factor at any K is set apart by a single
statement, so the rule refuses to select, and the verdict names what the
sorts show. One shared viewpoint. The one-factor fit is then the panel’s
report, and its array is the shared sort itself.

``` r
fit1 <- fit_bayesian(obesity_sorts, K = 1, seed = 11)
#> grid labels -4..4 recoded onto categories 1..9 (a monotone relabeling; the sorts are unchanged)
plot_factor_array(fit1)
```

<img src="man/figures/README-obesity-array-1.png" alt="" width="100%" />

Darker tiles are placements the posterior is more certain of. Each of
these fits takes a minute or two.

## When there are two viewpoints

Simulated sorts with two planted viewpoints, so the right answer is
known and the multi-factor reports have something to find.

``` r
sim <- generate_data(N = 14, J = 20, K = 2, noise_sd = 0.6,
                     primary_range = c(0.65, 0.9), seed = 7)
qdata <- qsort_data(sim$Y, distribution = sim$distribution)

lad <- fit_ladder(qdata, K_min = 2, K_max = 4, seed = 7, quiet = TRUE)
plot_choice_k(select_k(lad))
```

<img src="man/figures/README-choicek-1.png" alt="" width="100%" />

Two factors pass both checks, boxed on their row. Fit that model. The
sampler runs until a convergence check passes, and the printout reports
it.

``` r
fit <- fit_bayesian(qdata, K = 2, seed = 7)
#> grid labels -4..4 recoded onto categories 1..9 (a monotone relabeling; the sorts are unchanged)
fit
#> bayesqm fit: exact partition (rank-order) likelihood, PX-Gibbs
#>   14 participants, 20 statements, 2 factors; grid 1-2-2-3-4-3-2-2-1
#>   draws: 2000 kept (12000 iterations, burn 2000, thin 5)
#>   gate: passed (max Rhat 1.003; min ESS 1255 bulk / 1606 tail)
#>   alignment: pivot draw 332, mean congruence 0.89
#>   tables: compute_loadings(), compute_flags(), compute_factor_array(),
#>           compute_qdc(), claims()
```

What gets reported is decided by one rule. `claims()` keeps the most
probable flags, distinguishing statements, consensus statements, and
pairwise stars, and stops adding claims when the expected share of false
ones passes five percent.

``` r
claims(fit)
#> Selected claims at q = 0.05 (posterior expected FDR):
#>   flags             9 participants selected (expected false 0.41)
#>   distinguishing   11 listings selected (expected false 0.47)
#>   consensus         0 statements selected (expected false 0.00)
#>   stars             5 pairwise selected (expected false 0.17)
```

Behind the counts sit the full tables, and two of the views show the
result at a glance. The reported sort for each factor, on its own grid:

``` r
plot_factor_array(fit)
```

<img src="man/figures/README-array-1.png" alt="" width="100%" />

And statement by statement, where the two viewpoints separate and where
they agree:

``` r
plot_contrasts(fit)
```

<img src="man/figures/README-contrasts-1.png" alt="" width="100%" />

## What the package covers

**Import.** `read_qsort()` reads CSV, Excel (HTMLQ, FlashQ, and tablet
exports), PQMethod `.DAT`, Ken-Q JSON and Excel, KADE ZIP, and
Easy-HTMLQ Firebase JSON. Grids labeled `-4..+4`, from zero, or with any
distinct printed values work as they are, and the sorts are never
altered. `qsort_data()` builds the object from a plain matrix, and
`plot()` on it draws the pyramids.

**The tables.** `compute_loadings()` gives bounded loadings with
credible intervals, `compute_flags()` flag probabilities with an
explicit unclassified state, `compute_factor_array()` quota-exact
arrays, `compute_zscores()` statement scores, and `compute_qdc()`
distinguishing and consensus verdicts against a posterior critical
difference and the one-grid-column region. `factor_characteristics()` is
the per-factor block a results section quotes, and `claims()` selects
what gets reported. `rename_factors()` relabels every table at once, and
`rotate_factors()` and `flip_factor()` apply judgmental rotation to
every draw.

**The number of factors.** `fit_ladder()` fits the candidate models,
`select_k()` applies the two checks, and `plot_choice_k()` draws the
whole decision. `loo_ladder()` adds PSIS-LOO as directional
corroboration.

**Checks.** `check_fit()` runs the posterior-predictive checks, and
`check_persons()` separates sorts the model spans from shared viewpoints
it does not. Every fit passes a convergence check before it returns, and
`extend()` continues a chain draw for draw.

**Plots and draws.** The 11 views share one style, from the raw sorts to
the reported arrays, with `ggplot2::autoplot()` methods for loadings,
flags, contrasts, and the array; `bayesqm_set_colors()` themes them and
`save_bayesqm_plot()` writes the figures. `as.matrix()`, `as.array()`,
and `as.data.frame()` return the draws under Stan-style names, and the
`posterior::as_draws_*()` family is registered, so `bayesplot` and
`tidybayes` read a fit directly.

## Documentation

The package website is <https://rdazadda.github.io/bayesqm/>.
`vignette("bayesqm-intro")` walks one analysis end to end, and
`vignette("output-codebook")` defines every column of every reported
table. The articles go deeper: the validation checks, the decision rules
with a worked example of each verdict, and a start-to-finish walkthrough
of the obesity panel. The reference index groups every function by task,
and issues belong at <https://github.com/rdazadda/bayesqm/issues>.

## Citation

`citation("bayesqm")` gives the reference. Report the version with
`packageVersion("bayesqm")`.

## License

GPL (\>= 3); see <https://cran.r-project.org/web/licenses/GPL-3> for the
full text.

bayesqm is developed and maintained at the Center for Alaska Native
Health Research, University of Alaska Fairbanks.

# bayesqm

<!-- badges: start -->
[![License: GPL v3](https://img.shields.io/badge/License-GPLv3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)
<!-- badges: end -->

**Bayesian Q methodology: probabilistic factor analysis for the study
of subjective opinions.**

`bayesqm` is the first fully Bayesian framework for Q-methodology
factor analysis. Since Stephenson (1935), Q analysis has relied on
frequentist factor extraction with the standard error rule of Brown
(1980), which returns participant-to-factor assignments as binary
flagged / unflagged decisions and offers no measure of uncertainty
about them. `bayesqm` replaces that with posterior credible intervals
for every loading, probabilistic factor membership P(dominant_i = k | Y)
that makes cross-loading visible and quantifiable, and a principled
K-selection protocol built on element-wise PSIS-LOO. The model uses a
Student-t likelihood for robustness to idiosyncratic Q-sorts and
resolves rotational, sign, and label-permutation ambiguity through
the MatchAlign post-processing of Poworoznek et al. (2025). Posterior
sampling is via Stan (cmdstanr or rstan).

## What the package gives you

For each Q-sort dataset, `bayesqm` returns four things classical Q
analysis cannot:

1. **Posterior credible intervals for every participant loading**,
   replacing the binary flagged / unflagged classification of Brown
   (1980) with a continuous measure of assignment certainty.
2. **Probabilistic factor membership**, P(dominant_i = k | Y), making
   cross-loading visible and quantifiable rather than forcing each
   participant onto a single factor.
3. **A peak-plus-Sivula protocol for choosing K**, pairing the ELPD
   peak with the conservative parsimony diagnostic of Sivula et al.
   (2025) and surfacing the gap between them as informative about how
   strongly the data discriminate adjacent models.
4. **Posterior distinguishing and consensus statements**, reporting
   `P(D_j > delta | Y)` and `P(D_j < delta | Y)` from the posterior of
   an explicit viewpoint-divergence measure rather than a classical
   significance verdict.

## Installation

```r
# install.packages("remotes")
remotes::install_github("rdazadda/bayesqm")
```

### Stan backend

`bayesqm` fits models with Stan and needs a working backend:
`cmdstanr` (recommended) or `rstan`.

`cmdstanr` is not on CRAN and is installed from the Stan
R-universe. CmdStan itself must then be installed separately:

```r
install.packages("cmdstanr", repos = c("https://stan-dev.r-universe.dev", getOption("repos")))
cmdstanr::check_cmdstan_toolchain(fix = TRUE)
cmdstanr::install_cmdstan()
cmdstanr::cmdstan_version()   # verify
```

On Windows you also need Rtools; on macOS, the Xcode command-line
tools (`xcode-select --install`). See the Getting Started vignette
for the full setup walkthrough and the `rstan` alternative.

## Minimal workflow

```r
library(bayesqm)
qdata <- read_qsort("mystudy.csv")
fit   <- fit_bayesian(qdata, K = 3)
fit
```

`read_qsort()` auto-detects CSV, Excel (including HTMLQ, FlashQ, and
tablet-export variants), PQMethod `.DAT`, Ken-Q JSON or multi-sheet
Excel, KADE ZIP, and Easy-HTMLQ Firebase JSON.

`fit_bayesian()` returns an object of class `bayesqm_fit`. Standard R
accessors work as expected: `coef()`, `fitted()`, `residuals()`,
`sigma()`, `nobs()`, `family()`, `as.matrix()`, `summary()`,
`plot()`, plus `posterior_interval()` and `prior_summary()` from the
`rstantools` API.

## Choosing K

```r
run <- run_bayes(qdata, K_max = 5)
plot_elpd(run)
```

`run_bayes()` fits the model for K = 1..K_max and reports two
summaries:

- The **ELPD peak**: argmax of the expected log pointwise predictive
  density via element-wise PSIS-LOO (Vehtari, Gelman, & Gabry, 2017).
- The **Sivula parsimony diagnostic** (Sivula et al., 2025): a
  sequential rule that accepts a more complex K only when ΔELPD > 4
  and |ΔELPD| / SE(Δ) > 2.

The `$case` field labels their relationship as `agree`, `gap`, or
`reversed`. The ELPD peak is always the adopted K; the case label
indicates how confidently the data discriminate between adjacent
models, and a gap between the two is itself diagnostic information.

## Probabilistic factor membership

Rather than dichotomizing each loading into flagged or unflagged,
`bayesqm` reports the posterior probability that factor k is
participant i's dominant factor, P(dominant_i = k | Y).
`compute_dominant_sign()` adds the posterior probability that the
dominant loading is positive, so negative exemplars (a viewpoint held
by opposition) stay distinct. `classify_membership()` turns the
probabilities into a descriptive Strong / Moderate / Weak tier:

```r
compute_dominant_prob(fit$Lambda_draws)
compute_dominant_sign(fit$Lambda_draws)
classify_membership(fit$Lambda_draws)
plot_membership(fit)
```

Distinguishing and consensus statements come from the posterior of an
explicit viewpoint-divergence measure `D_j`, with the probabilities
`P(D_j > delta | Y)` and `P(D_j < delta | Y)` it implies. By default
`delta` is the reliability-adjusted critical difference from classical
Q analysis (Brown, 1980; Zabala & Pascual, 2016), computed from the
posterior dominant-factor counts (`critical_delta()`);
`suggest_delta()` (one forced-distribution category) is an
alternative:

```r
d <- critical_delta(fit$Lambda_draws)   # default separation (computed)
compute_divergence(fit$F_draws, delta = d)
plot_dist_cons(fit)                 # uses the fit's stored divergence summary
```

## Plotting

The package provides nine base-R plots and matching
`ggplot2::autoplot()` methods (registered when `ggplot2` and `ggdist`
are installed). All read their palette through `bayesqm_colors()` and
restore `par()` on exit:

```r
plot(fit)                       # cross-panel z-score dotchart
plot_loading_posterior(fit)     # loadings with 50% / 95% intervals
plot_membership(fit)            # dominant-factor probability heatmap
plot_dist_cons(fit)             # distinguishing/consensus divergence forest
plot_ppc(fit)                   # posterior predictive check on by-person correlations
plot_tucker(fit)                # MatchAlign alignment quality
plot_hyper(fit)                 # hyperparameter posteriors
plot_elpd(run)                  # ELPD across K with peak / Sivula annotations
```

For figure export, `save_bayesqm_plot()` opens the appropriate device
from the file extension and `caption_bayesqm()` returns a figure
caption that reports K, N, J, chains, coverage probability, and
convergence diagnostics.

## Compatibility with `qmethod`

`bayesqm_fit` deliberately mirrors the slot names from the `qmethod`
package (Zabala, 2015): `$dataset`, `$loa`, `$zsc`, `$zsc_n`,
`$f_char`, `$qdc`, `$flagged`. `$qdc` is the Bayesian divergence
table: per-viewpoint grid position and z-score with 95% CrI, then
`D_j` with 95% CrI, `pi_D`, and `pi_C`, not the classical
significance-label vocabulary. Intentional
Bayesian divergences are documented in `?bayesqm-package`:

- `$flagged` is defined probabilistically as
  P(argmax_k |λ_ik| = k) > 0.5, replacing Brown's (1980)
  significance-based rule.
- `$f_char$characteristics` omits the classical test-theory columns;
  factor-score uncertainty is already in the posterior credible
  intervals on `$ci_lower` and `$ci_upper`.

When you have substantive factor labels,
`rename_factors(fit, c("a", "b", "c"))` relabels every
factor-indexed slot in one call.

## Integration with `posterior` and `bayesplot`

`as.matrix(fit)`, `as.array(fit)`, and `as.data.frame(fit)` return
draws in Stan-style parameter naming (`Lambda[i,k]`, `F[j,k]`,
`nu`, `sigma`, `tau`). `as_draws_df()`, `as_draws_matrix()`, and
`as_draws_array()` are registered for `posterior::as_draws_*` when
`posterior` is installed, so `bayesplot` and `tidybayes` consume
`bayesqm_fit` objects natively.

## Citing the package

Run `citation("bayesqm")` for the canonical citation, or use:

> Azadda, R. D., AK-ACE Team, Hueffer, K., Peter, T., & Rasmus, S.
> (2026). bayesqm: Bayesian Q-Methodology Factor Analysis. R package
> version 0.1.0. https://github.com/rdazadda/bayesqm

BibTeX is available via `toBibtex(citation("bayesqm"))`.

## Where to look next

- `vignette("bayesqm-intro")` walks the full workflow end to end on
  a synthetic Q-sort.
- `?fit_bayesian` documents every prior and sampler option.
- `?run_bayes` covers the peak-plus-Sivula thresholds and the three
  case labels.
- `?bayesqm-membership` covers the probabilistic membership and
  distinguishing-statement summaries.
- Issues and feature requests:
  <https://github.com/rdazadda/bayesqm/issues>.

## Funding

This research was, in part, funded by the National Institutes of
Health (NIH) Agreement OT2HL158287 (Stacy Rasmus, Contact PI,
smrasmus@alaska.edu; Karsten Hueffer and Taa'aii Peter, MPIs). The
views and conclusions contained in this document are those of the
authors and should not be interpreted as representing the official
policies, either expressed or implied, of the NIH.

## License

Released under the GNU General Public License version 3 (GPL-3); see
<https://www.gnu.org/licenses/gpl-3.0.html> for the full text.

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  fig.width  = 7,
  fig.height = 4.5,
  dpi = 110
)

## ----message = FALSE, warning = FALSE-----------------------------------------
library(bayesqm)
library(ggplot2)

## ----setup-cmdstan, eval = FALSE----------------------------------------------
# install.packages("cmdstanr", repos = c("https://stan-dev.r-universe.dev", getOption("repos")))
# cmdstanr::check_cmdstan_toolchain(fix = TRUE)
# cmdstanr::install_cmdstan()
# cmdstanr::cmdstan_version()   # should print a version

## ----eval = FALSE-------------------------------------------------------------
# qdata <- read_qsort("mystudy.csv")   # CSV / Excel / HTMLQ / FlashQ
# qdata <- read_qsort("mystudy.DAT")   # PQMethod
# qdata <- read_qsort("mystudy.zip")   # KADE project

## -----------------------------------------------------------------------------
sim   <- generate_data(N = 33, J = 42, K = 3, seed = 1)
qdata <- qsort_data(sim$Y, distribution = sim$distribution)
qdata

## ----eval = FALSE-------------------------------------------------------------
# fit <- fit_bayesian(qdata, K = 3, chains = 4, iter = 2000,
#                     warmup = 1000, seed = 1)

## -----------------------------------------------------------------------------
fit <- demo_fit(N = 33, J = 42, K = 3, seed = 1)
fit

## -----------------------------------------------------------------------------
fit$diagnostics

## ----tucker, fig.cap = "MatchAlign alignment quality by factor."--------------
plot_tucker(fit)

## ----loadings, fig.cap = "Loadings with nested 50% and 95% credible intervals."----
plot_loading_posterior(fit)

## -----------------------------------------------------------------------------
head(compute_loadings(fit$Lambda_draws, prob = 0.95))

## ----zscores, fig.cap = "Posterior z-scores across factors with 50% and 95% credible intervals."----
plot(fit)

## ----zscore-one, fig.cap = "Posterior z-score for a single statement across factors."----
plot_zscore_posterior(fit, statement = 1)

## ----eval = FALSE-------------------------------------------------------------
# run <- run_bayes(qdata, K_max = 4, seed = 1,
#                  chains = 2, iter = 1500, warmup = 800)

## -----------------------------------------------------------------------------
run <- demo_run(K_max = 5, k_peak = 3, k_sivula = 1, case = "gap")
run

## ----elpd, fig.width = 7, fig.height = 4.2, fig.cap = "ΔELPD across K with the Sivula non-promotion band at |ΔELPD| < 4. Triangle marks the Sivula K, square marks the ELPD peak (the adopted K)."----
make_elpd_diff(run)

## -----------------------------------------------------------------------------
critical_delta(fit$Lambda_draws)
head(fit$qdc)

## ----dist-cons, fig.cap = "Posterior viewpoint divergence D_j with 95% credible intervals; statements ordered by P(D_j > delta | Y)."----
plot_dist_cons(fit)

## -----------------------------------------------------------------------------
head(classify_membership(fit$Lambda_draws))

## ----membership, fig.width = 6.5, fig.height = 5.5, fig.cap = "Blue tiles: posterior probability that each factor is dominant for a given participant (values printed in each cell). The right column shows the overall assignment verdict on an orange-red scale."----
make_dominant_panel(fit)

## ----ppc, fig.cap = "PPC on the by-person correlation matrix."----------------
plot_ppc(fit)

## ----hyper, fig.cap = "Posterior densities of nu, sigma, and tau."------------
plot_hyper(fit)

## ----eval = FALSE-------------------------------------------------------------
# save_bayesqm_plot("fig_loadings.pdf", plot_loading_posterior(fit))
# save_bayesqm_plot("fig_elpd.pdf",      plot_elpd(run),
#                   width = 3.5, height = 3)

## -----------------------------------------------------------------------------
caption_bayesqm(fit)

## ----eval = FALSE-------------------------------------------------------------
# bayesqm_set_colors("teal")   # "blue" (default), "red", "purple", "grey"
# plot(fit)
# bayesqm_set_colors("blue")

# fit_object.R
# The bayesqm_fit S3 class for the partition model: constructor, print,
# summary. The fit carries aligned draws, the raw draws (for realignment),
# the sampler state (for extend()), the gate report, and the alignment
# record; tables are computed from it by the compute_* functions.


#' @keywords internal
#' @noRd
new_bayesqm_fit <- function(call, Y, distribution, K, draws, draws_raw,
                            state, gate, align, prob, seed, settings,
                            fac_ids = NULL) {
  N <- ncol(Y); J <- nrow(Y)
  stmt_ids <- rownames(Y); if (is.null(stmt_ids)) stmt_ids <- paste0("S", seq_len(J))
  part_ids <- colnames(Y); if (is.null(part_ids)) part_ids <- paste0("P", seq_len(N))
  if (is.null(fac_ids)) fac_ids <- paste0("f", seq_len(K))

  name_draws <- function(d) {
    dimnames(d$F) <- list(NULL, stmt_ids, fac_ids)
    dimnames(d$Lambda) <- list(NULL, part_ids, fac_ids)
    dimnames(d$sigma) <- list(NULL, fac_ids)
    dimnames(d$s_i) <- list(NULL, part_ids)
    d
  }
  draws <- name_draws(draws)
  if (!is.null(draws_raw)) draws_raw <- name_draws(draws_raw)

  brief <- list(
    call         = call,
    date         = format(Sys.time(), "%Y-%m-%d %H:%M:%S"),
    pkg_version  = tryCatch(utils::packageVersion("bayesqm"),
                            error = function(e) "unknown"),
    model        = "exact partition (rank-order) likelihood, PX-Gibbs",
    K            = K,
    N            = N,
    J            = J,
    prob         = prob,
    seed         = seed,
    distribution = distribution,
    settings     = settings
  )

  structure(
    list(
      brief        = brief,
      dataset      = Y,
      distribution = distribution,
      draws        = draws,
      draws_raw    = draws_raw,
      state        = state,
      gate         = gate,
      align        = align
    ),
    class = "bayesqm_fit"
  )
}


# bounded loadings rho_ik = (S lambda_i)_k / sqrt(s_i^2 + 1), per draw; a
# deterministic transform of the aligned draws, computed on demand
#' @keywords internal
#' @noRd
.rho_draws <- function(fit) {
  d <- fit$draws
  T_ <- dim(d$F)[1]; J <- fit$brief$J; K <- fit$brief$K; N <- fit$brief$N
  rho <- array(NA_real_, c(T_, N, K), dimnames = dimnames(d$Lambda))
  for (t in seq_len(T_)) {
    F <- matrix(d$F[t, , ], J, K); L <- matrix(d$Lambda[t, , ], N, K)
    S <- crossprod(sweep(F, 2, colMeans(F))) / J
    q <- L %*% S
    rho[t, , ] <- q / sqrt(pmax(rowSums(q * L), 0) + 1)
  }
  rho
}


#' @export
print.bayesqm_fit <- function(x, ...) {
  b <- x$brief; g <- x$gate
  cat("bayesqm fit: ", b$model, "\n", sep = "")
  cat(sprintf("  %d participants, %d statements, %d factor%s; grid %s\n",
              b$N, b$J, b$K, if (b$K > 1) "s" else "",
              paste(b$distribution, collapse = "-")))
  cat(sprintf("  draws: %d kept (%d iterations, burn %d, thin %d%s)\n",
              dim(x$draws$F)[1], g$iterations, b$settings$burn,
              b$settings$thin,
              if (isTRUE(g$extended)) ", warm-extended" else ""))
  gate_line <- sprintf("max Rhat %.3f; min ESS %.0f bulk / %.0f tail",
                       g$rhat, g$ess_bulk, g$ess_tail)
  if (isTRUE(g$converged)) {
    cat("  gate: passed (", gate_line, ")\n", sep = "")
  } else {
    cat("  gate: NOT MET (", gate_line, ") - see extend()\n", sep = "")
  }
  cat(sprintf("  alignment: pivot draw %d, mean congruence %.2f\n",
              x$align$pivot, mean(x$align$congruence)))
  cat("  tables: compute_loadings(), compute_flags(), compute_factor_array(),\n",
      "          compute_qdc(), claims()\n", sep = "")
  invisible(x)
}


#' @export
summary.bayesqm_fit <- function(object, prob = NULL, ...) {
  print(object)
  prob <- if (is.null(prob)) object$brief$prob else prob
  rho <- .rho_draws(object)
  alpha <- 1 - prob
  m <- .summarize_draws(rho, mean)
  lo <- .summarize_draws(rho, quantile, probs = alpha / 2, names = FALSE)
  hi <- .summarize_draws(rho, quantile, probs = 1 - alpha / 2, names = FALSE)
  cat(sprintf("\nBounded loadings (posterior mean, %d%% interval), first %d participants:\n",
              round(100 * prob), min(object$brief$N, 8)))
  show <- seq_len(min(object$brief$N, 8))
  for (i in show) {
    entries <- sprintf("%s % .2f [% .2f, % .2f]",
                       colnames(m), m[i, ], lo[i, ], hi[i, ])
    cat(sprintf("  %-10s %s\n", rownames(m)[i], paste(entries, collapse = "  ")))
  }
  if (object$brief$N > 8)
    cat(sprintf("  ... %d more; compute_loadings() for the full table\n",
                object$brief$N - 8))
  invisible(object)
}

# The bayesqm_fit object: structure, dimnames, print, summary.

test_that("the fit carries draws, gate, and alignment with dimnames", {
  fit <- make_fake_fit(N = 6, J = 10, K = 2)
  expect_s3_class(fit, "bayesqm_fit")
  expect_named(fit, c("brief", "dataset", "distribution", "draws",
                      "draws_raw", "state", "gate", "align"))
  expect_identical(dimnames(fit$draws$Lambda)[[2]], paste0("P", 1:6))
  expect_identical(dimnames(fit$draws$F)[[2]], paste0("S", 1:10))
  expect_identical(dimnames(fit$draws$sigma)[[2]], c("f1", "f2"))
  expect_identical(fit$brief$K, 2)
  expect_identical(fit$brief$N, 6L)
  expect_identical(dim(fit$dataset), c(10L, 6L))
})

test_that("print names the model, the gate, and the alignment", {
  fit <- make_fake_fit()
  out <- paste(capture.output(print(fit)), collapse = "\n")
  expect_match(out, "exact partition \\(rank-order\\) likelihood")
  expect_match(out, "gate: passed")
  expect_match(out, "max Rhat 1.004")
  expect_match(out, "alignment: pivot draw 1")
  expect_match(out, "compute_loadings")
})

test_that("a failed gate prints NOT MET and an extended fit says so", {
  fit <- make_fake_fit(converged = FALSE, extended = TRUE)
  out <- paste(capture.output(print(fit)), collapse = "\n")
  expect_match(out, "gate: NOT MET")
  expect_match(out, "extend\\(\\)")
  expect_match(out, "warm-extended")
})

test_that("summary previews bounded loadings inside [-1, 1]", {
  fit <- make_fake_fit()
  out <- paste(capture.output(summary(fit)), collapse = "\n")
  expect_match(out, "Bounded loadings")
  rho <- .rho_draws(fit)
  expect_true(all(abs(rho) <= 1))
})

# ghk.R
# GHK evaluation of the partition likelihood, for person-level
# log-likelihoods and the PSIS-LOO bridge. The draw follows ascending
# category order with a running lower bound: that is exactly the cone
# event the quotas define.

#' @keywords internal
#' @noRd
ghk_loglik <- function(y, m, distr, R = 512, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  J <- length(y)
  ord <- order(y)
  cat_of <- y[ord]; mu <- m[ord]
  L <- rep(-Inf, R); cmax <- rep(-Inf, R); lw <- numeric(R)
  cur <- cat_of[1]
  for (idx in seq_len(J)) {
    if (cat_of[idx] != cur) { L <- cmax; cmax <- rep(-Inf, R); cur <- cat_of[idx] }
    plo <- pnorm(L - mu[idx])
    p <- 1 - plo
    lw <- lw + log(pmax(p, 1e-300))
    u <- mu[idx] + qnorm(pmin(pmax(plo + runif(R) * p, 1e-15), 1 - 1e-15))
    cmax <- pmax(cmax, pmax(u, L))
  }
  M <- max(lw)
  M + log(mean(exp(lw - M)))
}


#' Person-level partition log-likelihoods
#'
#' @description
#' GHK-simulated log-likelihood of each participant's sort under each
#' kept posterior draw (thinned to `draws`), the input PSIS-LOO needs.
#' The GHK seed depends only on the draw index and the person, never on
#' `K`, so log-likelihoods are common-random-number comparable across a
#' ladder of fits.
#'
#' @param fit A `bayesqm_fit`.
#' @param draws Posterior draws to evaluate (default 100, thinned evenly).
#' @param R GHK replications per evaluation (default 64).
#' @param seed Base seed for the GHK draws (default 1).
#'
#' @return A `draws x N` matrix of log-likelihoods.
#'
#' @export
loglik_person <- function(fit, draws = 100, R = 64, seed = 1) {
  assert_bayesqm_fit(fit)
  # the GHK seeds are deterministic; leave the caller's RNG stream untouched
  if (exists(".Random.seed", envir = globalenv())) {
    old_rng <- get(".Random.seed", envir = globalenv())
    on.exit(assign(".Random.seed", old_rng, envir = globalenv()), add = TRUE)
  }
  d <- fit$draws
  T_ <- dim(d$F)[1]; N <- fit$brief$N; J <- fit$brief$J; K <- fit$brief$K
  Y <- t(fit$dataset)
  td <- unique(round(seq(1, T_, length.out = min(draws, T_))))
  ll <- matrix(NA_real_, length(td), N)
  for (ti in seq_along(td)) {
    t <- td[ti]
    Eta <- matrix(d$Lambda[t, , ], N, K) %*% t(matrix(d$F[t, , ], J, K))
    ll[ti, ] <- vapply(seq_len(N), function(i)
      ghk_loglik(Y[i, ], Eta[i, ], fit$distribution, R = R,
                 seed = seed + 1000L * ti + i), numeric(1))
  }
  colnames(ll) <- dimnames(d$Lambda)[[2]]
  ll
}

# plot_sorts.R
# The data preview: every participant's completed sort drawn as their own
# pyramid, before any model runs. Adopted from the author's canhrQsort
# design: tiles stacked per grid column, a diverging ramp across the
# disagree-agree axis, luminance-adaptive statement labels, and the
# printed rank values on the axis.


# diverging ramp across grid columns (disagree -> agree)
#' @keywords internal
#' @noRd
.rank_ramp <- function(C) {
  grDevices::colorRampPalette(
    c("#6C3483", "#2E86C1", "#17A589", "#F4D03F",
      "#E67E22", "#E74C3C", "#922B21"))(C)
}

# white on dark tiles, near-black on light ones
#' @keywords internal
#' @noRd
.tile_text_col <- function(fill) {
  vapply(fill, function(col) {
    v <- grDevices::col2rgb(col)
    lum <- (0.299 * v[1] + 0.587 * v[2] + 0.114 * v[3]) / 255
    if (lum > 0.5) "gray20" else "white"
  }, character(1))
}


#' Preview every participant's sort
#'
#' @description
#' Draws each participant's completed sort as their own pyramid: one tile
#' per statement, stacked in its grid column, colored along the
#' disagree-agree axis. This is the look-at-the-decisions-first view:
#' run it straight after import, before any analysis, to see how every
#' participant grouped the statements. `plot(qdata)` does the same.
#'
#' @param x A `qsort_data` object or a `J x N` matrix of sorts.
#' @param participants Which participants to draw (names or indices);
#'   `NULL` (default) draws everyone.
#' @param per_page How many pyramids per page (default up to 12,
#'   arranged automatically); further participants continue on the next
#'   page.
#' @param labels Optional statement labels for the tiles (defaults to the
#'   statement ids).
#' @param cex Tile label size; `NULL` (default) adapts to the tallest
#'   column.
#'
#' @return The input, invisibly.
#'
#' @examples
#' distr <- c(1, 2, 3, 2, 1)
#' set.seed(1)
#' Y <- replicate(4, {
#'   r <- integer(9)
#'   r[order(rnorm(9))] <- rep(seq_along(distr), distr)
#'   r - 3                                  # printed labels -2..+2
#' })
#' plot_sorts(qsort_data(Y, distribution = distr, validate = FALSE))
#'
#' @export
plot_sorts <- function(x, participants = NULL, per_page = 12,
                       labels = NULL, cex = NULL) {
  qd <- if (inherits(x, "qsort_data")) x
        else qsort_data(as.matrix(x), validate = FALSE)
  Y <- qd$Y
  J <- nrow(Y); N <- ncol(Y)
  grid_vals <- sort(unique(as.vector(Y[is.finite(Y)])))
  C <- length(grid_vals)
  max_h <- max(vapply(seq_len(N), function(i)
    max(tabulate(match(Y[, i], grid_vals), nbins = C)), integer(1)))

  part_ids <- colnames(Y); if (is.null(part_ids)) part_ids <- paste0("P", seq_len(N))
  stmt_ids <- rownames(Y); if (is.null(stmt_ids)) stmt_ids <- paste0("S", seq_len(J))
  if (is.null(labels)) labels <- stmt_ids
  sel <- if (is.null(participants)) seq_len(N)
         else if (is.character(participants)) match(participants, part_ids)
         else as.integer(participants)
  if (anyNA(sel) || any(sel < 1 | sel > N))
    .bq_abort("unknown participant in `participants`.")

  fills <- .rank_ramp(C)
  txt <- .tile_text_col(fills)
  if (is.null(cex)) cex <- if (max_h <= 5) 0.9 else if (max_h <= 8) 0.7 else 0.55

  op <- par(no.readonly = TRUE)
  on.exit(par(op), add = TRUE)
  pages <- split(sel, ceiling(seq_along(sel) / per_page))
  for (pg in pages) {
    n <- length(pg)
    nc <- ceiling(sqrt(n)); nr <- ceiling(n / nc)
    par(mfrow = c(nr, nc), mar = c(2.5, 0.5, 2, 0.5))
    for (i in pg) {
      plot(NULL, xlim = c(0.5, C + 0.5), ylim = c(0.35, max_h + 0.65),
           axes = FALSE, xlab = "", ylab = "", main = part_ids[i],
           cex.main = 1, asp = 1)
      axis(1, at = seq_len(C), labels = grid_vals, cex.axis = 0.8,
           lwd = 0, lwd.ticks = 0, line = -1)
      for (ci in seq_len(C)) {
        js <- which(Y[, i] == grid_vals[ci])
        for (h in seq_along(js)) {
          rect(ci - 0.47, h - 0.47, ci + 0.47, h + 0.47,
               col = fills[ci], border = "white", lwd = 1.2)
          text(ci, h, labels[js[h]], col = txt[ci], cex = cex, font = 2)
        }
      }
    }
  }
  invisible(x)
}


#' @export
plot.qsort_data <- function(x, ...) plot_sorts(x, ...)


# ggplot2 version, faceted small multiples; registered on autoplot
#' @keywords internal
#' @noRd
autoplot.qsort_data <- function(object, participants = NULL, labels = NULL,
                                ...) {
  if (!requireNamespace("ggplot2", quietly = TRUE))
    .bq_abort("autoplot needs ggplot2; plot_sorts() does not.")
  Y <- object$Y
  J <- nrow(Y); N <- ncol(Y)
  grid_vals <- sort(unique(as.vector(Y[is.finite(Y)])))
  C <- length(grid_vals)
  part_ids <- colnames(Y); if (is.null(part_ids)) part_ids <- paste0("P", seq_len(N))
  stmt_ids <- rownames(Y); if (is.null(stmt_ids)) stmt_ids <- paste0("S", seq_len(J))
  if (is.null(labels)) labels <- stmt_ids
  sel <- if (is.null(participants)) seq_len(N)
         else if (is.character(participants)) match(participants, part_ids)
         else as.integer(participants)

  rows <- list()
  for (i in sel) {
    for (ci in seq_len(C)) {
      js <- which(Y[, i] == grid_vals[ci])
      if (length(js))
        rows[[length(rows) + 1]] <- data.frame(
          participant = part_ids[i], column = ci,
          height = seq_along(js), rank = grid_vals[ci],
          label = labels[js], stringsAsFactors = FALSE)
    }
  }
  d <- do.call(rbind, rows)
  d$rank <- factor(d$rank, levels = grid_vals)
  fills <- stats::setNames(.rank_ramp(C), as.character(grid_vals))
  d$txt <- .tile_text_col(fills[as.character(d$rank)])
  d$participant <- factor(d$participant, levels = part_ids[sel])

  ggplot2::ggplot(d, ggplot2::aes(x = .data$column, y = .data$height,
                                  fill = .data$rank)) +
    ggplot2::geom_tile(colour = "white", linewidth = 0.4,
                       width = 0.95, height = 0.95) +
    ggplot2::geom_text(ggplot2::aes(label = .data$label,
                                    colour = .data$txt),
                       size = 2.6, fontface = "bold") +
    ggplot2::scale_fill_manual(values = fills, drop = FALSE) +
    ggplot2::scale_colour_identity() +
    ggplot2::scale_x_continuous(breaks = seq_len(C), labels = grid_vals) +
    ggplot2::facet_wrap(~participant) +
    ggplot2::labs(x = NULL, y = NULL, fill = "rank") +
    ggplot2::theme_minimal() +
    ggplot2::theme(axis.text.y = ggplot2::element_blank(),
                   panel.grid = ggplot2::element_blank(),
                   legend.position = "none")
}

# claims.R
# One posterior false-discovery rule for every claim. claims() assembles
# the four claim families from the same internals the table functions use,
# so a selected column can never disagree across calls.


#' Selected claims at a common false-discovery level
#'
#' @description
#' Selects every claim — participant flags, distinguishing listings,
#' consensus statements, and pairwise stars — by the one posterior
#' false-discovery rule at level `q`, and reports the expected number of
#' false claims per family alongside. The same code path produces the
#' `selected` columns of [compute_flags()] and [compute_qdc()], so the
#' tables and this object always agree.
#'
#' @param fit A `bayesqm_fit`.
#' @param q Posterior expected false-discovery bound (default 0.05).
#' @param flag_floor,cons_floor Selection floors for flags (0.5) and
#'   consensus statements (0.95).
#'
#' @return A `bayesqm_claims` object: selected-only tables `flags`,
#'   `distinguishing`, `consensus`, and `stars`, each with its
#'   `expected_false` count, plus the level `q`.
#'
#' @examples
#' claims(demo_fit())
#'
#' @export
claims <- function(fit, q = 0.05, flag_floor = 0.5, cons_floor = 0.95) {
  assert_bayesqm_fit(fit)
  fl <- .flag_table(fit, q = q, floor = flag_floor)
  flags <- fl[fl$selected, c("participant", "factor", "sign", "flag_prob")]
  rownames(flags) <- NULL

  distinguishing <- consensus <- stars <- NULL
  ef <- c(flags = attr(fl, "expected_false"), distinguishing = 0,
          consensus = 0, stars = 0)
  if (fit$brief$K >= 2) {
    qt <- .qdc_tables(fit, q = q, cons_floor = cons_floor)
    fac_ids <- dimnames(fit$draws$sigma)[[2]]
    pd <- qt$pair_stats$pi_dist
    idx <- which(qt$dist_pub, arr.ind = TRUE)
    distinguishing <- data.frame(
      statement = rownames(pd)[idx[, 1]],
      factor = fac_ids[idx[, 2]],
      dist_prob = pd[idx],
      stringsAsFactors = FALSE
    )
    distinguishing <- distinguishing[order(distinguishing$factor,
                                           -distinguishing$dist_prob), ]
    rownames(distinguishing) <- NULL

    cons_rows <- qt$qdc$verdict == "consensus"
    consensus <- data.frame(statement = qt$qdc$statement[cons_rows],
                            consensus_prob = qt$qdc$consensus_prob[cons_rows],
                            stringsAsFactors = FALSE)
    rownames(consensus) <- NULL

    ct <- qt$contrasts
    stars <- ct[ct$selected,
                c("statement", "pair", "median", "exceed_prob", "stars")]
    rownames(stars) <- NULL
    ef["distinguishing"] <- qt$expected_false["distinguishing"]
    ef["consensus"] <- qt$expected_false["consensus"]
    ef["stars"] <- qt$expected_false["stars"]
  }

  structure(
    list(flags = flags, distinguishing = distinguishing,
         consensus = consensus, stars = stars,
         expected_false = ef, q = q),
    class = "bayesqm_claims"
  )
}


#' @export
print.bayesqm_claims <- function(x, ...) {
  cat(sprintf("Selected claims at q = %.2f (posterior expected FDR):\n", x$q))
  fam <- function(name, tab, unit) {
    n <- if (is.null(tab)) 0L else nrow(tab)
    cat(sprintf("  %-15s %3d %s selected (expected false %.2f)\n",
                name, n, unit, x$expected_false[[name]]))
  }
  fam("flags", x$flags, "participants")
  fam("distinguishing", x$distinguishing, "listings")
  fam("consensus", x$consensus, "statements")
  fam("stars", x$stars, "pairwise")
  invisible(x)
}

# zzz.R
# S3 method registration for generics that live in other packages.
# posterior is in Imports, so its registrations are unconditional; without
# them as_draws_df(fit) would fall back to the default method. ggplot2's
# .data pronoun is referenced inside autoplot; declare it for R CMD check.

utils::globalVariables(".data")

.onLoad <- function(libname, pkgname) {
  ns <- asNamespace("posterior")
  registerS3method("as_draws",        "bayesqm_fit",
                   as_draws.bayesqm_fit,        envir = ns)
  registerS3method("as_draws_df",     "bayesqm_fit",
                   as_draws_df.bayesqm_fit,     envir = ns)
  registerS3method("as_draws_matrix", "bayesqm_fit",
                   as_draws_matrix.bayesqm_fit, envir = ns)
  registerS3method("as_draws_array",  "bayesqm_fit",
                   as_draws_array.bayesqm_fit,  envir = ns)
  registerS3method("as_draws_rvars",  "bayesqm_fit",
                   as_draws_rvars.bayesqm_fit,  envir = ns)
  registerS3method("as_draws_list",   "bayesqm_fit",
                   as_draws_list.bayesqm_fit,   envir = ns)
  registerS3method("ndraws",          "bayesqm_fit",
                   ndraws.bayesqm_fit,          envir = ns)

  if (requireNamespace("ggplot2", quietly = TRUE)) {
    ns <- asNamespace("ggplot2")
    registerS3method("autoplot", "bayesqm_fit", autoplot.bayesqm_fit,
                     envir = ns)
    registerS3method("autoplot", "qsort_data", autoplot.qsort_data,
                     envir = ns)
  }
}

# ladder.R
# The two-signal choice of K: fit a ladder of K, judge each rung on
# posterior-predictive adequacy (extra-factor check in the central band,
# no mutual unspanned cluster) and on parsimony (every factor supported:
# at least two selected flags and one selected distinguishing listing),
# and select the smallest adequate K with all factors supported.
# select_k() reads only stored per-rung summaries, so it re-runs at
# another q without refitting.


#' Fit the model over a ladder of K
#'
#' @description
#' Fits [fit_bayesian()] at each K in the ladder and stores, per rung, the
#' fit together with the adequacy and support evidence that
#' [select_k()] consumes. Expect roughly one full fit's runtime per rung;
#' a message up front says how many rungs are coming.
#'
#' @param Y As in [fit_bayesian()].
#' @param K_max Largest K to fit. `NULL` (default) uses
#'   `min(6, floor(N / 3))`, capped at 3 when `N <= 12` — with a dozen
#'   sorts or fewer, the data cannot formally discriminate adjacent K.
#' @param K_min Smallest K to fit (default 2).
#' @param q False-discovery level for the support evidence stored on
#'   each rung (default 0.05); [select_k()] can re-select at another q.
#' @param screen_draws,screen_mixes Budget for the per-rung person
#'   check (defaults 30 and 60; the cluster signal needs no more).
#' @param quiet Suppress per-rung messages (default `FALSE`).
#' @param ... Passed to [fit_bayesian()] (iterations, seed, ...).
#'
#' @return A `bayesqm_ladder`: the per-rung fits and evidence.
#'
#' @export
fit_ladder <- function(Y, K_max = NULL, K_min = 2, q = 0.05,
                       screen_draws = 30, screen_mixes = 60,
                       quiet = FALSE, ...) {
  qd <- if (inherits(Y, "qsort_data")) Y
        else qsort_data(as.matrix(Y), validate = FALSE)
  N <- ncol(qd$Y)
  if (is.null(K_max))
    K_max <- if (N <= 12) min(3, N - 1) else min(6, floor(N / 3))
  ks <- seq.int(K_min, K_max)
  if (!length(ks) || K_max < K_min)
    .bq_abort("empty ladder: K_max (%d) is below K_min (%d).", K_max, K_min)
  if (!quiet)
    message(sprintf(
      "fitting %d rungs (K = %s); expect roughly one fit's runtime per rung",
      length(ks), paste(range(ks), collapse = "..")))

  rungs <- vector("list", length(ks))
  for (i in seq_along(ks)) {
    K <- ks[i]
    if (!quiet) message(sprintf("K = %d ...", K))
    fit <- fit_bayesian(qd, K = K, quiet = TRUE, ...)
    pc <- check_persons(fit, draws = screen_draws, mixes = screen_mixes)
    ck <- check_fit(fit, draws = 50)
    cl <- claims(fit, q = q)
    fac_ids <- dimnames(fit$draws$sigma)[[2]]
    supported <- vapply(fac_ids, function(f)
      sum(cl$flags$factor == f) >= 2 &&
        any(cl$distinguishing$factor == f), logical(1))
    rungs[[i]] <- list(
      K = K, fit = fit,
      t1b = ck$extra_factor$percentile,
      cluster = unspanned_cluster(pc),
      supported = supported,
      converged = fit$gate$converged,
      flags = cl$flags, distinguishing = cl$distinguishing
    )
  }
  structure(list(K = ks, rungs = rungs, q = q, N = N),
            class = "bayesqm_ladder")
}


#' Choose K by the two-signal rule
#'
#' @description
#' Signal one is posterior-predictive adequacy: the extra-factor check
#' sits inside the central band and the person check shows no mutual
#' unspanned cluster. Signal two is parsimony: every factor is supported,
#' meaning at least two selected flags and at least one selected
#' distinguishing listing. The selection is the smallest adequate K with
#' all factors supported. When no factor is supported at any rung the
#' verdict separates two opposite situations: a panel sharing nothing (no
#' factor ever attracts two flags) and a panel so unanimous that flags
#' abound but no statement distinguishes any pair — a single shared
#' viewpoint, reported as such. When adequacy and support never coincide,
#' both candidate solutions are reported rather than forcing a winner.
#'
#' @param ladder A `bayesqm_ladder`.
#' @param q Re-select the support evidence at this level; `NULL` (the
#'   default) keeps the level the ladder stored.
#' @param band Central adequacy band for the extra-factor percentile
#'   (default `c(0.05, 0.95)`).
#'
#' @return A `bayesqm_selection` with the verdict, the per-rung evidence
#'   table, and the selected `K` (or `NA`).
#'
#' @export
select_k <- function(ladder, q = NULL, band = c(0.05, 0.95)) {
  if (!inherits(ladder, "bayesqm_ladder"))
    .bq_abort("`ladder` must come from fit_ladder().")
  rungs <- ladder$rungs
  if (!is.null(q) && q != ladder$q) {
    for (i in seq_along(rungs)) {
      cl <- claims(rungs[[i]]$fit, q = q)
      fac_ids <- dimnames(rungs[[i]]$fit$draws$sigma)[[2]]
      rungs[[i]]$flags <- cl$flags
      rungs[[i]]$distinguishing <- cl$distinguishing
      rungs[[i]]$supported <- vapply(fac_ids, function(f)
        sum(cl$flags$factor == f) >= 2 &&
          any(cl$distinguishing$factor == f), logical(1))
    }
  }

  # per-factor evidence behind every support verdict, for the display
  detail <- do.call(rbind, lapply(rungs, function(r) {
    fac_ids <- dimnames(r$fit$draws$sigma)[[2]]
    data.frame(
      K = r$K, factor = seq_along(fac_ids),
      flags = vapply(fac_ids, function(f)
        sum(r$flags$factor == f), integer(1)),
      has_dist = vapply(fac_ids, function(f)
        any(r$distinguishing$factor == f), logical(1)),
      supported = r$supported, row.names = NULL
    )
  }))

  tab <- data.frame(
    K = ladder$K,
    converged = vapply(rungs, function(r) r$converged, logical(1)),
    extra_factor = vapply(rungs, function(r) r$t1b, numeric(1)),
    cluster = vapply(rungs, function(r) r$cluster, logical(1)),
    adequate = NA, factors_supported = vapply(rungs, function(r)
      sum(r$supported), integer(1)),
    all_supported = vapply(rungs, function(r) all(r$supported), logical(1))
  )
  tab$adequate <- tab$extra_factor >= band[1] & tab$extra_factor <= band[2] &
    !tab$cluster

  sel <- which(tab$adequate & tab$all_supported)
  K_star <- if (length(sel)) ladder$K[min(sel)] else NA_integer_

  verdict <-
    if (!any(unlist(lapply(rungs, function(r) r$supported)))) {
      # nothing supported can mean two opposite things: a panel sharing
      # nothing, or a panel so unanimous that no statement distinguishes
      # any pair of factors
      if (any(detail$flags >= 2) && !any(detail$has_dist)) "single_viewpoint"
      else "no_shared_structure"
    }
    else if (!is.na(K_star)) "selected"
    else if (any(tab$adequate) && any(tab$all_supported)) "tension"
    else if (any(tab$adequate)) "adequate_but_unsupported"
    else "no_adequate_rung"

  structure(
    list(K = K_star, verdict = verdict, table = tab, detail = detail,
         band = band, q = if (is.null(q)) ladder$q else q),
    class = "bayesqm_selection"
  )
}

#' @export
print.bayesqm_selection <- function(x, ...) {
  cat(sprintf("Choice of K, two-signal rule (q = %.2f, band %.2f-%.2f):\n",
              x$q, x$band[1], x$band[2]))
  print(x$table, digits = 2, row.names = FALSE)
  msg <- switch(x$verdict,
    selected = sprintf("selected K = %d: the smallest adequate rung with every factor supported.", x$K),
    single_viewpoint = "factors attract flags but no statement distinguishes any pair: the panel shares a single viewpoint. Report the one-factor solution (fit_bayesian with K = 1).",
    no_shared_structure = "no factor is supported at any rung: the data show no shared structure to factor.",
    tension = "adequacy and support never coincide; report both candidate solutions rather than forcing one.",
    adequate_but_unsupported = "adequate rungs exist but some factor is never supported there; the parsimony rule withholds a selection.",
    no_adequate_rung = "no rung passes the adequacy signal; look at the extra-factor percentiles and the person check."
  )
  cat("  ", msg, "\n", sep = "")
  invisible(x)
}


#' PSIS-LOO across the ladder, as directional corroboration
#'
#' @description
#' Person-level GHK log-likelihoods per rung, fed to PSIS-LOO. Reported as
#' directional corroboration only: with typical Q panels (N well below
#' 100), differences in expected log predictive density between adjacent K
#' have unreliable standard errors. The GHK seeds are common across rungs,
#' so comparisons are common-random-number paired.
#'
#' @param ladder A `bayesqm_ladder`.
#' @param draws,R As in [loglik_person()].
#'
#' @return A data frame: `K`, `elpd_loo`, `se`, `max_pareto_k`.
#'
#' @export
loo_ladder <- function(ladder, draws = 100, R = 64) {
  if (!inherits(ladder, "bayesqm_ladder"))
    .bq_abort("`ladder` must come from fit_ladder().")
  if (!requireNamespace("loo", quietly = TRUE))
    .bq_abort("loo_ladder() needs the loo package.")
  out <- lapply(ladder$rungs, function(r) {
    ll <- loglik_person(r$fit, draws = draws, R = R, seed = 1)
    lo <- suppressWarnings(loo::loo(ll))
    data.frame(K = r$K,
               elpd_loo = lo$estimates["elpd_loo", "Estimate"],
               se = lo$estimates["elpd_loo", "SE"],
               max_pareto_k = max(loo::pareto_k_values(lo)))
  })
  out <- do.call(rbind, out)
  message("directional corroboration only: elpd differences at N below ~100 have unreliable standard errors")
  out
}

# qsort_data.R
# The qsort_data class: a small container for a Q-sort dataset with its
# statement labels, participant IDs, forced distribution, and metadata.
# Also the validation helpers shared by the readers and the constructor.


#' Construct a validated qsort_data object
#'
#' @description
#' `qsort_data()` is the canonical constructor for a Q-sort dataset.
#' `validate_qsort()`, `check_distribution()`, and `infer_distribution()`
#' are the validation helpers used internally by the constructor and
#' by the file readers. `parse_distribution()` accepts a numeric vector,
#' a comma-/semicolon-/space-separated string, or a text file containing
#' one of those.
#'
#' @param Y A `J x N` numeric matrix (statements as rows, participants
#'   as columns) or a data frame.
#' @param statements,participants Optional character vectors of IDs;
#'   default to `S1..SJ` and `P1..PN`.
#' @param distribution Optional integer vector of forced-distribution
#'   counts. Inferred from `Y[, 1]` when `NULL`.
#' @param metadata Optional named list of study-level info.
#' @param source Provenance string stored on the object.
#' @param validate If `TRUE` (default), run [validate_qsort()] and emit
#'   warnings / messages for any issues found.
#' @param qdata A `qsort_data` object or bare matrix, passed to
#'   `validate_qsort()`.
#' @param x Numeric vector, character string, or path to a file
#'   containing the forced distribution, passed to
#'   `parse_distribution()`.
#'
#' @return `qsort_data()` returns a `qsort_data` S3 list with fields
#'   `Y`, `statements`, `participants`, `distribution`, `metadata`, and
#'   `source`. `validate_qsort()` returns a list with `valid`, `issues`,
#'   `warnings`, and `summary`. `check_distribution()` returns a list
#'   with `ok`, `non_conforming`, and `grid_values`.
#'   `infer_distribution()` and `parse_distribution()` return integer
#'   vectors.
#'
#' @name qsort_data
#' @aliases validate_qsort check_distribution infer_distribution parse_distribution
#' @export
qsort_data <- function(Y, statements = NULL, participants = NULL,
                       distribution = NULL, metadata = list(),
                       source = "manual", validate = TRUE) {
  if (is.data.frame(Y)) Y <- as.matrix(Y)
  if (!is.matrix(Y)) stop("Y must be a matrix or data frame.")
  storage.mode(Y) <- "double"

  J <- nrow(Y); N <- ncol(Y)
  if (J < 1 || N < 1) stop("Y is empty.")

  if (is.null(statements)) {
    statements <- rownames(Y)
    if (is.null(statements)) statements <- paste0("S", seq_len(J))
  }
  if (length(statements) != J)
    stop("statements length (", length(statements),
         ") does not match nrow(Y) (", J, ").")

  if (is.null(participants)) {
    participants <- colnames(Y)
    if (is.null(participants)) participants <- paste0("P", seq_len(N))
  }
  if (length(participants) != N)
    stop("participants length (", length(participants),
         ") does not match ncol(Y) (", N, ").")

  rownames(Y) <- as.character(statements)
  colnames(Y) <- as.character(participants)

  if (is.null(distribution)) distribution <- infer_distribution(Y)

  obj <- structure(
    list(Y            = Y,
         statements   = as.character(statements),
         participants = as.character(participants),
         distribution = as.integer(distribution),
         metadata     = as.list(metadata),
         source       = source),
    class = "qsort_data")

  if (validate) {
    v <- validate_qsort(obj)
    for (msg in v$issues)   warning(msg, call. = FALSE)
    for (msg in v$warnings) message(msg)
  }
  obj
}


#' @rdname qsort_data
#' @export
validate_qsort <- function(qdata, distribution = NULL) {
  Y <- if (inherits(qdata, "qsort_data")) qdata$Y else as.matrix(qdata)
  d <- if (!is.null(distribution)) distribution
       else if (inherits(qdata, "qsort_data")) qdata$distribution
       else NULL

  issues <- warns <- character(0)
  J <- nrow(Y); N <- ncol(Y)

  if (J < 5)
    issues <- c(issues, paste0("Only J=", J, " statements; at least 5 recommended."))
  if (N < 3)
    issues <- c(issues, paste0("Only N=", N, " participants; at least 3 required."))

  n_nonfinite <- sum(!is.finite(Y))
  if (n_nonfinite > 0)
    issues <- c(issues, paste0(n_nonfinite, " non-finite entries (NA/NaN/Inf) in Y."))

  if (!is.null(d) && length(d) > 0) {
    if (sum(d) != J) {
      issues <- c(issues, paste0("Forced distribution sums to ", sum(d),
                                 " but Y has J=", J, " statements."))
    } else {
      ck <- check_distribution(Y, d)
      if (!ck$ok) {
        n_bad <- length(ck$non_conforming)
        warns <- c(warns, paste0(
          n_bad, " of ", N, " participants do not match the forced distribution ",
          "(columns: ", paste(head(ck$non_conforming, 10), collapse = ", "),
          if (n_bad > 10) ", ..." else "", ")."))
      }
    }
  }

  list(valid    = length(issues) == 0,
       issues   = issues,
       warnings = warns,
       summary  = list(J = J, N = N, n_nonfinite = n_nonfinite))
}


#' @rdname qsort_data
#' @export
check_distribution <- function(Y, distribution) {
  n_pos <- length(distribution)
  # the printed labels are conventional (any monotone scheme: -4..+4,
  # 0-based, 1..C); take the ordered values the sorts actually use
  grid_vals <- sort(unique(Y[is.finite(Y)]))
  if (length(grid_vals) != n_pos)
    return(list(ok = FALSE, non_conforming = seq_len(ncol(Y)),
                grid_values = grid_vals))

  non_conforming <- integer(0)
  for (i in seq_len(ncol(Y))) {
    tab <- tabulate(match(Y[, i], grid_vals), nbins = n_pos)
    if (!all(tab == distribution)) non_conforming <- c(non_conforming, i)
  }
  list(ok             = length(non_conforming) == 0,
       non_conforming = non_conforming,
       grid_values    = grid_vals)
}


#' @rdname qsort_data
#' @export
infer_distribution <- function(Y) {
  vals <- Y[is.finite(Y)]
  if (length(vals) == 0) return(integer(0))
  grid_vals <- sort(unique(vals))
  # majority vote across participants, so one aberrant sort (even the
  # first) cannot define the design and get everyone else blamed
  counts <- apply(as.matrix(Y), 2, function(col)
    paste(tabulate(match(col[is.finite(col)], grid_vals),
                   nbins = length(grid_vals)), collapse = ","))
  tab <- table(counts)
  cands <- names(tab)[tab == max(tab)]
  modal <- counts[counts %in% cands][1]     # first occurrence breaks ties
  as.integer(strsplit(modal, ",", fixed = TRUE)[[1]])
}


#' @rdname qsort_data
#' @export
parse_distribution <- function(x) {
  if (is.numeric(x)) return(as.integer(x))
  if (!is.character(x) || length(x) != 1)
    stop("parse_distribution() expects a numeric vector or a single string.")

  if (file.exists(x)) x <- readLines(x, warn = FALSE)[1]
  x <- trimws(x)

  if (grepl("[,;]", x)) {
    v <- suppressWarnings(as.numeric(strsplit(x, "\\s*[,;]\\s*")[[1]]))
    if (!any(is.na(v))) return(as.integer(v))
  }

  v <- suppressWarnings(as.numeric(strsplit(x, "\\s+")[[1]]))
  if (!any(is.na(v))) return(as.integer(v))

  stop("Could not parse distribution from: ", x)
}


#' Print, summary, and matrix conversion for qsort_data
#'
#' @param x,object A `qsort_data` object.
#' @param ... Unused.
#' @return `print()` and `summary()` return the input invisibly;
#'   `as.matrix()` returns the `J x N` Q-sort matrix.
#' @name qsort_data-methods
#' @export
print.qsort_data <- function(x, ...) {
  cat("Q-sort data\n")
  cat("  statements  :", length(x$statements),
      "  participants :", length(x$participants), "\n")
  cat("  distribution:", paste(x$distribution, collapse = " "),
      "  (sum =", sum(x$distribution), ")\n")
  cat("  value range : [", min(x$Y, na.rm = TRUE), ", ",
      max(x$Y, na.rm = TRUE), "]\n", sep = "")
  cat("  source      :", x$source, "\n")
  invisible(x)
}


#' @rdname qsort_data-methods
#' @export
summary.qsort_data <- function(object, ...) {
  v <- validate_qsort(object)
  cat("Q-sort data summary\n")
  cat("  statements  :", length(object$statements), "\n")
  cat("  participants:", length(object$participants), "\n")
  cat("  distribution:", paste(object$distribution, collapse = " "), "\n")
  cat("  source      :", object$source, "\n")
  if (length(v$issues) > 0) {
    cat("  issues:\n")
    for (msg in v$issues)   cat("    - ", msg, "\n", sep = "")
  }
  if (length(v$warnings) > 0) {
    cat("  warnings:\n")
    for (msg in v$warnings) cat("    - ", msg, "\n", sep = "")
  }
  invisible(object)
}


#' @rdname qsort_data-methods
#' @export
as.matrix.qsort_data <- function(x, ...) x$Y

# Edge cases that live at the data boundary.

test_that("qsort_data infers distribution from column 1 when NULL", {
  Y <- matrix(c(1, 1, 2, 2, 3,
                1, 2, 2, 3, 3), ncol = 2)
  expect_warning(qd <- qsort_data(Y), "participants")
  expect_identical(qd$distribution, c(2L, 2L, 1L))
})

test_that("validate_qsort reports distribution mismatches", {
  Y <- matrix(c(1, 1, 2, 2, 3,
                1, 2, 2, 3, 3), ncol = 2)
  v <- validate_qsort(Y, distribution = c(2, 2, 1))
  expect_match(v$warnings, "do not match the forced distribution")
})

test_that("K = 1 flows through fit, print, and accessors", {
  fit <- make_fake_fit(N = 5, J = 9, K = 1, T = 80)
  expect_identical(dim(coef(fit)), c(5L, 1L))
  out <- paste(capture.output(print(fit)), collapse = "\n")
  expect_match(out, "1 factor;")
  expect_identical(ncol(as.matrix(fit)), 5L + 9L + 1L + 5L)
})

test_that("qsort_data constructs from a matrix", {
  set.seed(1L)
  grid <- c(-2, -1, 0, 1, 2)
  Y    <- cbind(grid, sample(grid), sample(grid), sample(grid))
  obj  <- qsort_data(Y, distribution = c(1, 1, 1, 1, 1))

  expect_s3_class(obj, "qsort_data")
  expect_equal(dim(obj$Y), c(5, 4))
  expect_length(obj$statements, 5)
  expect_length(obj$participants, 4)
  expect_equal(obj$distribution, c(1L, 1L, 1L, 1L, 1L))
})

test_that("infer_distribution recovers the grid counts from column 1", {
  y1 <- c(-2, -1, -1, 0, 0, 0, 1, 1, 2)
  Y  <- cbind(y1, sample(y1), sample(y1))
  expect_equal(infer_distribution(Y), c(1L, 2L, 3L, 2L, 1L))
})

test_that("parse_distribution handles numeric, string, and separators", {
  expect_equal(parse_distribution(c(1, 2, 3, 2, 1)), c(1L, 2L, 3L, 2L, 1L))
  expect_equal(parse_distribution("1 2 3 2 1"),       c(1L, 2L, 3L, 2L, 1L))
  expect_equal(parse_distribution("1,2,3,2,1"),       c(1L, 2L, 3L, 2L, 1L))
  expect_equal(parse_distribution("1;2;3;2;1"),       c(1L, 2L, 3L, 2L, 1L))
})

test_that("check_distribution flags non-conforming columns", {
  Y <- cbind(c(-1, -1, 0, 1, 1),
             c(-1,  0, 0, 0, 1))
  res <- check_distribution(Y, distribution = c(2, 1, 2))
  expect_false(res$ok)
  expect_equal(res$non_conforming, 2L)
})

test_that("validate_qsort reports too-small datasets", {
  Y <- matrix(0, nrow = 2, ncol = 2)
  v <- validate_qsort(Y, distribution = c(1, 1))
  expect_false(v$valid)
  expect_true(any(grepl("statements", v$issues)))
  expect_true(any(grepl("participants", v$issues)))
})

# align.R
# Post-processing applied to saved draws; the sampler state is never
# modified. Conventions (centered, unit-variance score columns, scale pushed
# to the loadings), MatchAlign with a polarity canon, a stabilized final
# orientation, and sigma carried through every rotation.

# center/scale F per draw, counter-scale Lambda and sigma
#' @keywords internal
#' @noRd
apply_conventions <- function(fit) {
  T_ <- dim(fit$F)[1]; K <- fit$K
  for (t in 1:T_) {
    F <- matrix(fit$F[t, , ], ncol = K)
    L <- matrix(fit$Lambda[t, , ], ncol = K)
    F <- sweep(F, 2, colMeans(F))
    d <- sqrt(colMeans(F^2))
    fit$F[t, , ] <- sweep(F, 2, d, "/")
    fit$Lambda[t, , ] <- sweep(L, 2, d, "*")
    fit$sigma[t, ] <- fit$sigma[t, ] * d
  }
  fit
}

# rotate the lambda scale the same way lambda was rotated (diagonal of the
# rotated covariance; exact for permutation/sign, diagonal approx after R)
#' @keywords internal
#' @noRd
rotate_sigma <- function(sig, R)
  sqrt(pmax(diag(t(R) %*% diag(sig^2, length(sig)) %*% R), 0))

#' MatchAlign post-processing for partition-model draws
#'
#' @description
#' Resolves rotational, sign, and label-permutation ambiguity in posterior
#' draws by the MatchAlign procedure of Poworoznek et al. (2025), adapted to
#' the partition model: varimax rotation of the loadings per draw, a pivot
#' draw at the median condition number (overridable), pivot polarity fixed
#' so each factor's loadings skew positive (defining sorts load positively),
#' greedy signed matching of loading columns to the pivot, and a
#' Procrustes rotation. A final pass re-orients every draw to the varimax
#' of the aligned mean loadings, re-signed to positive skew, so the
#' delivered orientation does not inherit one draw's sampling noise. One
#' rotation is applied to loadings and scores alike, and the loading scale
#' `sigma` is carried through.
#'
#' Called for its side effect on the draw arrays of an engine-level fit;
#' users normally receive already-aligned draws from [fit_bayesian()] and
#' need this only to realign under a different pivot.
#'
#' @param fit A fit carrying draw arrays `F` (`[T, J, K]`), `Lambda`
#'   (`[T, N, K]`), `sigma` (`[T, K]`), and `K`.
#' @param pivot Optional draw index to use as the alignment pivot. `NULL`
#'   (the default) selects the draw whose loading condition number sits at
#'   the median.
#'
#' @return The fit with aligned `F`, `Lambda`, and `sigma`, and the chosen
#'   `pivot` index appended.
#'
#' @references
#' Poworoznek, E., Anceschi, N., Ferrari, F., & Dunson, D. (2025).
#'   Efficiently Resolving Rotational Ambiguity in Bayesian Matrix
#'   Sampling with Matching. *Bayesian Analysis*.
#'
#' @export
matchalign <- function(fit, pivot = NULL) {
  if (inherits(fit, "bayesqm_fit")) {
    if (is.null(fit$draws_raw))
      .bq_abort("realigning needs the raw draws; refit with keep_raw = TRUE.")
    b <- fit$brief
    raw <- c(fit$draws_raw, list(N = b$N, J = b$J, K = b$K))
    pp <- postprocess(raw, pivot = pivot)
    keep_fields <- c("F", "Lambda", "sigma", "s_i")
    return(new_bayesqm_fit(
      call = b$call, Y = fit$dataset, distribution = fit$distribution,
      K = b$K, draws = pp[keep_fields], draws_raw = fit$draws_raw,
      state = fit$state, gate = fit$gate,
      align = list(pivot = pp$pivot, congruence = pp$congruence),
      prob = b$prob, seed = b$seed, settings = b$settings,
      fac_ids = dimnames(fit$draws$sigma)[[2]]
    ))
  }
  T_ <- dim(fit$F)[1]; K <- fit$K
  if (K > 1) for (t in 1:T_) {
    R <- stats::varimax(matrix(fit$Lambda[t, , ], ncol = K), normalize = FALSE)$rotmat
    fit$F[t, , ] <- matrix(fit$F[t, , ], ncol = K) %*% R
    fit$Lambda[t, , ] <- matrix(fit$Lambda[t, , ], ncol = K) %*% R
    fit$sigma[t, ] <- rotate_sigma(fit$sigma[t, ], R)
  }
  if (is.null(pivot)) {
    cond <- sapply(1:T_, function(t) {
      d <- svd(matrix(fit$Lambda[t, , ], ncol = K))$d
      d[1] / max(d[K], 1e-12)
    })
    pivot <- which.min(abs(cond - stats::median(cond)))
  }
  # polarity canon on the pivot: positive loading skew per column
  Lp <- matrix(fit$Lambda[pivot, , ], ncol = K)
  Fp <- matrix(fit$F[pivot, , ], ncol = K)
  for (k in 1:K) {
    sk <- sum(Lp[, k]^3)
    if (sk == 0) sk <- sum(Fp[, k]^3)
    if (sk < 0) { Fp[, k] <- -Fp[, k]; Lp[, k] <- -Lp[, k] }
  }
  fit$F[pivot, , ] <- Fp; fit$Lambda[pivot, , ] <- Lp
  congruence <- numeric(T_)
  for (t in 1:T_) {
    F <- matrix(fit$F[t, , ], ncol = K); L <- matrix(fit$Lambda[t, , ], ncol = K)
    if (K > 1) {
      # greedy signed match of loading columns to the pivot, by l2 distance
      dpos <- outer(colSums(L^2), colSums(Lp^2), "+") - 2 * crossprod(L, Lp)
      dneg <- outer(colSums(L^2), colSums(Lp^2), "+") + 2 * crossprod(L, Lp)
      dist <- pmin(dpos, dneg)
      perm <- integer(K); sgn <- numeric(K); avail <- rep(TRUE, K)
      for (b in order(apply(dist, 2, min))) {         # pivot columns, best first
        a <- which.min(ifelse(avail, dist[, b], Inf))
        perm[b] <- a; avail[a] <- FALSE
        sgn[b] <- if (dpos[a, b] <= dneg[a, b]) 1 else -1
      }
      F <- sweep(F[, perm, drop = FALSE], 2, sgn, "*")
      L <- sweep(L[, perm, drop = FALSE], 2, sgn, "*")
      sig <- fit$sigma[t, perm]
      sv <- svd(crossprod(L, Lp))
      R <- sv$u %*% t(sv$v)
      F <- F %*% R; L <- L %*% R
      fit$sigma[t, ] <- rotate_sigma(sig, R)
    } else {
      sgn <- sign(sum(L * Lp)); if (sgn == 0) sgn <- 1
      F <- F * sgn; L <- L * sgn
    }
    fit$F[t, , ] <- F; fit$Lambda[t, , ] <- L
    num <- colSums(L * Lp)
    den <- sqrt(colSums(L^2) * colSums(Lp^2))
    congruence[t] <- mean(abs(num) / pmax(den, 1e-12))
  }
  fit$pivot <- pivot
  fit$congruence <- congruence
  fit
}

# re-orient every draw to the varimax of the aligned-mean loadings,
# polarity re-fixed there; only the common orientation moves
#' @keywords internal
#' @noRd
stabilize_orientation <- function(fit) {
  K <- fit$K
  if (K < 2) return(fit)
  T_ <- dim(fit$F)[1]
  Lm <- apply(fit$Lambda, c(2, 3), mean)
  R <- stats::varimax(Lm, normalize = FALSE)$rotmat
  Lr <- Lm %*% R
  sgn <- vapply(1:K, function(k) {
    sk <- sum(Lr[, k]^3)
    if (sk == 0) 1 else sign(sk)
  }, numeric(1))
  R <- sweep(R, 2, sgn, "*")
  for (t in 1:T_) {
    fit$F[t, , ] <- matrix(fit$F[t, , ], ncol = K) %*% R
    fit$Lambda[t, , ] <- matrix(fit$Lambda[t, , ], ncol = K) %*% R
    fit$sigma[t, ] <- rotate_sigma(fit$sigma[t, ], R)
  }
  fit
}

# conventions -> align -> stable orientation -> conventions; the last pass
# restores exact unit-sd columns so f_jk keeps its z-score reading
#' @keywords internal
#' @noRd
postprocess <- function(fit, pivot = NULL)
  apply_conventions(stabilize_orientation(
    matchalign(apply_conventions(fit), pivot = pivot)))

# Alignment: conventions exactness, the polarity canon at the stabilized
# mean, sigma carriage, and invariance of the aligned draws to signed
# label permutations of the input.

align_fixture <- function(T_ = 40, N = 10, J = 12, K = 3, seed = 4) {
  set.seed(seed)
  base_F <- matrix(rnorm(J * K), J, K)
  base_L <- matrix(rnorm(N * K, 0, c(1.6, 1.0, 0.6)), N, K, byrow = TRUE)
  fit <- list(F = array(NA_real_, c(T_, J, K)),
              Lambda = array(NA_real_, c(T_, N, K)),
              sigma = matrix(NA_real_, T_, K),
              N = N, J = J, K = K)
  for (t in 1:T_) {
    th <- runif(1, 0, 2 * pi)
    R <- diag(K)
    R[1:2, 1:2] <- matrix(c(cos(th), sin(th), -sin(th), cos(th)), 2)
    p <- sample(K); s <- sample(c(-1, 1), K, replace = TRUE)
    M <- (R[, p] %*% diag(s, K))
    fit$F[t, , ] <- (base_F + matrix(rnorm(J * K, 0, .1), J, K)) %*% M
    fit$Lambda[t, , ] <- (base_L + matrix(rnorm(N * K, 0, .1), N, K)) %*% M
    fit$sigma[t, ] <- rotate_sigma(c(1.6, 1.0, 0.6), M)
  }
  fit
}

test_that("conventions are exact after the full postprocess", {
  pp <- postprocess(align_fixture())
  for (t in c(1, 17, 40)) {
    F <- matrix(pp$F[t, , ], ncol = 3)
    expect_lt(max(abs(colMeans(F))), 1e-10)
    expect_lt(max(abs(colMeans(F^2) - 1)), 1e-10)
  }
})

test_that("the polarity canon holds at the aligned mean", {
  pp <- postprocess(align_fixture())
  Lm <- apply(pp$Lambda, c(2, 3), mean)
  expect_true(all(colSums(Lm^3) > 0))
})

test_that("alignment is equivariant to signed label permutations of the input", {
  # the aligned posteriors must agree up to one GLOBAL label permutation
  # (the pivot's varimax column order follows its input order); signs are
  # fixed by the polarity canon, so the matched columns must agree exactly
  fit <- align_fixture()
  scr <- fit
  p <- c(3, 1, 2); s <- c(-1, 1, -1)
  for (t in seq_len(dim(fit$F)[1])) {
    scr$F[t, , ] <- sweep(matrix(fit$F[t, , p], ncol = 3), 2, s, "*")
    scr$Lambda[t, , ] <- sweep(matrix(fit$Lambda[t, , p], ncol = 3), 2, s, "*")
    scr$sigma[t, ] <- fit$sigma[t, p]
  }
  a <- postprocess(fit, pivot = 5)
  b <- postprocess(scr, pivot = 5)
  La <- apply(a$Lambda, c(2, 3), mean)
  Lb <- apply(b$Lambda, c(2, 3), mean)
  perm <- apply(abs(cor(La, Lb)), 1, which.max)
  expect_identical(sort(perm), 1:3)                    # a real permutation
  expect_equal(a$Lambda, b$Lambda[, , perm, drop = FALSE], tolerance = 1e-8)
  expect_equal(a$F, b$F[, , perm, drop = FALSE], tolerance = 1e-8)
  expect_equal(a$sigma, b$sigma[, perm, drop = FALSE], tolerance = 1e-8)
})

test_that("conventions hold for every draw and congruence to the pivot is high", {
  pp <- postprocess(align_fixture(), pivot = 5)
  for (t in seq_len(dim(pp$F)[1])) {
    F <- matrix(pp$F[t, , ], ncol = 3)
    expect_lt(max(abs(colMeans(F))), 1e-10)
    expect_lt(max(abs(colMeans(F^2) - 1)), 1e-10)
  }
  # the fixture is one structure plus noise, so alignment must recover it
  expect_gt(mean(pp$congruence), 0.9)
})

test_that("sigma is carried exactly through permutation and sign", {
  sig <- c(1.6, 1.0, 0.6)
  P <- diag(3)[, c(2, 3, 1)] %*% diag(c(-1, 1, -1))
  expect_equal(rotate_sigma(sig, P), sig[c(2, 3, 1)], tolerance = 1e-12)
})

test_that("K = 1 aligns by sign only", {
  set.seed(6)
  fit <- list(F = array(rnorm(20 * 8), c(20, 8, 1)),
              Lambda = array(rnorm(20 * 5), c(20, 5, 1)),
              sigma = matrix(1, 20, 1), N = 5, J = 8, K = 1)
  for (t in 1:20) {
    s <- sample(c(-1, 1), 1)
    fit$F[t, , 1] <- s * (1:8 / 4 + rnorm(8, 0, .1))
    fit$Lambda[t, , 1] <- s * (c(2, 1.5, 1, .5, .2) + rnorm(5, 0, .1))
  }
  pp <- postprocess(fit)
  Lm <- apply(pp$Lambda, c(2, 3), mean)
  expect_true(all(sign(Lm) == sign(Lm[1])))   # one common orientation
})

test_that("stored s_i is invariant to the whole alignment", {
  skip_if_not_installed("posterior")
  distr <- c(2, 3, 4, 2, 1)
  set.seed(11)
  Y <- quota_sort_rows(matrix(rnorm(8 * 12), 8, 12), distr)
  fit <- fit_partition(Y, distr, K = 2, n_iter = 400, burn = 100, thin = 5,
                       seed = 2)
  pp <- postprocess(fit)
  t <- 7
  F <- matrix(pp$F[t, , ], ncol = 2); L <- matrix(pp$Lambda[t, , ], ncol = 2)
  Fc <- sweep(F, 2, colMeans(F))
  expect_equal(sqrt(rowSums((L %*% (crossprod(Fc) / 12)) * L)),
               fit$s_i[t, ], tolerance = 1e-8)
})

# rotation.R
# Judgmental rotation and factor bookkeeping. Orientation after alignment
# is a reporting choice, as rotation is in classical Q: every aligned draw
# receives the same analyst-specified rotation, so posterior uncertainty
# propagates through the rotated summaries.


#' Rotate every aligned draw toward a target
#'
#' @description
#' Judgmental rotation: each aligned draw's statement scores are rotated
#' toward an analyst-specified target by Procrustes, and the loadings
#' receive the same rotation (its inverse transpose when `oblique = TRUE`),
#' so the latent utilities every draw implies are unchanged. Summaries
#' computed from the returned fit are summaries of the rotated solution,
#' with full posterior uncertainty.
#'
#' @param fit A `bayesqm_fit`.
#' @param target A `J x K` numeric matrix of target scores (for example, a
#'   hand-edited copy of the posterior-mean scores).
#' @param oblique Allow an oblique (non-orthogonal) rotation
#'   (default `FALSE`).
#'
#' @return The fit with rotated draws; `fit$align$rotated` records the
#'   call.
#'
#' @export
rotate_factors <- function(fit, target, oblique = FALSE) {
  assert_bayesqm_fit(fit)
  target <- as.matrix(target)
  J <- fit$brief$J; K <- fit$brief$K
  if (!all(dim(target) == c(J, K)))
    .bq_abort("target must be a %d x %d matrix of statement scores.", J, K)
  d <- fit$draws
  T_ <- dim(d$F)[1]
  for (t in seq_len(T_)) {
    F <- matrix(d$F[t, , ], J, K)
    L <- matrix(d$Lambda[t, , ], fit$brief$N, K)
    if (oblique) {
      R <- solve(crossprod(F), crossprod(F, target))
      Li <- L %*% t(solve(R))
    } else {
      sv <- svd(crossprod(F, target))
      R <- sv$u %*% t(sv$v)
      Li <- L %*% R
    }
    d$F[t, , ] <- F %*% R
    d$Lambda[t, , ] <- Li
    d$sigma[t, ] <- rotate_sigma(fit$draws$sigma[t, ], R)
  }
  fit$draws <- d
  fit$align$rotated <- if (oblique) "oblique target" else "orthogonal target"
  fit
}


#' Flip the pole of one factor
#'
#' @description
#' Reverses the sign of one factor's scores and loadings in every draw.
#' The polarity canon orients factors so defining sorts load positively;
#' this is the judgmental override.
#'
#' @param fit A `bayesqm_fit`.
#' @param factor Factor index or `"f2"`-style name.
#'
#' @return The fit with the factor's pole reversed.
#'
#' @export
flip_factor <- function(fit, factor) {
  assert_bayesqm_fit(fit)
  fac_ids <- dimnames(fit$draws$sigma)[[2]]
  k <- if (is.character(factor)) match(factor, fac_ids) else as.integer(factor)
  if (is.na(k) || k < 1 || k > fit$brief$K) .bq_abort("no such factor.")
  fit$draws$F[, , k] <- -fit$draws$F[, , k]
  fit$draws$Lambda[, , k] <- -fit$draws$Lambda[, , k]
  fit
}


#' Rename the factors
#'
#' @description
#' Replaces the `f1 ... fK` labels with substantive names everywhere the
#' fit carries them, so every downstream table and plot uses the new
#' names.
#'
#' @param fit A `bayesqm_fit`.
#' @param new_names Character vector of length `K`.
#'
#' @return The renamed fit.
#'
#' @export
rename_factors <- function(fit, new_names) {
  assert_bayesqm_fit(fit)
  K <- fit$brief$K
  if (length(new_names) != K || anyDuplicated(new_names))
    .bq_abort("new_names must be %d distinct names.", K)
  new_names <- as.character(new_names)
  relabel <- function(d) {
    if (is.null(d)) return(NULL)
    dimnames(d$F)[[3]] <- new_names
    dimnames(d$Lambda)[[3]] <- new_names
    dimnames(d$sigma)[[2]] <- new_names
    d
  }
  fit$draws <- relabel(fit$draws)
  fit$draws_raw <- relabel(fit$draws_raw)
  fit
}

# Judgmental rotation, pole flips, and renaming.

test_that("orthogonal rotation preserves the latent utilities", {
  fit <- make_fake_fit()
  target <- .summarize_draws(fit$draws$F, mean)[, c(2, 1)]   # swap the factors
  rot <- rotate_factors(fit, target)
  t <- 7
  U0 <- matrix(fit$draws$F[t, , ], 10, 2) %*%
    t(matrix(fit$draws$Lambda[t, , ], 6, 2))
  U1 <- matrix(rot$draws$F[t, , ], 10, 2) %*%
    t(matrix(rot$draws$Lambda[t, , ], 6, 2))
  expect_equal(U0, U1, tolerance = 1e-10)
  expect_identical(rot$align$rotated, "orthogonal target")
})

test_that("oblique rotation preserves the latent utilities", {
  fit <- make_fake_fit()
  target <- .summarize_draws(fit$draws$F, mean)
  target[, 1] <- target[, 1] + 0.3 * target[, 2]             # oblique mix
  rot <- rotate_factors(fit, target, oblique = TRUE)
  t <- 3
  U0 <- matrix(fit$draws$F[t, , ], 10, 2) %*%
    t(matrix(fit$draws$Lambda[t, , ], 6, 2))
  U1 <- matrix(rot$draws$F[t, , ], 10, 2) %*%
    t(matrix(rot$draws$Lambda[t, , ], 6, 2))
  expect_equal(U0, U1, tolerance = 1e-10)
})

test_that("rotation validates the target shape", {
  fit <- make_fake_fit()
  expect_error(rotate_factors(fit, matrix(0, 3, 2)), "10 x 2")
})

test_that("flip_factor reverses one pole everywhere", {
  fit <- make_fake_fit()
  fl <- flip_factor(fit, "f2")
  expect_identical(fl$draws$F[, , 2], -fit$draws$F[, , 2])
  expect_identical(fl$draws$Lambda[, , 2], -fit$draws$Lambda[, , 2])
  expect_identical(fl$draws$F[, , 1], fit$draws$F[, , 1])
  expect_error(flip_factor(fit, 5), "no such factor")
})

test_that("rename_factors relabels every carrier", {
  fit <- make_fake_fit()
  rn <- rename_factors(fit, c("Optimists", "Skeptics"))
  expect_identical(dimnames(rn$draws$F)[[3]], c("Optimists", "Skeptics"))
  expect_identical(dimnames(rn$draws$sigma)[[2]], c("Optimists", "Skeptics"))
  lo <- compute_loadings(rn)
  expect_true("Optimists_loading" %in% names(lo))
  expect_error(rename_factors(fit, c("a", "a")), "distinct")
})

# The public fitting interface: validation, migration errors, a real tiny
# fit, the gate warning, and the extend() identity through the public API.

tiny_data <- function(N = 6, J = 9, seed = 3) {
  distr <- get_distribution(J)
  set.seed(seed)
  Y <- t(quota_sort_rows(matrix(rnorm(N * J), N, J), distr))
  colnames(Y) <- paste0("P", seq_len(N))
  qsort_data(Y, distribution = distr, validate = FALSE)
}

test_that("removed 0.1.0 arguments give migration errors", {
  d <- tiny_data()
  expect_error(fit_bayesian(d, K = 2, robust = TRUE), "0.1.0 score-scale")
  expect_error(fit_bayesian(d, K = 2, iter = 2000), "iterations")
  expect_error(fit_bayesian(d, K = 2, chains = 4), "gated chain")
  expect_error(fit_bayesian(d, K = 2, banana = 1), "unused argument")
})

test_that("quota violations are refused by name", {
  d <- tiny_data()
  d$Y[1, 2] <- d$Y[1, 2] %% length(d$distribution) + 1
  err <- tryCatch(fit_bayesian(d, K = 2), error = function(e) conditionMessage(e))
  expect_match(err, "forced sorts matching the design grid")
  expect_match(err, "P2")
})

test_that("K and phase constraints are validated", {
  d <- tiny_data()
  expect_error(fit_bayesian(d, K = 6), "smaller than the number")
  expect_error(fit_bayesian(d, K = 2, iterations = 401, burn = 100, thin = 5),
               "multiple of thin")
})

test_that("a tiny fit returns a complete, converged object", {
  d <- tiny_data()
  fit <- fit_bayesian(d, K = 2, iterations = 500, burn = 100, thin = 5,
                      max_iterations = 500, seed = 4, rhat_max = Inf,
                      ess_min = 0, quiet = TRUE)
  expect_s3_class(fit, "bayesqm_fit")
  expect_identical(dim(fit$draws$F), c(80L, 9L, 2L))
  expect_true(fit$gate$converged)
  expect_false(fit$gate$extended)
  F <- matrix(fit$draws$F[5, , ], ncol = 2)
  expect_lt(max(abs(colMeans(F))), 1e-10)          # conventions applied
  expect_lt(max(abs(colMeans(F^2) - 1)), 1e-10)
  expect_identical(dim(fit$draws_raw$F), dim(fit$draws$F))
})

test_that("an unreachable gate warns and reports honestly", {
  d <- tiny_data()
  expect_warning(
    fit <- fit_bayesian(d, K = 2, iterations = 300, burn = 100, thin = 5,
                        max_iterations = 600, seed = 4, ess_min = 1e6,
                        quiet = TRUE),
    "gate was not met")
  expect_false(fit$gate$converged)
  expect_true(fit$gate$extended)
  expect_identical(fit$gate$iterations, 600L)
})

test_that("extend() reproduces one long chain through the public API", {
  d <- tiny_data()
  straight <- fit_bayesian(d, K = 2, iterations = 700, burn = 100, thin = 5,
                           max_iterations = 700, seed = 5, rhat_max = Inf,
                           ess_min = 0, quiet = TRUE)
  base <- fit_bayesian(d, K = 2, iterations = 400, burn = 100, thin = 5,
                       max_iterations = 400, seed = 5, rhat_max = Inf,
                       ess_min = 0, quiet = TRUE)
  runif(500)                                   # unrelated session RNG use
  grown <- extend(base, iterations = 300, quiet = TRUE)
  expect_identical(grown$draws_raw$F, straight$draws_raw$F)
  expect_identical(grown$draws_raw$Lambda, straight$draws_raw$Lambda)
  expect_identical(grown$draws_raw$sigma, straight$draws_raw$sigma)
  expect_equal(grown$draws$F, straight$draws$F, tolerance = 1e-12)
  expect_identical(grown$gate$iterations, 700L)
})

test_that("extend() requires the raw draws", {
  d <- tiny_data()
  fit <- fit_bayesian(d, K = 2, iterations = 300, burn = 100, thin = 5,
                      max_iterations = 300, seed = 4, rhat_max = Inf,
                      ess_min = 0, keep_raw = FALSE, quiet = TRUE)
  expect_error(extend(fit, 100), "keep_raw")
})

test_that("update() re-evaluates the stored call with new arguments", {
  d <- tiny_data()
  fit <- fit_bayesian(d, K = 2, iterations = 300, burn = 100, thin = 5,
                      max_iterations = 300, seed = 4, rhat_max = Inf,
                      ess_min = 0, quiet = TRUE)
  refit <- update(fit, seed = 9)
  expect_identical(refit$brief$seed, 9)
  expect_false(identical(refit$draws$F, fit$draws$F))
})

test_that("printed grid labels recode monotonically and change nothing", {
  d <- tiny_data()
  C <- length(d$distribution)
  ref <- fit_bayesian(d, K = 2, iterations = 300, burn = 100, thin = 5,
                      max_iterations = 300, seed = 4, rhat_max = Inf,
                      ess_min = 0, quiet = TRUE)
  shifted <- d$Y - ceiling(C / 2)                # -4..+4 style labels
  expect_message(
    fit <- fit_bayesian(shifted, K = 2, iterations = 300, burn = 100,
                        thin = 5, max_iterations = 300, seed = 4,
                        rhat_max = Inf, ess_min = 0),
    "monotone relabeling")
  expect_identical(fit$draws$F, ref$draws$F)     # identical inference
})

test_that("missing entries and alien value sets get their own errors", {
  d <- tiny_data()
  Yna <- d$Y; Yna[2, 3] <- NA
  expect_error(fit_bayesian(Yna, K = 2), "missing entr")
  Yalien <- d$Y * 10                             # 10,20,...: C values, fine
  idx <- which(Yalien == 20)                     # plenty of these remain
  Yalien[idx[1]] <- 3.7                          # now C+1 distinct values
  qa <- qsort_data(Yalien, distribution = d$distribution, validate = FALSE)
  expect_error(fit_bayesian(qa, K = 2), "distinct values")
})

test_that("zero-count grid columns collapse as a monotone relabeling", {
  distr <- c(2, 0, 3, 4)                          # interior zero, KADE-style
  set.seed(8)
  Yn <- t(quota_sort_rows(matrix(rnorm(6 * 9), 6, 9), c(2, 3, 4)))
  Yn[Yn >= 2] <- Yn[Yn >= 2] + 1                  # gap-coded on 1,3,4
  expect_message(
    fit <- fit_bayesian(qsort_data(Yn, distribution = distr, validate = FALSE),
                        K = 2, iterations = 300, burn = 100, thin = 5,
                        max_iterations = 300, seed = 4, rhat_max = Inf,
                        ess_min = 0),
    "zero-count")
  expect_equal(fit$distribution, c(2, 3, 4))
})

test_that("matchalign realigns a public fit under a chosen pivot", {
  d <- tiny_data()
  fit <- fit_bayesian(d, K = 2, iterations = 300, burn = 100, thin = 5,
                      max_iterations = 300, seed = 4, rhat_max = Inf,
                      ess_min = 0, quiet = TRUE)
  re <- matchalign(fit, pivot = 10)
  expect_s3_class(re, "bayesqm_fit")
  expect_identical(re$align$pivot, 10)
  expect_identical(re$draws_raw$F, fit$draws_raw$F)   # raw untouched
  slim <- fit_bayesian(d, K = 2, iterations = 300, burn = 100, thin = 5,
                       max_iterations = 300, seed = 4, rhat_max = Inf,
                       ess_min = 0, keep_raw = FALSE, quiet = TRUE)
  expect_error(matchalign(slim, pivot = 2), "keep_raw")
})

test_that("extend carries factor names and warns after judgmental rotation", {
  d <- tiny_data()
  fit <- fit_bayesian(d, K = 2, iterations = 300, burn = 100, thin = 5,
                      max_iterations = 300, seed = 5, rhat_max = Inf,
                      ess_min = 0, quiet = TRUE)
  named <- rename_factors(fit, c("Care", "Cost"))
  ext <- extend(named, 100, quiet = TRUE)
  expect_identical(dimnames(ext$draws$sigma)[[2]], c("Care", "Cost"))
  rot <- rotate_factors(fit, .summarize_draws(fit$draws$F, mean)[, 2:1])
  expect_warning(extend(rot, 100, quiet = TRUE), "not carried over")
})

test_that("K and budget arguments are validated up front", {
  d <- tiny_data()
  expect_error(fit_bayesian(d, K = 2.5), "whole number")
  expect_error(fit_bayesian(d, K = 2, iterations = 100, burn = 100), "exceed")
  expect_error(fit_bayesian(d, K = 2, thin = 0), "at least 1")
})

# import.R
# Format-specific readers for Q-sort data files. Every reader returns a
# qsort_data object in J x N orientation; read_qsort() dispatches to the
# right one by file extension and content (CSV, Excel, PQMethod .DAT, Ken-Q
# JSON / multi-sheet Excel, KADE ZIP, Easy-HTMLQ Firebase JSON).


#' Read Q-sort data from file
#'
#' @description
#' `read_qsort()` auto-detects the file format from extension and content
#' and dispatches to a specialised reader. The specialised readers are
#' also exported for explicit use:
#'
#' - `read_qsort_csv()`, `read_qsort_excel()` for generic CSV / Excel
#'   (with HTMLQ / FlashQ / Ken-Q auto-detection baked in)
#' - `read_pqmethod()` for PQMethod `.DAT` files
#' - `read_kenq()` for Ken-Q JSON or CSV
#' - `read_kenq_excel()` for multi-sheet Ken-Q Excel (Type 1 and Type 2,
#'   both old and Ver2 sub-formats)
#' - `read_kade_zip()` for KADE ZIP archives
#' - `read_easyhtml_firebase()` for Easy-HTMLQ Firebase JSON
#' - `read_statements()` for a standalone statement-text file
#'
#' All readers return a `qsort_data` object in `J x N` orientation
#' (statements as rows, participants as columns).
#'
#' @param file Path to the data file.
#' @param format For `read_qsort()`, `"auto"` (default) or one of
#'   `"csv"`, `"excel"`, `"pqmethod"`, `"kenq"`, `"kenq_excel"`,
#'   `"kade"`, `"easyhtml_firebase"`. For `read_kenq()`, one of
#'   `"auto"`, `"json"`, `"csv"`.
#' @param orientation For generic CSV/Excel: `"auto"`,
#'   `"statements_rows"`, or `"participants_rows"`.
#' @param id_col For generic CSV/Excel: `"auto"`, `"first"`, or `"none"`.
#' @param statements,distribution Optional overrides passed to
#'   [qsort_data()].
#' @param sheet Excel sheet name or index (default `1`).
#' @param statements_file For PQMethod, optional separate statements
#'   file.
#' @param column,id_column For `read_statements()`: column index or name.
#' @param ... Passed to the underlying reader.
#'
#' @return A `qsort_data` object, except `read_statements()` which
#'   returns a named character vector.
#'
#' @name read_qsort
#' @aliases read_qsort_csv read_qsort_excel read_pqmethod read_kenq read_kenq_excel read_kade_zip read_easyhtml_firebase read_statements
#' @export
read_qsort <- function(file, format = "auto", ...) {
  if (!file.exists(file)) stop("File not found: ", file)
  if (format == "auto") format <- detect_format(file)

  switch(format,
    csv      = read_qsort_csv(file, ...),
    excel    = read_qsort_excel(file, ...),
    pqmethod = read_pqmethod(file, ...),
    kenq     = read_kenq(file, ...),
    kenq_excel        = read_kenq_excel(file, ...),
    kade              = read_kade_zip(file, ...),
    easyhtml_firebase = read_easyhtml_firebase(file, ...),
    stop("Unknown format: ", format)
  )
}


#' @keywords internal
#' @noRd
detect_format <- function(file) {
  ext <- tolower(tools::file_ext(file))
  switch(ext,
    zip  = "kade",
    dat  = "pqmethod",
    json = detect_json_format(file),
    xlsx = ,
    xls  = "excel",
    csv  = ,
    tsv  = ,
    txt  = "csv",
    stop("No reader for extension: ", ext)
  )
}


#' @keywords internal
#' @noRd
detect_json_format <- function(file) {
  if (!requireNamespace("jsonlite", quietly = TRUE))
    stop("Package 'jsonlite' is required for JSON input.")
  d <- jsonlite::fromJSON(file, simplifyVector = FALSE)
  if (is.list(d) && "qSorts" %in% names(d))         return("kenq")
  if (is.list(d) && "respondentData" %in% names(d)) return("kenq")
  "easyhtml_firebase"
}


#' @rdname read_qsort
#' @export
read_qsort_csv <- function(file,
                           orientation = c("auto", "statements_rows", "participants_rows"),
                           id_col      = c("auto", "first", "none"),
                           statements = NULL, distribution = NULL, ...) {
  orientation <- match.arg(orientation)
  id_col      <- match.arg(id_col)
  raw <- utils::read.csv(file, stringsAsFactors = FALSE,
                         check.names = FALSE, ...)

  # HTMLQ and FlashQ are CSVs with distinctive headers; hand them off rather
  # than try to coerce the row/column structure.
  hdr <- tolower(names(raw))
  if (any(grepl("^uid$", hdr)) && any(grepl("datetime|timestamp", hdr)))
    return(parse_htmlq_frame(raw, variant = "htmlq",
                             source = paste0("htmlq:", basename(file))))
  if (any(grepl("^id$", hdr)) && any(grepl("^time$", hdr)))
    return(parse_htmlq_frame(raw, variant = "flashq",
                             source = paste0("flashq:", basename(file))))

  Y <- build_Y_from_frame(raw, orientation, id_col)
  qsort_data(Y, statements = statements, distribution = distribution,
             source = paste0("csv:", basename(file)))
}


#' @rdname read_qsort
#' @export
read_qsort_excel <- function(file, sheet = 1,
                             orientation = c("auto", "statements_rows", "participants_rows"),
                             id_col      = c("auto", "first", "none"),
                             statements = NULL, distribution = NULL, ...) {
  if (!requireNamespace("readxl", quietly = TRUE))
    stop("Package 'readxl' is required for Excel input.")
  orientation <- match.arg(orientation)
  id_col      <- match.arg(id_col)

  # Multi-sheet Ken-Q workbooks have sheets named "sorts", "statements",
  # "pattern", etc. Detect and hand off to the dedicated Ken-Q reader.
  sheets <- tolower(readxl::excel_sheets(file))
  if (any(sheets %in% c("sorts", "qsorts", "q-sorts", "q sorts")) &&
      length(sheets) >= 2)
    return(read_kenq_excel(file))

  raw <- as.data.frame(readxl::read_excel(file, sheet = sheet, ...),
                       stringsAsFactors = FALSE)

  # HTMLQ / FlashQ / Q-Sortware tablet exports are also shipped as .xlsx.
  # Their header signatures are the same as the CSV variants.
  hdr <- tolower(names(raw))
  if (any(grepl("^uid$", hdr)) && any(grepl("datetime|timestamp", hdr)))
    return(parse_htmlq_frame(raw, variant = "htmlq",
                             source = paste0("htmlq:", basename(file))))
  if (any(grepl("^id$", hdr)) && any(grepl("^time$", hdr)))
    return(parse_htmlq_frame(raw, variant = "flashq",
                             source = paste0("flashq:", basename(file))))

  Y <- build_Y_from_frame(raw, orientation, id_col)
  qsort_data(Y, statements = statements, distribution = distribution,
             source = paste0("excel:", basename(file)))
}


# Collapse a wide data frame down to a J x N numeric matrix with statements
# as rows. Three cases are handled:
#   (a) columns named qsort1..qsortN or p1..pN: those columns are
#       participants; any remaining column becomes the statement IDs.
#   (b) first column non-numeric: treated as an ID column; orientation
#       from the user's argument or the shape heuristic.
#   (c) all-numeric grid: orientation from the argument; the default
#       puts the longer dimension in the rows.
#' @keywords internal
#' @noRd
build_Y_from_frame <- function(df, orientation, id_col) {
  if (ncol(df) < 2) stop("Data has fewer than 2 columns.")

  nm <- tolower(names(df))
  qcols <- grep("^q(sort)?\\d+$", nm)
  pcols <- grep("^p\\d+$",        nm)
  use_cols <- if (length(qcols) >= 2) qcols
              else if (length(pcols) >= 2) pcols
              else integer(0)

  if (length(use_cols) > 0) {
    other <- setdiff(seq_len(ncol(df)), use_cols)
    ids <- if (length(other) >= 1) as.character(df[[other[1]]]) else NULL
    val <- df[, use_cols, drop = FALSE]
    mat <- suppressWarnings(as.matrix(sapply(val, as.numeric)))
    if (!is.null(ids)) rownames(mat) <- ids
    colnames(mat) <- names(df)[use_cols]
    return(mat)
  }

  first <- df[[1]]
  first_is_id <- switch(id_col,
    "first" = TRUE,
    "none"  = FALSE,
    "auto"  = is.character(first) || is.factor(first))
  if (first_is_id) {
    ids <- as.character(first)
    val <- df[, -1, drop = FALSE]
  } else {
    ids <- NULL
    val <- df
  }

  mat <- suppressWarnings(as.matrix(sapply(val, as.numeric)))
  if (!is.null(ids))   rownames(mat) <- ids
  colnames(mat) <- names(val)

  if (orientation == "auto")
    orientation <- if (nrow(mat) >= ncol(mat)) "participants_rows"
                   else                         "statements_rows"
  if (orientation == "participants_rows") mat <- t(mat)
  mat
}


# PQMethod .DAT files come in two header flavours. Simple (A): line 1 is a
# title, line 2 has n_statements / min / max. Extended (B): line 1 starts
# with numbers, line 2 has min / max followed by the distribution counts.
#' @rdname read_qsort
#' @export
read_pqmethod <- function(file, statements_file = NULL) {
  lines <- readLines(file, warn = FALSE)
  if (length(lines) < 3) stop("PQMethod file too short: ", file)

  line1 <- lines[1]
  l2_nums <- suppressWarnings(as.numeric(regmatches(lines[2],
    gregexpr("-?\\d+", lines[2]))[[1]]))
  if (length(l2_nums) < 3) stop("Invalid PQMethod header.")

  is_ext <- grepl("^\\s*\\d+\\s+\\d+\\s+", line1) && length(l2_nums) > 3

  if (is_ext) {
    title   <- trimws(sub("^\\s*[\\d\\s]+", "", line1, perl = TRUE))
    min_val <- l2_nums[1]
    max_val <- l2_nums[2]
    counts  <- l2_nums[-(1:2)]
    n_stmts <- as.integer(sum(counts))
  } else {
    title   <- trimws(line1)
    n_stmts <- l2_nums[1]
    min_val <- l2_nums[2]
    max_val <- l2_nums[3]
    counts  <- NULL
  }
  if (!is.finite(n_stmts) || n_stmts <= 0 || n_stmts > 500)
    stop("Bad PQMethod n_statements: ", n_stmts)

  # Data rows use fixed-width encoding: ID field followed by 2-character
  # chunks per sort value (negatives fit without a separator, e.g. " 5-1").
  # ID width = total line width minus 2 * n_statements.
  data_lines <- lines[-(1:2)]
  data_lines <- data_lines[nchar(trimws(data_lines)) > 0]
  id_width <- nchar(data_lines[1]) - 2L * n_stmts
  if (id_width < 2 || id_width > 40) id_width <- 10L

  sorts <- matrix(NA_real_, nrow = length(data_lines), ncol = n_stmts)
  ids   <- character(length(data_lines))
  for (r in seq_along(data_lines)) {
    ln <- data_lines[r]
    if (nchar(ln) >= id_width + 2L * n_stmts) {
      ids[r] <- trimws(substr(ln, 1, id_width))
      vstr   <- substr(ln, id_width + 1, id_width + 2L * n_stmts)
      for (j in seq_len(n_stmts))
        sorts[r, j] <- suppressWarnings(as.numeric(
          trimws(substr(vstr, (j - 1) * 2 + 1, j * 2))))
    } else {
      parts <- strsplit(trimws(ln), "\\s+")[[1]]
      ids[r] <- parts[1]
      v <- suppressWarnings(as.numeric(parts[-1]))[seq_len(n_stmts)]
      sorts[r, ] <- v
    }
  }
  rownames(sorts) <- ids
  colnames(sorts) <- paste0("S", seq_len(n_stmts))

  stmt_texts <- NULL
  if (!is.null(statements_file) && file.exists(statements_file))
    stmt_texts <- read_pqmethod_statements(statements_file)

  distribution <- if (!is.null(counts)) counts
                  else as.integer(tabulate(match(sorts[1, ],
                    seq(min_val, max_val)), nbins = max_val - min_val + 1L))

  qsort_data(Y = t(sorts),
             statements   = stmt_texts %||% colnames(sorts),
             participants = rownames(sorts),
             distribution = distribution,
             metadata = list(title = title, pqmethod_range = c(min_val, max_val)),
             source = paste0("pqmethod:", basename(file)))
}


#' @keywords internal
#' @noRd
read_pqmethod_statements <- function(file) {
  lines <- readLines(file, warn = FALSE)
  lines <- lines[nchar(trimws(lines)) > 0]
  trimws(sub("^\\s*\\d+[.):;\\s]+", "", lines))
}


#' @keywords internal
#' @noRd
parse_htmlq_frame <- function(df, variant = "htmlq", source = NULL) {
  col_names <- names(df)

  # Explicit statement columns first (s1, s2, ... or statement_1, ...); fall
  # back to any numeric column that isn't clearly an ID or timestamp.
  stmt_cols <- grep("^s\\d+$|^statement[_-]?\\d+$",
                    col_names, ignore.case = TRUE)
  if (length(stmt_cols) == 0) {
    num_cols <- which(vapply(df, is.numeric, logical(1)))
    drop_ids <- tolower(col_names[num_cols]) %in%
      c("uid", "id", "record_id", "time", "duration")
    stmt_cols <- num_cols[!drop_ids]
  }
  if (length(stmt_cols) == 0)
    stop("Could not identify statement columns in HTMLQ data.")

  sorts <- suppressWarnings(as.matrix(sapply(df[, stmt_cols, drop = FALSE],
                                             as.numeric)))
  colnames(sorts) <- col_names[stmt_cols]

  id_col <- grep("^uid$|^id$|^participant", col_names, ignore.case = TRUE)
  ids <- if (length(id_col) > 0) as.character(df[[id_col[1]]])
         else paste0("P", seq_len(nrow(sorts)))
  rownames(sorts) <- ids

  meta_cols <- setdiff(seq_along(col_names), c(stmt_cols, id_col))
  metadata <- if (length(meta_cols) > 0)
                as.list(df[, meta_cols, drop = FALSE]) else list()

  qsort_data(Y = t(sorts),
             statements   = colnames(sorts),
             participants = rownames(sorts),
             metadata = metadata,
             source = source %||% paste0(variant, ":<frame>"))
}


#' @rdname read_qsort
#' @export
read_kenq <- function(file, format = c("auto", "json", "csv")) {
  format <- match.arg(format)
  if (format == "auto")
    format <- if (tolower(tools::file_ext(file)) == "json") "json" else "csv"
  if (format == "json") parse_kenq_json(file) else parse_kenq_csv(file)
}


#' @keywords internal
#' @noRd
parse_kenq_json <- function(file) {
  if (!requireNamespace("jsonlite", quietly = TRUE))
    stop("Package 'jsonlite' is required for JSON input.")
  d <- jsonlite::fromJSON(file)

  if ("qSorts" %in% names(d)) {
    sorts <- if (is.matrix(d$qSorts)) d$qSorts else do.call(rbind, d$qSorts)
  } else if ("respondentData" %in% names(d)) {
    rd <- d$respondentData
    sorts <- if (is.data.frame(rd)) do.call(rbind, rd$sort)
             else do.call(rbind, lapply(rd, function(x) x$sort))
  } else {
    fb <- try_firebase_json(d)
    if (!is.null(fb)) return(fb)
    stop("Unrecognized Ken-Q / Easy-HTMLQ JSON structure.")
  }

  participants <- if ("participantNames" %in% names(d)) d$participantNames
                  else if (is.data.frame(d$respondentData))
                    d$respondentData$name %||% d$respondentData$id
                  else paste0("P", seq_len(nrow(sorts)))
  rownames(sorts) <- as.character(participants)

  stmts <- d$statementText %||% d$statements %||%
             paste0("S", seq_len(ncol(sorts)))
  colnames(sorts) <- if (length(stmts) == ncol(sorts)) stmts
                     else paste0("S", seq_len(ncol(sorts)))

  distribution <- d$sortPattern %||% d$distribution

  qsort_data(Y = t(sorts),
             statements = colnames(sorts),
             participants = rownames(sorts),
             distribution = distribution,
             metadata = list(kenq_version = d$version %||% "unknown"),
             source = paste0("kenq:", basename(file)))
}


#' @keywords internal
#' @noRd
parse_kenq_csv <- function(file) {
  df <- utils::read.csv(file, stringsAsFactors = FALSE, check.names = FALSE)
  parse_htmlq_frame(df, variant = "kenq",
                    source = paste0("kenq:", basename(file)))
}


# Easy-HTMLQ Firebase JSON: top-level keys are push IDs, each entry has a
# "sort" field holding pipe-delimited ranks.
#' @keywords internal
#' @noRd
try_firebase_json <- function(d) {
  if (is.data.frame(d) || !is.list(d)) return(NULL)
  has_sort <- vapply(d, function(x) is.list(x) && "sort" %in% names(x),
                     logical(1))
  if (!any(has_sort)) return(NULL)

  entries <- d[has_sort]
  rows <- list()
  names_out <- character()
  for (i in seq_along(entries)) {
    e <- entries[[i]]
    s <- gsub("\\+", "", as.character(e$sort))
    v <- suppressWarnings(as.numeric(strsplit(s, "\\|")[[1]]))
    if (length(v) == 0 || all(is.na(v))) next
    rows[[length(rows) + 1]] <- v
    key <- names(entries)[i]
    name <- e$name %||% e$email %||%
              substr(key, max(1, nchar(key) - 9), nchar(key))
    names_out <- c(names_out, as.character(name))
  }
  if (length(rows) == 0) return(NULL)

  sorts <- do.call(rbind, rows)
  rownames(sorts) <- names_out
  colnames(sorts) <- paste0("S", seq_len(ncol(sorts)))

  qsort_data(Y = t(sorts),
             statements = colnames(sorts),
             participants = rownames(sorts),
             metadata = list(format = "easyhtml_firebase"),
             source = "easyhtml:<json>")
}


#' @rdname read_qsort
#' @export
read_kenq_excel <- function(file) {
  if (!requireNamespace("readxl", quietly = TRUE))
    stop("Package 'readxl' is required for Excel input.")
  sheets <- readxl::excel_sheets(file)
  sheets_lower <- tolower(sheets)

  # Ken-Q templates ship empty primary sheets with populated "Example - "
  # sheets alongside; prefer the populated one.
  find_sheet <- function(targets) {
    i <- which(sheets_lower %in% targets)[1]
    if (!is.na(i)) {
      test <- readxl::read_excel(file, sheet = sheets[i], col_names = FALSE)
      has_data <- nrow(test) > 1 &&
        any(!is.na(unlist(test[2:min(nrow(test), 5), ])))
      if (has_data) return(sheets[i])
    }
    ex <- which(sheets_lower %in% paste0("example - ", targets))[1]
    if (!is.na(ex)) return(sheets[ex])
    if (!is.na(i)) return(sheets[i])
    NULL
  }

  sorts_sheet <- find_sheet(c("sorts", "qsorts", "q-sorts", "q sorts"))
  if (is.null(sorts_sheet))
    stop("Not a valid Ken-Q Excel file: no 'sorts' sheet.")

  stmts <- NULL
  sh <- find_sheet(c("statements", "statement"))
  if (!is.null(sh)) stmts <- parse_kenq_statements_sheet(file, sh)

  distribution <- NULL
  sh <- find_sheet(c("pattern", "patterns"))
  if (!is.null(sh)) {
    mult <- parse_kenq_pattern_sheet(file, sh)
    if (!is.null(mult)) distribution <- multiplier_to_distribution(mult)
  }

  project_name <- "Ken-Q Project"
  ni <- which(sheets_lower %in% c("name", "names"))[1]
  if (!is.na(ni)) {
    nr <- readxl::read_excel(file, sheet = sheets[ni], col_names = FALSE)
    if (nrow(nr) >= 1) {
      idx <- if (nrow(nr) >= 2 && !is.na(nr[[1]][2])) 2L else 1L
      project_name <- as.character(nr[[1]][idx])
    }
  }

  kenq_type <- NULL
  ti <- which(sheets_lower == "type")[1]
  if (!is.na(ti)) {
    tr <- readxl::read_excel(file, sheet = sheets[ti], col_names = FALSE)
    if (nrow(tr) >= 2) kenq_type <- suppressWarnings(as.integer(tr[[1]][2]))
  }

  raw <- as.data.frame(readxl::read_excel(file, sheet = sorts_sheet,
                                          col_names = FALSE),
                       stringsAsFactors = FALSE)
  if (nrow(raw) < 2) stop("Ken-Q 'sorts' sheet is empty.")

  n_stmts <- if (!is.null(stmts)) length(stmts) else NULL
  parsed  <- detect_and_parse_kenq_sorts(raw, n_stmts, kenq_type)
  sorts   <- parsed$sorts

  qsort_data(
    Y            = t(sorts),
    statements   = stmts %||% colnames(sorts),
    participants = rownames(sorts),
    distribution = distribution %||% parsed$distribution,
    metadata     = list(project_name = project_name,
                        kenq_format = parsed$format_type),
    source       = paste0("kenq-excel:", basename(file))
  )
}


# Pick a parser based on the "type" sheet when present, otherwise fall back
# to structural heuristics: a "Sort Pattern" row signals Type 2 (old); a
# header row with names in cols B+ signals Type 1 Ver2; a long header block
# (22 rows) followed by names + numeric data signals Type 1 Ver1.
#' @keywords internal
#' @noRd
detect_and_parse_kenq_sorts <- function(raw, n_stmts, kenq_type) {
  if (isTRUE(kenq_type == 1L)) return(parse_kenq_type1_ver2(raw, n_stmts))
  if (isTRUE(kenq_type == 2L)) {
    sp_row <- find_sort_pattern_row(raw)
    if (!is.null(sp_row))
      return(parse_kenq_type2_old(raw, sort_pattern_row = sp_row))
    return(parse_kenq_type2_ver2(raw))
  }

  sp_row <- find_sort_pattern_row(raw)
  if (!is.null(sp_row))
    return(parse_kenq_type2_old(raw, sort_pattern_row = sp_row))

  first_cell <- trimws(as.character(raw[1, 1]))
  first_blank  <- is.na(first_cell) || nchar(first_cell) == 0
  first_header <- !is.na(first_cell) &&
    grepl("respondent|sort\\s*value|name", first_cell, ignore.case = TRUE)
  if ((first_blank || first_header) && ncol(raw) > 1) {
    second <- as.character(raw[1, 2])
    if (!is.na(second) && !grepl("^-?\\d+\\.?\\d*$", trimws(second)))
      return(parse_kenq_type1_ver2(raw, n_stmts))
  }

  if (nrow(raw) > 25) {
    row2col2 <- as.character(raw[2, 2])
    if (!is.na(row2col2) && !grepl("^-?\\d+$", trimws(row2col2)) &&
        nchar(trimws(row2col2)) > 2) {
      sv_rows_ok <- TRUE
      for (r in seq.int(3L, min(22L, nrow(raw)))) {
        v <- suppressWarnings(as.numeric(raw[r, 1]))
        blank <- is.na(raw[r, 1]) ||
          nchar(trimws(as.character(raw[r, 1]))) == 0
        if (!is.na(v) || blank) next
        sv_rows_ok <- FALSE; break
      }
      if (sv_rows_ok) return(parse_kenq_type1_ver1(raw, n_stmts))
    }
  }

  first_txt <- !is.na(as.character(raw[1, 1])) &&
    !grepl("^-?\\d+\\.?\\d*$", trimws(as.character(raw[1, 1])))
  if (first_txt && ncol(raw) > 2) {
    sample <- suppressWarnings(as.numeric(raw[1, 2:ncol(raw)]))
    if (sum(!is.na(sample)) > (ncol(raw) - 1) * 0.5)
      return(parse_kenq_type2_ver2(raw))
  }

  stop("Could not determine Ken-Q Excel format (Type 1 or Type 2).")
}


#' @keywords internal
#' @noRd
find_sort_pattern_row <- function(raw) {
  for (i in seq_len(min(5, nrow(raw))))
    for (j in seq_len(min(5, ncol(raw)))) {
      v <- as.character(raw[i, j])
      if (!is.na(v) && grepl("sort\\s*pattern", v, ignore.case = TRUE))
        return(i)
    }
  NULL
}


#' @keywords internal
#' @noRd
parse_kenq_type1_ver2 <- function(raw, n_stmts) {
  na_mask <- vapply(raw[1, ], is.na, logical(1))
  names_row <- as.character(raw[1, ])
  ids <- names_row[-1]
  ids <- ids[!na_mask[-1] & nchar(trimws(ids)) > 0 &
             !grepl("^-?\\d+\\.?\\d*$", trimws(ids))]
  n_part <- length(ids)

  rows <- raw[-1, , drop = FALSE]
  sv <- suppressWarnings(as.numeric(rows[[1]]))
  ok <- !is.na(sv); sv <- sv[ok]
  stmt_data <- rows[ok, 2:(n_part + 1), drop = FALSE]

  if (is.null(n_stmts)) {
    all_nums <- suppressWarnings(
      as.integer(round(as.numeric(unlist(stmt_data)))))
    n_stmts <- max(all_nums, na.rm = TRUE)
  }

  sorts <- type1_to_sorts_matrix(sv, stmt_data, n_part, n_stmts, ids)
  dist  <- as.integer(tabulate(match(sv, seq(min(sv), max(sv))),
                               nbins = max(sv) - min(sv) + 1L))
  list(sorts = sorts, distribution = dist, format_type = "type1_ver2")
}


#' @keywords internal
#' @noRd
parse_kenq_type1_ver1 <- function(raw, n_stmts) {
  names_row_idx <- NULL
  if (nrow(raw) < 20L)
    stop("Ken-Q Type 1 Ver1: sheet has fewer than 20 rows.")
  for (i in seq.int(20L, min(28L, nrow(raw)))) {
    v <- as.character(raw[i, 2])
    if (!is.na(v) && nchar(trimws(v)) > 0 &&
        !grepl("^-?\\d+\\.?\\d*$", trimws(v))) {
      names_row_idx <- i; break
    }
  }
  if (is.null(names_row_idx))
    stop("Ken-Q Type 1 Ver1: cannot locate participant-names row.")

  ids <- as.character(raw[names_row_idx, ])[-1]
  ids <- ids[!is.na(ids) & nchar(trimws(ids)) > 0]
  n_part <- length(ids)

  rows <- raw[(names_row_idx + 1):nrow(raw), , drop = FALSE]
  valid <- apply(rows, 1, function(r) !all(is.na(r) |
                   nchar(trimws(as.character(r))) == 0))
  rows <- rows[valid, , drop = FALSE]

  sv <- suppressWarnings(as.numeric(rows[[1]]))
  ok <- !is.na(sv); sv <- sv[ok]
  stmt_data <- rows[ok, 2:(n_part + 1), drop = FALSE]

  if (is.null(n_stmts)) {
    all_nums <- suppressWarnings(
      as.integer(round(as.numeric(unlist(stmt_data)))))
    n_stmts <- max(all_nums, na.rm = TRUE)
  }

  sorts <- type1_to_sorts_matrix(sv, stmt_data, n_part, n_stmts, ids)
  dist  <- as.integer(tabulate(match(sv, seq(min(sv), max(sv))),
                               nbins = max(sv) - min(sv) + 1L))
  list(sorts = sorts, distribution = dist, format_type = "type1_ver1")
}


#' @keywords internal
#' @noRd
parse_kenq_type2_old <- function(raw, sort_pattern_row) {
  cells <- as.character(raw[sort_pattern_row, ])
  pat <- suppressWarnings(as.numeric(cells))
  pat <- pat[!is.na(pat)]
  dist <- if (length(pat) > 0)
            as.integer(tabulate(match(pat, seq(min(pat), max(pat))),
                                nbins = max(pat) - min(pat) + 1L))
          else NULL

  i <- sort_pattern_row + 1L
  while (i <= nrow(raw)) {
    f <- trimws(as.character(raw[i, 1]))
    if (!is.na(f) && nchar(f) > 0 &&
        !grepl("^-?\\d+\\.?\\d*$", f) &&
        !grepl("sort\\s*pattern", f, ignore.case = TRUE)) break
    i <- i + 1L
  }
  if (i > nrow(raw))
    stop("Ken-Q Type 2: no participant rows after Sort Pattern.")

  rows <- raw[i:nrow(raw), , drop = FALSE]
  valid <- apply(rows, 1, function(r) {
    f <- trimws(as.character(r[1]))
    !is.na(f) && nchar(f) > 0
  })
  rows <- rows[valid, , drop = FALSE]

  ids <- as.character(rows[[1]])
  sorts <- suppressWarnings(
    apply(rows[, -1, drop = FALSE], 2, as.numeric))
  if (!is.matrix(sorts)) sorts <- as.matrix(sorts)
  sorts <- sorts[, !apply(sorts, 2, function(c) all(is.na(c))),
                 drop = FALSE]
  rownames(sorts) <- ids

  list(sorts = sorts, distribution = dist, format_type = "type2_old")
}


#' @keywords internal
#' @noRd
parse_kenq_type2_ver2 <- function(raw) {
  valid <- apply(raw, 1, function(r) {
    f <- trimws(as.character(r[1]))
    !is.na(f) && nchar(f) > 0
  })
  raw <- raw[valid, , drop = FALSE]

  ids <- as.character(raw[[1]])
  sorts <- suppressWarnings(
    apply(raw[, -1, drop = FALSE], 2, as.numeric))
  if (!is.matrix(sorts)) sorts <- as.matrix(sorts)
  sorts <- sorts[, !apply(sorts, 2, function(c) all(is.na(c))),
                 drop = FALSE]
  rownames(sorts) <- ids
  list(sorts = sorts, distribution = NULL, format_type = "type2_ver2")
}


# In the Type 1 layout, each data row has a sort value in column 1 and, in
# the remaining columns, the statement number each participant placed at
# that sort value. This inverts that layout back to the canonical
# participant-by-statement matrix.
#' @keywords internal
#' @noRd
type1_to_sorts_matrix <- function(sv, stmt_data, n_part, n_stmts, ids) {
  sorts <- matrix(NA_real_, n_part, n_stmts,
                  dimnames = list(ids, paste0("S", seq_len(n_stmts))))
  for (p in seq_len(n_part)) {
    sn <- suppressWarnings(as.integer(round(as.numeric(stmt_data[, p]))))
    for (r in seq_along(sv))
      if (!is.na(sn[r]) && sn[r] >= 1L && sn[r] <= n_stmts)
        sorts[p, sn[r]] <- sv[r]
  }
  sorts
}


#' @keywords internal
#' @noRd
parse_kenq_statements_sheet <- function(file, sheet_name) {
  df <- readxl::read_excel(file, sheet = sheet_name)
  col <- which(tolower(names(df)) == "statements")[1]
  if (is.na(col)) col <- 1
  s <- as.character(df[[col]])
  s <- s[!is.na(s) & nchar(trimws(s)) > 0]
  s <- s[!grepl("^\\s*\\.?\\s*$", s)]
  if (length(s) == 0) NULL else s
}


#' @keywords internal
#' @noRd
parse_kenq_pattern_sheet <- function(file, sheet_name) {
  df <- readxl::read_excel(file, sheet = sheet_name, col_names = FALSE)
  for (r in c(2, 1, 3)) {
    if (r > nrow(df)) next
    txt <- paste(as.character(df[r, ]), collapse = ",")
    v <- suppressWarnings(as.numeric(
      strsplit(gsub("[^0-9,.-]", "", txt), ",")[[1]]))
    v <- v[!is.na(v)]
    if (length(v) >= 5) {
      if (length(v) < 20) v <- c(v, rep(0, 20 - length(v)))
      return(v[1:20])
    }
  }
  NULL
}


#' @keywords internal
#' @noRd
multiplier_to_distribution <- function(mult) {
  if (is.null(mult)) return(NULL)
  mult <- as.numeric(mult)
  nz <- which(mult > 0)
  if (length(nz) == 0) return(NULL)
  as.integer(mult[min(nz):max(nz)])
}


# A KADE archive holds four text files: sorts.txt (name + scores per row),
# statements.txt (one per line), pattern.txt (20-slot multiplier array),
# and name.txt (project title). The delimiter is comma or semicolon.
#' @rdname read_qsort
#' @export
read_kade_zip <- function(file) {
  tmp <- tempfile("kade_")
  dir.create(tmp)
  on.exit(unlink(tmp, recursive = TRUE))
  ex <- utils::unzip(file, exdir = tmp)
  bn <- tolower(basename(ex))

  sorts_file   <- ex[grepl("^sorts",     bn)][1]
  name_file    <- ex[grepl("^name",      bn)][1]
  stmts_file   <- ex[grepl("^statement", bn)][1]
  pattern_file <- ex[grepl("^pattern",   bn)][1]
  if (is.na(sorts_file)) stop("Not a valid KADE ZIP: missing sorts.txt")

  project_name <- "KADE Project"
  if (!is.na(name_file)) {
    ln <- trimws(readLines(name_file, warn = FALSE))
    ln <- ln[nchar(ln) > 0]
    if (length(ln) > 0) project_name <- ln[1]
  }

  stmts <- NULL
  if (!is.na(stmts_file)) {
    ln <- trimws(readLines(stmts_file, warn = FALSE))
    stmts <- ln[nchar(ln) > 0]
  }

  distribution <- NULL
  if (!is.na(pattern_file)) {
    ln <- trimws(readLines(pattern_file, warn = FALSE))
    ln <- ln[nchar(ln) > 0][1]
    delim <- if (length(gregexpr(";", ln)[[1]]) >
                 length(gregexpr(",", ln)[[1]])) ";" else ","
    mult <- suppressWarnings(as.numeric(strsplit(ln, delim)[[1]]))
    mult <- mult[!is.na(mult)]
    if (length(mult) > 0) {
      if (length(mult) < 20) mult <- c(mult, rep(0, 20 - length(mult)))
      distribution <- multiplier_to_distribution(mult[1:20])
    }
  }

  sl <- readLines(sorts_file, warn = FALSE)
  sl <- sl[nchar(trimws(sl)) > 0]
  if (length(sl) == 0) stop("KADE sorts.txt is empty.")
  delim <- if (length(gregexpr(";", sl[1])[[1]]) >
               length(gregexpr(",", sl[1])[[1]])) ";" else ","

  ids <- character()
  rows <- list()
  for (ln in sl) {
    parts <- trimws(strsplit(ln, delim)[[1]])
    ids <- c(ids, parts[1])
    rows[[length(rows) + 1]] <- suppressWarnings(as.numeric(parts[-1]))
  }
  sorts <- do.call(rbind, rows)
  rownames(sorts) <- ids
  if (!is.null(stmts) && length(stmts) == ncol(sorts))
    colnames(sorts) <- stmts
  else
    colnames(sorts) <- paste0("S", seq_len(ncol(sorts)))

  qsort_data(Y = t(sorts),
             statements   = colnames(sorts),
             participants = rownames(sorts),
             distribution = distribution,
             metadata = list(project_name = project_name),
             source = paste0("kade:", basename(file)))
}


#' @rdname read_qsort
#' @export
read_easyhtml_firebase <- function(file) {
  if (!requireNamespace("jsonlite", quietly = TRUE))
    stop("Package 'jsonlite' is required for JSON input.")
  d <- jsonlite::fromJSON(file, simplifyVector = FALSE)
  r <- try_firebase_json(d)
  if (is.null(r)) stop("Not a recognizable Easy-HTMLQ Firebase JSON.")
  r$source <- paste0("easyhtml:", basename(file))
  r
}


#' @rdname read_qsort
#' @export
read_statements <- function(file, column = 1, id_column = NULL) {
  ext <- tolower(tools::file_ext(file))
  if (ext %in% c("csv", "tsv")) {
    df <- utils::read.csv(file, stringsAsFactors = FALSE,
                          check.names = FALSE)
  } else if (ext %in% c("xlsx", "xls")) {
    if (!requireNamespace("readxl", quietly = TRUE))
      stop("Package 'readxl' is required for Excel input.")
    df <- as.data.frame(readxl::read_excel(file),
                        stringsAsFactors = FALSE)
  } else if (ext == "txt") {
    s <- trimws(readLines(file, warn = FALSE))
    s <- s[nchar(s) > 0]
    names(s) <- paste0("S", seq_along(s))
    return(s)
  } else {
    stop("Unsupported statements file: ", ext)
  }

  s <- as.character(df[[column]])
  names(s) <- if (!is.null(id_column)) as.character(df[[id_column]])
              else paste0("S", seq_along(s))
  s
}


#' @keywords internal
#' @noRd
`%||%` <- function(a, b) if (is.null(a)) b else a


# Dotted-name aliases for migration from the qmethod package, which uses
# import.pqmethod() / import.htmlq() / import.easyhtmlq(). Thin wrappers
# around the snake_case readers; scripts written against qmethod keep working.

#' qmethod-style import aliases
#'
#' @description
#' Thin aliases that forward to [read_pqmethod()], [read_qsort()] (HTMLQ
#' auto-detection), [read_kenq()], and [read_easyhtml_firebase()]. These
#' exist only so scripts written against the `qmethod` package continue
#' to work; new code should call the `read_*` functions directly.
#'
#' @param file Path to the data file.
#' @param ... Passed to the underlying reader.
#'
#' @return A `qsort_data` object.
#'
#' @name import-aliases
#' @aliases import.pqmethod import.htmlq import.kenq import.easyhtmlq
#' @export
import.pqmethod <- function(file, ...) read_pqmethod(file, ...)

#' @rdname import-aliases
#' @export
import.htmlq <- function(file, ...) read_qsort(file, format = "csv", ...)

#' @rdname import-aliases
#' @export
import.kenq <- function(file, ...) read_kenq(file, ...)

#' @rdname import-aliases
#' @export
import.easyhtmlq <- function(file, ...) read_easyhtml_firebase(file, ...)

test_that("read_qsort_csv reads a generic wide CSV with statement rows", {
  tmp <- tempfile(fileext = ".csv")
  on.exit(unlink(tmp), add = TRUE)

  # 5 statements x 4 participants; first column is statement ID.
  writeLines(c(
    "id,P1,P2,P3,P4",
    "S1,-2,-1,0,1",
    "S2,-1,-2,1,0",
    "S3,0,0,-1,-2",
    "S4,1,1,-2,2",
    "S5,2,2,2,-1"
  ), tmp)

  d <- read_qsort_csv(tmp)
  expect_s3_class(d, "qsort_data")
  expect_equal(dim(d$Y), c(5, 4))
  expect_equal(rownames(d$Y), c("S1", "S2", "S3", "S4", "S5"))
})

test_that("read_qsort_csv auto-detects HTMLQ header signatures", {
  tmp <- tempfile(fileext = ".csv")
  on.exit(unlink(tmp), add = TRUE)

  writeLines(c(
    "uid,datetime,s1,s2,s3,s4,s5",
    "u001,2025-01-01,-2,-1,0,1,2",
    "u002,2025-01-02,-1,0,1,2,-2",
    "u003,2025-01-03,0,1,2,-2,-1"
  ), tmp)

  d <- read_qsort_csv(tmp)
  expect_s3_class(d, "qsort_data")
  expect_equal(ncol(d$Y), 3)  # 3 participants
  expect_equal(nrow(d$Y), 5)
  expect_true(grepl("^htmlq:", d$source))
})

test_that("read_qsort auto-dispatches from file extension", {
  tmp <- tempfile(fileext = ".csv")
  on.exit(unlink(tmp), add = TRUE)

  writeLines(c(
    "id,P1,P2,P3",
    "S1,-1,0,1",
    "S2,0,1,-1",
    "S3,1,-1,0"
  ), tmp)

  d <- read_qsort(tmp)
  expect_s3_class(d, "qsort_data")
})

test_that("read_qsort errors on unknown extension", {
  tmp <- tempfile(fileext = ".xyz")
  on.exit(unlink(tmp), add = TRUE)
  writeLines("anything", tmp)
  expect_error(read_qsort(tmp), "No reader for extension")
})

test_that("read_qsort errors when the file does not exist", {
  expect_error(read_qsort("does_not_exist_12345.csv"), "File not found")
})

test_that("read_statements reads a .txt statements file", {
  tmp <- tempfile(fileext = ".txt")
  on.exit(unlink(tmp), add = TRUE)
  writeLines(c("First statement.",
               "Second statement.",
               "Third statement."), tmp)
  s <- read_statements(tmp)
  expect_length(s, 3)
  expect_equal(unname(s[1]), "First statement.")
  expect_equal(names(s), c("S1", "S2", "S3"))
})

test_that("parse_distribution reads a distribution from a file", {
  tmp <- tempfile(fileext = ".txt")
  on.exit(unlink(tmp), add = TRUE)
  writeLines("1 2 3 2 1", tmp)
  expect_equal(parse_distribution(tmp), c(1L, 2L, 3L, 2L, 1L))
})

test_that("dotted import aliases forward to the snake_case readers", {
  tmp <- tempfile(fileext = ".csv")
  on.exit(unlink(tmp), add = TRUE)
  writeLines(c(
    "uid,datetime,s1,s2,s3",
    "u1,2025-01-01,-1,0,1",
    "u2,2025-01-02,0,1,-1",
    "u3,2025-01-03,1,-1,0"
  ), tmp)
  d <- import.htmlq(tmp)
  expect_s3_class(d, "qsort_data")
})

#' @keywords internal
#' @aliases bayesqm-package
"_PACKAGE"

#' bayesqm: Bayesian Q Methodology
#'
#' @description
#' A Bayesian analysis for Q methodology, alongside the classical one. The
#' forced Q sort is modeled as an ordered partition of the statements
#' through an exact rank-order likelihood: the design quotas fix the
#' partition margins, so the probability of the observed sorting event is
#' the full observed-data likelihood. The posterior is sampled by a
#' parameter-expanded Gibbs sampler in R with no compiled code, gated on rotation-invariant
#' convergence diagnostics, aligned by MatchAlign, and returned as the
#' familiar Q tables with uncertainty attached.
#'
#' @details
#' The typical workflow:
#'
#' \enumerate{
#'   \item **Import data.** [read_qsort()] auto-detects CSV, Excel,
#'     PQMethod `.DAT`, Ken-Q JSON / multi-sheet Excel, KADE ZIP, or
#'     Easy-HTMLQ Firebase JSON. [qsort_data()] constructs the object
#'     directly from a matrix. The exact likelihood needs forced sorts:
#'     every participant must match the design grid.
#'   \item **Fit.** [fit_bayesian()] returns a `bayesqm_fit`; [extend()]
#'     warm-continues a chain the convergence gate flagged.
#'   \item **Read the tables.** [compute_loadings()] (bounded loadings
#'     with credible intervals), [compute_flags()] (flag probabilities
#'     with an unclassified state), [compute_zscores()],
#'     [compute_factor_array()] (quota-exact arrays),
#'     [compute_qdc()] (distinguishing / consensus / indeterminate),
#'     [crib_sheet()], and [claims()] — one posterior false-discovery
#'     rule selecting every claim.
#'   \item **Check the model.** [check_fit()] (agreement, extra-factor,
#'     and paired-comparison checks) and [check_persons()] (the person check,
#'     against mixed-replication bands).
#'   \item **Choose K.** [fit_ladder()] and [select_k()] run the
#'     two-signal workflow; [loo_ladder()] adds directional
#'     corroboration.
#'   \item **Report.** The `plot_*` views, [rotate_factors()] for
#'     judgmental rotation, [rename_factors()], and the standard R
#'     accessors (`coef()`, `fitted()`, `as_draws_df()`,
#'     [posterior_interval()], [prior_summary()]).
#' }
#'
#' @section Relationship to classical Q analysis:
#' The questions and the reporting conventions are Q's own; what the
#' Bayesian account adds is a probability behind each familiar verdict.
#' Vocabulary carries over: flags, factor arrays, defining sorts,
#' distinguishing and consensus statements. Two deliberate differences in
#' the tables:
#'
#' - Factor characteristics do not print eigenvalues or explained
#'   variance, which have no counterpart in a generative model; report
#'   the defining-sort counts from [claims()] and the extra-factor check
#'   of [check_fit()] in their place.
#' - Consensus is a positive finding (an equivalence region of one grid
#'   column, [delta_grid()]), not the complement of distinguishing, so a
#'   statement can be distinguishing, consensus, or neither.
#'
#' @references
#' Poworoznek, E., Anceschi, N., Ferrari, F., & Dunson, D. (2025).
#'   Efficiently Resolving Rotational Ambiguity in Bayesian Matrix
#'   Sampling with Matching. *Bayesian Analysis*.
#'
#' Vehtari, A., Gelman, A., Simpson, D., Carpenter, B., & Bürkner, P.-C.
#'   (2021). Rank-Normalization, Folding, and Localization: An Improved
#'   R-hat for Assessing Convergence of MCMC. *Bayesian Analysis*, 16(2),
#'   667-718.
#'
#' @name bayesqm-package
NULL

# plots.R
# Base-graphics views of the partition-model fit, one per user question.
# Every plot reads its palette through bayesqm_colors(), restores par()
# on exit, and returns its input invisibly. Factor identity always rides
# on the qualitative palette plus position or symbol, never colour alone.


#' @keywords internal
#' @noRd
.bq_par <- function(...) {
  op <- par(no.readonly = TRUE)
  par(font.axis = 2, font.lab = 2, font.main = 2, cex.main = 1.1)
  par(...)
  op
}


# every legend sits in a white box with a grey border
#' @keywords internal
#' @noRd
.bq_legend <- function(...)
  legend(..., bg = "white", box.col = "gray70", box.lwd = 0.8)


#' Bounded loadings with credible intervals
#'
#' @description
#' One row per participant, one interval per factor on the bounded
#' loading scale, with selected flags marked. The dotted rules sit at
#' the classical descriptive cut-off `1.96 / sqrt(J)` (reference).
#'
#' @param fit A `bayesqm_fit`.
#' @param q False-discovery level for the flag marks (default 0.05).
#'
#' @return `fit`, invisibly.
#'
#' @export
plot_loading_posterior <- function(fit, q = 0.05) {
  assert_bayesqm_fit(fit)
  cols <- bayesqm_colors()
  lo <- compute_loadings(fit)
  fl <- compute_flags(fit, q = q)
  K <- fit$brief$K; N <- fit$brief$N
  fac_ids <- dimnames(fit$draws$sigma)[[2]]
  thr <- 1.96 / sqrt(fit$brief$J)

  inner <- compute_loadings(fit, prob = 0.5)     # nested 50% interval

  op <- .bq_par(mar = c(4, 6, 3, 1))
  on.exit(par(op), add = TRUE)
  plot(NULL, xlim = c(-1, 1), ylim = c(0.5, N + 0.5), yaxt = "n",
       xlab = "bounded loading", ylab = "",
       main = "Participant loadings, 50% and 95% intervals")
  axis(2, at = seq_len(N), labels = lo$participant, las = 1, cex.axis = 0.8)
  abline(v = 0, col = cols$grey)
  abline(v = c(-thr, thr), col = cols$gridgrey, lty = 3)
  text(thr, N + 0.5, expression(bold(1.96 / sqrt(J))), pos = 4, cex = 0.7,
       col = cols$grey, xpd = TRUE)
  off <- seq(-0.22, 0.22, length.out = K)
  for (k in seq_len(K)) {
    ck <- cols$qual[(k - 1) %% length(cols$qual) + 1]
    y <- seq_len(N) + off[k]
    segments(lo[[paste0(fac_ids[k], "_lower")]], y,
             lo[[paste0(fac_ids[k], "_upper")]], y, col = ck, lwd = 1.2)
    segments(inner[[paste0(fac_ids[k], "_lower")]], y,
             inner[[paste0(fac_ids[k], "_upper")]], y, col = ck, lwd = 3.2)
    pub_k <- fl$selected & fl$factor == fac_ids[k]
    points(lo[[paste0(fac_ids[k], "_loading")]], y,
           pch = ifelse(pub_k, 19, 21), col = ck, bg = "white", cex = 0.9)
  }
  .bq_legend("topleft", legend = c(fac_ids, "selected flag"),
             col = c(cols$qual[seq_len(K)], cols$grey),
             pch = c(rep(15, K), 19), cex = 0.8)
  invisible(fit)
}


#' Flag probabilities with the unclassified state
#'
#' @description
#' One row per participant, one dot per factor at its posterior flag
#' probability (both poles combined; a minus sign marks a dominant
#' negative pole), and a grey square for the probability of remaining
#' unclassified. Filled dots are selected flags, and the dotted rule is
#' the 0.5 selection floor.
#'
#' @param fit A `bayesqm_fit`.
#' @param q False-discovery level for the selection marks (default 0.05).
#'
#' @return `fit`, invisibly.
#'
#' @export
plot_flags <- function(fit, q = 0.05) {
  assert_bayesqm_fit(fit)
  cols <- bayesqm_colors()
  fl <- compute_flags(fit, q = q)
  phi <- attr(fl, "phi")
  K <- fit$brief$K; N <- fit$brief$N
  fac_ids <- dimnames(fit$draws$sigma)[[2]]
  kq <- cols$qual[(seq_len(K) - 1) %% length(cols$qual) + 1]

  op <- .bq_par(mar = c(4, 6, 3, 1))
  on.exit(par(op), add = TRUE)
  plot(NULL, xlim = c(0, 1.05), ylim = c(0.5, N + 1.4), yaxt = "n",
       xlab = "posterior flag probability", ylab = "",
       main = "Flag probabilities")
  axis(2, at = seq_len(N), labels = rownames(phi), las = 1, cex.axis = 0.8)
  abline(v = 0.5, col = cols$grey, lty = 3)
  text(0.5, 0.56, "selection floor", pos = 4, cex = 0.7,
       col = cols$grey, font = 3)
  off <- seq(-0.26, 0.26, length.out = K + 1)
  for (k in seq_len(K)) {
    pk <- phi[, 2 * k - 1] + phi[, 2 * k]
    neg <- phi[, 2 * k] > phi[, 2 * k - 1]
    y <- seq_len(N) + off[k]
    sel <- fl$selected & fl$factor == fac_ids[k]
    points(pk, y, pch = ifelse(sel, 19, 21), col = kq[k], bg = "white",
           cex = 1.05, lwd = 1.4)
    show <- neg & pk > 0.02
    if (any(show))
      text(pk[show] + 0.02, y[show], "-", col = kq[k], font = 2)
  }
  points(phi[, 2 * K + 1], seq_len(N) + off[K + 1], pch = 22,
         col = cols$grey, bg = cols$gridgrey, cex = 1.0)
  .bq_legend("top", ncol = min(K + 2, 4),
             legend = c(fac_ids, "unclassified", "filled: selected"),
             col = c(kq, cols$grey, cols$grey),
             pt.bg = c(rep("white", K), cols$gridgrey, "white"),
             pch = c(rep(21, K), 22, 19), cex = 0.75)
  invisible(fit)
}


#' The factor array on its grid
#'
#' @description
#' The reported array for one factor drawn as the physical sorting grid:
#' columns are grid positions with their quota heights, each cell carries
#' its statement, and cell shading shows how certain the placement is
#' (the posterior probability of that statement landing in that column).
#'
#' @param fit A `bayesqm_fit`.
#' @param factor Which factor to draw (index or `"f2"`-style name;
#'   default 1).
#' @param labels Optional statement labels (defaults to the statement
#'   ids).
#'
#' @return `fit`, invisibly.
#'
#' @export
plot_factor_array <- function(fit, factor = 1, labels = NULL) {
  assert_bayesqm_fit(fit)
  cols <- bayesqm_colors()
  distr <- fit$distribution
  C <- length(distr)
  fac_ids <- dimnames(fit$draws$sigma)[[2]]
  k <- if (is.character(factor)) match(factor, fac_ids) else as.integer(factor)
  if (is.na(k) || k < 1 || k > fit$brief$K) .bq_abort("no such factor.")

  ar <- compute_factor_array(fit)
  slot <- ar[[paste0(fac_ids[k], "_grid")]]
  pcol <- attr(ar, "certainty")[, k]
  if (is.null(labels)) labels <- ar$statement

  op <- .bq_par(mar = c(4, 1, 3.5, 1))
  on.exit(par(op), add = TRUE)
  plot(NULL, xlim = c(0.5, C + 0.5), ylim = c(0, max(distr) + 0.5),
       axes = FALSE, xlab = "grid column", ylab = "",
       main = sprintf("Factor %s array", fac_ids[k]))
  mtext("darker tiles are more certain of their column", side = 3,
        line = 0.2, cex = 0.8, font = 3, col = cols$grey)
  col_labels <- if (C %% 2 == 0)
    c(seq(-C / 2, -1), seq(1, C / 2)) else seq_len(C) - (C + 1) / 2
  axis(1, at = seq_len(C), labels = col_labels)
  shade <- colorRampPalette(.bq_certainty)(101)
  for (cc in seq_len(C)) {
    js <- which(slot == cc)
    js <- js[order(-pcol[js])]
    for (h in seq_along(js)) {
      fill <- shade[round(100 * pcol[js[h]]) + 1]
      rect(cc - 0.45, h - 1 + 0.05, cc + 0.45, h - 0.05,
           col = fill, border = cols$grey)
      text(cc, h - 0.5, labels[js[h]], cex = 0.7,
           col = .tile_text_col(fill))
    }
  }
  invisible(fit)
}


#' Statement contrasts between two factors
#'
#' @description
#' Per statement, the posterior contrast interval for one factor pair,
#' the posterior critical difference `delta_kl` (dashed), and the
#' grid-width consensus region ([delta_grid()], shaded). Verdict
#' colours: selected distinguishing listings, consensus statements,
#' indeterminate.
#'
#' @param fit A `bayesqm_fit` with `K >= 2`.
#' @param pair Which factor pair to draw (index into the pair list or
#'   `"f1-f2"`-style name; default 1).
#' @param q,cons_floor As in [compute_qdc()].
#'
#' @return `fit`, invisibly.
#'
#' @export
plot_contrasts <- function(fit, pair = 1, q = 0.05, cons_floor = 0.95) {
  assert_bayesqm_fit(fit)
  cols <- bayesqm_colors()
  qdc <- compute_qdc(fit, q = q, cons_floor = cons_floor)
  ct50 <- attr(compute_qdc(fit, q = q, cons_floor = cons_floor,
                           prob = 0.5), "contrasts")
  ct <- attr(qdc, "contrasts")
  dkl <- attr(qdc, "delta_kl")
  dg <- attr(qdc, "delta_grid")
  pair_ids <- unique(ct$pair)
  p <- if (is.character(pair)) match(pair, pair_ids) else as.integer(pair)
  if (is.na(p) || p < 1 || p > length(pair_ids)) .bq_abort("no such pair.")
  keep <- ct$pair == pair_ids[p]
  ct <- ct[keep, ]; ct50 <- ct50[keep, ]
  o <- order(ct$median)
  ct <- ct[o, ]; ct50 <- ct50[o, ]
  verdict <- qdc$verdict[o]
  vcol <- ifelse(grepl("^distinguishing", verdict),
                 .bq_verdict[["distinguishing"]],
                 ifelse(verdict == "consensus", .bq_verdict[["consensus"]],
                        .bq_verdict[["indeterminate"]]))
  J <- nrow(ct)

  op <- .bq_par(mar = c(4, 6, 3, 1))
  on.exit(par(op), add = TRUE)
  xr <- range(c(ct$lower, ct$upper, dkl[p], -dkl[p]))
  plot(NULL, xlim = xr, ylim = c(0.5, J + 0.5), yaxt = "n",
       xlab = sprintf("score contrast (%s)", pair_ids[p]), ylab = "",
       main = sprintf("Statement contrasts, %s", pair_ids[p]))
  rect(-dg, 0, dg, J + 1, col = adjustcolor(cols$fill, 0.5), border = NA)
  abline(v = c(-dkl[p], dkl[p]), col = cols$accent, lty = 2)
  abline(v = 0, col = cols$grey)
  text(dkl[p], J + 0.5, expression(bold(+delta[kl])), pos = 4, cex = 0.75,
       col = cols$accent, xpd = TRUE)
  text(-dkl[p], J + 0.5, expression(bold(-delta[kl])), pos = 2, cex = 0.75,
       col = cols$accent, xpd = TRUE)
  axis(2, at = seq_len(J), labels = ct$statement, las = 1, cex.axis = 0.7)
  segments(ct$lower, seq_len(J), ct$upper, seq_len(J), col = vcol, lwd = 1.2)
  segments(ct50$lower, seq_len(J), ct50$upper, seq_len(J), col = vcol,
           lwd = 3.2)
  points(ct$median, seq_len(J), pch = 19, col = vcol, cex = 0.8)
  .bq_legend("topleft",
             legend = expression("distinguishing", "consensus",
                                 "indeterminate",
                                 "consensus region (one grid column)",
                                 delta[kl] * ", critical difference"),
             col = c(unname(.bq_verdict), cols$grey, cols$accent),
             pch = c(19, 19, 19, 22, NA),
             lty = c(NA, NA, NA, NA, 2),
             pt.bg = c(NA, NA, NA, adjustcolor(cols$fill, 0.7), NA),
             pt.cex = c(0.9, 0.9, 0.9, 1.6, NA), cex = 0.72)
  invisible(fit)
}


#' The two-signal choice-of-K display
#'
#' @description
#' The whole decision in one strip, one row per ladder rung. The left
#' span shows the adequacy signal, the extra-factor percentile inside
#' its shaded band, with a warning ring where the person check found a
#' mutual unspanned cluster. The right span shows one chip per factor,
#' coloured when the factor is supported and grey when not, with the
#' reason inside: `f` means fewer than two selected flags, `d` means no
#' selected distinguishing statement. Each row ends with its own verdict
#' in words, the selected row is boxed, and when no K passes both checks
#' the conclusion is written across the bottom of the plot.
#'
#' @param x A `bayesqm_selection` from [select_k()].
#'
#' @return `x`, invisibly.
#'
#' @export
plot_choice_k <- function(x) {
  if (!inherits(x, "bayesqm_selection"))
    .bq_abort("plot_choice_k() takes a bayesqm_selection from select_k().")
  cols <- bayesqm_colors()
  tab <- x$table
  det <- x$detail
  nK <- nrow(tab)
  Kmax <- max(tab$K)
  kq <- cols$qual[(seq_len(Kmax) - 1) %% length(cols$qual) + 1]
  chipw <- 0.45 / Kmax
  cx0 <- 1.12
  vx0 <- cx0 + Kmax * chipw + 0.06
  xmax <- vx0 + 0.62

  row_word <- function(r) {
    if (!tab$adequate[r]) return("not adequate")
    if (!tab$all_supported[r]) return("adequate, but unsupported")
    if (!is.na(x$K) && tab$K[r] == x$K) return("report this one")
    "also passes"
  }
  concl <- switch(x$verdict,
    no_shared_structure = "No factor is supported at any K. Report no factor solution.",
    single_viewpoint = "The sorts agree as one shared viewpoint, consider K = 1.",
    adequate_but_unsupported = "Adequate, but no K has every factor supported.",
    no_adequate_rung = "No K is adequate. Try a larger K or read the person check.",
    tension = "The two checks disagree, read the rows before choosing.",
    NULL)

  op <- .bq_par(mar = c(6, 4.5, 4, 1))
  on.exit(par(op), add = TRUE)
  y0 <- if (is.null(concl)) 0.5 else -0.15
  plot(NULL, xlim = c(0, xmax), ylim = c(y0, nK + 0.9), axes = FALSE,
       xlab = "", ylab = "", main = "Choice of K, two signals")
  mtext(sprintf("verdict: %s", gsub("_", " ", x$verdict)), side = 3,
        line = 0.3, cex = 0.85, font = 3, col = cols$grey)

  # left span: adequacy
  rect(x$band[1], 0.5, x$band[2], nK + 0.5,
       col = adjustcolor(cols$fill, 0.4), border = NA)
  text(mean(x$band), nK + 0.7, "adequate range", cex = 0.7,
       col = cols$grey, font = 3)
  axis(1, at = seq(0, 1, 0.25), cex.axis = 0.8)
  axis(2, at = seq_len(nK), labels = paste0("K = ", tab$K), las = 1,
       tick = FALSE)
  mtext("extra-factor percentile", side = 1, line = 2.3, at = 0.5,
        cex = 0.85, font = 2)
  segments(0, seq_len(nK), 1, seq_len(nK), col = "grey92")
  points(tab$extra_factor, seq_len(nK),
         pch = ifelse(tab$adequate, 19, 21), col = cols$dark,
         bg = "white", cex = 1.4, lwd = 1.4)
  if (any(tab$cluster))
    points(tab$extra_factor[tab$cluster], which(tab$cluster), pch = 1,
           col = cols$accent, cex = 2.6, lwd = 2)

  # right span: one chip per factor, then the row's verdict in words
  abline(v = 1.06, col = "grey85")
  for (r in seq_len(nK)) {
    dr <- det[det$K == tab$K[r], ]
    for (k in seq_len(nrow(dr))) {
      xc <- cx0 + (k - 0.5) * chipw
      ok <- dr$supported[k]
      rect(xc - 0.46 * chipw, r - 0.3, xc + 0.46 * chipw, r + 0.3,
           col = if (ok) kq[k] else "grey92",
           border = if (ok) cols$grey else "grey80")
      if (!ok) {
        why <- paste0(if (dr$flags[k] < 2) "f" else "",
                      if (!dr$has_dist[k]) "d" else "")
        text(xc, r, why, col = cols$accent, cex = 0.8, font = 2)
      }
    }
    win <- !is.na(x$K) && tab$K[r] == x$K
    text(vx0, r, row_word(r), adj = 0, cex = 0.75,
         col = if (win) cols$accent else cols$grey,
         font = if (win) 2 else 3, xpd = TRUE)
    if (win)
      rect(-0.02, r - 0.45, xmax, r + 0.45, border = cols$accent, lwd = 2,
           xpd = TRUE)
  }
  text(cx0 + Kmax * chipw / 2, nK + 0.7, "every factor supported",
       cex = 0.7, col = cols$grey, font = 3)
  axis(1, at = cx0 + (seq_len(Kmax) - 0.5) * chipw,
       labels = seq_len(Kmax), tick = FALSE, cex.axis = 0.7, line = -0.7)
  mtext("factor", side = 1, line = 2.3, at = cx0 + Kmax * chipw / 2,
        cex = 0.85, font = 2)
  if (!is.null(concl))
    text(xmax / 2, 0.12, concl, cex = 0.8, font = 2, col = cols$dark,
         xpd = TRUE)
  .bq_legend(x = grconvertX(0.5, "ndc", "user"),
             y = grconvertY(0.012, "ndc", "user"),
             xjust = 0.5, yjust = 0, xpd = NA, ncol = 2,
             legend = c("in band: adequate", "coloured chip: supported",
                        "f / d: too few flags / nothing distinguishes"),
             pch = c(19, 22, NA),
             col = c(cols$dark, cols$grey, cols$accent),
             pt.bg = c(NA, kq[1], NA),
             pt.cex = c(1, 1.4, NA), cex = 0.7)
  invisible(x)
}


#' The person check against the mixed bands
#'
#' @description
#' The `m` (model agreement) by `w` (person agreement) scatter with the
#' mixed-replication bands; verdict colours, and arrows from unspanned
#' persons to their nearest partners.
#'
#' @param x A `bayesqm_persons` from [check_persons()].
#'
#' @return `x`, invisibly.
#'
#' @export
plot_person_check <- function(x) {
  if (!inherits(x, "bayesqm_persons"))
    .bq_abort("plot_person_check() takes a bayesqm_persons from check_persons().")
  cols <- bayesqm_colors()
  band_m <- attr(x, "band_m"); band_w <- attr(x, "band_w")
  vmap <- c(fits = "#17A589", no_shared = "grey72",
            unspanned = "#E67E22", atypical = "#6C3483")
  vlab <- c(fits = "fits the factors", no_shared = "no shared viewpoint",
            unspanned = "unspanned viewpoint", atypical = "atypical")

  op <- .bq_par(mar = c(4, 4, 3, 1))
  on.exit(par(op), add = TRUE)
  plot(NULL, xlim = range(c(x$m, band_m)), ylim = c(0, 1),
       xlab = "model agreement m", ylab = "person agreement w",
       main = "Person check")
  rect(band_m[1], -0.04, band_m[2], 1.04,
       col = adjustcolor(cols$fill, 0.45), border = NA)
  abline(v = band_m, col = cols$grey, lty = 3)
  text(mean(band_m), 0.02, "reference band", cex = 0.7, col = cols$grey,
       font = 3)
  segments(x$m, band_w[, 1], x$m, band_w[, 2], col = cols$gridgrey)
  points(x$m, x$w, pch = 19, col = vmap[x$verdict], cex = 1.1)
  text(x$m, x$w, x$participant, pos = 3, cex = 0.7, col = cols$grey)
  unsp <- which(x$verdict == "unspanned")
  for (i in unsp) {
    j <- if (!is.null(x$partner_index)) x$partner_index[i]
         else match(x$partner[i], x$participant)
    arrows(x$m[i], x$w[i], x$m[j], x$w[j], length = 0.08, col = cols$accent)
  }
  .bq_legend("bottomright", legend = unname(vlab), col = vmap, pch = 19,
             cex = 0.75)
  invisible(x)
}


#' Convergence and alignment view
#'
#' @description
#' Traces of the invariant person spreads `s_i` for the persons with the
#' lowest effective sample size — the series the convergence gate reads —
#' plus the per-draw congruence of the aligned draws to the pivot, with
#' the gate report in the margin. A congruence histogram hugging 1 says
#' the alignment found one common orientation; a long left tail says some
#' draws sit far from it.
#'
#' @param fit A `bayesqm_fit`.
#' @param n_series How many worst-ESS persons to trace (default 3).
#'
#' @return `fit`, invisibly.
#'
#' @export
plot_convergence <- function(fit, n_series = 3) {
  assert_bayesqm_fit(fit)
  cols <- bayesqm_colors()
  s <- fit$draws$s_i
  ess <- apply(s, 2, function(x) conv_stats(x)["bulk"])
  worst <- order(ess)[seq_len(min(n_series, ncol(s)))]
  g <- fit$gate

  op <- .bq_par(mfrow = c(length(worst) + 1, 1), mar = c(2, 4, 1.5, 1),
                oma = c(2, 0, 2, 0))
  on.exit(par(op), add = TRUE)
  for (i in worst) {
    plot(s[, i], type = "l", col = cols$dark, xlab = "", ylab = "s_i",
         main = sprintf("%s (bulk ESS %.0f)", colnames(s)[i], ess[i]),
         cex.main = 0.9)
  }
  cg <- fit$align$congruence
  hist(cg, breaks = 30, col = cols$fill, border = cols$grey,
       xlim = c(min(cg, 0.8), 1), xlab = "", ylab = "draws",
       main = sprintf("alignment congruence (mean %.2f)", mean(cg)),
       cex.main = 0.9)
  mtext(sprintf("gate %s: max Rhat %.3f, min ESS %.0f bulk / %.0f tail (%d iterations%s)",
                if (g$converged) "passed" else "NOT MET", g$rhat, g$ess_bulk,
                g$ess_tail, g$iterations,
                if (g$extended) ", warm-extended" else ""),
        outer = TRUE, side = 3, cex = 0.85)
  invisible(fit)
}


#' Posterior-predictive check display
#'
#' @description
#' The replicated distributions behind [check_fit()]: the agreement RMSE
#' against its double-replicate reference, and the observed extra-factor
#' eigenvalue in its replicated distribution.
#'
#' @param x A `bayesqm_checks` from [check_fit()].
#'
#' @return `x`, invisibly.
#'
#' @export
plot_ppc <- function(x) {
  if (!inherits(x, "bayesqm_checks"))
    .bq_abort("plot_ppc() takes a bayesqm_checks from check_fit().")
  cols <- bayesqm_colors()
  op <- .bq_par(mfrow = c(1, 2), mar = c(4, 4, 3, 1))
  on.exit(par(op), add = TRUE)

  a <- x$agreement
  hist(a$ref, breaks = 20, col = cols$fill, border = cols$grey,
       xlim = range(c(a$ref, a$obs)), xlab = "agreement RMSE",
       main = sprintf("Agreement check (p = %.2f)", a$p))
  hist(a$obs, breaks = 20, col = adjustcolor(cols$qual[2], 0.55),
       border = NA, add = TRUE)
  .bq_legend("topright", legend = c("reference", "observed"),
             fill = c(cols$fill, adjustcolor(cols$qual[2], 0.55)),
             border = c(cols$grey, NA), cex = 0.75)

  e <- x$extra_factor
  hist(e$replicated, breaks = 20, col = cols$fill, border = cols$grey,
       xlim = range(c(e$replicated, e$observed)),
       xlab = expression(e[K + 1]),
       main = sprintf("Extra-factor check (percentile %.2f)", e$percentile))
  abline(v = e$observed, col = cols$accent, lwd = 2)
  .bq_legend("topright", legend = c("replicated", "observed"),
             fill = c(cols$fill, NA), border = c(cols$grey, NA),
             lty = c(NA, 1), lwd = c(NA, 2),
             col = c(NA, cols$accent), cex = 0.75)
  invisible(x)
}


#' @rdname plot_loading_posterior
#' @param x,... A `bayesqm_fit` and arguments passed on.
#' @export
plot.bayesqm_fit <- function(x, ...) plot_loading_posterior(x, ...)

# The plot suite: every view draws without error on a complete fit, par()
# is restored, and each returns its input invisibly.

expect_plots_cleanly <- function(code) {
  tmp <- tempfile(fileext = ".pdf")
  grDevices::pdf(tmp)
  op <- par(no.readonly = TRUE)
  out <- force(code)
  expect_identical(par("mfrow"), op$mfrow)
  grDevices::dev.off()
  unlink(tmp)
  invisible(out)
}

test_that("every fit view draws cleanly", {
  fit <- demo_fit()
  expect_plots_cleanly(expect_invisible(plot_loading_posterior(fit)))
  expect_plots_cleanly(expect_invisible(plot_flags(fit)))
  expect_plots_cleanly(expect_invisible(plot_factor_array(fit)))
  expect_plots_cleanly(expect_invisible(plot_factor_array(fit, factor = "f2")))
  expect_plots_cleanly(expect_invisible(plot_contrasts(fit)))
  expect_plots_cleanly(expect_invisible(plot_convergence(fit)))
  expect_plots_cleanly(expect_invisible(plot(fit)))
})

test_that("check and selection views draw cleanly", {
  fit <- demo_fit()
  ck <- check_fit(fit, draws = 15)
  expect_plots_cleanly(expect_invisible(plot_ppc(ck)))
  pc <- check_persons(fit, draws = 10, mixes = 20)
  expect_plots_cleanly(expect_invisible(plot_person_check(pc)))
})

test_that("plot inputs are validated", {
  fit <- demo_fit()
  expect_error(plot_factor_array(fit, factor = 9), "no such factor")
  expect_error(plot_contrasts(fit, pair = "f9-f1"), "no such pair")
  expect_error(plot_ppc(fit), "bayesqm_checks")
  expect_error(plot_person_check(fit), "bayesqm_persons")
})

test_that("deprecated plot names forward with a deprecation signal", {
  fit <- demo_fit()
  tmp <- tempfile(fileext = ".pdf")
  grDevices::pdf(tmp)
  expect_warning(plot_membership(fit), "plot_flags")
  expect_warning(plot_dist_cons(fit), "plot_contrasts")
  expect_error(plot_hyper(fit), "factor_characteristics")
  grDevices::dev.off()
  unlink(tmp)
})

test_that("autoplot serves the flagship four", {
  skip_if_not_installed("ggplot2")
  fit <- demo_fit()
  for (ty in c("loadings", "flags", "contrasts", "array")) {
    p <- ggplot2::autoplot(fit, type = ty)
    expect_s3_class(p, "ggplot")
  }
})

test_that("caption carries provenance and the gate verdict", {
  fit <- demo_fit()
  cap <- caption_bayesqm(fit)
  expect_match(cap, "Exact partition")
  expect_match(cap, "N = 8, J = 13, K = 2")
  expect_match(cap, "gate passed")
  expect_false(grepl("gate", caption_bayesqm(fit, include_gate = FALSE)))
})

test_that("plot_sorts previews every participant on any labeling", {
  distr <- c(1, 2, 3, 2, 1)
  set.seed(2)
  Y <- t(quota_sort_rows(matrix(rnorm(5 * 9), 5, 9), distr)) - 3   # -2..+2
  qd <- qsort_data(Y, distribution = distr, validate = FALSE)
  expect_plots_cleanly(expect_invisible(plot_sorts(qd)))
  expect_plots_cleanly(expect_invisible(plot(qd)))                 # dispatch
  expect_plots_cleanly(expect_invisible(plot_sorts(qd, participants = c(2, 4))))
  expect_plots_cleanly(expect_invisible(plot_sorts(qd, per_page = 2)))
  expect_error(plot_sorts(qd, participants = "nobody"), "unknown participant")
  # asymmetric grid
  distr2 <- c(1, 2, 4, 2)
  Y2 <- t(quota_sort_rows(matrix(rnorm(3 * 9), 3, 9), distr2))
  expect_plots_cleanly(
    expect_invisible(plot_sorts(qsort_data(Y2, distribution = distr2,
                                           validate = FALSE))))
})

test_that("autoplot previews sorts as faceted pyramids", {
  skip_if_not_installed("ggplot2")
  distr <- c(1, 2, 3, 2, 1)
  set.seed(2)
  Y <- t(quota_sort_rows(matrix(rnorm(4 * 9), 4, 9), distr))
  qd <- qsort_data(Y, distribution = distr, validate = FALSE)
  p <- ggplot2::autoplot(qd)
  expect_s3_class(p, "ggplot")
})

test_that("statement views draw cleanly and validate", {
  fit <- demo_fit()
  expect_plots_cleanly(expect_invisible(plot_zscores(fit)))
  expect_plots_cleanly(expect_invisible(plot_zscores(fit, order_by = "score")))
  expect_plots_cleanly(expect_invisible(plot_statement(fit, "S3")))
  expect_plots_cleanly(expect_invisible(plot_statement(fit, 5)))
  expect_error(plot_statement(fit, "S99"), "no such statement")
  tmp <- tempfile(fileext = ".pdf"); grDevices::pdf(tmp)
  expect_warning(plot_zscore_posterior(fit, 2), "plot_statement")
  grDevices::dev.off(); unlink(tmp)
  fit1 <- make_fake_fit(N = 5, J = 9, K = 1, T = 80)
  expect_plots_cleanly(expect_invisible(plot_zscores(fit1)))   # K = 1 path
})

# accessors.R
# Standard R accessors and draws-ecosystem bridges for the bayesqm_fit.
# Parameter names in the flat draw representation follow the Stan-style
# convention: Lambda[i,k], F[j,k], sigma[k], s_i[i].


# flatten the aligned draw arrays into a T x P matrix with bracketed names
#' @keywords internal
#' @noRd
flatten_draws_to_matrix <- function(fit) {
  d <- fit$draws
  T_ <- dim(d$F)[1]
  N <- fit$brief$N; J <- fit$brief$J; K <- fit$brief$K
  cols <- c(
    outer(seq_len(N), seq_len(K), function(i, k) sprintf("Lambda[%d,%d]", i, k)),
    outer(seq_len(J), seq_len(K), function(j, k) sprintf("F[%d,%d]", j, k)),
    sprintf("sigma[%d]", seq_len(K)),
    sprintf("s_i[%d]", seq_len(N))
  )
  m <- cbind(matrix(d$Lambda, T_, N * K),
             matrix(d$F, T_, J * K),
             matrix(d$sigma, T_, K),
             matrix(d$s_i, T_, N))
  colnames(m) <- cols
  m
}


#' Posterior-mean bounded loadings
#'
#' @param object A `bayesqm_fit`.
#' @param ... Unused.
#' @return An `N x K` matrix of posterior-mean bounded loadings (the
#'   correlation-scale loading of the accompanying paper).
#' @export
coef.bayesqm_fit <- function(object, ...) {
  .summarize_draws(.rho_draws(object), mean)
}

#' Posterior-mean reconstruction on the utility scale
#'
#' @param object A `bayesqm_fit`.
#' @param ... Unused.
#' @return A `J x N` matrix of posterior-mean latent utilities
#'   `F Lambda'`.
#' @export
fitted.bayesqm_fit <- function(object, ...) {
  Fm <- .summarize_draws(object$draws$F, mean)
  Lm <- .summarize_draws(object$draws$Lambda, mean)
  out <- Fm %*% t(Lm)
  dimnames(out) <- dimnames(object$dataset)
  out
}

#' @export
residuals.bayesqm_fit <- function(object, ...) {
  .bq_abort(paste0(
    "a rank likelihood has no residual scale: the data are the ordering ",
    "events themselves. Use check_fit() for posterior-predictive checks."))
}

#' @export
nobs.bayesqm_fit <- function(object, ...) object$brief$N

#' Posterior-mean loading scales
#'
#' @param object A `bayesqm_fit`.
#' @param ... Unused.
#' @return Named numeric vector of posterior-mean `sigma_k`.
#' @export
sigma.bayesqm_fit <- function(object, ...) {
  out <- colMeans(object$draws$sigma)
  names(out) <- dimnames(object$draws$sigma)[[2]]
  out
}

#' @export
family.bayesqm_fit <- function(object, ...) {
  structure(list(family = "exact partition (rank-order) likelihood",
                 link = "latent Gaussian utilities"),
            class = "bayesqm_family")
}

#' @export
print.bayesqm_family <- function(x, ...) {
  cat("Family:", x$family, "\n")
  cat("Link:  ", x$link, "\n")
  invisible(x)
}

#' @export
as.matrix.bayesqm_fit <- function(x, ...) flatten_draws_to_matrix(x)

#' @export
as.array.bayesqm_fit <- function(x, ...) {
  m <- flatten_draws_to_matrix(x)
  array(m, c(nrow(m), 1L, ncol(m)),
        dimnames = list(NULL, NULL, colnames(m)))
}

#' @export
as.data.frame.bayesqm_fit <- function(x, ...) {
  as.data.frame(flatten_draws_to_matrix(x))
}

#' @export
update.bayesqm_fit <- function(object, ...) {
  cl <- object$brief$call
  # the fit carries its data, so refitting never depends on the original
  # data object still existing in the caller's session
  cl$Y <- qsort_data(object$dataset, distribution = object$distribution,
                     validate = FALSE)
  new_args <- list(...)
  for (nm in names(new_args)) cl[[nm]] <- new_args[[nm]]
  eval(cl, parent.frame())
}


#' Posterior interval generic
#'
#' @description
#' Generic for posterior credible intervals, defined here so the package
#' needs no Stan-ecosystem dependency. Compatible with the generic of the
#' same name in \pkg{rstantools}.
#'
#' @param object A fitted model object.
#' @param ... Passed to methods.
#'
#' @return See the method documentation.
#' @export
posterior_interval <- function(object, ...) UseMethod("posterior_interval")

#' Credible intervals for bayesqm_fit parameters
#'
#' @description
#' Posterior credible intervals for any subset of parameters in a
#' `bayesqm_fit`. Method for [posterior_interval()].
#'
#' @param object A `bayesqm_fit`.
#' @param prob Coverage probability (default 0.95).
#' @param pars Optional character vector of exact parameter names.
#' @param regex_pars Optional regex; matching parameter names are included
#'   in addition to those named in `pars`.
#' @param ... Unused.
#'
#' @return A matrix with one row per parameter and two columns for the
#'   lower and upper interval bounds.
#'
#' @method posterior_interval bayesqm_fit
#' @export
posterior_interval.bayesqm_fit <- function(object, prob = 0.95,
                                           pars = NULL, regex_pars = NULL,
                                           ...) {
  m <- flatten_draws_to_matrix(object)
  if (!is.null(pars) || !is.null(regex_pars)) {
    keep <- logical(ncol(m))
    if (!is.null(pars))       keep <- keep | (colnames(m) %in% pars)
    if (!is.null(regex_pars)) keep <- keep | grepl(regex_pars, colnames(m))
    if (!any(keep)) .bq_abort("no parameters matched pars / regex_pars.")
    m <- m[, keep, drop = FALSE]
  }
  alpha <- 1 - prob
  out <- t(apply(m, 2, quantile, probs = c(alpha / 2, 1 - alpha / 2),
                 names = FALSE, na.rm = TRUE))
  colnames(out) <- sprintf("%.1f%%", 100 * c(alpha / 2, 1 - alpha / 2))
  out
}


#' Prior summary generic
#'
#' @description
#' Generic for summarizing the priors a model was fit with, defined here so
#' the package needs no Stan-ecosystem dependency. Compatible with the
#' generic of the same name in \pkg{rstantools}.
#'
#' @param object A fitted model object.
#' @param ... Passed to methods.
#'
#' @return See the method documentation.
#' @export
prior_summary <- function(object, ...) UseMethod("prior_summary")

#' Prior summary for a bayesqm_fit
#'
#' @description
#' The partition model's priors, as a printable `bayesqm_prior` object.
#' Method for [prior_summary()].
#'
#' @param object A `bayesqm_fit`.
#' @param ... Unused.
#'
#' @return A `bayesqm_prior` data frame with columns `parameter` and
#'   `prior`.
#'
#' @method prior_summary bayesqm_fit
#' @export
prior_summary.bayesqm_fit <- function(object, ...) {
  sc <- object$brief$settings$sigma_scale
  out <- data.frame(
    parameter = c("f_jk (statement scores)",
                  "lambda_ik (loadings)",
                  "sigma_k (loading scales)"),
    prior = c("Normal(0, 1), column conventions applied to draws",
              "Normal(0, sigma_k^2)",
              sprintf("half-Normal(0, %g)", sc)),
    stringsAsFactors = FALSE
  )
  class(out) <- c("bayesqm_prior", "data.frame")
  out
}

#' @export
print.bayesqm_prior <- function(x, ...) {
  cat("Priors of the partition model:\n")
  for (r in seq_len(nrow(x)))
    cat(sprintf("  %-26s %s\n", x$parameter[r], x$prior[r]))
  invisible(x)
}


# posterior-package bridge; registered in .onLoad when posterior is present
#' @keywords internal
#' @noRd
as_draws.bayesqm_fit <- function(x, ...) {
  posterior::as_draws_matrix(flatten_draws_to_matrix(x))
}

#' @keywords internal
#' @noRd
as_draws_df.bayesqm_fit <- function(x, ...) {
  posterior::as_draws_df(flatten_draws_to_matrix(x))
}

#' @keywords internal
#' @noRd
as_draws_matrix.bayesqm_fit <- function(x, ...) {
  posterior::as_draws_matrix(flatten_draws_to_matrix(x))
}

#' @keywords internal
#' @noRd
as_draws_array.bayesqm_fit <- function(x, ...) {
  posterior::as_draws_array(as.array(x))
}

#' @keywords internal
#' @noRd
as_draws_rvars.bayesqm_fit <- function(x, ...) {
  posterior::as_draws_rvars(as_draws_df.bayesqm_fit(x))
}

#' @keywords internal
#' @noRd
as_draws_list.bayesqm_fit <- function(x, ...) {
  posterior::as_draws_list(as_draws_df.bayesqm_fit(x))
}

#' @keywords internal
#' @noRd
ndraws.bayesqm_fit <- function(x, ...) dim(x$draws$F)[1]

# Standard R accessors on the new fit object.

test_that("coef returns bounded loadings with names", {
  fit <- make_fake_fit()
  co <- coef(fit)
  expect_identical(dim(co), c(6L, 2L))
  expect_identical(rownames(co), paste0("P", 1:6))
  expect_true(all(abs(co) <= 1))
})

test_that("fitted reconstructs on the utility scale, residuals refuse", {
  fit <- make_fake_fit()
  fv <- fitted(fit)
  expect_identical(dim(fv), dim(fit$dataset))
  expect_identical(dimnames(fv), dimnames(fit$dataset))
  expect_error(residuals(fit), "no residual scale")
})

test_that("nobs, sigma, family behave", {
  fit <- make_fake_fit()
  expect_identical(nobs(fit), 6L)
  s <- sigma(fit)
  expect_named(s, c("f1", "f2"))
  expect_true(all(s > 0))
  out <- paste(capture.output(print(family(fit))), collapse = "\n")
  expect_match(out, "exact partition")
})

test_that("the flat draw matrix follows the bracketed naming convention", {
  fit <- make_fake_fit(T = 50)
  m <- as.matrix(fit)
  expect_identical(nrow(m), 50L)
  expect_true(all(c("Lambda[1,1]", "F[10,2]", "sigma[2]", "s_i[6]") %in%
                    colnames(m)))
  expect_identical(ncol(m), 6L * 2L + 10L * 2L + 2L + 6L)
})

test_that("posterior_interval subsets and validates", {
  fit <- make_fake_fit()
  pi_all <- posterior_interval(fit)
  expect_identical(colnames(pi_all), c("2.5%", "97.5%"))
  pi_sig <- posterior_interval(fit, regex_pars = "^sigma")
  expect_identical(nrow(pi_sig), 2L)
  expect_error(posterior_interval(fit, pars = "nope"), "matched")
})

test_that("prior_summary prints the partition priors", {
  fit <- make_fake_fit()
  out <- paste(capture.output(print(prior_summary(fit))), collapse = "\n")
  expect_match(out, "sigma_k")
  expect_match(out, "half-Normal")
})

# colors.R
# A bayesplot-style colour-scheme system. Plots in this package read
# their palette through bayesqm_colors(); setting a scheme with
# bayesqm_set_colors() reshapes every plot without touching user code.
# A custom named list can also be supplied.


.bq_env <- new.env(parent = emptyenv())
.bq_env$scheme <- "sorts"
.bq_env$custom <- NULL


# sorts-ramp semantic colours shared by the verdict-coloured plots
.bq_verdict <- c(distinguishing = "#922B21", consensus = "#17A589",
                 indeterminate = "grey72")


# placement-certainty scale for the factor array: light floor so low
# certainty stays visibly pale, dark navy top
.bq_certainty <- c("#DEEBF7", "#4292C6", "#08306B")


# Built-in schemes. Each provides six named slots consumed by the
# plotting code: a dark line/point colour, an accent colour (used
# sparingly for warning-rules and decision marks), a grey for axes /
# zero rules, a lighter grey for descriptive cut-offs, a "fill" for
# histograms / shaded bands, and "qual", the qualitative palette for
# factor identity, drawn from the plot_sorts() colour ramp (never
# cycled, shared by all schemes so factor colours stay stable when the
# scheme changes).
.bq_schemes <- list(
  sorts = list(
    dark     = "#1B4F72",
    accent   = "#922B21",
    grey     = "grey40",
    gridgrey = "grey75",
    fill     = "#D6EAF8",
    qual     = c("#2E86C1", "#E67E22", "#17A589", "#6C3483",
                 "#E74C3C", "#922B21", "#F4D03F", "#5D6D7E")
  ),
  blue = list(
    dark     = "#03306b",
    accent   = "#8F272C",
    grey     = "grey40",
    gridgrey = "grey75",
    fill     = "#b3cde0",
    qual     = c("#2E86C1", "#E67E22", "#17A589", "#6C3483",
                 "#E74C3C", "#922B21", "#F4D03F", "#5D6D7E")
  ),
  teal = list(
    dark     = "#00441b",
    accent   = "#b2182b",
    grey     = "grey40",
    gridgrey = "grey75",
    fill     = "#a8ddb5",
    qual     = c("#2E86C1", "#E67E22", "#17A589", "#6C3483",
                 "#E74C3C", "#922B21", "#F4D03F", "#5D6D7E")
  ),
  red = list(
    dark     = "#67001f",
    accent   = "#2166ac",
    grey     = "grey40",
    gridgrey = "grey75",
    fill     = "#f4a582",
    qual     = c("#2E86C1", "#E67E22", "#17A589", "#6C3483",
                 "#E74C3C", "#922B21", "#F4D03F", "#5D6D7E")
  ),
  purple = list(
    dark     = "#3f007d",
    accent   = "#b35806",
    grey     = "grey40",
    gridgrey = "grey75",
    fill     = "#bcbddc",
    qual     = c("#2E86C1", "#E67E22", "#17A589", "#6C3483",
                 "#E74C3C", "#922B21", "#F4D03F", "#5D6D7E")
  ),
  grey = list(
    dark     = "#1a1a1a",
    accent   = "#8F272C",
    grey     = "grey40",
    gridgrey = "grey75",
    fill     = "#cccccc",
    qual     = c("#2E86C1", "#E67E22", "#17A589", "#6C3483",
                 "#E74C3C", "#922B21", "#F4D03F", "#5D6D7E")
  )
)


#' Get or set the bayesqm colour scheme
#'
#' @description
#' Every plot in the package reads its palette through
#' `bayesqm_colors()`. Call `bayesqm_set_colors()` to switch the active
#' scheme for every subsequent plot. The available built-in schemes are
#' `"sorts"` (default, the [plot_sorts()] colour family), `"blue"`,
#' `"teal"`, `"red"`, `"purple"`, and `"grey"`.
#' For full control, pass a named list with slots `dark`, `accent`,
#' `grey`, `gridgrey`, and `fill`.
#'
#' @param scheme Character name of a built-in scheme, or a named list
#'   of colours with the slot names listed in the description.
#'
#' @return `bayesqm_colors()` returns the active palette as a named
#'   list. `bayesqm_set_colors()` returns the previous scheme name,
#'   invisibly.
#'
#' @examples
#' bayesqm_colors()
#' old <- bayesqm_set_colors("teal")
#' bayesqm_colors()[["fill"]]
#' bayesqm_set_colors(old)
#'
#' @name bayesqm-colors
#' @aliases bayesqm_colors bayesqm_set_colors
#' @export
bayesqm_colors <- function() {
  if (identical(.bq_env$scheme, "custom") && !is.null(.bq_env$custom))
    return(.bq_env$custom)
  .bq_schemes[[.bq_env$scheme]]
}


#' @rdname bayesqm-colors
#' @export
bayesqm_set_colors <- function(scheme) {
  required <- c("dark", "accent", "grey", "gridgrey", "fill")

  if (is.character(scheme) && length(scheme) == 1L) {
    if (!scheme %in% names(.bq_schemes))
      stop("Unknown scheme '", scheme, "'. Available: ",
           paste(names(.bq_schemes), collapse = ", "))
    old <- .bq_env$scheme
    .bq_env$scheme <- scheme
    .bq_env$custom <- NULL
    return(invisible(old))
  }

  if (is.list(scheme)) {
    missing <- setdiff(required, names(scheme))
    if (length(missing))
      stop("Custom scheme is missing slot(s): ",
           paste(missing, collapse = ", "))
    if (is.null(scheme$qual)) scheme$qual <- .bq_schemes$blue$qual
    old <- .bq_env$scheme
    .bq_env$custom <- scheme[c(required, "qual")]
    .bq_env$scheme <- "custom"
    return(invisible(old))
  }

  stop("scheme must be a character name or a named list of colours.")
}

test_that("bayesqm_colors returns a complete palette", {
  p <- bayesqm_colors()
  expect_type(p, "list")
  for (slot in c("dark", "accent", "grey", "gridgrey", "fill", "qual"))
    expect_true(slot %in% names(p), info = paste0("missing: ", slot))
})

test_that("bayesqm_set_colors switches between built-in schemes", {
  old <- bayesqm_set_colors("teal")
  on.exit(bayesqm_set_colors(old), add = TRUE)

  p <- bayesqm_colors()
  expect_equal(p$dark, "#00441b")

  bayesqm_set_colors("red")
  expect_equal(bayesqm_colors()$dark, "#67001f")
})

test_that("bayesqm_set_colors accepts a custom list", {
  old <- bayesqm_set_colors(list(
    dark     = "#000000",
    accent   = "#ff0000",
    grey     = "#aaaaaa",
    gridgrey = "#cccccc",
    fill     = "#dddddd"
  ))
  on.exit(bayesqm_set_colors(old), add = TRUE)

  expect_equal(bayesqm_colors()$dark,   "#000000")
  expect_equal(bayesqm_colors()$accent, "#ff0000")
})

test_that("bayesqm_set_colors rejects unknown schemes and incomplete lists", {
  expect_error(bayesqm_set_colors("nonexistent"), "Unknown scheme")
  expect_error(bayesqm_set_colors(list(dark = "#fff", accent = "#000")),
               "missing slot")
  expect_error(bayesqm_set_colors(42), "character name or a named list")
})

test_that("the active scheme is what bayesqm_colors() serves", {
  old <- bayesqm_set_colors("red")
  on.exit(bayesqm_set_colors(old), add = TRUE)
  active <- bayesqm_colors()
  expect_true(all(c("dark", "accent", "grey", "gridgrey", "fill")
                  %in% names(active)))
  bayesqm_set_colors("blue")
  expect_false(identical(bayesqm_colors()$dark, active$dark))
})

# utils.R
# Internal helpers shared across the package. Not exported.


# Apply `fun` over the last two margins of a [Td, dim2, dim3] draws array,
# preserving the dim2 x dim3 matrix shape (avoids the apply() drop-when-K=1
# bug) and propagating dimnames.
.summarize_draws <- function(draws, fun, ...) {
  out <- apply(draws, c(2, 3), fun, ...)
  dim(out) <- dim(draws)[2:3]
  dimnames(out) <- dimnames(draws)[2:3]
  out
}


# Internal stop with a "[bayesqm]" prefix, sprintf semantics, and the
# caller's frame as call.
.bq_abort <- function(fmt, ..., call = sys.call(-1L)) {
  cond <- structure(
    class   = c("bayesqm_error", "error", "condition"),
    list(message = paste0("[bayesqm] ", sprintf(fmt, ...)),
         call    = call))
  stop(cond)
}


assert_bayesqm_fit <- function(x, var_name = deparse(substitute(x))) {
  if (!inherits(x, "bayesqm_fit"))
    .bq_abort("`%s` must be a bayesqm_fit object, not %s.",
              var_name, class(x)[1L])
  invisible(x)
}

# helpers.R
# Data generation and evaluation utilities for Bayesian Q methodology.


#' Simulate Q-sort data
#'
#' @description
#' `generate_data()` is the top-level data-generating function used
#' by the package's simulation studies and tests. It builds a
#' loading matrix (`generate_loadings()`), factor scores, noise of the
#' chosen type (`generate_noise()`), and discretises the continuous
#' signal onto a forced Q-sort grid (`discretize_to_grid()`). See also
#' `get_distribution()` for the standard forced-distribution lookup.
#'
#' @param N,J,K Numbers of participants, statements, and factors.
#' @param noise_sd Residual SD.
#' @param error_type One of `"normal"`, `"t"`, `"contaminated"`.
#' @param nu Degrees of freedom for `error_type = "t"`.
#' @param contam_prop,contam_scale Contamination rate and scale.
#' @param loading_type `"simple"` or `"complex"`.
#' @param primary_range,cross_range Uniform ranges for primary and
#'   cross-loadings.
#' @param seed Optional RNG seed; restored on exit.
#' @param Y_cont Continuous scores (for `discretize_to_grid()`).
#' @param distr Integer forced-distribution counts.
#' @param type For `generate_noise()`, one of `"normal"`, `"t"`,
#'   `"contaminated"`. For `generate_loadings()`, `"simple"` or
#'   `"complex"`.
#' @param sd Residual SD for `generate_noise()`.
#'
#' @return `generate_data()` returns a list with `Y`, `Lambda_true`,
#'   `F_true`, `distribution`, `N`, `J`, `K`. The component helpers
#'   return their respective raw objects.
#'
#' @name generate_data
#' @aliases generate_loadings generate_noise discretize_to_grid get_distribution
#' @export
generate_data <- function(N, J, K, noise_sd = 1, error_type = "normal",
                          nu = 5, contam_prop = 0.10, contam_scale = 4,
                          loading_type = "simple",
                          primary_range = c(0.55, 0.85),
                          cross_range  = c(-0.15, 0.15),
                          seed = NULL) {
  if (!is.null(seed)) set.seed(seed)

  distr  <- get_distribution(J)
  Lambda <- generate_loadings(N, K, primary_range = primary_range,
                              cross_range = cross_range,
                              type = loading_type)
  Fmat <- matrix(rnorm(J * K), nrow = J, ncol = K)
  mu   <- Fmat %*% t(Lambda)
  E    <- generate_noise(J, N, type = error_type, sd = noise_sd,
                         nu = nu, contam_prop = contam_prop,
                         contam_scale = contam_scale)
  Y <- discretize_to_grid(mu + E, distr)

  list(Y = Y, Lambda_true = Lambda, F_true = Fmat,
       distribution = distr, N = N, J = J, K = K)
}


#' @rdname generate_data
#' @export
generate_loadings <- function(N, K, primary_range = c(0.55, 0.85),
                              cross_range = c(-0.15, 0.15),
                              type = "simple") {
  Lambda <- matrix(0, N, K)
  assignment <- rep(seq_len(K), length.out = N)

  for (i in seq_len(N)) {
    kp <- assignment[i]
    Lambda[i, kp] <- runif(1, primary_range[1], primary_range[2])
    for (k in seq_len(K))
      if (k != kp) Lambda[i, k] <- runif(1, cross_range[1], cross_range[2])
  }

  if (type == "complex" && K >= 2) {
    nc  <- max(1, round(0.20 * N))
    idx <- sample.int(N, nc)
    for (i in idx) {
      kp <- assignment[i]
      ks <- sample(setdiff(seq_len(K), kp), 1)
      Lambda[i, ks] <- runif(1, 0.25, 0.45)
    }
  }

  colnames(Lambda) <- paste0("V", seq_len(K))
  rownames(Lambda) <- paste0("p", seq_len(N))
  Lambda
}


#' @rdname generate_data
#' @export
generate_noise <- function(J, N, type = "normal", sd = 1,
                           nu = 5, contam_prop = 0.10, contam_scale = 4) {
  n <- J * N
  E <- switch(type,
    normal = rnorm(n, 0, sd),
    t = rt(n, df = nu) * sd,
    contaminated = {
      is_outlier <- rbinom(n, 1, contam_prop)
      ifelse(is_outlier == 1, rnorm(n, 0, contam_scale * sd), rnorm(n, 0, sd))
    },
    stop("Unknown error type: ", type)
  )
  matrix(E, nrow = J, ncol = N)
}


#' @rdname generate_data
#' @export
discretize_to_grid <- function(Y_cont, distr) {
  J <- nrow(Y_cont); N <- ncol(Y_cont)
  n_pos <- length(distr)

  if (n_pos %% 2 == 1) {
    half <- (n_pos - 1) / 2
    grid_vals <- seq(-half, half)
  } else {
    half <- n_pos / 2
    grid_vals <- c(seq(-half, -1), seq(1, half))
  }
  scores <- rep(grid_vals, times = distr)

  if (sum(distr) != J)
    stop("Forced distribution must sum to J. Got ", sum(distr), " for J = ", J)

  Y <- matrix(NA_integer_, J, N)
  for (n in seq_len(N)) {
    rnk    <- rank(Y_cont[, n], ties.method = "random")
    Y[, n] <- scores[rnk]
  }
  Y
}


#' @rdname generate_data
#' @export
get_distribution <- function(J) {
  tbl <- list(
    "9"  = c(1, 1, 2, 1, 2, 1, 1),
    "13" = c(1, 1, 2, 2, 3, 2, 1, 1),
    "15" = c(1, 2, 2, 3, 3, 2, 2),
    "19" = c(1, 2, 2, 3, 3, 3, 2, 2, 1),
    "20" = c(1, 2, 2, 3, 4, 3, 2, 2, 1),
    "23" = c(1, 2, 3, 3, 4, 3, 3, 2, 2),
    "30" = c(2, 3, 3, 4, 6, 4, 3, 3, 2),
    "33" = c(1, 3, 4, 5, 5, 5, 4, 3, 3),
    "34" = c(2, 3, 4, 5, 6, 5, 4, 3, 2),
    "40" = c(2, 3, 4, 5, 6, 6, 5, 4, 3, 2),
    "42" = c(2, 3, 4, 5, 7, 7, 5, 4, 3, 2),
    "47" = c(2, 3, 4, 5, 6, 7, 6, 5, 4, 3, 2),
    "48" = c(2, 3, 4, 5, 7, 6, 7, 5, 4, 3, 2),
    "60" = c(2, 4, 5, 7, 8, 8, 8, 7, 5, 4, 2)
  )
  key <- as.character(J)
  if (key %in% names(tbl)) return(tbl[[key]])

  for (w in c(7, 9, 11, 5, 13)) {
    hw   <- (w - 1) / 2
    base <- c(seq_len(hw), hw + 1, rev(seq_len(hw)))
    if (J >= sum(base)) {
      base[(w + 1) / 2] <- base[(w + 1) / 2] + (J - sum(base))
      return(base)
    }
  }

  d <- rep(J %/% 7, 7)
  d[4] <- d[4] + (J - sum(d))
  d
}


#' Simulation-study assessment helpers
#'
#' @description
#' `assess_recovery()` compares a point estimate and optional posterior
#' draws to a known loading truth, returning RMSE, per-factor RMSE,
#' bias, Tucker's congruence, credible-interval coverage, width, and
#' Gneiting-Raftery interval score. `assess_classification()` compares
#' a logical flag matrix to the true factor assignments using optimal
#' permutation matching.
#'
#' @param Lambda_hat Estimated loading matrix (N x K).
#' @param Lambda_true True loading matrix.
#' @param Lambda_draws Optional array of shape `[T, N, K]` of posterior
#'   draws, used for coverage / interval metrics.
#' @param prob Credible-interval probability.
#' @param flags Logical matrix of factor assignments.
#'
#' @return `assess_recovery()` returns a list of metrics;
#'   `assess_classification()` returns a list with `accuracy` and
#'   `per_factor`.
#'
#' @name assess_recovery
#' @aliases assess_classification
#' @export
assess_recovery <- function(Lambda_hat, Lambda_true, Lambda_draws = NULL,
                            prob = 0.95) {
  K <- ncol(Lambda_true)
  L_aligned <- procrustes_rotation(Lambda_hat, Lambda_true)

  diff     <- L_aligned - Lambda_true
  rmse     <- sqrt(mean(diff^2))
  rmse_fac <- sqrt(colMeans(diff^2))
  bias     <- mean(diff)
  bias_fac <- colMeans(diff)

  tuck <- numeric(K)
  for (k in seq_len(K))
    tuck[k] <- tucker_congruence(L_aligned[, k], Lambda_true[, k])

  coverage <- ci_width <- is_score <- NA_real_
  ci_lo <- ci_hi <- NULL

  if (!is.null(Lambda_draws)) {
    nd <- dim(Lambda_draws)[1]
    alpha <- 1 - prob

    # Single Procrustes from posterior mean to truth, applied to all draws
    R <- attr(procrustes_rotation(Lambda_hat, Lambda_true), "rotation")
    aligned_draws <- Lambda_draws
    for (s in seq_len(nd))
      aligned_draws[s, , ] <- Lambda_draws[s, , ] %*% R

    dm <- dim(aligned_draws)[2:3]
    ci_lo <- apply(aligned_draws, c(2, 3), quantile, probs =     alpha / 2)
    ci_hi <- apply(aligned_draws, c(2, 3), quantile, probs = 1 - alpha / 2)
    dim(ci_lo) <- dm; dim(ci_hi) <- dm

    inside   <- (Lambda_true >= ci_lo) & (Lambda_true <= ci_hi)
    coverage <- mean(inside)
    ci_width <- mean(ci_hi - ci_lo)

    # Interval score (Gneiting & Raftery, 2007)
    is_score <- mean((ci_hi - ci_lo) +
                     (2 / alpha) * pmax(ci_lo - Lambda_true, 0) +
                     (2 / alpha) * pmax(Lambda_true - ci_hi, 0))
  }

  list(rmse = rmse, rmse_factor = rmse_fac,
       bias = bias, bias_factor = bias_fac,
       tucker = tuck, ci_lower = ci_lo, ci_upper = ci_hi,
       coverage = coverage, ci_width = ci_width,
       interval_score = is_score)
}


#' @rdname assess_recovery
#' @export
assess_classification <- function(flags, Lambda_true) {
  N <- nrow(Lambda_true); K <- ncol(Lambda_true)
  true_assign <- apply(abs(Lambda_true), 1, which.max)

  if (is.null(flags) || all(!flags))
    return(list(accuracy = 0, per_factor = rep(0, K)))

  est <- apply(flags, 1, function(row) {
    w <- which(row)
    if (length(w) == 1) w else if (length(w) == 0) NA_integer_ else w[1]
  })

  # Align estimated labels to true labels via optimal permutation
  conf <- matrix(0, K, K)
  valid <- !is.na(est)
  for (i in which(valid))
    conf[est[i], true_assign[i]] <- conf[est[i], true_assign[i]] + 1

  if (requireNamespace("clue", quietly = TRUE)) {
    perm <- as.integer(clue::solve_LSAP(conf, maximum = TRUE))
  } else {
    perm <- integer(K); used <- logical(K)
    for (k in seq_len(K)) {
      bj <- 0; bv <- -1
      for (j in seq_len(K))
        if (!used[j] && conf[k, j] > bv) { bv <- conf[k, j]; bj <- j }
      perm[k] <- bj; used[bj] <- TRUE
    }
  }

  ea <- rep(NA_integer_, N)
  ea[valid] <- perm[est[valid]]
  matched <- !is.na(ea) & (ea == true_assign)

  pf <- numeric(K)
  for (k in seq_len(K)) {
    idx   <- which(true_assign == k)
    pf[k] <- mean(matched[idx], na.rm = TRUE)
  }

  list(accuracy = mean(matched, na.rm = TRUE), per_factor = pf)
}


#' Tucker's congruence and orthogonal Procrustes rotation
#'
#' @description
#' Linear-algebra utilities shared by MatchAlign and the recovery
#' assessments. `tucker_congruence(x, y)` computes `sum(x*y) /
#' sqrt(sum(x^2) * sum(y^2))`. `procrustes_rotation(X, Target)` finds
#' the orthogonal `R` minimising `||X R - Target||_F` and returns
#' `X %*% R` with `R` attached as attribute `"rotation"`.
#'
#' @param x,y Numeric vectors (for Tucker).
#' @param X,Target Matrices (for Procrustes).
#' @return Tucker returns a scalar; Procrustes returns the rotated
#'   matrix with a `"rotation"` attribute.
#' @name tucker_congruence
#' @aliases procrustes_rotation
#' @export
tucker_congruence <- function(x, y) {
  d <- sqrt(sum(x^2) * sum(y^2))
  if (d < 1e-12) NA_real_ else sum(x * y) / d
}


#' @rdname tucker_congruence
#' @export
procrustes_rotation <- function(X, Target) {
  X <- as.matrix(X); Target <- as.matrix(Target)
  sv  <- svd(t(X) %*% Target)
  R   <- sv$u %*% t(sv$v)
  out <- X %*% R
  attr(out, "rotation") <- R
  out
}

test_that("generate_loadings produces an N x K matrix in [-1, 1]-ish range", {
  set.seed(1L)
  L <- generate_loadings(N = 30, K = 3, type = "simple")
  expect_equal(dim(L), c(30, 3))
  expect_true(all(abs(L) <= 1))
})

test_that("generate_loadings with complex type adds secondary loadings for some participants", {
  set.seed(1L)
  L <- generate_loadings(N = 30, K = 3, type = "complex")
  # At least some participants have two moderate loadings
  n_complex <- sum(apply(abs(L), 1, function(r) sum(r > 0.2) >= 2))
  expect_gt(n_complex, 0L)
})

test_that("generate_noise returns a J x N matrix for each error type", {
  set.seed(1L)
  for (type in c("normal", "t", "contaminated")) {
    E <- generate_noise(J = 10, N = 5, type = type)
    expect_equal(dim(E), c(10, 5))
  }
})

test_that("generate_noise rejects an unknown error type", {
  expect_error(generate_noise(J = 5, N = 5, type = "nope"),
               "Unknown error type")
})

test_that("assess_recovery compares estimate to truth and returns metrics", {
  set.seed(1L)
  Ltrue <- generate_loadings(N = 25, K = 2)
  Lhat  <- Ltrue + matrix(rnorm(50, 0, 0.05), 25, 2)
  res <- assess_recovery(Lhat, Ltrue)
  expect_true(is.finite(res$rmse))
  expect_length(res$rmse_factor, 2)
  expect_length(res$tucker, 2)
  expect_true(all(res$tucker > 0.9))
})

test_that("assess_recovery with posterior draws returns coverage and interval score", {
  set.seed(1L)
  Ltrue <- generate_loadings(N = 15, K = 2)
  T     <- 120
  draws <- array(rep(c(Ltrue), each = T), c(T, 15, 2)) +
           array(rnorm(T * 15 * 2, 0, 0.1), c(T, 15, 2))
  Lhat  <- apply(draws, c(2, 3), mean)
  res <- assess_recovery(Lhat, Ltrue, Lambda_draws = draws, prob = 0.9)
  expect_true(is.finite(res$coverage))
  expect_true(is.finite(res$ci_width))
  expect_true(is.finite(res$interval_score))
  expect_true(res$coverage >= 0 && res$coverage <= 1)
})

test_that("assess_classification with all TRUE flags recovers the true partition", {
  set.seed(1L)
  Ltrue <- generate_loadings(N = 12, K = 3)
  true_assign <- apply(abs(Ltrue), 1, which.max)
  flags <- matrix(FALSE, 12, 3)
  for (i in seq_len(12)) flags[i, true_assign[i]] <- TRUE
  res <- assess_classification(flags, Ltrue)
  expect_equal(res$accuracy, 1)
})

test_that("procrustes_rotation attaches the rotation matrix", {
  set.seed(1L)
  T <- matrix(rnorm(20), 10, 2)
  X <- T %*% qr.Q(qr(matrix(rnorm(4), 2, 2)))
  out <- procrustes_rotation(X, T)
  R <- attr(out, "rotation")
  expect_equal(dim(R), c(2, 2))
  expect_lt(max(abs(crossprod(R) - diag(2))), 1e-8)
})

test_that("tucker_congruence handles zero-norm inputs safely", {
  expect_true(is.na(tucker_congruence(c(0, 0), c(1, 2))))
  expect_true(is.na(tucker_congruence(c(1, 2), c(0, 0))))
})

test_that("generate_data returns the declared shape", {
  dat <- generate_data(N = 12, J = 20, K = 2, seed = 1L)
  expect_equal(dim(dat$Y),           c(20, 12))
  expect_equal(dim(dat$Lambda_true), c(12, 2))
  expect_equal(dim(dat$F_true),      c(20, 2))
  expect_equal(sum(dat$distribution), 20)
})

test_that("generate_data is reproducible for a given seed", {
  d1 <- generate_data(N = 8, J = 15, K = 2, seed = 7)
  d2 <- generate_data(N = 8, J = 15, K = 2, seed = 7)
  expect_identical(d1$Y, d2$Y)
  expect_identical(d1$Lambda_true, d2$Lambda_true)
})

test_that("generate_data without seed leaves RNG advanced (not reset)", {
  set.seed(42)
  x1 <- runif(1)

  set.seed(42)
  generate_data(N = 8, J = 15, K = 2)
  x2 <- runif(1)

  expect_false(identical(x1, x2))
})

test_that("get_distribution returns J-summing integer counts", {
  for (J in c(9L, 15L, 20L, 30L, 47L)) {
    d <- get_distribution(J)
    expect_equal(sum(d), J)
    expect_true(all(d >= 1))
  }
})

test_that("discretize_to_grid honours the forced distribution", {
  Y <- discretize_to_grid(matrix(rnorm(60), 20, 3), distr = get_distribution(20))
  for (j in seq_len(ncol(Y))) {
    tab <- as.integer(table(Y[, j]))
    expect_equal(sort(tab), sort(get_distribution(20)))
  }
})

# plot_zscores.R
# The whole-panel statement view (the classic Q z-score chart, with the
# posterior behind it) and the single-statement drill-down.


#' Statement scores across factors, whole panel
#'
#' @description
#' Every statement's score under every factor, with nested 50% and 95%
#' credible intervals — the classic Q z-score chart carrying its
#' uncertainty. Statements are ordered by how strongly the factors
#' diverge on them, so the top of the chart is where the viewpoints
#' disagree and the bottom is where they concur; the left margin marks
#' each statement's verdict.
#'
#' @param fit A `bayesqm_fit`.
#' @param order_by `"divergence"` (default; by the largest median pairwise
#'   contrast) or `"score"` (by the first factor's median score).
#' @param q,cons_floor As in [compute_qdc()] (used for the verdict marks
#'   when `K >= 2`).
#'
#' @return `fit`, invisibly.
#'
#' @export
plot_zscores <- function(fit, order_by = c("divergence", "score"),
                         q = 0.05, cons_floor = 0.95) {
  assert_bayesqm_fit(fit)
  order_by <- match.arg(order_by)
  cols <- bayesqm_colors()
  K <- fit$brief$K; J <- fit$brief$J
  fac_ids <- dimnames(fit$draws$sigma)[[2]]
  kq <- cols$qual[(seq_len(K) - 1) %% length(cols$qual) + 1]

  zs <- compute_zscores(fit)
  zs50 <- compute_zscores(fit, prob = 0.5)

  verdict <- rep("", J)
  if (K >= 2) {
    qdc <- compute_qdc(fit, q = q, cons_floor = cons_floor)
    verdict <- qdc$verdict
    ct <- attr(qdc, "contrasts")
    div <- vapply(zs$statement, function(s)
      max(abs(ct$median[ct$statement == s])), numeric(1))
  } else {
    div <- abs(zs[[paste0(fac_ids[1], "_zsc")]])
  }
  o <- if (order_by == "divergence") order(div)
       else order(zs[[paste0(fac_ids[1], "_zsc")]])
  zs <- zs[o, ]; zs50 <- zs50[o, ]; verdict <- verdict[o]

  vcol <- ifelse(grepl("^distinguishing", verdict),
                 .bq_verdict[["distinguishing"]],
                 ifelse(verdict == "consensus", .bq_verdict[["consensus"]],
                        NA))

  op <- .bq_par(mar = c(4, 6, 3, 1))
  on.exit(par(op), add = TRUE)
  xr <- range(unlist(zs[, grep("_lower|_upper", names(zs))]))
  plot(NULL, xlim = xr, ylim = c(0.5, J + 0.5), yaxt = "n",
       xlab = "statement score (z units)", ylab = "",
       main = "Statement scores by factor, 50% and 95% intervals")
  abline(v = 0, col = cols$grey)
  axis(2, at = seq_len(J), labels = zs$statement, las = 1, cex.axis = 0.7)
  off <- seq(-0.25, 0.25, length.out = K)
  for (k in seq_len(K)) {
    fk <- fac_ids[k]
    y <- seq_len(J) + off[k]
    segments(zs[[paste0(fk, "_lower")]], y, zs[[paste0(fk, "_upper")]], y,
             col = kq[k], lwd = 1.2)
    segments(zs50[[paste0(fk, "_lower")]], y, zs50[[paste0(fk, "_upper")]], y,
             col = kq[k], lwd = 3)
    points(zs[[paste0(fk, "_zsc")]], y, pch = 19, col = kq[k], cex = 0.7)
  }
  mark <- which(!is.na(vcol))
  if (length(mark))
    points(rep(xr[1], length(mark)), mark, pch = 15, col = vcol[mark],
           cex = 0.8, xpd = TRUE)
  .bq_legend("bottomright",
             legend = c(fac_ids, "distinguishing", "consensus"),
             col = c(kq, .bq_verdict[["distinguishing"]],
                     .bq_verdict[["consensus"]]),
             pch = c(rep(19, K), 15, 15), cex = 0.75)
  invisible(fit)
}


#' One statement, in depth
#'
#' @description
#' The posterior of a single statement's score under each factor, drawn
#' as full densities with the medians marked, plus the statement's
#' verdict and its pairwise contrasts in the margin text. The drill-down
#' for the statement a write-up centers on.
#'
#' @param fit A `bayesqm_fit`.
#' @param statement Statement id or index.
#' @param q,cons_floor As in [compute_qdc()].
#'
#' @return `fit`, invisibly.
#'
#' @export
plot_statement <- function(fit, statement, q = 0.05, cons_floor = 0.95) {
  assert_bayesqm_fit(fit)
  cols <- bayesqm_colors()
  K <- fit$brief$K
  stmt_ids <- dimnames(fit$draws$F)[[2]]
  j <- if (is.character(statement)) match(statement, stmt_ids)
       else as.integer(statement)
  if (is.na(j) || j < 1 || j > fit$brief$J) .bq_abort("no such statement.")
  fac_ids <- dimnames(fit$draws$sigma)[[2]]
  kq <- cols$qual[(seq_len(K) - 1) %% length(cols$qual) + 1]

  dens <- lapply(seq_len(K), function(k) density(fit$draws$F[, j, k]))
  med <- vapply(seq_len(K), function(k) median(fit$draws$F[, j, k]), numeric(1))

  sub <- ""
  if (K >= 2) {
    qdc <- compute_qdc(fit, q = q, cons_floor = cons_floor)
    ct <- attr(qdc, "contrasts")
    ct <- ct[ct$statement == stmt_ids[j], ]
    dkl <- attr(qdc, "delta_kl")
    sub <- sprintf("verdict %s. contrasts %s, critical difference %s",
                   qdc$verdict[j],
                   paste(sprintf("%s %.2f", ct$pair, ct$median), collapse = ", "),
                   paste(sprintf("%.2f", dkl), collapse = " / "))
  }

  op <- .bq_par(mar = c(4, 4, 4, 1))
  on.exit(par(op), add = TRUE)
  plot(NULL, xlim = range(unlist(lapply(dens, `[[`, "x"))),
       ylim = c(0, 1.08 * max(unlist(lapply(dens, `[[`, "y")))),
       xlab = "statement score (z units)", ylab = "density",
       main = sprintf("Statement %s", stmt_ids[j]))
  mtext(sub, side = 3, line = 0.2, cex = 0.75, col = cols$grey)
  abline(v = 0, col = cols$grey)
  for (k in seq_len(K)) {
    polygon(dens[[k]], col = adjustcolor(kq[k], 0.25), border = kq[k],
            lwd = 1.5)
    segments(med[k], 0, med[k], max(dens[[k]]$y), col = kq[k], lwd = 2,
             lty = 2)
  }
  .bq_legend("topright", legend = fac_ids, fill = adjustcolor(kq, 0.4),
             border = kq, cex = 0.8)
  invisible(fit)
}


#' @rdname plot_statement
#' @param ... Passed on.
#' @export
plot_zscore_posterior <- function(...) {
  .Deprecated("plot_statement", package = "bayesqm")
  plot_statement(...)
}

# deprecated.R
# Defunct 0.1.0 entry points. Every function here computed a quantity of
# the removed score-scale model that has no honest counterpart under the
# partition likelihood, so each errors with a pointer instead of silently
# returning different numbers under an old name. Scheduled for deletion at
# 0.3.0.

#' @keywords internal
#' @noRd
.defunct_020 <- function(what, instead) {
  .bq_abort(paste0(
    "%s was removed with the 0.1.0 score-scale model; the partition ",
    "likelihood of bayesqm 0.2.0 computes its answer differently. %s."),
    what, instead, call = sys.call(-2L))
}

#' Defunct 0.1.0 functions
#'
#' @description
#' These 0.1.0 entry points belonged to the removed Student-t score-scale
#' model and now signal an error naming their 0.2.0 replacement:
#' flag probabilities and signs come from `compute_flags()`,
#' distinguishing/consensus verdicts from `compute_qdc()`, the critical
#' difference from the posterior itself, factor enumeration from
#' `fit_ladder()` and `select_k()`, and scalar summaries from
#' `as_draws_df()`.
#'
#' @param ... Ignored.
#' @return Always signals an error.
#' @name bayesqm-defunct
#' @keywords internal
NULL

#' @rdname bayesqm-defunct
#' @export
run_bayes <- function(...)
  .defunct_020("run_bayes()", "Use fit_ladder() and select_k()")

#' @rdname bayesqm-defunct
#' @export
plot_hyper <- function(...)
  .defunct_020("plot_hyper()",
               "Factor scale summaries live in factor_characteristics()")

#' @rdname bayesqm-defunct
#' @export
select_k_peak <- function(...)
  .defunct_020("select_k_peak()",
               "The two-signal workflow in select_k() replaces ELPD-peak selection")

#' @rdname bayesqm-defunct
#' @export
select_k_sivula <- function(...)
  .defunct_020("select_k_sivula()",
               "The two-signal workflow in select_k() replaces ELPD-based selection; loo_ladder() keeps PSIS-LOO as directional corroboration")

#' @rdname bayesqm-defunct
#' @export
compute_dominant_prob <- function(...)
  .defunct_020("compute_dominant_prob()",
               "Flag probabilities with an explicit unclassified state come from compute_flags()")

#' @rdname bayesqm-defunct
#' @export
compute_dominant_sign <- function(...)
  .defunct_020("compute_dominant_sign()", "compute_flags() reports flag signs")

#' @rdname bayesqm-defunct
#' @export
compute_threshold_prob <- function(...)
  .defunct_020("compute_threshold_prob()",
               "compute_flags() applies the exceedance condition inside the flag rule")

#' @rdname bayesqm-defunct
#' @export
classify_membership <- function(...)
  .defunct_020("classify_membership()",
               "compute_flags() reports flag probabilities and the unclassified state")

#' @rdname bayesqm-defunct
#' @export
compute_divergence <- function(...)
  .defunct_020("compute_divergence()",
               "compute_qdc() gives distinguishing, consensus, and indeterminate verdicts")

#' @rdname bayesqm-defunct
#' @export
critical_delta <- function(...)
  .defunct_020("critical_delta()",
               "compute_qdc() computes the critical difference from the posterior contrast spread")

#' @rdname bayesqm-defunct
#' @export
suggest_delta <- function(...)
  .defunct_020("suggest_delta()",
               "delta_grid() is the grid-width equivalence region (population-sd definition; values differ slightly from 0.1.0)")

#' @rdname bayesqm-defunct
#' @export
compute_posterior_scalars <- function(...)
  .defunct_020("compute_posterior_scalars()",
               "Use as_draws_df() with posterior::summarise_draws()")

#' @rdname bayesqm-defunct
#' @export
demo_run <- function(...)
  .defunct_020("demo_run()", "Use demo_fit(); a ladder demo returns with fit_ladder()")

#' @rdname bayesqm-defunct
#' @export
plot_elpd <- function(...)
  .defunct_020("plot_elpd()",
               "PSIS-LOO is directional corroboration only; loo_ladder() prints the table, and plot_choice_k() shows the two-signal decision")

#' @rdname bayesqm-defunct
#' @export
make_elpd_diff <- function(...)
  .defunct_020("make_elpd_diff()",
               "PSIS-LOO is directional corroboration only; see loo_ladder() and plot_choice_k()")

#' @rdname bayesqm-defunct
#' @export
make_ppc_ridge <- function(...)
  .defunct_020("make_ppc_ridge()",
               "Per-rung posterior-predictive evidence lives in select_k() and plot_choice_k(); plot_ppc() shows a single fit's checks")

#' @rdname bayesqm-defunct
#' @export
make_dominant_panel <- function(...)
  .defunct_020("make_dominant_panel()",
               "plot_flags() shows the flag probabilities with the unclassified state")

# Model checks, the person check, the GHK bridge, and the two-signal
# ladder. Real-sampler pieces run at toy scale; the GHK total-probability
# and full-ladder runs sit behind skip_on_cran.

test_that("check_fit returns proper probabilities and prints plainly", {
  fit <- demo_fit()
  ck <- check_fit(fit, draws = 20)
  expect_true(ck$agreement$p >= 0 && ck$agreement$p <= 1)
  expect_true(ck$extra_factor$percentile >= 0 && ck$extra_factor$percentile <= 1)
  expect_true(ck$paired$p >= 0 && ck$paired$p <= 1)
  expect_identical(length(ck$paired$p_j), 13L)
  out <- paste(capture.output(print(ck)), collapse = "\n")
  expect_match(out, "agreement check \\(T1a\\)")
  expect_match(out, "extra-factor check \\(T1b\\)")
  expect_match(out, "paired-comparison check \\(T2\\)")
  expect_match(out, "diagnostics to read")
})

test_that("check_persons screens with verdicts and bands", {
  fit <- demo_fit()
  pc <- check_persons(fit, draws = 15, mixes = 30)
  expect_identical(nrow(pc), 8L)
  expect_true(all(pc$verdict %in% c("fits", "no_shared", "unspanned", "atypical")))
  expect_true(all(pc$m >= -1 & pc$m <= 1))
  expect_identical(length(attr(pc, "band_m")), 2L)
  expect_identical(dim(attr(pc, "band_w")), c(8L, 2L))
})

test_that("unspanned_cluster demands mutual partners", {
  pc <- data.frame(participant = c("P1", "P2", "P3"),
                   partner = c("P2", "P3", "P1"),
                   verdict = c("unspanned", "unspanned", "fits"),
                   stringsAsFactors = FALSE)
  expect_false(unspanned_cluster(pc))         # P1 -> P2 but P2 -> P3
  pc$partner <- c("P2", "P1", "P1")
  expect_true(unspanned_cluster(pc))          # P1 <-> P2
})

test_that("loglik_person returns finite log-likelihoods with CRN seeds", {
  fit <- demo_fit()
  ll <- loglik_person(fit, draws = 10, R = 32, seed = 1)
  expect_identical(dim(ll), c(10L, 8L))
  expect_true(all(is.finite(ll)))
  expect_true(all(ll < 0))
  ll2 <- loglik_person(fit, draws = 10, R = 32, seed = 1)
  expect_identical(ll, ll2)                   # deterministic given the seed
})

test_that("GHK probabilities sum to one over all sorts of a toy grid", {
  skip_on_cran()
  distr <- c(1, 2, 1)
  sorts <- list()
  rec <- function(prefix, left) {
    if (!any(left > 0)) { sorts[[length(sorts) + 1]] <<- prefix; return(invisible()) }
    for (c in which(left > 0)) {
      l2 <- left; l2[c] <- l2[c] - 1L
      rec(c(prefix, c), l2)
    }
  }
  rec(integer(0), as.integer(distr))
  set.seed(2)
  m <- rnorm(4, 0, 0.8)
  tot <- sum(vapply(sorts, function(y)
    exp(ghk_loglik(y, m, distr, R = 8192, seed = 9)), numeric(1)))
  expect_equal(tot, 1, tolerance = 0.05)      # the paper's own G1b bar
})

test_that("the ladder fits, selects, and re-selects without refitting", {
  skip_on_cran()
  set.seed(21)
  N <- 9; J <- 13; K0 <- 2
  distr <- get_distribution(J)
  L0 <- matrix(0, N, K0)
  L0[cbind(seq_len(N), rep_len(seq_len(K0), N))] <- 1.4
  F0 <- matrix(rnorm(J * K0), J, K0)
  Y <- t(quota_sort_rows(L0 %*% t(F0) + matrix(rnorm(N * J), N, J), distr))
  qd <- qsort_data(Y, distribution = distr, validate = FALSE)

  lad <- fit_ladder(qd, K_min = 2, K_max = 3, iterations = 700, burn = 100,
                    thin = 3, max_iterations = 700, seed = 3,
                    rhat_max = Inf, ess_min = 0, quiet = TRUE)
  expect_s3_class(lad, "bayesqm_ladder")
  expect_identical(lad$K, 2:3)

  sel <- select_k(lad)
  expect_s3_class(sel, "bayesqm_selection")
  expect_true(sel$verdict %in% c("selected", "tension",
                                 "adequate_but_unsupported", "no_adequate_rung",
                                 "no_shared_structure", "single_viewpoint"))
  expect_identical(nrow(sel$table), 2L)
  out <- paste(capture.output(print(sel)), collapse = "\n")
  expect_match(out, "two-signal rule")

  sel2 <- select_k(lad, q = 0.2)              # re-select, no refit
  expect_identical(sel2$q, 0.2)
})

test_that("a noise panel reaches the no-shared-structure branch", {
  skip_on_cran()
  set.seed(5)
  N <- 8; J <- 13
  distr <- get_distribution(J)
  Y <- t(quota_sort_rows(matrix(rnorm(N * J), N, J), distr))
  qd <- qsort_data(Y, distribution = distr, validate = FALSE)
  lad <- fit_ladder(qd, K_min = 2, K_max = 2, iterations = 500, burn = 100,
                    thin = 4, max_iterations = 500, seed = 7,
                    rhat_max = Inf, ess_min = 0, quiet = TRUE)
  sel <- select_k(lad)
  expect_true(sel$verdict %in% c("no_shared_structure", "no_adequate_rung",
                                 "adequate_but_unsupported", "single_viewpoint"))
  expect_true(is.na(sel$K))
})

test_that("the selection carries per-factor evidence and plots the decision", {
  skip_on_cran()
  set.seed(21)
  N <- 9; J <- 13; K0 <- 2
  distr <- get_distribution(J)
  L0 <- matrix(0, N, K0)
  L0[cbind(seq_len(N), rep_len(seq_len(K0), N))] <- 1.4
  F0 <- matrix(rnorm(J * K0), J, K0)
  Y <- t(quota_sort_rows(L0 %*% t(F0) + matrix(rnorm(N * J), N, J), distr))
  qd <- qsort_data(Y, distribution = distr, validate = FALSE)
  lad <- fit_ladder(qd, K_min = 2, K_max = 3, iterations = 700, burn = 100,
                    thin = 3, max_iterations = 700, seed = 3,
                    rhat_max = Inf, ess_min = 0, quiet = TRUE)
  sel <- select_k(lad)
  expect_identical(names(sel$detail),
                   c("K", "factor", "flags", "has_dist", "supported"))
  expect_identical(nrow(sel$detail), 2L + 3L)      # per-factor rows per rung
  expect_identical(sel$detail$supported,
                   sel$detail$flags >= 2 & sel$detail$has_dist)
  tmp <- tempfile(fileext = ".pdf"); grDevices::pdf(tmp)
  expect_invisible(plot_choice_k(sel))
  grDevices::dev.off(); unlink(tmp)
})

test_that("dead plot names error with pointers; plot_tucker forwards", {
  expect_error(plot_elpd(), "plot_choice_k")
  expect_error(make_elpd_diff(), "loo_ladder")
  expect_error(make_ppc_ridge(), "select_k")
  expect_error(make_dominant_panel(), "plot_flags")
  fit <- demo_fit()
  tmp <- tempfile(fileext = ".pdf"); grDevices::pdf(tmp)
  expect_warning(plot_tucker(fit), "plot_convergence")
  grDevices::dev.off(); unlink(tmp)
})

test_that("unanimity and emptiness get opposite null verdicts", {
  fake_rung <- function(K, flags_n, has_dist) {
    fit <- make_fake_fit(N = 6, J = 10, K = K)
    fac <- dimnames(fit$draws$sigma)[[2]]
    list(K = K, fit = fit, t1b = 0.5, cluster = FALSE,
         converged = TRUE,
         supported = rep(FALSE, K),
         flags = data.frame(participant = if (flags_n) paste0("P", seq_len(flags_n)) else character(0),
                            factor = rep(fac[1], flags_n),
                            sign = rep(1, flags_n),
                            flag_prob = rep(0.9, flags_n)),
         distinguishing = data.frame(statement = character(0),
                                     factor = character(0),
                                     dist_prob = numeric(0)))
  }
  unanimous <- structure(list(K = 2L, rungs = list(fake_rung(2, 5, FALSE)),
                              q = 0.05, N = 6), class = "bayesqm_ladder")
  expect_identical(select_k(unanimous)$verdict, "single_viewpoint")
  empty <- structure(list(K = 2L, rungs = list(fake_rung(2, 0, FALSE)),
                          q = 0.05, N = 6), class = "bayesqm_ladder")
  expect_identical(select_k(empty)$verdict, "no_shared_structure")
})

# Defunct 0.1.0 entry points must error with a pointer, never return
# different numbers under an old name.

test_that("defunct stubs error with their replacement named", {
  expect_error(run_bayes(), "fit_ladder")
  expect_error(select_k_peak(), "select_k")
  expect_error(select_k_sivula(), "loo_ladder")
  expect_error(compute_dominant_prob(), "compute_flags")
  expect_error(compute_dominant_sign(), "compute_flags")
  expect_error(compute_threshold_prob(), "compute_flags")
  expect_error(classify_membership(), "compute_flags")
  expect_error(compute_divergence(), "compute_qdc")
  expect_error(critical_delta(), "compute_qdc")
  expect_error(suggest_delta(), "delta_grid")
  expect_error(compute_posterior_scalars(), "as_draws_df")
  expect_error(demo_run(), "demo_fit")
})

# gibbs.R
# The exact partition (rank-order) likelihood engine: quota sort, PX-Gibbs
# sampler with warm-started continuation, and the convergence gate on the
# invariant person spreads s_i.

#' Grid width on the z scale
#'
#' @description
#' The width of one grid column on the standardized score scale,
#' `1 / sd(forced positions)` with the population standard deviation. This
#' is the equivalence-region half-width used for consensus verdicts: a
#' score contrast smaller than one grid column is not expressible in a
#' completed sort.
#'
#' @param distribution Integer vector of forced-distribution counts (the
#'   number of statements allowed in each grid column).
#'
#' @return A single numeric value.
#'
#' @examples
#' delta_grid(c(2, 3, 4, 5, 7, 7, 5, 4, 3, 2))
#'
#' @export
delta_grid <- function(distribution) {
  v <- rep(seq_along(distribution), distribution)
  1 / sqrt(mean((v - mean(v))^2))
}

# rank a utility vector onto the forced grid; ties impossible a.s.
#' @keywords internal
#' @noRd
quota_sort <- function(u, distr) {
  y <- integer(length(u))
  y[order(u)] <- rep(seq_along(distr), distr)
  y
}

# all rows at once; identical to row-wise quota_sort, no RNG involved
#' @keywords internal
#' @noRd
quota_sort_rows <- function(U, distr) {
  Y <- array(0L, dim(U))
  Y[order(row(U), U)] <- rep(rep(seq_along(distr), distr), nrow(U))
  Y
}

# midrank of each category under the quotas; Spearman on quota sorts equals
# Pearson on these recodes
#' @keywords internal
#' @noRd
midranks <- function(distr) {
  hi <- cumsum(distr)
  (hi + c(1, head(hi, -1) + 1)) / 2
}

# inverse-cdf truncated normal; the clamp guards the extreme-tail case where
# both cdf values round to the same double
#' @keywords internal
#' @noRd
rtnorm <- function(m, lo, hi) {
  plo <- pnorm(lo, m, 1); phi <- pnorm(hi, m, 1)
  u <- runif(length(m), plo, phi)
  pmin(pmax(qnorm(pmin(pmax(u, 1e-15), 1 - 1e-15), m, 1), lo), hi)
}

# stepping-out slice sampler (Neal) for one scalar. The shrink loop carries
# an iteration cap: draws are identical on every healthy path, but a
# degenerate spin aborts with a diagnosable error instead of freezing the
# session.
#' @keywords internal
#' @noRd
slice1 <- function(x0, logf, w = 0.5, lower = 1e-6, upper = 50) {
  y <- logf(x0) - rexp(1)
  L <- max(lower, x0 - w * runif(1)); R <- min(upper, L + w)
  while (L > lower && logf(L) > y) L <- max(lower, L - w)
  while (R < upper && logf(R) > y) R <- min(upper, R + w)
  it <- 0L
  repeat {
    x1 <- runif(1, L, R)
    if (logf(x1) > y) return(x1)
    if (x1 < x0) L <- x1 else R <- x1
    it <- it + 1L
    if (it > 2e5)
      stop("slice1: shrink loop exceeded 2e5 iterations (x0=",
           signif(x0, 6), ", y=", signif(y, 6), ")")
  }
}

# start U consistent with the observed sort: Blom scores, random within category
#' @keywords internal
#' @noRd
init_U <- function(Y) {
  N <- nrow(Y); J <- ncol(Y)
  blom <- qnorm(((1:J) - 0.375) / (J + 0.25))
  U <- matrix(0, N, J)
  for (i in 1:N) {
    ord <- order(Y[i, ], stats::runif(J))       # ascending category, random within
    U[i, ord] <- blom
  }
  U
}

# PX-Gibbs for the partition likelihood. Y is persons x statements in
# category codes 1..C obeying the quotas. state = NULL starts a fresh chain
# (seed, random init, burn); passing the state returned by an earlier call
# continues that chain instead — no reseed, no burn, and the captured RNG
# state is restored, so a continued chain is draw-for-draw identical to one
# long run even across sessions.
#' @keywords internal
#' @noRd
fit_partition <- function(Y, distr, K,
                          n_iter = 12000, burn = 2000, thin = 5,
                          seed = NULL, sigma_scale = 1, state = NULL,
                          init_scale = 0.1) {
  N <- nrow(Y); J <- ncol(Y); C <- length(distr)
  stopifnot(all(apply(Y, 1, function(r) all(tabulate(r, C) == distr))))
  cat_idx <- lapply(1:C, function(c) lapply(1:N, function(i) which(Y[i, ] == c)))

  if (is.null(state)) {
    if (!is.null(seed)) set.seed(seed)
    U <- init_U(Y)
    F <- matrix(rnorm(J * K, 0, init_scale), J, K)
    L <- matrix(rnorm(N * K, 0, init_scale), N, K)
    sig <- rep(1, K)
  } else {
    burn <- 0
    assign(".Random.seed", state$rng, envir = globalenv())
    U <- state$U; F <- state$F; L <- state$L; sig <- state$sig
  }

  # last keep lands on n_iter when (n_iter - burn) is a multiple of thin, so
  # a continuation block keeps the same phase
  keep <- seq(burn + thin, n_iter, by = thin)
  Fd <- array(NA_real_, c(length(keep), J, K))
  Ld <- array(NA_real_, c(length(keep), N, K))
  Sd <- matrix(NA_real_, length(keep), K)
  inv_spread <- matrix(NA_real_, length(keep), N)   # s_i, invariant diagnostic
  s <- 0L

  for (it in 1:n_iter) {
    # utilities, category by category; the lower bound is the running max of
    # the category just drawn, the upper the current min of the one above
    M <- L %*% t(F)
    for (i in 1:N) {
      ui <- U[i, ]; Mi <- M[i, ]
      lo <- -Inf
      for (c in 1:C) {
        jj <- cat_idx[[c]][[i]]
        hi <- if (c == C) Inf else min(ui[cat_idx[[c + 1]][[i]]])
        vals <- rtnorm(Mi[jj], lo, hi)
        ui[jj] <- vals
        lo <- max(vals)
      }
      U[i, ] <- ui
    }

    # PX working scale, rescale u, then lambda; V does not depend on i
    V <- chol2inv(chol(crossprod(F) + diag(1 / sig^2, K)))
    A <- U %*% F
    r <- rowSums(U^2) - rowSums((A %*% V) * A)
    v <- 1 / sqrt(rgamma(N, shape = J / 2, rate = pmax(r, 1e-12) / 2))
    U <- U / v
    A <- A / v
    cV <- chol(V)
    L <- A %*% V + matrix(rnorm(N * K), N, K) %*% cV

    # statement scores
    Vf <- chol2inv(chol(crossprod(L) + diag(K)))
    F <- t(U) %*% L %*% Vf + matrix(rnorm(J * K), J, K) %*% chol(Vf)

    # sigma_k, slice on the half-normal conditional; the bracket scales
    # with the prior so a wide sigma_scale is never silently truncated
    for (k in 1:K) {
      ssq <- sum(L[, k]^2)
      sig[k] <- slice1(sig[k], function(x)
        -N * log(x) - ssq / (2 * x^2) - x^2 / (2 * sigma_scale^2),
        upper = max(50, 10 * sigma_scale))
    }

    if (it %in% keep) {
      s <- s + 1L
      Fd[s, , ] <- F; Ld[s, , ] <- L; Sd[s, ] <- sig
      Fc <- sweep(F, 2, colMeans(F))
      inv_spread[s, ] <- sqrt(rowSums((L %*% (crossprod(Fc) / J)) * L))
    }
  }
  list(F = Fd, Lambda = Ld, sigma = Sd, s_i = inv_spread,
       state = list(U = U, F = F, L = L, sig = sig,
                    rng = get(".Random.seed", envir = globalenv())),
       N = N, J = J, K = K, distr = distr)
}

# gate statistics for one person's kept series. The single chain is split
# into two halves and posterior splits each again, giving the four-segment
# rank-normalized (and folded) split-Rhat of Vehtari et al. (2021); ESS uses
# Geyer's (1992) initial monotone sequence truncation. All deterministic, so
# checking mid-chain consumes no RNG.
#' @keywords internal
#' @noRd
conv_stats <- function(x) {
  m <- matrix(x[seq_len(2L * (length(x) %/% 2L))], ncol = 2)
  c(rhat = posterior::rhat(m),
    bulk = posterior::ess_bulk(m),
    tail = posterior::ess_tail(m))
}

# the gate: over persons, max Rhat and min bulk/tail ESS
#' @keywords internal
#' @noRd
check_conv <- function(fit, rhat_max = 1.01, ess_min = 400) {
  st <- apply(fit$s_i, 2, conv_stats)
  rhat <- suppressWarnings(max(st["rhat", ], na.rm = TRUE))
  bulk <- suppressWarnings(min(st["bulk", ], na.rm = TRUE))
  tl   <- suppressWarnings(min(st["tail", ], na.rm = TRUE))
  list(rhat = rhat, ess = bulk, ess_tail = tl,
       ok = is.finite(rhat) && rhat < rhat_max &&
            is.finite(bulk) && bulk >= ess_min &&
            is.finite(tl)   && tl >= ess_min)
}

# append a continuation block to a fit, draw arrays along the draw dimension
#' @keywords internal
#' @noRd
grow_fit <- function(fit, ext) {
  bind3 <- function(a, b) {
    d <- dim(a)
    out <- array(NA_real_, c(d[1] + dim(b)[1], d[2], d[3]))
    out[seq_len(d[1]), , ] <- a
    out[-seq_len(d[1]), , ] <- b
    out
  }
  fit$F <- bind3(fit$F, ext$F)
  fit$Lambda <- bind3(fit$Lambda, ext$Lambda)
  fit$sigma <- rbind(fit$sigma, ext$sigma)
  fit$s_i <- rbind(fit$s_i, ext$s_i)
  fit$state <- ext$state
  fit
}

# gated fit, a fixed-ESS sequential stopping rule: first check at the floor,
# then warm-started doubling of total length until the gate passes or the
# cap. The burn is paid once and no draws are discarded; the gate is always
# recomputed on the entire kept chain. Never errors on nonconvergence — the
# caller reads converged and warns.
#' @keywords internal
#' @noRd
fit_partition_gated <- function(Y, distr, K, seed = NULL, sigma_scale = 1,
                                n_iter = 12000, burn = 2000, thin = 5,
                                n_max = 48000, rhat_max = 1.01, ess_min = 400,
                                init_scale = 0.1, quiet = TRUE) {
  fit <- fit_partition(Y, distr, K, n_iter = n_iter, burn = burn, thin = thin,
                       seed = seed, sigma_scale = sigma_scale,
                       init_scale = init_scale)
  cv <- check_conv(fit, rhat_max, ess_min)
  total <- as.integer(n_iter)
  while (!cv$ok && total < n_max) {
    if (!quiet)
      message(sprintf("gate not met at %d iterations (Rhat %.3f, ESS %.0f); extending to %d",
                      total, cv$rhat, min(cv$ess, cv$ess_tail), 2L * total))
    ext <- fit_partition(Y, distr, K, n_iter = total, burn = 0, thin = thin,
                         sigma_scale = sigma_scale, state = fit$state)
    fit <- grow_fit(fit, ext)
    total <- 2L * total
    cv <- check_conv(fit, rhat_max, ess_min)
  }
  fit$converged <- cv$ok; fit$extended <- total > n_iter; fit$iters <- total
  fit$rhat <- cv$rhat; fit$ess <- cv$ess; fit$ess_tail <- cv$ess_tail
  fit
}

# The partition-likelihood engine: quota sort, grid constants, the PX-Gibbs
# sampler's warm-start contract, and the convergence gate. The bitwise
# identity tests are frozen: any change to the sampler's RNG consumption
# order must fail here before it ships.

toy_sort <- function(N = 8, J = 12, distr = c(2, 3, 4, 2, 1), seed = 1) {
  set.seed(seed)
  quota_sort_rows(matrix(rnorm(N * J), N, J), distr)
}

test_that("delta_grid pins the paper's constants", {
  expect_equal(delta_grid(c(2, 3, 4, 5, 7, 7, 5, 4, 3, 2)), 0.4268636566,
               tolerance = 1e-9)
  expect_equal(delta_grid(c(1, 2, 2, 3, 4, 3, 2, 2, 1)), 0.4714045208,
               tolerance = 1e-9)
  expect_equal(delta_grid(c(4, 5, 7, 9, 10, 10, 10, 9, 7, 5, 4)), 0.3720326659,
               tolerance = 1e-9)
})

test_that("quota_sort obeys the quotas, order, and permutation equivariance", {
  distr <- c(2, 3, 4, 5, 7, 7, 5, 4, 3, 2)
  set.seed(2)
  u <- rnorm(42)
  y <- quota_sort(u, distr)
  expect_identical(tabulate(y, 10), as.integer(distr))
  expect_true(all(diff(y[order(u)]) >= 0))
  p <- sample(42)
  expect_identical(quota_sort(u[p], distr), y[p])
  U <- matrix(rnorm(5 * 42), 5, 42)
  expect_identical(quota_sort_rows(U, distr),
                   t(apply(U, 1, quota_sort, distr = distr)))
})

test_that("midranks are the tied ranks of the grid positions", {
  distr <- c(2, 3, 4, 2, 1)
  y <- rep(seq_along(distr), distr)
  expect_equal(midranks(distr), unique(rank(y)))
  expect_equal(sum(midranks(distr) * distr), sum(seq_len(sum(distr))))
})

test_that("a continued chain is draw-for-draw identical to one long run", {
  distr <- c(2, 3, 4, 2, 1)
  Y <- toy_sort(distr = distr)
  straight <- fit_partition(Y, distr, K = 2, n_iter = 1200, burn = 200,
                            thin = 5, seed = 7)
  base <- fit_partition(Y, distr, K = 2, n_iter = 600, burn = 200,
                        thin = 5, seed = 7)
  ext <- fit_partition(Y, distr, K = 2, n_iter = 600, burn = 0, thin = 5,
                       state = base$state)
  grown <- grow_fit(base, ext)
  expect_identical(straight$F, grown$F)
  expect_identical(straight$Lambda, grown$Lambda)
  expect_identical(straight$sigma, grown$sigma)
  expect_identical(straight$s_i, grown$s_i)
  expect_identical(straight$state, grown$state)
})

test_that("the identity survives unrelated RNG use between fit and extension", {
  distr <- c(2, 3, 4, 2, 1)
  Y <- toy_sort(distr = distr)
  straight <- fit_partition(Y, distr, K = 2, n_iter = 1200, burn = 200,
                            thin = 5, seed = 7)
  base <- fit_partition(Y, distr, K = 2, n_iter = 600, burn = 200,
                        thin = 5, seed = 7)
  runif(1000); sample(1e6, 50)              # a user's session happens here
  ext <- fit_partition(Y, distr, K = 2, n_iter = 600, burn = 0, thin = 5,
                       state = base$state)
  expect_identical(straight$F, grow_fit(base, ext)$F)
})

test_that("the gate consumes no RNG", {
  distr <- c(2, 3, 4, 2, 1)
  Y <- toy_sort(distr = distr)
  fit <- fit_partition(Y, distr, K = 2, n_iter = 400, burn = 100, thin = 5,
                       seed = 3)
  before <- .Random.seed
  cv <- check_conv(fit)
  expect_identical(.Random.seed, before)
  expect_true(is.finite(cv$rhat))
})

test_that("the ladder extends to the cap and reports honestly", {
  distr <- c(2, 3, 4, 2, 1)
  Y <- toy_sort(distr = distr)
  # unreachable bar: every rung fails, the ladder stops exactly at the cap
  fit <- fit_partition_gated(Y, distr, K = 2, seed = 5, n_iter = 400,
                             burn = 100, thin = 5, n_max = 1600,
                             ess_min = 1e6)
  expect_false(fit$converged)
  expect_true(fit$extended)
  expect_identical(fit$iters, 1600L)
  expect_identical(dim(fit$F)[1], as.integer((1600 - 100) / 5))
  # trivial bar: the floor suffices, nothing extends
  fit2 <- fit_partition_gated(Y, distr, K = 2, seed = 5, n_iter = 400,
                              burn = 100, thin = 5, n_max = 1600,
                              rhat_max = Inf, ess_min = 0)
  expect_true(fit2$converged)
  expect_false(fit2$extended)
  expect_identical(fit2$iters, 400L)
})

test_that("fit_partition refuses sorts that violate the quotas", {
  distr <- c(2, 3, 4, 2, 1)
  Y <- toy_sort(distr = distr)
  Y[1, 1] <- Y[1, 1] %% 5 + 1               # break one person's counts
  expect_error(fit_partition(Y, distr, K = 2, n_iter = 200, burn = 50,
                             thin = 5, seed = 1))
})

test_that("the stored s_i matches its definition on the kept draws", {
  distr <- c(2, 3, 4, 2, 1)
  Y <- toy_sort(distr = distr)
  fit <- fit_partition(Y, distr, K = 2, n_iter = 400, burn = 100, thin = 5,
                       seed = 9)
  t <- 10
  F <- matrix(fit$F[t, , ], ncol = 2)
  L <- matrix(fit$Lambda[t, , ], ncol = 2)
  Fc <- sweep(F, 2, colMeans(F))
  expect_equal(fit$s_i[t, ],
               sqrt(rowSums((L %*% (crossprod(Fc) / ncol(Y))) * L)),
               tolerance = 1e-12)
})

# demo_fit.R
# A tiny real fit for examples, tests, and vignettes: seconds of sampling,
# no convergence gate, fixed seed. Everything a bayesqm_fit carries is
# present, at toy scale.


#' A small demonstration fit
#'
#' @description
#' Runs the partition-model sampler on a small synthetic forced-sort
#' dataset with a fixed seed and the convergence gate disabled. It exists
#' so examples and tests have a complete `bayesqm_fit` in about a second;
#' it is no substitute for [fit_bayesian()] at its defaults on real data.
#'
#' @param N,J,K Panel size, statement count, factors (defaults 8, 13, 2).
#' @param draws Kept posterior draws (default 200).
#' @param seed Random seed (default 1).
#'
#' @return A `bayesqm_fit`.
#'
#' @examples
#' fit <- demo_fit()
#' fit
#'
#' @export
demo_fit <- function(N = 8, J = 13, K = 2, draws = 200, seed = 1) {
  distr <- get_distribution(J)
  set.seed(seed)
  L0 <- matrix(0, N, K)
  grp <- rep_len(seq_len(K), N)
  L0[cbind(seq_len(N), grp)] <- 1.2
  F0 <- matrix(rnorm(J * K), J, K)
  U <- L0 %*% t(F0) + matrix(rnorm(N * J), N, J)
  Y <- t(quota_sort_rows(U, distr))
  rownames(Y) <- paste0("S", seq_len(J))
  colnames(Y) <- paste0("P", seq_len(N))

  thin <- 2L
  burn <- 100L
  fit_bayesian(qsort_data(Y, distribution = distr, validate = FALSE),
               K = K, iterations = burn + thin * draws, burn = burn,
               thin = thin, max_iterations = burn + thin * draws,
               seed = seed + 1, rhat_max = Inf, ess_min = 0,
               quiet = TRUE)
}

# demo_fit: the substrate for examples; fast, complete, reproducible.

test_that("demo_fit returns a complete converged fit in seconds", {
  fit <- demo_fit()
  expect_s3_class(fit, "bayesqm_fit")
  expect_identical(fit$brief$N, 8L)
  expect_identical(fit$brief$J, 13L)
  expect_identical(dim(fit$draws$F)[1], 200L)
  expect_true(fit$gate$converged)
})

test_that("demo_fit is reproducible by seed", {
  a <- demo_fit(seed = 2)
  b <- demo_fit(seed = 2)
  expect_identical(a$draws$F, b$draws$F)
  expect_false(identical(a$draws$F, demo_fit(seed = 3)$draws$F))
})

# data.R
# Documentation for the shipped datasets.


#' Childhood obesity Q sorts
#'
#' @description
#' The childhood obesity Q dataset from Akhtar-Danesh (2023): 33
#' participants each force-sorted 42 statements about childhood
#' obesity onto a nine-column grid (quotas 2-4-5-6-8-6-5-4-2, printed
#' values -4 to +4). A real, published panel at typical Q scale, used
#' in the package documentation.
#'
#' @format A [qsort_data] object: the 42 x 33 integer matrix of printed
#'   grid values with statement and participant ids, and the forced
#'   distribution.
#'
#' @source Akhtar-Danesh, N. (2023). Impact of factor rotation on
#'   Q-methodology analysis. \emph{PLOS ONE}, 18(9), e0290728.
#'   \doi{10.1371/journal.pone.0290728}
#'
#' @examples
#' obesity_sorts
#' plot(obesity_sorts, participants = 1:4)
"obesity_sorts"


#' Grizzly bear reintroduction Q sorts
#'
#' @description
#' The grizzly bear reintroduction Q dataset from Easter et al. (2025):
#' 67 participants each force-sorted 41 statements about reintroducing
#' grizzly bears to the North Cascades onto an eleven-column grid
#' (quotas 1-2-3-5-6-7-6-5-3-2-1, printed values -5 to +5). A real,
#' published panel where the two-signal rule selects a two-factor
#' solution, used in the package documentation.
#'
#' @format A [qsort_data] object: the 41 x 67 integer matrix of printed
#'   grid values with statement and participant ids, and the forced
#'   distribution.
#'
#' @source Easter, T. S., Santo, A. R., Sage, A. H., Carter, N. H.,
#'   Chan, K. M. A., & Ransom, J. I. (2025). Divergent values and
#'   perspectives drive three distinct viewpoints on grizzly bear
#'   reintroduction in Washington, the United States. \emph{People and
#'   Nature}, 7, 127--145. \doi{10.1002/pan3.10748}. Data from the
#'   Dryad repository under CC0, \doi{10.5061/dryad.73n5tb369}.
#'
#' @examples
#' grizzly_sorts
#' plot(grizzly_sorts, participants = 1:4)
"grizzly_sorts"

# autoplot.R
# ggplot2 versions of the flagship views, registered on ggplot2's
# autoplot generic in .onLoad. ggplot2 is a soft dependency; the base
# plot_* functions carry every view regardless.

#' @keywords internal
#' @noRd
autoplot.bayesqm_fit <- function(object,
                                 type = c("loadings", "flags", "contrasts",
                                          "array"),
                                 ...) {
  if (!requireNamespace("ggplot2", quietly = TRUE))
    .bq_abort("autoplot needs ggplot2; the base plot_* functions do not.")
  type <- match.arg(type)
  cols <- bayesqm_colors()
  fac_ids <- dimnames(object$draws$sigma)[[2]]
  K <- object$brief$K
  kq <- cols$qual[(seq_len(K) - 1) %% length(cols$qual) + 1]
  qual <- stats::setNames(kq, fac_ids)

  if (type == "loadings") {
    lo <- compute_loadings(object)
    long <- do.call(rbind, lapply(fac_ids, function(f)
      data.frame(participant = lo$participant, factor = f,
                 loading = lo[[paste0(f, "_loading")]],
                 lower = lo[[paste0(f, "_lower")]],
                 upper = lo[[paste0(f, "_upper")]])))
    p <- ggplot2::ggplot(long, ggplot2::aes(
           x = .data$loading, y = .data$participant,
           colour = .data$factor, shape = .data$factor)) +
      ggplot2::geom_vline(xintercept = 0, colour = cols$grey) +
      ggplot2::geom_errorbar(ggplot2::aes(xmin = .data$lower,
                                          xmax = .data$upper),
                             width = 0, orientation = "y",
                             position = ggplot2::position_dodge(0.6)) +
      ggplot2::geom_point(position = ggplot2::position_dodge(0.6)) +
      ggplot2::scale_colour_manual(values = qual) +
      ggplot2::xlim(-1, 1) +
      ggplot2::labs(x = "bounded loading", y = NULL)
    return(p)
  }

  if (type == "flags") {
    fl <- compute_flags(object, ...)
    phi <- attr(fl, "phi")
    long <- data.frame(
      participant = rep(rownames(phi), ncol(phi)),
      state = rep(colnames(phi), each = nrow(phi)),
      prob = as.vector(phi))
    long$state <- factor(long$state, levels = rev(colnames(phi)))
    fill <- c(stats::setNames(rep(kq, each = 2),
                              colnames(phi)[seq_len(2 * K)]),
              unclassified = cols$gridgrey)
    p <- ggplot2::ggplot(long, ggplot2::aes(
           x = .data$prob, y = .data$participant, fill = .data$state)) +
      ggplot2::geom_col(colour = "white", linewidth = 0.2) +
      ggplot2::scale_fill_manual(values = fill) +
      ggplot2::labs(x = "posterior probability", y = NULL, fill = NULL)
    return(p)
  }

  if (type == "contrasts") {
    qdc <- compute_qdc(object, ...)
    ct <- attr(qdc, "contrasts")
    dkl <- attr(qdc, "delta_kl")
    dg <- attr(qdc, "delta_grid")
    verdict <- ifelse(grepl("^distinguishing", qdc$verdict), "distinguishing",
                      qdc$verdict)
    ct$verdict <- verdict[match(ct$statement, qdc$statement)]
    p <- ggplot2::ggplot(ct, ggplot2::aes(
           x = .data$median, y = stats::reorder(.data$statement, .data$median),
           colour = .data$verdict)) +
      ggplot2::annotate("rect", xmin = -dg, xmax = dg, ymin = -Inf,
                        ymax = Inf, fill = cols$fill, alpha = 0.4) +
      ggplot2::geom_vline(data = data.frame(pair = rep(names(dkl), each = 2),
                                            x = as.vector(rbind(-dkl, dkl))),
                          ggplot2::aes(xintercept = .data$x),
                          colour = cols$accent, linetype = 2) +
      ggplot2::geom_errorbar(ggplot2::aes(xmin = .data$lower,
                                          xmax = .data$upper),
                             width = 0, orientation = "y") +
      ggplot2::geom_point() +
      ggplot2::facet_wrap(~pair) +
      ggplot2::scale_colour_manual(values = c(
        distinguishing = .bq_verdict[["distinguishing"]],
        consensus = .bq_verdict[["consensus"]],
        indeterminate = .bq_verdict[["indeterminate"]])) +
      ggplot2::labs(x = "score contrast", y = NULL, colour = NULL)
    return(p)
  }

  # array
  ar <- compute_factor_array(object)
  cert <- attr(ar, "certainty")
  long <- do.call(rbind, lapply(seq_len(K), function(k) {
    slot <- ar[[paste0(fac_ids[k], "_grid")]]
    height <- stats::ave(slot, slot, FUN = seq_along)
    data.frame(statement = ar$statement, factor = fac_ids[k],
               column = slot, height = height,
               certainty = cert[, k])
  }))
  p <- ggplot2::ggplot(long, ggplot2::aes(
         x = .data$column, y = .data$height, fill = .data$certainty)) +
    ggplot2::geom_tile(colour = cols$grey, width = 0.9, height = 0.9) +
    ggplot2::geom_text(ggplot2::aes(label = .data$statement), size = 2.6) +
    ggplot2::scale_fill_gradientn(colours = .bq_certainty,
                                  limits = c(0, 1)) +
    ggplot2::facet_wrap(~factor) +
    ggplot2::labs(x = "grid column", y = NULL, fill = "certainty") +
    ggplot2::theme(axis.text.y = ggplot2::element_blank(),
                   axis.ticks.y = ggplot2::element_blank())
  p
}

# summaries.R
# The posterior tables: bounded loadings, flag probabilities with an
# unclassified state, statement scores, quota-respecting factor arrays,
# and the distinguishing/consensus verdicts. Every selected column in
# every table comes from the one fdr_select() path, so claims() and the
# table functions can never disagree.


# selection rule: largest prefix with posterior expected FDR <= q
#' @keywords internal
#' @noRd
fdr_select <- function(p, q = 0.05, floor = NA) {
  keep <- if (is.na(floor)) rep(TRUE, length(p)) else p >= floor
  idx <- order(p, decreasing = TRUE)
  sel <- logical(length(p))
  err <- cumsum(1 - p[idx]) / seq_along(idx)
  ok <- which(err <= q & keep[idx])
  if (length(ok)) sel[idx[seq_len(max(ok))]] <- keep[idx[seq_len(max(ok))]]
  list(selected = sel, expected_false = sum(1 - p[sel]))
}


# per-draw signed flag column per person, 2K + 1 for unclassified. A person
# is a flag candidate for k+/k- in a draw when factor k carries the
# majority of the squared loading AND the bounded loading clears the Brown
# floor 1.96/sqrt(J).
#' @keywords internal
#' @noRd
.flag_draws <- function(fit) {
  d <- fit$draws
  T_ <- dim(d$F)[1]; J <- fit$brief$J; K <- fit$brief$K; N <- fit$brief$N
  floorJ <- 1.96 / sqrt(J)
  colidx <- matrix(NA_integer_, T_, N)
  for (t in seq_len(T_)) {
    F <- matrix(d$F[t, , ], J, K); L <- matrix(d$Lambda[t, , ], N, K)
    S <- crossprod(sweep(F, 2, colMeans(F))) / J
    qv <- L %*% S
    rho <- qv / sqrt(pmax(rowSums(qv * L), 0) + 1)
    L2 <- L^2
    dom <- max.col(L2, ties.method = "first")
    dompass <- L2[cbind(seq_len(N), dom)] >
      (rowSums(L2) - L2[cbind(seq_len(N), dom)])
    r_dom <- rho[cbind(seq_len(N), dom)]
    flagged <- dompass & (abs(r_dom) > floorJ)
    colidx[t, ] <- ifelse(flagged,
                          2L * (dom - 1L) + ifelse(r_dom >= 0, 1L, 2L),
                          2L * K + 1L)
  }
  colidx
}


# phi: N x (2K + 1) matrix of signed flag-candidate probabilities plus the
# unclassified state
#' @keywords internal
#' @noRd
.phi_matrix <- function(fit) {
  d <- fit$draws
  K <- fit$brief$K
  colidx <- .flag_draws(fit)
  phi <- t(apply(colidx, 2, tabulate, nbins = 2L * K + 1L)) / nrow(colidx)
  fac_ids <- dimnames(d$sigma)[[2]]
  colnames(phi) <- c(rbind(paste0(fac_ids, "+"), paste0(fac_ids, "-")),
                     "unclassified")
  rownames(phi) <- dimnames(d$Lambda)[[2]]
  phi
}


# quota arrays per draw: g[t, j, k] is statement j's grid column under
# factor k's scores in draw t
#' @keywords internal
#' @noRd
.g_draws <- function(fit, negative = FALSE) {
  d <- fit$draws
  T_ <- dim(d$F)[1]; J <- fit$brief$J; K <- fit$brief$K
  distr <- fit$distribution
  g <- array(NA_integer_, c(T_, J, K))
  for (t in seq_len(T_)) for (k in seq_len(K))
    g[t, , k] <- quota_sort(if (negative) -d$F[t, , k] else d$F[t, , k], distr)
  g
}


# contrast machinery shared by compute_qdc and claims: the posterior
# critical difference delta_kl per pair, exceedance probabilities, the
# joint distinguishing and consensus events, and the contrast intervals.
# prob governs the DISPLAY interval only; the critical difference keeps
# its z_.975.
#' @keywords internal
#' @noRd
.pair_stats <- function(fit, prob = NULL) {
  prob <- if (is.null(prob)) fit$brief$prob else prob
  alpha <- 1 - prob
  d <- fit$draws
  T_ <- dim(d$F)[1]; J <- fit$brief$J; K <- fit$brief$K
  if (K < 2)
    .bq_abort("distinguishing and consensus verdicts need K >= 2 factors.")
  dgrid <- delta_grid(fit$distribution)
  pairs <- combn(K, 2)
  P <- ncol(pairs)
  fac_ids <- dimnames(d$sigma)[[2]]
  pair_ids <- paste(fac_ids[pairs[1, ]], fac_ids[pairs[2, ]], sep = "-")

  delta_kl <- delta_kl99 <- numeric(P)
  pi_pair <- pi_pair99 <- d_med <- d_lo <- d_hi <-
    matrix(NA_real_, J, P, dimnames = list(dimnames(d$F)[[2]], pair_ids))
  exc <- vector("list", P)
  for (p in seq_len(P)) {
    D <- matrix(d$F[, , pairs[1, p]] - d$F[, , pairs[2, p]], T_, J)
    sdj <- apply(D, 2, sd)
    delta_kl[p] <- qnorm(0.975) * mean(sdj)
    delta_kl99[p] <- qnorm(0.995) * mean(sdj)
    exc[[p]] <- abs(D) > delta_kl[p]
    pi_pair[, p] <- colMeans(exc[[p]])
    pi_pair99[, p] <- colMeans(abs(D) > delta_kl99[p])
    qd <- apply(D, 2, quantile, probs = c(alpha / 2, .5, 1 - alpha / 2))
    d_lo[, p] <- qd[1, ]; d_med[, p] <- qd[2, ]; d_hi[, p] <- qd[3, ]
  }
  names(delta_kl) <- names(delta_kl99) <- pair_ids

  slice <- function(k) matrix(d$F[, , k], T_, J)
  mxF <- Reduce(pmax, lapply(seq_len(K), slice))
  mnF <- Reduce(pmin, lapply(seq_len(K), slice))
  pi_cons <- colMeans((mxF - mnF) < dgrid)
  names(pi_cons) <- dimnames(d$F)[[2]]

  pi_dist <- matrix(NA_real_, J, K,
                    dimnames = list(dimnames(d$F)[[2]], fac_ids))
  for (k in seq_len(K)) {
    touch <- which(pairs[1, ] == k | pairs[2, ] == k)
    pi_dist[, k] <- colMeans(Reduce(`&`, exc[touch]))
  }

  list(pairs = pairs, pair_ids = pair_ids, delta_kl = delta_kl,
       delta_kl99 = delta_kl99, delta_grid = dgrid, pi_pair = pi_pair,
       pi_pair99 = pi_pair99, pi_dist = pi_dist, pi_cons = pi_cons,
       d_med = d_med, d_lo = d_lo, d_hi = d_hi)
}


#' Bounded participant loadings with credible intervals
#'
#' @description
#' The correlation-scale loading of the accompanying paper,
#' `rho = (S lambda)_k / sqrt(s^2 + 1)`, summarized per participant and
#' factor, with the posterior-mean person spread `s_i` alongside.
#'
#' @param fit A `bayesqm_fit`.
#' @param prob Credible-interval probability; defaults to the fit's.
#'
#' @return A data frame with one row per participant: `participant`, then
#'   `f{k}_loading`, `f{k}_lower`, `f{k}_upper` per factor, then `spread`
#'   (posterior-mean `s_i`).
#'
#' @examples
#' compute_loadings(demo_fit())
#'
#' @export
compute_loadings <- function(fit, prob = NULL) {
  assert_bayesqm_fit(fit)
  prob <- if (is.null(prob)) fit$brief$prob else prob
  alpha <- 1 - prob
  rho <- .rho_draws(fit)
  m <- .summarize_draws(rho, mean)
  lo <- .summarize_draws(rho, quantile, probs = alpha / 2, names = FALSE)
  hi <- .summarize_draws(rho, quantile, probs = 1 - alpha / 2, names = FALSE)
  out <- data.frame(participant = rownames(m), stringsAsFactors = FALSE)
  for (k in seq_len(ncol(m))) {
    fk <- colnames(m)[k]
    out[[paste0(fk, "_loading")]] <- m[, k]
    out[[paste0(fk, "_lower")]] <- lo[, k]
    out[[paste0(fk, "_upper")]] <- hi[, k]
  }
  out$spread <- colMeans(fit$draws$s_i)
  rownames(out) <- NULL
  out
}


#' Factor characteristics
#'
#' @description
#' The per-factor block of the accompanying paper: modal and mean number
#' of defining sorts per posterior draw with a credible interval, the
#' selected flag count, the mean posterior spread of the statement
#' scores, and the mean replicate reliability
#' `R_i = s_i^2 / (1 + s_i^2)` of the flagged participants. The
#' statement-score correlations between factors ride along as an
#' attribute.
#'
#' @param fit A `bayesqm_fit`.
#' @param prob Credible-interval probability for the defining-sort counts
#'   (default 0.90).
#' @param q,floor The flag rule fixing the flagged set, as in
#'   [compute_flags()].
#'
#' @return A data frame with one row per factor: `factor`, `flagged`
#'   (selected flags), `defining_modal`, `defining_mean`,
#'   `defining_lower`, `defining_upper`, `score_spread`, and
#'   `reliability`. Attribute `score_correlations` holds the posterior
#'   mean K x K correlation matrix of the statement scores.
#'
#' @examples
#' factor_characteristics(demo_fit())
#'
#' @export
factor_characteristics <- function(fit, prob = 0.90, q = 0.05, floor = 0.5) {
  assert_bayesqm_fit(fit)
  d <- fit$draws
  T_ <- dim(d$F)[1]; J <- fit$brief$J; K <- fit$brief$K
  fac_ids <- dimnames(d$sigma)[[2]]
  alpha <- 1 - prob

  colidx <- .flag_draws(fit)
  counts <- vapply(seq_len(K), function(k)
    rowSums(colidx == 2L * k - 1L | colidx == 2L * k), numeric(T_))

  fl <- .flag_table(fit, q = q, floor = floor)
  rel_i <- colMeans(d$s_i^2 / (1 + d$s_i^2))

  Csum <- matrix(0, K, K)
  for (t in seq_len(T_)) Csum <- Csum + cor(matrix(d$F[t, , ], J, K))
  score_cor <- Csum / T_
  dimnames(score_cor) <- list(fac_ids, fac_ids)

  qs <- apply(counts, 2, quantile, probs = c(alpha / 2, 1 - alpha / 2))
  out <- data.frame(factor = fac_ids, stringsAsFactors = FALSE)
  out$flagged <- vapply(fac_ids, function(f)
    sum(fl$selected & fl$factor == f), integer(1))
  out$defining_modal <- vapply(seq_len(K), function(k) {
    tb <- table(counts[, k])
    as.integer(names(tb)[which.max(tb)])
  }, integer(1))
  out$defining_mean <- colMeans(counts)
  out$defining_lower <- qs[1, ]
  out$defining_upper <- qs[2, ]
  out$score_spread <- vapply(seq_len(K), function(k)
    mean(apply(matrix(d$F[, , k], T_, J), 2, sd)), numeric(1))
  out$reliability <- vapply(fac_ids, function(f) {
    idx <- which(fl$selected & fl$factor == f)
    if (length(idx)) mean(rel_i[idx]) else NA_real_
  }, numeric(1))
  rownames(out) <- NULL
  attr(out, "score_correlations") <- score_cor
  out
}


# the flag table both compute_flags() and claims() serve from
#' @keywords internal
#' @noRd
.flag_table <- function(fit, q, floor) {
  phi <- .phi_matrix(fit)
  K <- fit$brief$K; N <- fit$brief$N
  cand <- phi[, -(2 * K + 1), drop = FALSE]
  res <- fdr_select(as.vector(cand), q = q, floor = floor)
  selmat <- matrix(res$selected, nrow = N)
  best <- apply(cand, 1, which.max)
  fac_ids <- dimnames(fit$draws$sigma)[[2]]
  out <- data.frame(
    participant = rownames(phi),
    factor = fac_ids[ceiling(best / 2)],
    sign = ifelse(best %% 2 == 1, 1, -1),
    flag_prob = cand[cbind(seq_len(N), best)],
    unclassified_prob = phi[, 2 * K + 1],
    selected = rowSums(selmat) > 0,
    stringsAsFactors = FALSE
  )
  rownames(out) <- NULL
  attr(out, "expected_false") <- res$expected_false
  attr(out, "phi") <- phi
  out
}

#' Flag probabilities with an explicit unclassified state
#'
#' @description
#' For each participant, the posterior probability that the classical flag
#' rule fires on each signed factor, with the probability of remaining
#' unclassified alongside. Selected flags come from all signed
#' candidates by the posterior false-discovery rule at level `q`.
#'
#' @param fit A `bayesqm_fit`.
#' @param q Posterior expected false-discovery bound (default 0.05).
#' @param floor Minimum flag probability a candidate needs before it can
#'   be selected (default 0.5, so at most one signed flag per participant).
#'
#' @return A data frame with one row per participant: modal signed
#'   candidate (`factor`, `sign`), its probability `flag_prob`,
#'   `unclassified_prob`, and `selected`. The full probability matrix is
#'   attached as `attr(, "phi")` and the expected number of false
#'   selected flags as `attr(, "expected_false")`.
#'
#' @examples
#' compute_flags(demo_fit())
#'
#' @export
compute_flags <- function(fit, q = 0.05, floor = 0.5) {
  assert_bayesqm_fit(fit)
  out <- .flag_table(fit, q = q, floor = floor)
  thr <- 1.96 / sqrt(fit$brief$J)
  if (thr > 0.8)
    message(sprintf(paste0(
      "with J = %d statements the descriptive cut-off 1.96/sqrt(J) = %.2f ",
      "sits near the ceiling of the bounded loading scale, so flags can ",
      "rarely fire at this statement count"), fit$brief$J, thr))
  out
}


#' Statement scores with credible intervals
#'
#' @param fit A `bayesqm_fit`.
#' @param prob Credible-interval probability; defaults to the fit's.
#'
#' @return A data frame with one row per statement: `statement`, then
#'   `f{k}_zsc`, `f{k}_lower`, `f{k}_upper` per factor.
#'
#' @examples
#' compute_zscores(demo_fit())
#'
#' @export
compute_zscores <- function(fit, prob = NULL) {
  assert_bayesqm_fit(fit)
  prob <- if (is.null(prob)) fit$brief$prob else prob
  alpha <- 1 - prob
  Fd <- fit$draws$F
  m <- .summarize_draws(Fd, mean)
  lo <- .summarize_draws(Fd, quantile, probs = alpha / 2, names = FALSE)
  hi <- .summarize_draws(Fd, quantile, probs = 1 - alpha / 2, names = FALSE)
  out <- data.frame(statement = rownames(m), stringsAsFactors = FALSE)
  for (k in seq_len(ncol(m))) {
    fk <- colnames(m)[k]
    out[[paste0(fk, "_zsc")]] <- m[, k]
    out[[paste0(fk, "_lower")]] <- lo[, k]
    out[[paste0(fk, "_upper")]] <- hi[, k]
  }
  rownames(out) <- NULL
  out
}


#' Quota-respecting factor arrays
#'
#' @description
#' The reported array for each factor: statements sorted onto the design
#' grid by their posterior column probabilities (mean grid column,
#' tie-broken by posterior-mean score), which is quota-exact by
#' construction. When the \pkg{clue} package is installed, a footrule
#' assignment cross-check is computed and its disagreement rate attached.
#'
#' @param fit A `bayesqm_fit`.
#' @param negative Also return the negative-pole arrays (default `FALSE`);
#'   on a symmetric grid the mirror is exact.
#'
#' @return A data frame with one row per statement: `statement`, then
#'   `f{k}_grid` per factor (and `f{k}_grid_neg` when `negative = TRUE`).
#'   `attr(, "footrule_disagreement")` gives the share of cells where the
#'   footrule assignment differs (`NA` without \pkg{clue}).
#'
#' @examples
#' compute_factor_array(demo_fit())
#'
#' @export
compute_factor_array <- function(fit, negative = FALSE) {
  assert_bayesqm_fit(fit)
  distr <- fit$distribution
  J <- fit$brief$J; K <- fit$brief$K
  C <- length(distr)
  g <- .g_draws(fit)
  cbar <- apply(g, c(2, 3), mean)
  Fm <- .summarize_draws(fit$draws$F, mean)
  slots <- rep(seq_len(C), distr)
  Ahat <- matrix(NA_integer_, J, K)
  for (k in seq_len(K)) {
    o <- order(cbar[, k], Fm[, k])          # tie-break by posterior mean score
    Ahat[o, k] <- slots
  }

  fr <- NA_real_
  if (requireNamespace("clue", quietly = TRUE)) {
    Afr <- matrix(NA_integer_, J, K)
    for (k in seq_len(K)) {
      cost <- vapply(slots, function(a) colMeans(abs(g[, , k] - a)), numeric(J))
      Afr[, k] <- slots[clue::solve_LSAP(cost)]
    }
    fr <- mean(Afr != Ahat)
  }

  out <- data.frame(statement = dimnames(fit$draws$F)[[2]],
                    stringsAsFactors = FALSE)
  fac_ids <- dimnames(fit$draws$sigma)[[2]]
  certainty <- matrix(NA_real_, J, K,
                      dimnames = list(out$statement, fac_ids))
  for (k in seq_len(K)) {
    out[[paste0(fac_ids[k], "_grid")]] <- Ahat[, k]
    certainty[, k] <- vapply(seq_len(J), function(j)
      mean(g[, j, k] == Ahat[j, k]), numeric(1))
  }

  if (negative) {
    if (all(distr == rev(distr))) {
      for (k in seq_len(K))
        out[[paste0(fac_ids[k], "_grid_neg")]] <- (C + 1L) - Ahat[, k]
    } else {
      gn <- .g_draws(fit, negative = TRUE)
      cbn <- apply(gn, c(2, 3), mean)
      for (k in seq_len(K)) {
        An <- integer(J)
        o <- order(cbn[, k], -Fm[, k])
        An[o] <- slots
        out[[paste0(fac_ids[k], "_grid_neg")]] <- An
      }
    }
  }
  rownames(out) <- NULL
  attr(out, "footrule_disagreement") <- fr
  attr(out, "certainty") <- certainty
  out
}


# the verdict tables compute_qdc() and claims() serve from
#' @keywords internal
#' @noRd
.qdc_tables <- function(fit, q, cons_floor, prob = NULL) {
  ps <- .pair_stats(fit, prob = prob)
  J <- fit$brief$J; K <- fit$brief$K
  fac_ids <- dimnames(fit$draws$sigma)[[2]]
  stmt_ids <- dimnames(fit$draws$F)[[2]]

  dl <- fdr_select(as.vector(ps$pi_dist), q = q)
  dist_pub <- matrix(dl$selected, J, K)
  cl <- fdr_select(ps$pi_cons, q = q, floor = cons_floor)
  sl <- fdr_select(as.vector(ps$pi_pair), q = q)
  star_pub <- matrix(sl$selected, J, ncol(ps$pi_pair))
  sl99 <- fdr_select(as.vector(ps$pi_pair99), q = q)
  star99 <- star_pub & matrix(sl99$selected, J, ncol(ps$pi_pair))

  # column placements per draw give the on-grid counterpart per pair
  g <- .g_draws(fit)
  p_grid <- matrix(NA_real_, J, ncol(ps$pi_pair))
  for (p in seq_len(ncol(ps$pi_pair)))
    p_grid[, p] <- colMeans(abs(g[, , ps$pairs[1, p]] -
                                g[, , ps$pairs[2, p]]) >= 1)

  verdict <- character(J)
  for (j in seq_len(J)) {
    dk <- which(dist_pub[j, ])
    verdict[j] <-
      if (length(dk)) paste0("distinguishing (",
                             paste(fac_ids[dk], collapse = ", "), ")")
      else if (cl$selected[j]) "consensus"
      else "indeterminate"
  }

  qdc <- data.frame(statement = stmt_ids, stringsAsFactors = FALSE)
  for (k in seq_len(K))
    qdc[[paste0(fac_ids[k], "_dist_prob")]] <- ps$pi_dist[, k]
  qdc$consensus_prob <- ps$pi_cons
  qdc$verdict <- verdict
  rownames(qdc) <- NULL

  contrasts <- data.frame(
    statement = rep(stmt_ids, ncol(ps$pi_pair)),
    pair = rep(ps$pair_ids, each = J),
    median = as.vector(ps$d_med),
    lower = as.vector(ps$d_lo),
    upper = as.vector(ps$d_hi),
    exceed_prob = as.vector(ps$pi_pair),
    diff_column_prob = as.vector(p_grid),
    selected = as.vector(star_pub),
    stars = ifelse(as.vector(star99), "**",
                   ifelse(as.vector(star_pub), "*", "")),
    stringsAsFactors = FALSE
  )

  list(qdc = qdc, contrasts = contrasts, pair_stats = ps,
       dist_pub = dist_pub, cons_pub = cl$selected,
       expected_false = c(distinguishing = dl$expected_false,
                          consensus = cl$expected_false,
                          stars = sl$expected_false))
}

#' Distinguishing and consensus verdicts
#'
#' @description
#' The statement verdict table. Distinguishing-for-a-factor is the joint
#' event that every contrast touching that factor exceeds the posterior
#' critical difference `delta_kl` (computed from the posterior spread of
#' the score contrasts); consensus is the equivalence event that all
#' scores sit within one grid column ([delta_grid()]) of each other. Both
#' families are selected through the posterior false-discovery rule, and a
#' statement can be distinguishing, consensus, or indeterminate.
#'
#' @param fit A `bayesqm_fit` with `K >= 2`.
#' @param q Posterior expected false-discovery bound (default 0.05).
#' @param cons_floor Minimum consensus probability before a statement can
#'   be selected as consensus (default 0.95).
#' @param prob Credible-interval probability for the contrast intervals;
#'   defaults to the fit's. The critical difference itself keeps its
#'   z_0.975 definition regardless.
#'
#' @return A data frame with one row per statement: per-factor
#'   distinguishing probabilities, `consensus_prob`, and the `verdict`.
#'   Attributes: `delta_kl` and `delta_kl99` (per pair), `delta_grid`,
#'   `contrasts` (the per-pair contrast table with intervals, the
#'   probability `diff_column_prob` that the two viewpoints place the
#'   statement in different grid columns, and `stars`, `*` for selected
#'   claims and `**` for those that also clear the `z_0.995` critical
#'   difference at probability .99), and `expected_false` per family.
#'
#' @examples
#' compute_qdc(demo_fit())
#'
#' @export
compute_qdc <- function(fit, q = 0.05, cons_floor = 0.95, prob = NULL) {
  assert_bayesqm_fit(fit)
  qt <- .qdc_tables(fit, q = q, cons_floor = cons_floor, prob = prob)
  out <- qt$qdc
  attr(out, "delta_kl") <- qt$pair_stats$delta_kl
  attr(out, "delta_kl99") <- qt$pair_stats$delta_kl99
  attr(out, "delta_grid") <- qt$pair_stats$delta_grid
  attr(out, "contrasts") <- qt$contrasts
  attr(out, "expected_false") <- qt$expected_false
  out
}


#' Extreme-placement probabilities per statement and factor
#'
#' @description
#' The crib sheet: for each statement and factor, the posterior
#' probability of landing in the top or bottom grid column of that
#' factor's array, and of carrying the highest or lowest score across
#' factors.
#'
#' @param fit A `bayesqm_fit`.
#'
#' @return A long data frame: `statement`, `factor`, `p_top`, `p_bottom`,
#'   `p_highest`, `p_lowest`.
#'
#' @examples
#' head(crib_sheet(demo_fit()))
#'
#' @export
crib_sheet <- function(fit) {
  assert_bayesqm_fit(fit)
  d <- fit$draws
  T_ <- dim(d$F)[1]; J <- fit$brief$J; K <- fit$brief$K
  C <- length(fit$distribution)
  g <- .g_draws(fit)
  fac_ids <- dimnames(d$sigma)[[2]]
  stmt_ids <- dimnames(d$F)[[2]]
  slice <- function(k) matrix(d$F[, , k], T_, J)
  out <- vector("list", K)
  for (k in seq_len(K)) {
    others <- setdiff(seq_len(K), k)
    hi <- lo <- rep(NA_real_, J)
    if (length(others)) {
      mx <- Reduce(pmax, lapply(others, slice))
      mn <- Reduce(pmin, lapply(others, slice))
      hi <- colMeans(slice(k) > mx)
      lo <- colMeans(slice(k) < mn)
    }
    out[[k]] <- data.frame(
      statement = stmt_ids, factor = fac_ids[k],
      p_top = colMeans(g[, , k] == C),
      p_bottom = colMeans(g[, , k] == 1),
      p_highest = hi, p_lowest = lo,
      stringsAsFactors = FALSE
    )
  }
  out <- do.call(rbind, out)
  rownames(out) <- NULL
  out
}

# The summary tables and the one publication rule. The coherence tests are
# the contract: claims() and the table functions must always agree.

test_that("compute_loadings returns the wide schema on the bounded scale", {
  fit <- make_fake_fit()
  lo <- compute_loadings(fit)
  expect_identical(names(lo)[1], "participant")
  expect_true(all(c("f1_loading", "f1_lower", "f1_upper", "f2_loading",
                    "spread") %in% names(lo)))
  vals <- unlist(lo[, grep("_loading|_lower|_upper", names(lo))])
  expect_true(all(abs(vals) <= 1))
  expect_true(all(lo$spread > 0))
})

test_that("flags carry probabilities, the unclassified state, and the FDR bound", {
  fit <- make_fake_fit()
  fl <- compute_flags(fit, q = 0.05)
  expect_identical(nrow(fl), 6L)
  expect_true(all(fl$flag_prob >= 0 & fl$flag_prob <= 1))
  expect_true(all(fl$unclassified_prob >= 0 & fl$unclassified_prob <= 1))
  phi <- attr(fl, "phi")
  expect_identical(dim(phi), c(6L, 5L))
  expect_equal(unname(rowSums(phi)), rep(1, 6), tolerance = 1e-12)
  expect_lte(attr(fl, "expected_false"), 0.05 * max(1, sum(fl$selected)))
})

test_that("fdr_select keeps the expected-FDR promise and the floor", {
  p <- c(.99, .97, .9, .6, .55, .3)
  r <- fdr_select(p, q = 0.05)
  expect_true(all(r$selected[1:2]))
  expect_lte(r$expected_false / max(1, sum(r$selected)), 0.05)
  rf <- fdr_select(p, q = 0.5, floor = 0.56)
  expect_false(rf$selected[5])               # under the floor, never selected
})

test_that("factor arrays are quota-exact and mirror on a symmetric grid", {
  fit <- make_fake_fit(N = 6, J = 9, K = 2)
  ar <- compute_factor_array(fit, negative = TRUE)
  distr <- fit$distribution
  for (k in c("f1_grid", "f2_grid")) {
    expect_identical(tabulate(ar[[k]], length(distr)), as.integer(distr))
  }
  if (all(distr == rev(distr))) {
    expect_identical(ar$f1_grid_neg, length(distr) + 1L - ar$f1_grid)
  }
  skip_if_not_installed("clue")
  expect_true(is.finite(attr(ar, "footrule_disagreement")))
})

test_that("qdc gives three-state verdicts with landmarks attached", {
  fit <- make_fake_fit()
  qdc <- compute_qdc(fit)
  expect_true(all(grepl("^distinguishing|^consensus$|^indeterminate$",
                        qdc$verdict)))
  expect_true(all(attr(qdc, "delta_kl") > 0))
  expect_equal(attr(qdc, "delta_grid"), delta_grid(fit$distribution))
  ct <- attr(qdc, "contrasts")
  expect_true(all(ct$lower <= ct$median & ct$median <= ct$upper))
  expect_identical(nrow(ct), fit$brief$J * 1L)      # K = 2, one pair
})

test_that("qdc refuses K = 1", {
  fit <- make_fake_fit(N = 5, J = 9, K = 1, T = 60)
  expect_error(compute_qdc(fit), "K >= 2")
})

test_that("claims() agrees with the table functions exactly", {
  fit <- make_fake_fit()
  q <- 0.10
  cl <- claims(fit, q = q)
  fl <- compute_flags(fit, q = q)
  expect_identical(cl$flags$participant, fl$participant[fl$selected])
  qdc <- compute_qdc(fit, q = q)
  dist_stmts <- qdc$statement[grepl("^distinguishing", qdc$verdict)]
  expect_identical(sort(unique(cl$distinguishing$statement)),
                   sort(dist_stmts))
  cons_stmts <- qdc$statement[qdc$verdict == "consensus"]
  expect_identical(cl$consensus$statement, cons_stmts)
  ct <- attr(qdc, "contrasts")
  expect_identical(nrow(cl$stars), sum(ct$selected))
})

test_that("claims print with all four families and the level", {
  fit <- make_fake_fit()
  out <- paste(capture.output(print(claims(fit, q = 0.05))), collapse = "\n")
  expect_match(out, "q = 0.05")
  expect_match(out, "flags")
  expect_match(out, "distinguishing")
  expect_match(out, "consensus")
  expect_match(out, "stars")
})

test_that("crib_sheet probabilities are proper and complete", {
  fit <- make_fake_fit(N = 6, J = 9, K = 2)
  cs <- crib_sheet(fit)
  expect_identical(nrow(cs), 9L * 2L)
  probs <- unlist(cs[, c("p_top", "p_bottom", "p_highest", "p_lowest")])
  expect_true(all(probs >= 0 & probs <= 1))
})

test_that("the summary layer runs end to end on a real tiny fit", {
  fit <- demo_fit()
  expect_s3_class(compute_loadings(fit), "data.frame")
  expect_s3_class(compute_flags(fit), "data.frame")
  expect_s3_class(compute_zscores(fit), "data.frame")
  expect_s3_class(compute_factor_array(fit), "data.frame")
  expect_s3_class(compute_qdc(fit), "data.frame")
  expect_s3_class(claims(fit), "bayesqm_claims")
})


test_that("factor_characteristics reports the per-factor block", {
  fit <- make_fake_fit()
  ch <- factor_characteristics(fit)
  K <- fit$brief$K; N <- fit$brief$N
  expect_equal(nrow(ch), K)
  expect_true(all(c("factor", "flagged", "defining_modal", "defining_mean",
                    "defining_lower", "defining_upper", "score_spread",
                    "reliability") %in% names(ch)))
  expect_true(all(ch$defining_mean >= 0 & ch$defining_mean <= N))
  expect_true(all(ch$defining_lower <= ch$defining_mean + 1e-9))
  expect_true(all(ch$defining_upper >= ch$defining_mean - 1e-9))
  expect_true(all(ch$flagged >= 0 & ch$flagged <= N))
  expect_true(all(ch$score_spread > 0))
  ok <- !is.na(ch$reliability)
  expect_true(all(ch$reliability[ok] > 0 & ch$reliability[ok] < 1))
  sc <- attr(ch, "score_correlations")
  expect_equal(dim(sc), c(K, K))
  expect_equal(unname(diag(sc)), rep(1, K), tolerance = 1e-8)
  expect_equal(sc, t(sc), tolerance = 1e-12)
})

test_that("contrasts carry the on-grid probability and two-level stars", {
  fit <- make_fake_fit()
  qdc <- compute_qdc(fit)
  co <- attr(qdc, "contrasts")
  expect_true(all(c("diff_column_prob", "stars") %in% names(co)))
  expect_true(all(co$diff_column_prob >= 0 & co$diff_column_prob <= 1))
  expect_true(all(co$stars %in% c("", "*", "**")))
  expect_identical(co$selected, co$stars != "")    # stars grade selected claims
  d1 <- attr(qdc, "delta_kl"); d99 <- attr(qdc, "delta_kl99")
  expect_identical(names(d1), names(d99))
  expect_true(all(d99 > d1))
})

# fit_bayesian.R
# The public fitting interface: strict forced-sort validation, the gated
# PX-Gibbs run, alignment, and warm-started continuation via extend().


# every column must obey the design quotas exactly; the exact likelihood is
# a statement about the sorting event the quotas define
#' @keywords internal
#' @noRd
.assert_forced_sorts <- function(Y, distribution) {
  C <- length(distribution)
  ok <- apply(Y, 2, function(y) all(tabulate(y, C) == distribution))
  if (all(ok)) return(invisible(TRUE))
  bad <- colnames(Y); if (is.null(bad)) bad <- paste0("P", seq_len(ncol(Y)))
  .bq_abort(paste0(
    "the exact rank-order likelihood requires forced sorts matching the ",
    "design grid (%s), and %d sort%s do%s not: %s. Free-distribution ",
    "sorts are outside this model's scope."),
    paste(distribution, collapse = "-"), sum(!ok),
    if (sum(!ok) > 1) "s" else "", if (sum(!ok) > 1) "" else "es",
    paste(bad[!ok], collapse = ", "))
}

# 0.1.0 fitting arguments, caught by name with a migration pointer. The
# caller passes the ORIGINAL call names, because R's partial matching would
# otherwise bind `iter` to `iterations` before the dots ever see it.
#' @keywords internal
#' @noRd
.check_removed_args <- function(supplied) {
  removed <- c(stan_dir = "the Stan model is gone; nothing replaces this",
               robust = "the partition likelihood has no error family to choose",
               nu = "the partition likelihood has no error family to choose",
               chains = "the sampler runs one gated chain; see `iterations`",
               iter = "use `iterations`",
               warmup = "use `burn`",
               adapt_delta = "there is no NUTS tuning; the sampler is Gibbs",
               max_draws = "use `thin`",
               prior_loading_scale = "use `sigma_scale`",
               prior_sigma_scale = "use `sigma_scale`",
               prior_nu_alpha = "the partition likelihood has no nu",
               prior_nu_beta = "the partition likelihood has no nu",
               use_half_cauchy = "the loading-scale prior is half-normal",
               delta = "the critical difference is computed by compute_qdc()")
  hit <- intersect(supplied, names(removed))
  if (length(hit))
    .bq_abort(paste0(
      "`%s` belonged to the 0.1.0 score-scale model, which bayesqm 0.2.0 ",
      "replaces with the exact partition likelihood: %s."),
      hit[1], removed[hit[1]])
}


#' Fit the exact partition-likelihood model to forced Q sorts
#'
#' @description
#' Models each completed sort as a forced ordered partition of the
#' statements and samples the posterior of the latent factor model by a
#' parameter-expanded Gibbs sampler. Convergence is gated on
#' rotation-invariant person spreads (rank-normalized split-R-hat and
#' bulk/tail effective sample size); a chain failing the gate is
#' warm-extended at successive doublings up to `max_iterations`. Draws are
#' aligned by MatchAlign with a polarity canon, so defining sorts load
#' positively.
#'
#' @param Y A `qsort_data` object, or a `J x N` numeric matrix of grid
#'   positions with statements as rows and participants as columns. Every
#'   sort must obey the forced distribution exactly.
#' @param K Integer number of factors.
#' @param iterations First-check chain length (default 12000, the settings
#'   frozen in the accompanying paper).
#' @param burn Burn-in iterations (default 2000), paid once; extensions
#'   continue the chain.
#' @param thin Keep every `thin`-th post-burn draw (default 5).
#' @param max_iterations Total-iteration cap for the gated extension ladder
#'   (default 48000).
#' @param seed Optional integer seed; fits are exactly reproducible given
#'   the seed.
#' @param sigma_scale Half-normal prior scale for the per-factor loading
#'   scales (default 1).
#' @param rhat_max,ess_min Gate thresholds (defaults 1.01 and 400).
#' @param prob Credible-interval probability stored on the fit
#'   (default 0.95).
#' @param keep_raw Keep the pre-alignment draws on the fit (default
#'   `TRUE`); required by [extend()] and by realignment under a different
#'   pivot.
#' @param pivot Optional draw index for the alignment pivot; `NULL` (the
#'   default) uses the median-condition-number rule.
#' @param quiet Suppress progress messages (default `FALSE`).
#' @param ... Unused; supplying a removed 0.1.0 argument gives a migration
#'   error.
#'
#' @return A `bayesqm_fit` carrying aligned draws, the gate report, the
#'   alignment record, and (when `keep_raw = TRUE`) the raw draws and
#'   sampler state.
#'
#' @examples
#' fit <- demo_fit()
#' fit
#'
#' @export
fit_bayesian <- function(Y, K, iterations = 12000, burn = 2000, thin = 5,
                         max_iterations = 48000, seed = NULL,
                         sigma_scale = 1, rhat_max = 1.01, ess_min = 400,
                         prob = 0.95, keep_raw = TRUE, pivot = NULL,
                         quiet = FALSE, ...) {
  cl <- match.call()
  .check_removed_args(names(as.list(sys.call()))[-1])
  dots <- list(...)
  if (length(dots))
    .bq_abort("unused argument `%s`.", names(dots)[1])

  if (inherits(Y, "qsort_data")) {
    distribution <- Y$distribution
    Y <- Y$Y
  } else {
    Y <- as.matrix(Y)
    distribution <- infer_distribution(Y)
  }
  if (is.null(distribution) || !length(distribution))
    .bq_abort("no forced distribution: supply qsort_data with a distribution.")
  if (!is.numeric(Y))
    .bq_abort(paste0("Y is not numeric (first column class: %s); drop ID or ",
                     "text columns, or import with read_qsort()/qsort_data()."),
              class(Y[, 1])[1])
  if (anyNA(Y))
    .bq_abort("Y has %d missing entr%s; the partition likelihood needs complete sorts.",
              sum(is.na(Y)), if (sum(is.na(Y)) > 1) "ies" else "y")
  # zero-count columns (some import patterns keep interior zeros) hold no
  # statements by definition; dropping them is a monotone relabeling, and
  # the recode below maps whatever labels the sorts use onto what remains
  if (any(distribution == 0)) {
    if (!quiet)
      message(sprintf("dropping %d zero-count grid column(s) from the distribution",
                      sum(distribution == 0)))
    distribution <- distribution[distribution != 0]
  }
  # printed grid labels (-4..+4, 0-based, or any monotone scheme) are
  # conventional; recode them onto categories 1..C, which changes nothing
  C <- length(distribution)
  vals <- sort(unique(as.vector(Y)))
  if (!all(vals %in% seq_len(C))) {
    if (length(vals) == C) {
      Y <- matrix(match(Y, vals), nrow(Y), ncol(Y), dimnames = dimnames(Y))
      if (!quiet)
        message(sprintf("grid labels %s..%s recoded onto categories 1..%d (a monotone relabeling; the sorts are unchanged)",
                        vals[1], vals[C], C))
    } else {
      .bq_abort(paste0("the sorts use %d distinct values (%s, ...) but the ",
                       "design grid has %d columns; supply sorts on the ",
                       "design's ordered categories."),
                length(vals), paste(utils::head(vals, 4), collapse = ", "), C)
    }
  }
  .assert_forced_sorts(Y, distribution)

  N <- ncol(Y); J <- nrow(Y)
  if (length(K) != 1 || is.na(K) || K != round(K))
    .bq_abort("K must be a single whole number.")
  K <- as.integer(K)
  if (K < 1 || K >= N)
    .bq_abort("K must be at least 1 and smaller than the number of participants (%d).", N)
  if (K >= J)
    .bq_abort("K must be smaller than the number of statements (%d); a %d-statement panel cannot support %d factors.",
              J, J, K)
  for (arg in c("iterations", "burn", "thin", "max_iterations")) {
    v <- get(arg)
    if (length(v) != 1 || is.na(v) || v != round(v) || v < 0)
      .bq_abort("`%s` must be a single non-negative whole number.", arg)
  }
  if (thin < 1) .bq_abort("`thin` must be at least 1.")
  if (iterations <= burn)
    .bq_abort("`iterations` (%d) must exceed `burn` (%d).", iterations, burn)
  if (max_iterations < iterations)
    .bq_abort("`max_iterations` must be at least `iterations`.")
  if (max_iterations >= .Machine$integer.max / 2)
    .bq_abort("`max_iterations` is beyond any sane budget.")
  if ((iterations - burn) %% thin != 0)
    .bq_abort("(iterations - burn) must be a multiple of thin, so the keep grid stays phase-continuous under extension.")

  raw <- fit_partition_gated(t(Y), distribution, K, seed = seed,
                             sigma_scale = sigma_scale,
                             n_iter = iterations, burn = burn, thin = thin,
                             n_max = max_iterations, rhat_max = rhat_max,
                             ess_min = ess_min, quiet = quiet)
  gate <- list(converged = raw$converged, extended = raw$extended,
               iterations = raw$iters, floor = as.integer(iterations),
               cap = as.integer(max_iterations),
               rhat = raw$rhat, ess_bulk = raw$ess, ess_tail = raw$ess_tail,
               rhat_max = rhat_max, ess_min = ess_min)
  if (!gate$converged)
    warning(sprintf(paste0(
      "[bayesqm] the convergence gate was not met at the %d-iteration cap ",
      "(max Rhat %.3f, min ESS %.0f); summaries remain available but read ",
      "them cautiously, and consider extend()."),
      gate$cap, gate$rhat, min(gate$ess_bulk, gate$ess_tail)), call. = FALSE)

  keep_fields <- c("F", "Lambda", "sigma", "s_i")
  raw_draws <- if (keep_raw) raw[keep_fields] else NULL
  pp <- postprocess(raw, pivot = pivot)

  new_bayesqm_fit(
    call = cl, Y = Y, distribution = distribution, K = K,
    draws = pp[keep_fields], draws_raw = raw_draws,
    state = if (keep_raw) raw$state else NULL,
    gate = gate,
    align = list(pivot = pp$pivot, congruence = pp$congruence),
    prob = prob, seed = seed,
    settings = list(burn = as.integer(burn), thin = as.integer(thin),
                    sigma_scale = sigma_scale)
  )
}


#' Continue sampling a fitted chain
#'
#' @description
#' Warm-extends the fitted chain by further iterations and re-runs the
#' convergence gate and the alignment on the grown draw set. The extension
#' restores the sampler's saved random-number state, so the result is
#' draw-for-draw identical to having run one longer chain, whatever
#' happened in the session in between.
#'
#' @param fit A `bayesqm_fit` created with `keep_raw = TRUE`.
#' @param iterations Additional iterations; `NULL` (the default) doubles
#'   the chain. Must be a multiple of the fit's `thin`.
#' @param pivot,quiet As in [fit_bayesian()].
#'
#' @return The extended `bayesqm_fit`.
#'
#' @export
extend <- function(fit, iterations = NULL, pivot = NULL, quiet = FALSE) {
  assert_bayesqm_fit(fit)
  if (is.null(fit$draws_raw) || is.null(fit$state))
    .bq_abort("extend() needs the raw draws and sampler state; refit with keep_raw = TRUE.")
  b <- fit$brief
  thin <- b$settings$thin
  if (is.null(iterations)) iterations <- fit$gate$iterations
  if (iterations %% thin != 0)
    .bq_abort("iterations must be a multiple of thin (%d).", thin)

  raw <- c(fit$draws_raw,
           list(state = fit$state, N = b$N, J = b$J, K = b$K,
                distr = fit$distribution))
  ext <- fit_partition(t(fit$dataset), fit$distribution, b$K,
                       n_iter = iterations, burn = 0, thin = thin,
                       sigma_scale = b$settings$sigma_scale,
                       state = fit$state)
  raw <- grow_fit(raw, ext)
  cv <- check_conv(raw, fit$gate$rhat_max, fit$gate$ess_min)

  gate <- fit$gate
  gate$converged <- cv$ok
  gate$extended <- TRUE
  gate$iterations <- gate$iterations + as.integer(iterations)
  gate$rhat <- cv$rhat; gate$ess_bulk <- cv$ess; gate$ess_tail <- cv$ess_tail
  if (!quiet)
    message(sprintf("extended to %d iterations; gate %s (max Rhat %.3f, min ESS %.0f)",
                    gate$iterations, if (cv$ok) "passed" else "still not met",
                    cv$rhat, min(cv$ess, cv$ess_tail)))

  if (!is.null(fit$align$rotated))
    warning("[bayesqm] extension re-aligns the grown chain canonically; the ",
            "judgmental rotation is not carried over - re-apply ",
            "rotate_factors() (or flip_factor()) on the extended fit.",
            call. = FALSE)

  keep_fields <- c("F", "Lambda", "sigma", "s_i")
  raw_draws <- raw[keep_fields]
  pp <- postprocess(raw, pivot = pivot)

  new_bayesqm_fit(
    call = b$call, Y = fit$dataset, distribution = fit$distribution,
    K = b$K, draws = pp[keep_fields], draws_raw = raw_draws,
    state = raw$state, gate = gate,
    align = list(pivot = pp$pivot, congruence = pp$congruence),
    prob = b$prob, seed = b$seed, settings = b$settings,
    fac_ids = dimnames(fit$draws$sigma)[[2]]
  )
}

# caption.R
# One-line figure caption carrying the fit's provenance.


#' Caption text for figures from a fit
#'
#' @description
#' A single string naming the model, the panel, the draws, and the gate
#' verdict — the provenance a figure caption should carry.
#'
#' @param fit A `bayesqm_fit`.
#' @param include_gate Append the gate report (default `TRUE`).
#'
#' @return A length-one character string.
#'
#' @examples
#' caption_bayesqm(demo_fit())
#'
#' @export
caption_bayesqm <- function(fit, include_gate = TRUE) {
  assert_bayesqm_fit(fit)
  b <- fit$brief; g <- fit$gate
  out <- sprintf(
    "Exact partition (rank-order) likelihood; N = %d, J = %d, K = %d; %d posterior draws.",
    b$N, b$J, b$K, dim(fit$draws$F)[1])
  if (include_gate)
    out <- paste(out, sprintf(
      "Convergence gate %s (max Rhat %.3f, min ESS %.0f).",
      if (g$converged) "passed" else "not met", g$rhat,
      min(g$ess_bulk, g$ess_tail)))
  out
}


# one-release aliases for the 0.1.0 plot names whose concept survived

#' @rdname plot_flags
#' @param ... Passed on.
#' @export
plot_membership <- function(...) {
  .Deprecated("plot_flags", package = "bayesqm")
  plot_flags(...)
}

#' @rdname plot_contrasts
#' @param ... Passed on.
#' @export
plot_dist_cons <- function(...) {
  .Deprecated("plot_contrasts", package = "bayesqm")
  plot_contrasts(...)
}

#' @rdname plot_convergence
#' @param ... Passed on.
#' @export
plot_tucker <- function(...) {
  .Deprecated("plot_convergence", package = "bayesqm")
  plot_convergence(...)
}

# checks.R
# Posterior-predictive model checks and the person check. Replicates
# come from the fitted model's own draws through the same quota-sort engine
# that generated the data; Spearman agreement runs as Pearson on the fixed
# midrank recode, since every sort shares one tie pattern.

#' @keywords internal
#' @noRd
rank_recode <- function(Y, distr) matrix(midranks(distr)[Y], nrow(Y))

#' @keywords internal
#' @noRd
spearman_R <- function(Y, distr) {
  R <- cor(t(rank_recode(Y, distr)))
  R[!is.finite(R)] <- 0; diag(R) <- 1
  R
}

# eigenvalue of the person-correlation matrix one past K
#' @keywords internal
#' @noRd
e_next <- function(Y, distr, K) {
  ev <- eigen(spearman_R(Y, distr), symmetric = TRUE, only.values = TRUE)$values
  if (K + 1 <= length(ev)) ev[K + 1] else NA_real_
}


#' Posterior-predictive checks of the fitted model
#'
#' @description
#' Three checks, each comparing the observed sorts with replicates drawn
#' from the fitted model:
#' the agreement check (T1a) compares the observed person-agreement matrix
#' with replicated ones against a double-replicate reference; the
#' extra-factor check (T1b) locates the observed `(K+1)`-th eigenvalue in
#' its replicated distribution; and the paired-comparison check (T2)
#' compares standardized statement-pair margins, ties counted one half.
#' These are diagnostics to read, not tests to pass.
#'
#' @param fit A `bayesqm_fit`.
#' @param draws Posterior draws used for replication (default 100).
#'
#' @return A `bayesqm_checks` list: `agreement` (`p`), `extra_factor`
#'   (`percentile`), and `paired` (`p`, per-statement `p_j`), with a print
#'   method.
#'
#' @examples
#' check_fit(demo_fit(), draws = 20)
#'
#' @export
check_fit <- function(fit, draws = 100) {
  assert_bayesqm_fit(fit)
  d <- fit$draws
  T_ <- dim(d$F)[1]; N <- fit$brief$N; J <- fit$brief$J; K <- fit$brief$K
  distr <- fit$distribution
  Y <- t(fit$dataset)
  td <- unique(round(seq(1, T_, length.out = min(draws, T_))))

  # T1a: RMSE between observed and replicated agreement matrices, located
  # against a second-replicate reference
  off <- upper.tri(matrix(0, N, N))
  R_obs <- spearman_R(Y, distr)
  rmse <- function(A, B) sqrt(mean((A[off] - B[off])^2))

  # T2: all j < j' margins, ties one half, standardized discrepancy
  pr <- combn(J, 2); a <- pr[1, ]; b <- pr[2, ]
  sel_j <- lapply(seq_len(J), function(j) which(a == j | b == j))
  margins <- function(Ym) colMeans((Ym[, a] > Ym[, b]) + 0.5 * (Ym[, a] == Ym[, b]))
  p_obs <- margins(Y)

  e_obs <- e_next(Y, distr, K)

  t1a_obs <- t1a_ref <- t2o <- t2r <- numeric(length(td))
  e_rep <- numeric(length(td))
  exj <- matrix(NA_real_, length(td), J)
  for (ti in seq_along(td)) {
    t <- td[ti]
    F <- matrix(d$F[t, , ], J, K); L <- matrix(d$Lambda[t, , ], N, K)
    M <- L %*% t(F)
    Y1 <- quota_sort_rows(M + matrix(rnorm(N * J), N, J), distr)
    Y2 <- quota_sort_rows(M + matrix(rnorm(N * J), N, J), distr)
    R1 <- spearman_R(Y1, distr)
    t1a_obs[ti] <- rmse(R_obs, R1)
    t1a_ref[ti] <- rmse(spearman_R(Y2, distr), R1)
    e_rep[ti] <- e_next(Y1, distr, K)
    phat <- colMeans(pnorm((L %*% t(F[a, , drop = FALSE] - F[b, , drop = FALSE])) / sqrt(2)))
    phat <- pmin(pmax(phat, 1e-6), 1 - 1e-6)
    p_rep <- margins(Y1)
    do <- (p_obs - phat)^2 / (phat * (1 - phat))
    dr <- (p_rep - phat)^2 / (phat * (1 - phat))
    t2o[ti] <- sum(do); t2r[ti] <- sum(dr)
    for (j in seq_len(J)) exj[ti, j] <- sum(dr[sel_j[[j]]]) >= sum(do[sel_j[[j]]])
  }

  structure(
    list(agreement = list(p = mean(t1a_ref >= t1a_obs),
                          obs = t1a_obs, ref = t1a_ref),
         extra_factor = list(percentile = mean(e_rep <= e_obs),
                             observed = e_obs, replicated = e_rep),
         paired = list(p = mean(t2r >= t2o),
                       p_j = stats::setNames(colMeans(exj),
                                             dimnames(d$F)[[2]]),
                       obs = t2o, rep = t2r),
         draws = length(td)),
    class = "bayesqm_checks"
  )
}

#' @export
print.bayesqm_checks <- function(x, ...) {
  cat(sprintf("Posterior-predictive checks (%d replicated draws):\n", x$draws))
  cat(sprintf("  agreement check (T1a):        p = %.2f\n", x$agreement$p))
  cat(sprintf("  extra-factor check (T1b):     observed e_(K+1) at percentile %.2f\n",
              x$extra_factor$percentile))
  cat(sprintf("  paired-comparison check (T2): p = %.2f (worst statement %.2f)\n",
              x$paired$p, min(x$paired$p_j)))
  cat("  diagnostics to read, not tests to pass\n")
  invisible(x)
}


#' The person check against mixed-replication bands
#'
#' @description
#' For each participant, the model-agreement statistic `m` (mean over
#' draws of the best absolute Spearman agreement with any factor array)
#' and the person-agreement statistic `w` (best absolute agreement with
#' any other sort), each located against bands from mixed replicates
#' (fresh persons drawn from the fitted model). Verdicts: `fits`,
#' `no_shared` (below the model band, inside the person band),
#' `unspanned` (below the model band but above the person band, a shared
#' viewpoint the fitted factors do not span), and `atypical`. Unspanned
#' persons name their nearest partner.
#'
#' @param fit A `bayesqm_fit`.
#' @param draws Draw grid for the arrays (default 60).
#' @param mixes Mixed replicates for the bands (default 150).
#'
#' @return A `bayesqm_persons` data frame: `participant`, `m`, `w`,
#'   `partner`, `verdict`, with the band limits attached as attributes.
#'
#' @examples
#' check_persons(demo_fit(), draws = 20, mixes = 40)
#'
#' @export
check_persons <- function(fit, draws = 60, mixes = 150) {
  assert_bayesqm_fit(fit)
  d <- fit$draws
  T_ <- dim(d$F)[1]; N <- fit$brief$N; J <- fit$brief$J; K <- fit$brief$K
  distr <- fit$distribution
  Y <- t(fit$dataset)
  td <- unique(round(seq(1, T_, length.out = min(draws, T_))))
  nd <- length(td)
  Yr <- rank_recode(Y, distr)

  AR <- matrix(0, J, K * nd)
  for (ti in seq_len(nd)) for (k in seq_len(K))
    AR[, (ti - 1) * K + k] <- midranks(distr)[quota_sort(d$F[td[ti], , k], distr)]
  draw_of <- rep(seq_len(nd), each = K)
  max_by_draw <- function(v) tapply(abs(v), draw_of, max)
  CC <- cor(t(Yr), AR)
  m_i <- vapply(seq_len(N), function(i) mean(max_by_draw(CC[i, ])), numeric(1))
  W <- abs(cor(t(Yr))); diag(W) <- NA
  w_i <- apply(W, 1, max, na.rm = TRUE)
  w_part <- apply(W, 1, which.max)

  mix_m <- numeric(mixes); mixw <- matrix(NA_real_, mixes, N)
  for (bb in seq_len(mixes)) {
    t <- td[((bb - 1) %% nd) + 1]
    F <- matrix(d$F[t, , ], J, K)
    lam <- rnorm(K, 0, d$sigma[t, ])
    ymix <- midranks(distr)[quota_sort(as.vector(F %*% lam) + rnorm(J), distr)]
    cc <- as.vector(cor(ymix, AR))
    mix_m[bb] <- mean(max_by_draw(cc))
    mixw[bb, ] <- abs(as.vector(cor(ymix, t(Yr))))
  }
  band_m <- quantile(mix_m, c(0.05, 0.95))
  band_w <- t(vapply(seq_len(N), function(i)
    quantile(apply(mixw[, -i, drop = FALSE], 1, max), c(0.05, 0.95)),
    numeric(2)))

  verdict <- rep("fits", N)
  low_m <- m_i < band_m[1]
  verdict[low_m & w_i > band_w[, 2]] <- "unspanned"
  verdict[low_m & w_i <= band_w[, 2] & w_i >= band_w[, 1]] <- "no_shared"
  verdict[low_m & w_i < band_w[, 1]] <- "atypical"

  ids <- dimnames(d$Lambda)[[2]]
  if (anyDuplicated(ids))
    message("participant names are not unique; partner references use the column index")
  out <- data.frame(participant = ids, m = m_i, w = w_i,
                    partner = ids[w_part], partner_index = w_part,
                    verdict = verdict, stringsAsFactors = FALSE)
  rownames(out) <- NULL
  attr(out, "band_m") <- band_m
  attr(out, "band_w") <- band_w
  class(out) <- c("bayesqm_persons", "data.frame")
  out
}

#' @export
print.bayesqm_persons <- function(x, ...) {
  counts <- table(factor(x$verdict,
                         levels = c("fits", "no_shared", "unspanned", "atypical")))
  cat("Person check:",
      paste(sprintf("%s %d", names(counts), counts), collapse = ", "), "\n")
  unsp <- x$participant[x$verdict == "unspanned"]
  if (length(unsp)) {
    cat("  unspanned persons and their nearest partners:\n")
    for (i in which(x$verdict == "unspanned"))
      cat(sprintf("    %s -> %s (w = %.2f)\n", x$participant[i],
                  x$partner[i], x$w[i]))
  }
  print.data.frame(x, digits = 2, row.names = FALSE)
  invisible(x)
}

# Algorithm 2's T3 co-fire condition: two or more unspanned persons that
# point at each other as nearest partners (mutual)
#' @keywords internal
#' @noRd
unspanned_cluster <- function(pc) {
  idx <- which(pc$verdict == "unspanned")
  pi_ <- if (!is.null(pc$partner_index)) pc$partner_index
         else match(pc$partner, pc$participant)
  length(idx) >= 2 && any(vapply(idx, function(i)
    pi_[i] %in% idx && pi_[pi_[i]] == i, logical(1)))
}

# save_plot.R
# Opens the appropriate graphics device for the file extension, evaluates
# the (lazily evaluated) plot expression on it, closes the device, and
# returns the path. Defaults to a 7.2 x 5 inch figure at 300 dpi for
# raster formats.


#' Save a bayesqm plot to file
#'
#' @description
#' Opens a graphics device chosen from the file extension (`.pdf`,
#' `.svg`, `.png`, `.tiff`, `.jpeg`), evaluates `expr` so whatever it
#' draws lands on that device, and closes the device. `expr` is lazily
#' evaluated, so a call like `save_bayesqm_plot("fig.pdf", plot(fit))`
#' does not draw to the current screen device first. If `expr` returns
#' a `ggplot` object (for example, from [ggplot2::autoplot()]), it is
#' `print()`ed onto the device.
#'
#' @param file Output path. Extension determines the device.
#' @param expr Plotting expression (lazily evaluated).
#' @param width,height Dimensions in inches. Defaults to a 7.2 x 5
#'   inch two-column journal figure.
#' @param dpi Resolution for raster formats (`png`, `tiff`, `jpeg`).
#'
#' @return The `file` path, invisibly.
#'
#' @examples
#' f <- file.path(tempdir(), "fig.pdf")
#' save_bayesqm_plot(f, plot(1:10))
#' unlink(f)
#'
#' @export
save_bayesqm_plot <- function(file, expr,
                              width  = 7.2,
                              height = 5,
                              dpi    = 300) {
  ext <- tolower(tools::file_ext(file))
  if (!nzchar(ext))
    stop("file must have an extension (pdf, svg, png, tiff, jpeg).")

  switch(ext,
    pdf  = grDevices::pdf(file, width = width, height = height),
    svg  = grDevices::svg(file, width = width, height = height),
    png  = grDevices::png(file,  width = width  * dpi,
                                 height = height * dpi,
                                 res    = dpi),
    tiff = grDevices::tiff(file, width = width  * dpi,
                                 height = height * dpi,
                                 res    = dpi),
    jpeg = ,
    jpg  = grDevices::jpeg(file, width = width  * dpi,
                                 height = height * dpi,
                                 res    = dpi,
                                 quality = 95),
    stop("Unsupported extension '", ext,
         "'. Use pdf, svg, png, tiff, or jpeg."))

  on.exit(grDevices::dev.off(), add = TRUE)

  result <- force(expr)
  if (inherits(result, c("ggplot", "gg"))) print(result)

  invisible(file)
}

# save_bayesqm_plot: device handling, extension dispatch, error paths.

test_that("save_bayesqm_plot writes a pdf and returns the path", {
  tmp <- tempfile(fileext = ".pdf")
  out <- save_bayesqm_plot(tmp, plot(1:10))
  expect_identical(out, tmp)
  expect_true(file.exists(tmp))
  unlink(tmp)
})

test_that("save_bayesqm_plot dispatches by extension", {
  for (ext in c(".pdf", ".png")) {
    tmp <- tempfile(fileext = ext)
    save_bayesqm_plot(tmp, plot(1:10), width = 4, height = 3)
    expect_true(file.exists(tmp))
    unlink(tmp)
  }
})

test_that("save_bayesqm_plot errors on missing or unknown extension", {
  expect_error(save_bayesqm_plot(tempfile(), plot(1:10)))
  expect_error(save_bayesqm_plot(tempfile(fileext = ".xyz"), plot(1:10)))
})

test_that("save_bayesqm_plot prints a ggplot automatically", {
  skip_if_not_installed("ggplot2")
  tmp <- tempfile(fileext = ".pdf")
  p <- ggplot2::ggplot(data.frame(x = 1:3, y = 1:3),
                       ggplot2::aes(x, y)) + ggplot2::geom_point()
  save_bayesqm_plot(tmp, p)
  expect_true(file.exists(tmp))
  unlink(tmp)
})

test_that("save_bayesqm_plot closes the device even on error", {
  ndev <- length(grDevices::dev.list())
  tmp <- tempfile(fileext = ".pdf")
  try(save_bayesqm_plot(tmp, stop("boom")), silent = TRUE)
  expect_identical(length(grDevices::dev.list()), ndev)
  unlink(tmp)
})

# Construct a synthetic bayesqm_fit without sampling. Draws respect the
# column conventions (centered, unit-sd score columns), so every accessor,
# print method, and summary function works against a fit built this way.

make_fake_fit <- function(N = 6, J = 10, K = 2, T = 120, seed = 1L,
                          converged = TRUE, extended = FALSE) {
  set.seed(seed)
  distr <- get_distribution(J)
  L0 <- matrix(0, N, K)
  L0[cbind(seq_len(N), rep_len(seq_len(K), N))] <- 1.2

  Fd <- array(NA_real_, c(T, J, K))
  Ld <- array(NA_real_, c(T, N, K))
  Sd <- matrix(abs(rnorm(T * K, 1, 0.1)), T, K)
  si <- matrix(NA_real_, T, N)
  for (t in seq_len(T)) {
    F <- matrix(rnorm(J * K), J, K)
    F <- sweep(F, 2, colMeans(F))
    F <- sweep(F, 2, sqrt(colMeans(F^2)), "/")
    L <- L0 + matrix(rnorm(N * K, 0, 0.15), N, K)
    Fd[t, , ] <- F; Ld[t, , ] <- L
    si[t, ] <- sqrt(rowSums((L %*% (crossprod(F) / J)) * L))
  }
  draws <- list(F = Fd, Lambda = Ld, sigma = Sd, s_i = si)

  Fm <- apply(Fd, c(2, 3), mean); dim(Fm) <- c(J, K)
  Lm <- apply(Ld, c(2, 3), mean); dim(Lm) <- c(N, K)
  Y <- quota_sort_rows(t(Fm %*% t(Lm)) + matrix(rnorm(N * J, 0, .1), N, J), distr)
  Y <- t(Y)
  rownames(Y) <- paste0("S", seq_len(J))
  colnames(Y) <- paste0("P", seq_len(N))

  gate <- list(converged = converged, extended = extended,
               iterations = if (extended) 1600L else 800L, floor = 800L,
               cap = 3200L, rhat = 1.004, ess_bulk = 450, ess_tail = 430,
               rhat_max = 1.01, ess_min = 400)

  new_bayesqm_fit(
    call = quote(fit_bayesian(Y, K = K)),
    Y = Y, distribution = distr, K = K,
    draws = draws, draws_raw = NULL, state = NULL,
    gate = gate,
    align = list(pivot = 1L, congruence = rep(0.95, T)),
    prob = 0.95, seed = seed,
    settings = list(burn = 200L, thin = 5L, sigma_scale = 1)
  )
}

# posterior-package bridges registered in .onLoad.

test_that("as_draws_df round-trips the flat matrix", {
  fit <- make_fake_fit(T = 40)
  dd <- posterior::as_draws_df(fit)
  expect_identical(posterior::ndraws(dd), 40L)
  expect_true("Lambda[1,1]" %in% posterior::variables(dd))
})

test_that("as_draws_matrix and as_draws_rvars work", {
  fit <- make_fake_fit(T = 40)
  m <- posterior::as_draws_matrix(fit)
  expect_identical(dim(m), c(40L, ncol(as.matrix(fit))))
  rv <- posterior::as_draws_rvars(fit)
  expect_identical(dim(rv$Lambda), c(6L, 2L))
})

test_that("ndraws dispatches on the fit", {
  fit <- make_fake_fit(T = 40)
  expect_identical(posterior::ndraws(fit), 40L)
})

---
title: "Getting started with bayesqm"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 2
bibliography: REFERENCES.bib
link-citations: true
vignette: >
  %\VignetteIndexEntry{Getting started with bayesqm}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  fig.width  = 7,
  fig.height = 4.5,
  dpi = 110
)
```

This vignette walks through one Q study end to end in **bayesqm**:
import, fit, diagnose, interpret, select K, and export. The running
example is a synthetic 33-participant, 42-statement Q-sort.

```{r, message = FALSE, warning = FALSE}
library(bayesqm)
library(ggplot2)
```

## Setup

bayesqm performs inference with Stan, so you need a working Stan
backend before the modelling functions will run. The package
supports cmdstanr (recommended) and rstan.

cmdstanr is distributed through the Stan R-universe rather than
CRAN, and installing the R package does not install CmdStan itself;
that is a separate step. Run the following once (not evaluated
here):

```{r setup-cmdstan, eval = FALSE}
install.packages("cmdstanr", repos = c("https://stan-dev.r-universe.dev", getOption("repos")))
cmdstanr::check_cmdstan_toolchain(fix = TRUE)
cmdstanr::install_cmdstan()
cmdstanr::cmdstan_version()   # should print a version
```

On Windows you additionally need Rtools matching your R version; on
macOS, run `xcode-select --install`. If you prefer rstan, install
it from CRAN with a C++ toolchain; bayesqm uses whichever backend
is available.

The remainder of this vignette uses small precomputed demonstration
objects (`demo_fit()`, `demo_run()`) so it renders without a Stan
backend. To reproduce the examples on real data you will need the
backend installed as above.

## The Q-sort data

A Q study asks each participant to rank-order a set of statements
into a forced distribution. `bayesqm` represents that as a `J × N`
numeric matrix (statements as rows, participants as columns) plus a
vector of forced-distribution counts. `read_qsort()` auto-detects
the file format:

```{r, eval = FALSE}
qdata <- read_qsort("mystudy.csv")   # CSV / Excel / HTMLQ / FlashQ
qdata <- read_qsort("mystudy.DAT")   # PQMethod
qdata <- read_qsort("mystudy.zip")   # KADE project
```

The example dataset:

```{r}
sim   <- generate_data(N = 33, J = 42, K = 3, seed = 1)
qdata <- qsort_data(sim$Y, distribution = sim$distribution)
qdata
```

## Fitting the model

`fit_bayesian()` samples the posterior of a low-rank factor model
with a Student-t likelihood (by default) and a hierarchical normal
prior on the loadings, then resolves rotational ambiguity with the
MatchAlign post-processing of @PoworoznekEtAl2025:

```{r, eval = FALSE}
fit <- fit_bayesian(qdata, K = 3, chains = 4, iter = 2000,
                    warmup = 1000, seed = 1)
```

```{r}
fit <- demo_fit(N = 33, J = 42, K = 3, seed = 1)
fit
```

`summary(fit)` adds factor characteristics, the PSIS-LOO ELPD (when
available), a divergence summary, and the MatchAlign
diagnostic.

## Diagnostics

Convergence statistics are stored on `fit$diagnostics`. The
recommended thresholds are R-hat below 1.01, bulk and tail effective
sample sizes above 400, and zero divergent transitions
[@VehtariEtAl2021]:

```{r}
fit$diagnostics
```

`plot_tucker()` visualises the per-draw Tucker phi between each
aligned factor column and the MatchAlign pivot. Values near 1
indicate a stable alignment; bimodality signals residual
label-switching.

```{r tucker, fig.cap = "MatchAlign alignment quality by factor."}
plot_tucker(fit)
```

## Factor loadings

Each participant has a posterior distribution over their loading on
every factor. `plot_loading_posterior()` draws the loading forest,
with nested 50% and 95% credible-interval whiskers and the classical
@Brown1980 cut-off as a faint reference. Filled points are
participants with `P(factor k dominant) > 0.5`.

```{r loadings, fig.cap = "Loadings with nested 50% and 95% credible intervals."}
plot_loading_posterior(fit)
```

The same information as a tidy data frame:

```{r}
head(compute_loadings(fit$Lambda_draws, prob = 0.95))
```

## Factor z-scores

`plot()` produces a cross-panel z-score dotchart; rows share an
order (by factor 1's posterior mean, configurable via `sort_by`) so
factors can be compared horizontally.

```{r zscores, fig.cap = "Posterior z-scores across factors with 50% and 95% credible intervals."}
plot(fit)
```

For a single statement across factors, `plot_zscore_posterior()`
shows the per-factor view:

```{r zscore-one, fig.cap = "Posterior z-score for a single statement across factors."}
plot_zscore_posterior(fit, statement = 1)
```

## Choosing K

`run_bayes()` fits the model for each K in a range and applies the
**peak-plus-Sivula** protocol: the ELPD peak is the automated choice
and the Sivula parsimony rule [@SivulaEtAl2025] is reported alongside
as a diagnostic.

On real data:

```{r, eval = FALSE}
run <- run_bayes(qdata, K_max = 4, seed = 1,
                 chains = 2, iter = 1500, warmup = 800)
```

```{r}
run <- demo_run(K_max = 5, k_peak = 3, k_sivula = 1, case = "gap")
run
```

```{r elpd, fig.width = 7, fig.height = 4.2, fig.cap = "ΔELPD across K with the Sivula non-promotion band at |ΔELPD| < 4. Triangle marks the Sivula K, square marks the ELPD peak (the adopted K)."}
make_elpd_diff(run)
```

The ELPD peak is always the adopted K. The Sivula diagnostic is
reported alongside as a parsimony check. The `run$case` field labels
the relationship between the two: when Sivula lands on the same K as
the peak, the case is `agree`; when Sivula points to a smaller K
than the peak, the case is `gap`; when Sivula exceeds the peak, the
case is `reversed` (rare in practice).

## Distinguishing, consensus, and membership

`bayesqm` replaces the classical z-score test with the posterior of an
explicit viewpoint-divergence measure. For each statement, D_j is the
mean absolute pairwise difference of the K viewpoint scores; the
package reports P(D_j > delta | Y) and P(D_j < delta | Y), the
probabilities that the viewpoints diverge meaningfully or practically
agree. By default the separation delta is the reliability-adjusted
critical difference from classical Q analysis [@Brown1980], computed
from the posterior dominant-factor counts (`critical_delta()`);
`suggest_delta()` (one forced-distribution category on the standardised
scale) is an alternative, and results are reported with sensitivity
across the choice of delta. No fixed probability cutoff defines a
statement.

The full distinguishing/consensus table is stored on `fit$qdc` (per
viewpoint: grid position and z-score with 95% CrI, then `D_j` with
95% CrI, `pi_D`, `pi_C`). `critical_delta()` returns the separation
the fit used:

```{r}
critical_delta(fit$Lambda_draws)
head(fit$qdc)
```

```{r dist-cons, fig.cap = "Posterior viewpoint divergence D_j with 95% credible intervals; statements ordered by P(D_j > delta | Y)."}
plot_dist_cons(fit)
```

Participant-level membership is also probabilistic.
`classify_membership()` turns posterior dominance into a
`Strong / Moderate / Weak` tier:

```{r}
head(classify_membership(fit$Lambda_draws))
```

```{r membership, fig.width = 6.5, fig.height = 5.5, fig.cap = "Blue tiles: posterior probability that each factor is dominant for a given participant (values printed in each cell). The right column shows the overall assignment verdict on an orange-red scale."}
make_dominant_panel(fit)
```

## Posterior predictive check

`plot_ppc()` shows the posterior distribution of the RMSE between
`cor(Y_rep)` and `cor(Y_obs)`. A well-specified model puts most
posterior mass at small RMSE.

```{r ppc, fig.cap = "PPC on the by-person correlation matrix."}
plot_ppc(fit)
```

## Hyperparameters

```{r hyper, fig.cap = "Posterior densities of nu, sigma, and tau."}
plot_hyper(fit)
```

## Reporting and exporting

`save_bayesqm_plot()` writes any plot to PDF, SVG, PNG, TIFF, or
JPEG at the chosen dimensions:

```{r, eval = FALSE}
save_bayesqm_plot("fig_loadings.pdf", plot_loading_posterior(fit))
save_bayesqm_plot("fig_elpd.pdf",      plot_elpd(run),
                  width = 3.5, height = 3)
```

`caption_bayesqm()` returns a figure caption with model
configuration, sample sizes, interval probability, and convergence
diagnostics:

```{r}
caption_bayesqm(fit)
```

Standard R accessors work on the fit (`coef`, `fitted`, `residuals`,
`sigma`, `family`, `as.matrix`), and `as.matrix(fit)` returns a
Stan-style draws matrix that `posterior` and `bayesplot` consume
natively.

## Theming

Every plot reads its palette through `bayesqm_colors()`. Switch the
scheme once and every subsequent base-R plot follows:

```{r, eval = FALSE}
bayesqm_set_colors("teal")   # "blue" (default), "red", "purple", "grey"
plot(fit)
bayesqm_set_colors("blue")
```

## Where next

- `?fit_bayesian`: every prior and sampler option
- `?run_bayes`: peak-plus-Sivula thresholds
- `?bayesqm-membership`: the full set of membership summaries
- `?rename_factors`: relabel `f1..fK` with substantive factor names
- `ggplot2::autoplot()`: ggplot2 / ggdist versions of every view
  above, when `ggplot2` and `ggdist` are installed

## References

---
title: "Validation"
bibliography: ../REFERENCES.bib
csl: ../apa.csl
link-citations: true
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>",
                      fig.width = 7, fig.height = 5)
library(bayesqm)
```

This article checks the implementation two ways. It fits a panel with
planted viewpoints and measures how well the truth comes back, and it
runs a real panel through both this package and `qmethod` to compare
the outputs wherever the two compute the same object. Every number on
this page is computed when the site builds.

## Recovering planted truth

`generate_data()` builds a panel with known viewpoints. 14 sorters, 20
statements, two planted factors, the same panel the README analyzes.

```{r truth-fit}
sim <- generate_data(N = 14, J = 20, K = 2, noise_sd = 0.6,
                     primary_range = c(0.65, 0.9), seed = 7)
qdata <- qsort_data(sim$Y, distribution = sim$distribution)
fit <- fit_bayesian(qdata, K = 2, seed = 7)
```

The model reports loadings on a bounded correlation scale, so the
planted loadings go onto the same scale before the comparison, and
the posterior draws supply the interval checks.
`assess_recovery()` aligns the estimate to the truth and returns the
error, the congruence per factor, a 0-to-1 agreement between loading
patterns, and the coverage of the credible intervals.

```{r truth-recovery}
rho_true  <- sim$Lambda_true / sqrt(1 + rowSums(sim$Lambda_true^2))
lo        <- compute_loadings(fit)
rho_hat   <- as.matrix(lo[, paste0("f", 1:2, "_loading")])
s2        <- fit$draws$s_i^2
rho_draws <- fit$draws$Lambda /
  sqrt(1 + array(s2, dim(fit$draws$Lambda)))

rec <- assess_recovery(rho_hat, rho_true, rho_draws)
round(rec$tucker, 3)
round(unlist(rec[c("rmse", "coverage", "ci_width")]), 3)
```

Congruence at these levels says the planted viewpoints come back as
themselves, and coverage near the 95 percent the intervals promise
says the uncertainty is honest. The statement scores tell the same
story.

```{r truth-scores}
zs <- compute_zscores(fit)
F_hat <- as.matrix(zs[, paste0("f", 1:2, "_zsc")])
R <- attr(procrustes_rotation(rho_hat, rho_true), "rotation")
round(diag(cor(scale(sim$F_true), F_hat %*% R)), 3)
```

A single run shows the machinery working, it does not prove
calibration. The generator scales to any panel shape, so the check can
be repeated at your own study's size before you commit to an
analysis.

## Corresponding with classical analysis

`qmethod` [@Zabala2014] is the field's reference implementation of the
classical workflow, principal components, varimax rotation, and the
standard flagging and scoring rules. Where both packages compute the
same object the two should track each other, and the childhood
obesity panel of @AkhtarDanesh2023 shows what that looks like on a
real study.

```{r obesity-fits}
data(obesity_sorts)
qm  <- qmethod::qmethod(as.data.frame(obesity_sorts$Y), nfactors = 3,
                        silent = TRUE)
fb  <- fit_bayesian(obesity_sorts, K = 3, seed = 11)
lo3 <- compute_loadings(fb)
rho <- as.matrix(lo3[, paste0("f", 1:3, "_loading")])
```

Find each classical factor's closest posterior counterpart by
congruence, then compare the loadings and the statement scores.

```{r obesity-compare}
loa <- as.matrix(qm$loa)
cong <- abs(t(apply(loa, 2, function(a)
  apply(rho, 2, function(b) tucker_congruence(a, b)))))
closest <- apply(cong, 1, which.max)
zb <- as.matrix(compute_zscores(fb)[, paste0("f", 1:3, "_zsc")])
zq <- as.matrix(qm$zsc)

data.frame(
  classical  = colnames(loa),
  closest    = paste0("f", closest),
  congruence = round(cong[cbind(1:3, closest)], 2),
  score_cor  = round(diag(cor(zq, zb[, closest])), 2)
)
```

The dominant factor corresponds tightly, in loadings and in scores.
The two weaker classical factors correspond progressively less, and
they are exactly the factors the decision layer declines to support.
Notice also that two classical factors share the same closest
posterior counterpart, which is what a split of one viewpoint looks
like from the posterior's side.

The arrays tell the same story. The share of statements both analyses
place in the same grid column:

```{r obesity-arrays}
ab <- compute_factor_array(fb)
gb <- as.matrix(ab[, paste0("f", 1:3, "_grid")])
gq <- as.matrix(qm$zsc_n) + 5     # printed -4..4 onto categories 1..9
round(colMeans(gq == gb[, closest]), 2)
```

Strong for the dominant pair, far weaker for the rest.

The two differ where they are designed to differ. The classical
analysis flags a participant or it does not, while `compute_flags()`
returns the probability of that assignment, and `claims()` refuses to
report a flag the posterior cannot stand behind. On this panel the
classical account reports three factors, and the decision layer reads
the same numbers as one shared viewpoint, a divergence the package
front page shows in full. Point summaries correspond where both
methods compute the same object. The Bayesian layer adds the
uncertainty and the rules.

## References

---
title: "Getting started with bayesqm"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 2
vignette: >
  %\VignetteIndexEntry{Getting started with bayesqm}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: REFERENCES.bib
csl: apa.csl
link-citations: true
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>",
                      fig.width = 7, fig.height = 5)
library(bayesqm)
```

You have a stack of completed Q sorts and a practical question. How
many viewpoints does the panel hold, who holds each one, and which
statements set them apart? `bayesqm` answers from a model of the
sorting event itself. Each participant placed every statement into a
fixed grid, the quotas make that placement an ordered partition of the
statement set, and the package computes the probability of the
observed partition exactly. The tables that come back are the ones a
Q study reports, with the uncertainty of every entry.

This vignette walks one analysis end to end on a small demonstration
fit, so it knits in seconds. On real data, replace `demo_fit()` with
`fit_bayesian()`. A fit at typical panel sizes takes a minute or two.

## The sorts

`read_qsort()` reads the common formats. CSV and Excel grids, PQMethod
`.DAT`, Ken-Q JSON and multi-sheet Excel, KADE ZIP exports, and
Easy-HTMLQ Firebase JSON.

```{r, eval = FALSE}
qdata <- read_qsort("mystudy.dat")        # PQMethod file, for example
qdata <- qsort_data(Y, distribution = c(2, 3, 4, 5, 4, 3, 2))
```

One scope note up front. The exact likelihood is a statement about the
forced sorting event, so `fit_bayesian()` requires every sort to match
the design grid and names any participant who does not.
Free-distribution studies, which Ken-Q, HTMLQ, and KADE permit, are
outside this model's scope. The import functions still read them for
inspection.

Before any model runs, look at what the participants actually did.
`plot(qdata)` draws every completed sort as its own pyramid, one tile
per statement, the color giving the column it was placed in. Here on
`obesity_sorts`, the childhood obesity panel of @AkhtarDanesh2023 that
ships with the package. 33 participants, 42 statements, a nine-column
grid.

```{r, fig.height = 6}
data(obesity_sorts)
plot(obesity_sorts, participants = 1:6)
```

A reversed sorter, a pattern shared by nobody, a data-entry slip, all
of it shows up here first, while it is still a data question rather
than a modeling one.

## The fit

```{r}
fit <- demo_fit(seed = 1)
fit
```

The header carries what a fit is. The likelihood, the panel, the
draws, the convergence check, and the alignment. Convergence is judged
on summaries that do not depend on rotation, and a chain that misses
the bar is extended at successive doublings up to a cap. If it still
misses, the fit warns, and `extend(fit)` continues exactly where it
stopped, draw for draw identical to one longer run. Rotational, sign,
and label ambiguity is resolved by the MatchAlign procedure of
@PoworoznekEtAl2025 with a polarity rule, so defining sorts load
positively and every summary comes from one common orientation.

## Who holds each viewpoint

Loadings on a bounded correlation scale, with credible intervals.

```{r}
head(compute_loadings(fit))
```

A flag probability is the posterior share of draws in which a
participant defines the factor, and the unclassified state keeps
cross-loading and abstention visible instead of forcing a yes or no.

```{r}
compute_flags(fit)
```

## What each viewpoint says

The factor array is the viewpoint written as a completed sort,
quota-exact by construction.

```{r}
head(compute_factor_array(fit))
```

```{r}
plot_factor_array(fit)
```

Darker tiles are placements the posterior is more certain of.

## Where they differ and where they agree

The critical difference is computed from the posterior spread of each
score contrast, and consensus is a positive finding, the event that
every factor places a statement within one grid column of the others.
A statement can be distinguishing, consensus, or neither.

```{r}
qdc <- compute_qdc(fit)
table(qdc$verdict)
```

```{r}
plot_contrasts(fit)
```

## What gets reported

One rule decides. `claims()` keeps the most probable flags,
distinguishing statements, consensus statements, and pairwise stars,
and stops adding claims when the expected share of false ones passes
the level you set.

```{r}
claims(fit, q = 0.05)
```

## The per-factor block

`factor_characteristics()` is the summary a results section quotes.
How many sorts define each factor, with an interval, how spread the
factor's statement scores are, and how reliable its defining sorts
are.

```{r}
factor_characteristics(fit)
```

## Checking the model

```{r}
check_fit(fit, draws = 30)
```

The person check separates sorts the model spans from shared
viewpoints it does not.

```{r}
check_persons(fit, draws = 15, mixes = 30)
```

## How many viewpoints

`fit_ladder()` fits a ladder of candidate K and `select_k()` applies
two checks together. Adequacy asks whether K factors account for the
shared structure in the panel. Support asks whether every factor earns
its place, meaning at least two selected flags and one selected
distinguishing statement. Expect one fit's runtime per rung.

```{r, eval = FALSE}
ladder <- fit_ladder(qdata)
select_k(ladder)
plot_choice_k(select_k(ladder))
```

`loo_ladder()` adds PSIS-LOO [@VehtariEtAl2017] as directional
corroboration only. At typical Q panel sizes its standard errors
cannot certify adjacent K.

## Coming from PQMethod or qmethod

The outputs you know have direct counterparts. Flagging, automatic or
by hand, becomes `compute_flags()`, and `claims()` selects the flags
at your false-discovery level. The loadings table is
`compute_loadings()`, the factor arrays and z-scores are
`compute_factor_array()` and `compute_zscores()`, and the
distinguishing and consensus statements are the three-way verdicts of
`compute_qdc()`. In place of eigenvalue rules and parallel analysis,
the number of factors comes from `fit_ladder()` and `select_k()`.
Explained variance alone has no counterpart in a generative model.
Report the defining-sort counts from `factor_characteristics()` and
the extra-factor check from `check_fit()` instead.

## Reproducibility

Fits are exactly reproducible given a seed, and `extend()` preserves
that. It restores the sampler's saved random-number state, so
continuing a chain after any amount of unrelated R work gives the same
draws as one uninterrupted run.

## Where next

The remaining views are `plot_zscores()` for the whole statement
panel, `plot_statement()` for one statement in depth,
`plot_loading_posterior()`, `plot_flags()`, `plot_person_check()`,
`plot_convergence()`, and `plot_ppc()`, with `ggplot2::autoplot()`
methods for loadings, flags, contrasts, and the array. The reference
index at <https://rdazadda.github.io/bayesqm/> groups every function
by task.

## References

---
title: "The decision rules"
bibliography: ../REFERENCES.bib
csl: ../apa.csl
link-citations: true
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>",
                      fig.width = 7, fig.height = 5)
library(bayesqm)
```

Two rules sit behind everything the package reports. `claims()`
decides what a fitted model may report, and `select_k()` decides how
many factors to fit at all. This article works through both.

## One rule for every claim

A candidate claim is anything the analysis might report. A flag, a
distinguishing statement, a consensus statement, a pairwise star. Each
carries a posterior probability, and the rule is the same for all
four kinds. Rank the candidates by probability, add them from the
top, and stop when the expected share of false claims among those
added passes the level q. Every claim short of probability one
contributes its shortfall, so the expected number of false claims is
the sum of one minus the probability across the selected claims, and
the table reports that number next to the count.

The demonstration fit makes the arithmetic visible. Its strongest
flag candidates:

```{r rank}
fit <- demo_fit(seed = 1)
fl <- compute_flags(fit)
head(fl[order(-fl$flag_prob), c("participant", "factor", "flag_prob")], 5)
```

At q = 0.05 none of these probabilities is high enough. Even the
strongest candidate would put the expected false share past five
percent on its own, so the rule reports nothing rather than something
it cannot stand behind.

```{r strict}
claims(fit, q = 0.05)
```

Loosen the level and the rule starts admitting candidates, stating
the expected number of false claims it now carries.

```{r loose}
claims(fit, q = 0.25)
```

That is the whole mechanism. The level q is the only setting, and the
expected number of false claims is what it controls.

## Two checks for the number of factors

`fit_ladder()` fits every candidate K and `select_k()` reads each fit
twice. The adequacy check asks whether K factors account for the
shared structure, judged by where the next unused eigenvalue falls
against the model's own replications, with a warning when the person
check finds a cluster of sorters no factor spans. The support check
asks whether every factor earns its place, at least two selected
flags and at least one selected distinguishing statement. The
selected K is the smallest candidate that passes both.

On a simulated panel with two planted viewpoints, the decision looks
like this:

```{r ladder, fig.height = 5.5}
sim <- generate_data(N = 14, J = 20, K = 2, noise_sd = 0.6,
                     primary_range = c(0.65, 0.9), seed = 7)
qdata <- qsort_data(sim$Y, distribution = sim$distribution)
lad <- fit_ladder(qdata, K_min = 2, K_max = 4, seed = 7, quiet = TRUE)
plot_choice_k(select_k(lad))
```

Each row ends with its own verdict. K = 4 is adequate but two of its
factors attract nothing, K = 3 fails the adequacy check, and K = 2
passes both, so it is boxed and named.

When no row passes both checks, the rule refuses to select and names
what it sees instead. A panel whose flags pile onto one factor while
no statement separates any pair reads as a single viewpoint, the
verdict the childhood obesity panel receives in its walkthrough. A
panel where no factor attracts even two flags reads as no shared
structure. On the grizzly bear panel, the package's second shipped
dataset, the same standard resolves the other way, K = 2 passes both
checks and the rule selects two viewpoints. And when adequacy and
support each hold somewhere but never together, the verdict is
tension, an instruction to look at the rows and make the choice
openly.

A refusal is a finding. The data can support fewer viewpoints than
the analyst hoped, the rules let that outcome through, and that is
what makes the solutions they do select worth believing.

---
title: "Output codebook"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 2
vignette: >
  %\VignetteIndexEntry{Output codebook}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(bayesqm)
fit <- demo_fit(seed = 1)
```

Every column of every table the package reports, defined in one
place. The examples run on the small demonstration fit, so the
numbers here are illustrations, not findings.

## compute_loadings()

```{r}
head(compute_loadings(fit), 3)
```

One row per participant. `f*_loading` is the posterior mean loading
on the bounded correlation scale, between -1 and 1. `f*_lower` and
`f*_upper` bound the credible interval at the fit's stored
probability, 95 percent by default. `spread` is the posterior mean of
the participant's person spread, the size of their systematic signal.
Calling with `prob = 0.5` returns the same table with 50 percent
intervals for nested plotting.

## compute_flags()

```{r}
head(compute_flags(fit), 3)
```

One row per participant. `factor` is the participant's most probable
factor, `sign` its pole, 1 for a defining sort and -1 for a mirrored
one. `flag_prob` is the posterior probability that the participant
defines that factor, both poles combined, and `unclassified_prob` the
probability that no factor claims them. `selected` marks the flags
the false-discovery rule reports at the chosen level. The table
carries two attributes, `expected_false`, the expected number of
false flags among the selected, and `phi`, the full probability
matrix over every factor, pole, and the unclassified state.

## compute_zscores()

```{r}
head(compute_zscores(fit), 3)
```

One row per statement. `f*_zsc` is the posterior mean statement score
in z units under each factor, with `f*_lower` and `f*_upper` its
interval. Scores are computed from the aligned draws, so they are
comparable across factors.

## compute_factor_array()

```{r}
head(compute_factor_array(fit), 3)
```

One row per statement. `f*_grid` is the reported grid column for the
statement under each factor, quota-exact by construction, numbered as
consecutive categories from 1 to the number of columns. Attribute `certainty` is the posterior
probability of each placement, the shading in
`plot_factor_array()`, and `footrule_disagreement` records how far
the reported array sits from the unconstrained posterior ranking.

## compute_qdc()

```{r}
qdc <- compute_qdc(fit)
head(qdc, 3)
```

One row per statement. `f*_dist_prob` is the probability that the
statement distinguishes that factor from every other, and
`consensus_prob` the probability that all factors place it within one
grid column of each other. `verdict` is the resulting three-way call,
distinguishing with its factors named, consensus, or indeterminate.

The `contrasts` attribute is the long table behind the verdicts:

```{r}
head(attr(qdc, "contrasts"), 3)
```

One row per statement and factor pair. `median`, `lower`, and `upper`
summarize the posterior score contrast. `exceed_prob` is the
probability that the contrast exceeds the critical difference,
`diff_column_prob` the probability that the two factors place the
statement in different grid columns, `selected` the false-discovery
selection at the first level, and `stars` the two-level marking, one
star for a selected contrast and two when the stricter level is also
cleared. The attributes `delta_kl`, `delta_kl99`, and `delta_grid`
carry the two critical differences and the one-column consensus
region.

## factor_characteristics()

```{r}
factor_characteristics(fit)
```

One row per factor. `flagged` counts the selected flags, and
`defining_modal`, `defining_mean`, `defining_lower`, and
`defining_upper` summarize the posterior number of defining sorts.
`score_spread` is the average posterior spread of the factor's
statement scores, and `reliability` the mean replicate reliability of
the factor's flagged participants, `NA` when no participant is
flagged. The `score_correlations` attribute holds the posterior mean
correlations between factor score columns.

## claims()

```{r}
claims(fit, q = 0.25)
```

Four tables and a rule. `flags`, `distinguishing`, `consensus`, and
`stars` list every claim the false-discovery rule selects at level
`q`, each row carrying its posterior probability, and
`expected_false` gives the expected number of false claims inside
each family.

## check_fit() and check_persons()

```{r}
check_fit(fit, draws = 20)
```

`agreement` compares the observed person-to-model agreement with its
replicated reference and reports a two-sided probability. `paired`
does the same for paired comparisons. `extra_factor` reports where
the next unused eigenvalue falls among the model's replications, the
percentile the choice-of-K rule reads.

```{r}
pc <- check_persons(fit, draws = 10, mixes = 20)
head(pc, 3)
```

One row per participant. `m` is agreement with the model's
reconstruction, `w` agreement with their own mixed replicates, and
`verdict` the resulting call, with `partner` naming the nearest
other sorter when a shared viewpoint sits outside the model.

## crib_sheet()

```{r}
head(crib_sheet(fit), 3)
```

One row per statement and factor. `p_top` and `p_bottom` are the
probabilities of landing in the most extreme agree and disagree
columns, `p_highest` and `p_lowest` of being that factor's single
most and least agreed statement. These are the shortlists used when
interpreting a factor.

## select_k()

Run on a ladder of fits, `select_k()` returns `K`, the verdict, and a
table with one row per candidate. `extra_factor` is the
posterior-predictive percentile, `adequate` whether it sits in the
band with no unspanned cluster, `factors_supported` how many factors
earn at least two selected flags and one selected distinguishing
statement, and `all_supported` whether every factor does. The
`detail` element carries the same support information factor by
factor, and the plot method draws the whole decision.

---
title: "The obesity panel, start to finish"
bibliography: ../REFERENCES.bib
csl: ../apa.csl
link-citations: true
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>",
                      fig.width = 7, fig.height = 5)
library(bayesqm)
```

This walkthrough runs the childhood obesity panel of @AkhtarDanesh2023
from raw sorts to write-up. The package front page shows this panel's
headline, the ladder refusing a multi-factor solution and reading the
panel as one shared viewpoint. Here the analysis goes all the way,
with the checks and the tables a study would quote.

## The sorts

33 participants sorted 42 statements about childhood obesity onto a
nine-column grid.

```{r sorts, fig.height = 6}
data(obesity_sorts)
obesity_sorts
plot(obesity_sorts, participants = 1:12)
```

## The ladder, row by row

Fit two, three, and four factors and let the two checks read each
candidate.

```{r ladder, fig.height = 5.5}
sel <- select_k(fit_ladder(obesity_sorts, K_min = 2, K_max = 4,
                           seed = 11, quiet = TRUE))
plot_choice_k(sel)
```

Read the rows. Every dot sits in the band, so every K accounts for
the panel. On the right, one chip at every K carries a `d` alone,
that factor attracts flags but not one statement distinguishes it
from the others, and every other chip carries `fd`, neither flags nor
a distinguishing statement. Flags piling onto one factor while
nothing separates the factors is the signature of a single shared
viewpoint, and the verdict says so. The grizzly bear panel, the
package's second shipped dataset, resolves the other way, the rule
selects two viewpoints there, with one factor holding both sides of
the reintroduction question. The
classical analysis of the obesity panel reports three factors, and
the validation article shows how the two accounts relate, factor by
factor.

## The one-factor fit

```{r fit}
fit1 <- fit_bayesian(obesity_sorts, K = 1, seed = 11)
fit1
```

Who carries the shared viewpoint, and how strongly:

```{r loadings, fig.height = 7}
plot_loading_posterior(fit1)
```

With one factor there are no pairs to distinguish, so the claims are
the flags:

```{r claims}
claims(fit1)
```

```{r fchar}
factor_characteristics(fit1)
```

## The shared sort

The reported array is the viewpoint written as a completed sort, and
with most of the panel behind it, the placements are firm.

```{r array, fig.height = 6}
plot_factor_array(fit1)
```

A write-up quotes the poles. `crib_sheet()` gives the probability of
each statement landing in the extreme columns and of being the single
most or least agreed statement:

```{r crib}
cs <- crib_sheet(fit1)
head(cs[order(-cs$p_top), ], 3)
head(cs[order(-cs$p_bottom), ], 3)
```

## The checks

Does one factor account for the panel, and does it span the people?

```{r checks, fig.height = 5.5}
check_fit(fit1, draws = 40)
pc <- check_persons(fit1)
table(pc$verdict)
plot_person_check(pc)
```

One factor sits comfortably inside every check, and the person check
spans the whole panel. The single viewpoint is not a compromise the
refusal forced, it is the model the data support.

## What you report

The study's findings are the shared array, the pole shortlists, and
the loadings with their flags, each carrying its uncertainty. The
write-up also owes the reader the ladder, because the single
viewpoint is a decision the checks made, not an assumption. The
classical account of this panel reports three factors, and the two
readings are not in contradiction so much as at different standards
of evidence. The correspondence between them, where it is tight and
where it dissolves, is worked through in the validation article.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot_factor_array}
\alias{plot_factor_array}
\title{The factor array on its grid}
\usage{
plot_factor_array(fit, factor = 1, labels = NULL)
}
\arguments{
\item{fit}{A \code{bayesqm_fit}.}

\item{factor}{Which factor to draw (index or \code{"f2"}-style name;
default 1).}

\item{labels}{Optional statement labels (defaults to the statement
ids).}
}
\value{
\code{fit}, invisibly.
}
\description{
The reported array for one factor drawn as the physical sorting grid:
columns are grid positions with their quota heights, each cell carries
its statement, and cell shading shows how certain the placement is
(the posterior probability of that statement landing in that column).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ladder.R
\name{select_k}
\alias{select_k}
\title{Choose K by the two-signal rule}
\usage{
select_k(ladder, q = NULL, band = c(0.05, 0.95))
}
\arguments{
\item{ladder}{A \code{bayesqm_ladder}.}

\item{q}{Re-select the support evidence at this level; \code{NULL} (the
default) keeps the level the ladder stored.}

\item{band}{Central adequacy band for the extra-factor percentile
(default \code{c(0.05, 0.95)}).}
}
\value{
A \code{bayesqm_selection} with the verdict, the per-rung evidence
table, and the selected \code{K} (or \code{NA}).
}
\description{
Signal one is posterior-predictive adequacy: the extra-factor check
sits inside the central band and the person check shows no mutual
unspanned cluster. Signal two is parsimony: every factor is supported,
meaning at least two selected flags and at least one selected
distinguishing listing. The selection is the smallest adequate K with
all factors supported. When no factor is supported at any rung the
verdict separates two opposite situations: a panel sharing nothing (no
factor ever attracts two flags) and a panel so unanimous that flags
abound but no statement distinguishes any pair — a single shared
viewpoint, reported as such. When adequacy and support never coincide,
both candidate solutions are reported rather than forcing a winner.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R
\name{prior_summary.bayesqm_fit}
\alias{prior_summary.bayesqm_fit}
\title{Prior summary for a bayesqm_fit}
\usage{
\method{prior_summary}{bayesqm_fit}(object, ...)
}
\arguments{
\item{object}{A \code{bayesqm_fit}.}

\item{...}{Unused.}
}
\value{
A \code{bayesqm_prior} data frame with columns \code{parameter} and
\code{prior}.
}
\description{
The partition model's priors, as a printable \code{bayesqm_prior} object.
Method for \code{\link[=prior_summary]{prior_summary()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_zscores.R
\name{plot_statement}
\alias{plot_statement}
\alias{plot_zscore_posterior}
\title{One statement, in depth}
\usage{
plot_statement(fit, statement, q = 0.05, cons_floor = 0.95)

plot_zscore_posterior(...)
}
\arguments{
\item{fit}{A \code{bayesqm_fit}.}

\item{statement}{Statement id or index.}

\item{q, cons_floor}{As in \code{\link[=compute_qdc]{compute_qdc()}}.}

\item{...}{Passed on.}
}
\value{
\code{fit}, invisibly.
}
\description{
The posterior of a single statement's score under each factor, drawn
as full densities with the medians marked, plus the statement's
verdict and its pairwise contrasts in the margin text. The drill-down
for the statement a write-up centers on.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ladder.R
\name{fit_ladder}
\alias{fit_ladder}
\title{Fit the model over a ladder of K}
\usage{
fit_ladder(
  Y,
  K_max = NULL,
  K_min = 2,
  q = 0.05,
  screen_draws = 30,
  screen_mixes = 60,
  quiet = FALSE,
  ...
)
}
\arguments{
\item{Y}{As in \code{\link[=fit_bayesian]{fit_bayesian()}}.}

\item{K_max}{Largest K to fit. \code{NULL} (default) uses
\code{min(6, floor(N / 3))}, capped at 3 when \code{N <= 12} — with a dozen
sorts or fewer, the data cannot formally discriminate adjacent K.}

\item{K_min}{Smallest K to fit (default 2).}

\item{q}{False-discovery level for the support evidence stored on
each rung (default 0.05); \code{\link[=select_k]{select_k()}} can re-select at another q.}

\item{screen_draws, screen_mixes}{Budget for the per-rung person
check (defaults 30 and 60; the cluster signal needs no more).}

\item{quiet}{Suppress per-rung messages (default \code{FALSE}).}

\item{...}{Passed to \code{\link[=fit_bayesian]{fit_bayesian()}} (iterations, seed, ...).}
}
\value{
A \code{bayesqm_ladder}: the per-rung fits and evidence.
}
\description{
Fits \code{\link[=fit_bayesian]{fit_bayesian()}} at each K in the ladder and stores, per rung, the
fit together with the adequacy and support evidence that
\code{\link[=select_k]{select_k()}} consumes. Expect roughly one full fit's runtime per rung;
a message up front says how many rungs are coming.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/claims.R
\name{claims}
\alias{claims}
\title{Selected claims at a common false-discovery level}
\usage{
claims(fit, q = 0.05, flag_floor = 0.5, cons_floor = 0.95)
}
\arguments{
\item{fit}{A \code{bayesqm_fit}.}

\item{q}{Posterior expected false-discovery bound (default 0.05).}

\item{flag_floor, cons_floor}{Selection floors for flags (0.5) and
consensus statements (0.95).}
}
\value{
A \code{bayesqm_claims} object: selected-only tables \code{flags},
\code{distinguishing}, \code{consensus}, and \code{stars}, each with its
\code{expected_false} count, plus the level \code{q}.
}
\description{
Selects every claim — participant flags, distinguishing listings,
consensus statements, and pairwise stars — by the one posterior
false-discovery rule at level \code{q}, and reports the expected number of
false claims per family alongside. The same code path produces the
\code{selected} columns of \code{\link[=compute_flags]{compute_flags()}} and \code{\link[=compute_qdc]{compute_qdc()}}, so the
tables and this object always agree.
}
\examples{
claims(demo_fit())

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/caption.R, R/plots.R
\name{plot_membership}
\alias{plot_membership}
\alias{plot_flags}
\title{Flag probabilities with the unclassified state}
\usage{
plot_membership(...)

plot_flags(fit, q = 0.05)
}
\arguments{
\item{...}{Passed on.}

\item{fit}{A \code{bayesqm_fit}.}

\item{q}{False-discovery level for the selection marks (default 0.05).}
}
\value{
\code{fit}, invisibly.
}
\description{
One row per participant, one dot per factor at its posterior flag
probability (both poles combined; a minus sign marks a dominant
negative pole), and a grey square for the probability of remaining
unclassified. Filled dots are selected flags, and the dotted rule is
the 0.5 selection floor.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/align.R
\name{matchalign}
\alias{matchalign}
\title{MatchAlign post-processing for partition-model draws}
\usage{
matchalign(fit, pivot = NULL)
}
\arguments{
\item{fit}{A fit carrying draw arrays \code{F} (\verb{[T, J, K]}), \code{Lambda}
(\verb{[T, N, K]}), \code{sigma} (\verb{[T, K]}), and \code{K}.}

\item{pivot}{Optional draw index to use as the alignment pivot. \code{NULL}
(the default) selects the draw whose loading condition number sits at
the median.}
}
\value{
The fit with aligned \code{F}, \code{Lambda}, and \code{sigma}, and the chosen
\code{pivot} index appended.
}
\description{
Resolves rotational, sign, and label-permutation ambiguity in posterior
draws by the MatchAlign procedure of Poworoznek et al. (2025), adapted to
the partition model: varimax rotation of the loadings per draw, a pivot
draw at the median condition number (overridable), pivot polarity fixed
so each factor's loadings skew positive (defining sorts load positively),
greedy signed matching of loading columns to the pivot, and a
Procrustes rotation. A final pass re-orients every draw to the varimax
of the aligned mean loadings, re-signed to positive skew, so the
delivered orientation does not inherit one draw's sampling noise. One
rotation is applied to loadings and scores alike, and the loading scale
\code{sigma} is carried through.

Called for its side effect on the draw arrays of an engine-level fit;
users normally receive already-aligned draws from \code{\link[=fit_bayesian]{fit_bayesian()}} and
need this only to realign under a different pivot.
}
\references{
Poworoznek, E., Anceschi, N., Ferrari, F., & Dunson, D. (2025).
Efficiently Resolving Rotational Ambiguity in Bayesian Matrix
Sampling with Matching. \emph{Bayesian Analysis}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ghk.R
\name{loglik_person}
\alias{loglik_person}
\title{Person-level partition log-likelihoods}
\usage{
loglik_person(fit, draws = 100, R = 64, seed = 1)
}
\arguments{
\item{fit}{A \code{bayesqm_fit}.}

\item{draws}{Posterior draws to evaluate (default 100, thinned evenly).}

\item{R}{GHK replications per evaluation (default 64).}

\item{seed}{Base seed for the GHK draws (default 1).}
}
\value{
A \verb{draws x N} matrix of log-likelihoods.
}
\description{
GHK-simulated log-likelihood of each participant's sort under each
kept posterior draw (thinned to \code{draws}), the input PSIS-LOO needs.
The GHK seed depends only on the draw index and the person, never on
\code{K}, so log-likelihoods are common-random-number comparable across a
ladder of fits.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot_ppc}
\alias{plot_ppc}
\title{Posterior-predictive check display}
\usage{
plot_ppc(x)
}
\arguments{
\item{x}{A \code{bayesqm_checks} from \code{\link[=check_fit]{check_fit()}}.}
}
\value{
\code{x}, invisibly.
}
\description{
The replicated distributions behind \code{\link[=check_fit]{check_fit()}}: the agreement RMSE
against its double-replicate reference, and the observed extra-factor
eigenvalue in its replicated distribution.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R
\name{sigma.bayesqm_fit}
\alias{sigma.bayesqm_fit}
\title{Posterior-mean loading scales}
\usage{
\method{sigma}{bayesqm_fit}(object, ...)
}
\arguments{
\item{object}{A \code{bayesqm_fit}.}

\item{...}{Unused.}
}
\value{
Named numeric vector of posterior-mean \code{sigma_k}.
}
\description{
Posterior-mean loading scales
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/demo_fit.R
\name{demo_fit}
\alias{demo_fit}
\title{A small demonstration fit}
\usage{
demo_fit(N = 8, J = 13, K = 2, draws = 200, seed = 1)
}
\arguments{
\item{N, J, K}{Panel size, statement count, factors (defaults 8, 13, 2).}

\item{draws}{Kept posterior draws (default 200).}

\item{seed}{Random seed (default 1).}
}
\value{
A \code{bayesqm_fit}.
}
\description{
Runs the partition-model sampler on a small synthetic forced-sort
dataset with a fixed seed and the convergence gate disabled. It exists
so examples and tests have a complete \code{bayesqm_fit} in about a second;
it is no substitute for \code{\link[=fit_bayesian]{fit_bayesian()}} at its defaults on real data.
}
\examples{
fit <- demo_fit()
fit

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checks.R
\name{check_fit}
\alias{check_fit}
\title{Posterior-predictive checks of the fitted model}
\usage{
check_fit(fit, draws = 100)
}
\arguments{
\item{fit}{A \code{bayesqm_fit}.}

\item{draws}{Posterior draws used for replication (default 100).}
}
\value{
A \code{bayesqm_checks} list: \code{agreement} (\code{p}), \code{extra_factor}
(\code{percentile}), and \code{paired} (\code{p}, per-statement \code{p_j}), with a print
method.
}
\description{
Three checks, each comparing the observed sorts with replicates drawn
from the fitted model:
the agreement check (T1a) compares the observed person-agreement matrix
with replicated ones against a double-replicate reference; the
extra-factor check (T1b) locates the observed \code{(K+1)}-th eigenvalue in
its replicated distribution; and the paired-comparison check (T2)
compares standardized statement-pair margins, ties counted one half.
These are diagnostics to read, not tests to pass.
}
\examples{
check_fit(demo_fit(), draws = 20)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/caption.R
\name{caption_bayesqm}
\alias{caption_bayesqm}
\title{Caption text for figures from a fit}
\usage{
caption_bayesqm(fit, include_gate = TRUE)
}
\arguments{
\item{fit}{A \code{bayesqm_fit}.}

\item{include_gate}{Append the gate report (default \code{TRUE}).}
}
\value{
A length-one character string.
}
\description{
A single string naming the model, the panel, the draws, and the gate
verdict — the provenance a figure caption should carry.
}
\examples{
caption_bayesqm(demo_fit())

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaries.R
\name{factor_characteristics}
\alias{factor_characteristics}
\title{Factor characteristics}
\usage{
factor_characteristics(fit, prob = 0.9, q = 0.05, floor = 0.5)
}
\arguments{
\item{fit}{A \code{bayesqm_fit}.}

\item{prob}{Credible-interval probability for the defining-sort counts
(default 0.90).}

\item{q, floor}{The flag rule fixing the flagged set, as in
\code{\link[=compute_flags]{compute_flags()}}.}
}
\value{
A data frame with one row per factor: \code{factor}, \code{flagged}
(selected flags), \code{defining_modal}, \code{defining_mean},
\code{defining_lower}, \code{defining_upper}, \code{score_spread}, and
\code{reliability}. Attribute \code{score_correlations} holds the posterior
mean K x K correlation matrix of the statement scores.
}
\description{
The per-factor block of the accompanying paper: modal and mean number
of defining sorts per posterior draw with a credible interval, the
selected flag count, the mean posterior spread of the statement
scores, and the mean replicate reliability
\code{R_i = s_i^2 / (1 + s_i^2)} of the flagged participants. The
statement-score correlations between factors ride along as an
attribute.
}
\examples{
factor_characteristics(demo_fit())

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{obesity_sorts}
\alias{obesity_sorts}
\title{Childhood obesity Q sorts}
\format{
A \link{qsort_data} object: the 42 x 33 integer matrix of printed
grid values with statement and participant ids, and the forced
distribution.
}
\source{
Akhtar-Danesh, N. (2023). Impact of factor rotation on
Q-methodology analysis. \emph{PLOS ONE}, 18(9), e0290728.
\doi{10.1371/journal.pone.0290728}
}
\usage{
obesity_sorts
}
\description{
The childhood obesity Q dataset from Akhtar-Danesh (2023): 33
participants each force-sorted 42 statements about childhood
obesity onto a nine-column grid (quotas 2-4-5-6-8-6-5-4-2, printed
values -4 to +4). A real, published panel at typical Q scale, used
in the package documentation.
}
\examples{
obesity_sorts
plot(obesity_sorts, participants = 1:4)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bayesqm-package.R
\docType{package}
\name{bayesqm-package}
\alias{bayesqm}
\alias{bayesqm-package}
\title{bayesqm: Bayesian Q Methodology: Exact Rank-Order Likelihood for Forced Q Sorts}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

A Bayesian analysis for Q methodology, alongside the classical one. Models the forced Q sort as an ordered partition of the statements through an exact rank-order likelihood (the design quotas fix the partition margins, so the likelihood of the observed sorting event is exact), fits it by a parameter-expanded Gibbs sampler in R with no compiled code and a convergence gate on rotation-invariant functionals, resolves rotational ambiguity via the MatchAlign post-processing of Poworoznek et al. (2025) \doi{10.1214/25-BA1544}, and returns the familiar Q tables as posterior summaries: credible intervals for bounded participant loadings, flag probabilities with an explicit unclassified state, quota-respecting factor arrays, distinguishing and consensus statements judged against a posterior critical difference and a grid-width equivalence region, one posterior false-discovery rule for all published claims, and a two-signal posterior-predictive workflow for the number of factors.

A Bayesian analysis for Q methodology, alongside the classical one. The
forced Q sort is modeled as an ordered partition of the statements
through an exact rank-order likelihood: the design quotas fix the
partition margins, so the probability of the observed sorting event is
the full observed-data likelihood. The posterior is sampled by a
parameter-expanded Gibbs sampler in R with no compiled code, gated on rotation-invariant
convergence diagnostics, aligned by MatchAlign, and returned as the
familiar Q tables with uncertainty attached.
}
\details{
The typical workflow:

\enumerate{
\item \strong{Import data.} \code{\link[=read_qsort]{read_qsort()}} auto-detects CSV, Excel,
PQMethod \code{.DAT}, Ken-Q JSON / multi-sheet Excel, KADE ZIP, or
Easy-HTMLQ Firebase JSON. \code{\link[=qsort_data]{qsort_data()}} constructs the object
directly from a matrix. The exact likelihood needs forced sorts:
every participant must match the design grid.
\item \strong{Fit.} \code{\link[=fit_bayesian]{fit_bayesian()}} returns a \code{bayesqm_fit}; \code{\link[=extend]{extend()}}
warm-continues a chain the convergence gate flagged.
\item \strong{Read the tables.} \code{\link[=compute_loadings]{compute_loadings()}} (bounded loadings
with credible intervals), \code{\link[=compute_flags]{compute_flags()}} (flag probabilities
with an unclassified state), \code{\link[=compute_zscores]{compute_zscores()}},
\code{\link[=compute_factor_array]{compute_factor_array()}} (quota-exact arrays),
\code{\link[=compute_qdc]{compute_qdc()}} (distinguishing / consensus / indeterminate),
\code{\link[=crib_sheet]{crib_sheet()}}, and \code{\link[=claims]{claims()}} — one posterior false-discovery
rule selecting every claim.
\item \strong{Check the model.} \code{\link[=check_fit]{check_fit()}} (agreement, extra-factor,
and paired-comparison checks) and \code{\link[=check_persons]{check_persons()}} (the person check,
against mixed-replication bands).
\item \strong{Choose K.} \code{\link[=fit_ladder]{fit_ladder()}} and \code{\link[=select_k]{select_k()}} run the
two-signal workflow; \code{\link[=loo_ladder]{loo_ladder()}} adds directional
corroboration.
\item \strong{Report.} The \verb{plot_*} views, \code{\link[=rotate_factors]{rotate_factors()}} for
judgmental rotation, \code{\link[=rename_factors]{rename_factors()}}, and the standard R
accessors (\code{coef()}, \code{fitted()}, \code{as_draws_df()},
\code{\link[=posterior_interval]{posterior_interval()}}, \code{\link[=prior_summary]{prior_summary()}}).
}
}
\section{Relationship to classical Q analysis}{

The questions and the reporting conventions are Q's own; what the
Bayesian account adds is a probability behind each familiar verdict.
Vocabulary carries over: flags, factor arrays, defining sorts,
distinguishing and consensus statements. Two deliberate differences in
the tables:
\itemize{
\item Factor characteristics do not print eigenvalues or explained
variance, which have no counterpart in a generative model; report
the defining-sort counts from \code{\link[=claims]{claims()}} and the extra-factor check
of \code{\link[=check_fit]{check_fit()}} in their place.
\item Consensus is a positive finding (an equivalence region of one grid
column, \code{\link[=delta_grid]{delta_grid()}}), not the complement of distinguishing, so a
statement can be distinguishing, consensus, or neither.
}
}

\references{
Poworoznek, E., Anceschi, N., Ferrari, F., & Dunson, D. (2025).
Efficiently Resolving Rotational Ambiguity in Bayesian Matrix
Sampling with Matching. \emph{Bayesian Analysis}.

Vehtari, A., Gelman, A., Simpson, D., Carpenter, B., & Bürkner, P.-C.
(2021). Rank-Normalization, Folding, and Localization: An Improved
R-hat for Assessing Convergence of MCMC. \emph{Bayesian Analysis}, 16(2),
667-718.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/rdazadda/bayesqm}
  \item \url{https://rdazadda.github.io/bayesqm/}
  \item Report bugs at \url{https://github.com/rdazadda/bayesqm/issues}
}

}
\author{
\strong{Maintainer}: Raymond Dacosta Azadda \email{rdazadda@alaska.edu} (\href{https://orcid.org/0009-0002-4384-4556}{ORCID})

Authors:
\itemize{
  \item Raymond Dacosta Azadda \email{rdazadda@alaska.edu} (\href{https://orcid.org/0009-0002-4384-4556}{ORCID})
  \item Henry Ofoe Agbi-Kaiser (\href{https://orcid.org/0009-0008-6127-9136}{ORCID})
  \item Hannah D. Robinson (\href{https://orcid.org/0009-0000-5159-1363}{ORCID})
  \item  AK-ACE Team
  \item Karsten Hueffer
  \item Taa'aii Peter
  \item Stacy Rasmus
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaries.R
\name{compute_factor_array}
\alias{compute_factor_array}
\title{Quota-respecting factor arrays}
\usage{
compute_factor_array(fit, negative = FALSE)
}
\arguments{
\item{fit}{A \code{bayesqm_fit}.}

\item{negative}{Also return the negative-pole arrays (default \code{FALSE});
on a symmetric grid the mirror is exact.}
}
\value{
A data frame with one row per statement: \code{statement}, then
\verb{f\{k\}_grid} per factor (and \verb{f\{k\}_grid_neg} when \code{negative = TRUE}).
\code{attr(, "footrule_disagreement")} gives the share of cells where the
footrule assignment differs (\code{NA} without \pkg{clue}).
}
\description{
The reported array for each factor: statements sorted onto the design
grid by their posterior column probabilities (mean grid column,
tie-broken by posterior-mean score), which is quota-exact by
construction. When the \pkg{clue} package is installed, a footrule
assignment cross-check is computed and its disagreement rate attached.
}
\examples{
compute_factor_array(demo_fit())

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R
\name{prior_summary}
\alias{prior_summary}
\title{Prior summary generic}
\usage{
prior_summary(object, ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{...}{Passed to methods.}
}
\value{
See the method documentation.
}
\description{
Generic for summarizing the priors a model was fit with, defined here so
the package needs no Stan-ecosystem dependency. Compatible with the
generic of the same name in \pkg{rstantools}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ladder.R
\name{loo_ladder}
\alias{loo_ladder}
\title{PSIS-LOO across the ladder, as directional corroboration}
\usage{
loo_ladder(ladder, draws = 100, R = 64)
}
\arguments{
\item{ladder}{A \code{bayesqm_ladder}.}

\item{draws, R}{As in \code{\link[=loglik_person]{loglik_person()}}.}
}
\value{
A data frame: \code{K}, \code{elpd_loo}, \code{se}, \code{max_pareto_k}.
}
\description{
Person-level GHK log-likelihoods per rung, fed to PSIS-LOO. Reported as
directional corroboration only: with typical Q panels (N well below
100), differences in expected log predictive density between adjacent K
have unreliable standard errors. The GHK seeds are common across rungs,
so comparisons are common-random-number paired.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_sorts.R
\name{plot_sorts}
\alias{plot_sorts}
\title{Preview every participant's sort}
\usage{
plot_sorts(x, participants = NULL, per_page = 12, labels = NULL, cex = NULL)
}
\arguments{
\item{x}{A \code{qsort_data} object or a \verb{J x N} matrix of sorts.}

\item{participants}{Which participants to draw (names or indices);
\code{NULL} (default) draws everyone.}

\item{per_page}{How many pyramids per page (default up to 12,
arranged automatically); further participants continue on the next
page.}

\item{labels}{Optional statement labels for the tiles (defaults to the
statement ids).}

\item{cex}{Tile label size; \code{NULL} (default) adapts to the tallest
column.}
}
\value{
The input, invisibly.
}
\description{
Draws each participant's completed sort as their own pyramid: one tile
per statement, stacked in its grid column, colored along the
disagree-agree axis. This is the look-at-the-decisions-first view:
run it straight after import, before any analysis, to see how every
participant grouped the statements. \code{plot(qdata)} does the same.
}
\examples{
distr <- c(1, 2, 3, 2, 1)
set.seed(1)
Y <- replicate(4, {
  r <- integer(9)
  r[order(rnorm(9))] <- rep(seq_along(distr), distr)
  r - 3                                  # printed labels -2..+2
})
plot_sorts(qsort_data(Y, distribution = distr, validate = FALSE))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_bayesian.R
\name{fit_bayesian}
\alias{fit_bayesian}
\title{Fit the exact partition-likelihood model to forced Q sorts}
\usage{
fit_bayesian(
  Y,
  K,
  iterations = 12000,
  burn = 2000,
  thin = 5,
  max_iterations = 48000,
  seed = NULL,
  sigma_scale = 1,
  rhat_max = 1.01,
  ess_min = 400,
  prob = 0.95,
  keep_raw = TRUE,
  pivot = NULL,
  quiet = FALSE,
  ...
)
}
\arguments{
\item{Y}{A \code{qsort_data} object, or a \verb{J x N} numeric matrix of grid
positions with statements as rows and participants as columns. Every
sort must obey the forced distribution exactly.}

\item{K}{Integer number of factors.}

\item{iterations}{First-check chain length (default 12000, the settings
frozen in the accompanying paper).}

\item{burn}{Burn-in iterations (default 2000), paid once; extensions
continue the chain.}

\item{thin}{Keep every \code{thin}-th post-burn draw (default 5).}

\item{max_iterations}{Total-iteration cap for the gated extension ladder
(default 48000).}

\item{seed}{Optional integer seed; fits are exactly reproducible given
the seed.}

\item{sigma_scale}{Half-normal prior scale for the per-factor loading
scales (default 1).}

\item{rhat_max, ess_min}{Gate thresholds (defaults 1.01 and 400).}

\item{prob}{Credible-interval probability stored on the fit
(default 0.95).}

\item{keep_raw}{Keep the pre-alignment draws on the fit (default
\code{TRUE}); required by \code{\link[=extend]{extend()}} and by realignment under a different
pivot.}

\item{pivot}{Optional draw index for the alignment pivot; \code{NULL} (the
default) uses the median-condition-number rule.}

\item{quiet}{Suppress progress messages (default \code{FALSE}).}

\item{...}{Unused; supplying a removed 0.1.0 argument gives a migration
error.}
}
\value{
A \code{bayesqm_fit} carrying aligned draws, the gate report, the
alignment record, and (when \code{keep_raw = TRUE}) the raw draws and
sampler state.
}
\description{
Models each completed sort as a forced ordered partition of the
statements and samples the posterior of the latent factor model by a
parameter-expanded Gibbs sampler. Convergence is gated on
rotation-invariant person spreads (rank-normalized split-R-hat and
bulk/tail effective sample size); a chain failing the gate is
warm-extended at successive doublings up to \code{max_iterations}. Draws are
aligned by MatchAlign with a polarity canon, so defining sorts load
positively.
}
\examples{
fit <- demo_fit()
fit

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaries.R
\name{compute_flags}
\alias{compute_flags}
\title{Flag probabilities with an explicit unclassified state}
\usage{
compute_flags(fit, q = 0.05, floor = 0.5)
}
\arguments{
\item{fit}{A \code{bayesqm_fit}.}

\item{q}{Posterior expected false-discovery bound (default 0.05).}

\item{floor}{Minimum flag probability a candidate needs before it can
be selected (default 0.5, so at most one signed flag per participant).}
}
\value{
A data frame with one row per participant: modal signed
candidate (\code{factor}, \code{sign}), its probability \code{flag_prob},
\code{unclassified_prob}, and \code{selected}. The full probability matrix is
attached as \code{attr(, "phi")} and the expected number of false
selected flags as \code{attr(, "expected_false")}.
}
\description{
For each participant, the posterior probability that the classical flag
rule fires on each signed factor, with the probability of remaining
unclassified alongside. Selected flags come from all signed
candidates by the posterior false-discovery rule at level \code{q}.
}
\examples{
compute_flags(demo_fit())

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaries.R
\name{compute_qdc}
\alias{compute_qdc}
\title{Distinguishing and consensus verdicts}
\usage{
compute_qdc(fit, q = 0.05, cons_floor = 0.95, prob = NULL)
}
\arguments{
\item{fit}{A \code{bayesqm_fit} with \code{K >= 2}.}

\item{q}{Posterior expected false-discovery bound (default 0.05).}

\item{cons_floor}{Minimum consensus probability before a statement can
be selected as consensus (default 0.95).}

\item{prob}{Credible-interval probability for the contrast intervals;
defaults to the fit's. The critical difference itself keeps its
z_0.975 definition regardless.}
}
\value{
A data frame with one row per statement: per-factor
distinguishing probabilities, \code{consensus_prob}, and the \code{verdict}.
Attributes: \code{delta_kl} and \code{delta_kl99} (per pair), \code{delta_grid},
\code{contrasts} (the per-pair contrast table with intervals, the
probability \code{diff_column_prob} that the two viewpoints place the
statement in different grid columns, and \code{stars}, \code{*} for selected
claims and \verb{**} for those that also clear the \code{z_0.995} critical
difference at probability .99), and \code{expected_false} per family.
}
\description{
The statement verdict table. Distinguishing-for-a-factor is the joint
event that every contrast touching that factor exceeds the posterior
critical difference \code{delta_kl} (computed from the posterior spread of
the score contrasts); consensus is the equivalence event that all
scores sit within one grid column (\code{\link[=delta_grid]{delta_grid()}}) of each other. Both
families are selected through the posterior false-discovery rule, and a
statement can be distinguishing, consensus, or indeterminate.
}
\examples{
compute_qdc(demo_fit())

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaries.R
\name{compute_loadings}
\alias{compute_loadings}
\title{Bounded participant loadings with credible intervals}
\usage{
compute_loadings(fit, prob = NULL)
}
\arguments{
\item{fit}{A \code{bayesqm_fit}.}

\item{prob}{Credible-interval probability; defaults to the fit's.}
}
\value{
A data frame with one row per participant: \code{participant}, then
\verb{f\{k\}_loading}, \verb{f\{k\}_lower}, \verb{f\{k\}_upper} per factor, then \code{spread}
(posterior-mean \code{s_i}).
}
\description{
The correlation-scale loading of the accompanying paper,
\verb{rho = (S lambda)_k / sqrt(s^2 + 1)}, summarized per participant and
factor, with the posterior-mean person spread \code{s_i} alongside.
}
\examples{
compute_loadings(demo_fit())

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_bayesian.R
\name{extend}
\alias{extend}
\title{Continue sampling a fitted chain}
\usage{
extend(fit, iterations = NULL, pivot = NULL, quiet = FALSE)
}
\arguments{
\item{fit}{A \code{bayesqm_fit} created with \code{keep_raw = TRUE}.}

\item{iterations}{Additional iterations; \code{NULL} (the default) doubles
the chain. Must be a multiple of the fit's \code{thin}.}

\item{pivot, quiet}{As in \code{\link[=fit_bayesian]{fit_bayesian()}}.}
}
\value{
The extended \code{bayesqm_fit}.
}
\description{
Warm-extends the fitted chain by further iterations and re-runs the
convergence gate and the alignment on the grown draw set. The extension
restores the sampler's saved random-number state, so the result is
draw-for-draw identical to having run one longer chain, whatever
happened in the session in between.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/save_plot.R
\name{save_bayesqm_plot}
\alias{save_bayesqm_plot}
\title{Save a bayesqm plot to file}
\usage{
save_bayesqm_plot(file, expr, width = 7.2, height = 5, dpi = 300)
}
\arguments{
\item{file}{Output path. Extension determines the device.}

\item{expr}{Plotting expression (lazily evaluated).}

\item{width, height}{Dimensions in inches. Defaults to a 7.2 x 5
inch two-column journal figure.}

\item{dpi}{Resolution for raster formats (\code{png}, \code{tiff}, \code{jpeg}).}
}
\value{
The \code{file} path, invisibly.
}
\description{
Opens a graphics device chosen from the file extension (\code{.pdf},
\code{.svg}, \code{.png}, \code{.tiff}, \code{.jpeg}), evaluates \code{expr} so whatever it
draws lands on that device, and closes the device. \code{expr} is lazily
evaluated, so a call like \code{save_bayesqm_plot("fig.pdf", plot(fit))}
does not draw to the current screen device first. If \code{expr} returns
a \code{ggplot} object (for example, from \code{\link[ggplot2:autoplot]{ggplot2::autoplot()}}), it is
\code{print()}ed onto the device.
}
\examples{
f <- file.path(tempdir(), "fig.pdf")
save_bayesqm_plot(f, plot(1:10))
unlink(f)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/caption.R, R/plots.R
\name{plot_dist_cons}
\alias{plot_dist_cons}
\alias{plot_contrasts}
\title{Statement contrasts between two factors}
\usage{
plot_dist_cons(...)

plot_contrasts(fit, pair = 1, q = 0.05, cons_floor = 0.95)
}
\arguments{
\item{...}{Passed on.}

\item{fit}{A \code{bayesqm_fit} with \code{K >= 2}.}

\item{pair}{Which factor pair to draw (index into the pair list or
\code{"f1-f2"}-style name; default 1).}

\item{q, cons_floor}{As in \code{\link[=compute_qdc]{compute_qdc()}}.}
}
\value{
\code{fit}, invisibly.
}
\description{
Per statement, the posterior contrast interval for one factor pair,
the posterior critical difference \code{delta_kl} (dashed), and the
grid-width consensus region (\code{\link[=delta_grid]{delta_grid()}}, shaded). Verdict
colours: selected distinguishing listings, consensus statements,
indeterminate.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot_loading_posterior}
\alias{plot_loading_posterior}
\alias{plot.bayesqm_fit}
\title{Bounded loadings with credible intervals}
\usage{
plot_loading_posterior(fit, q = 0.05)

\method{plot}{bayesqm_fit}(x, ...)
}
\arguments{
\item{fit}{A \code{bayesqm_fit}.}

\item{q}{False-discovery level for the flag marks (default 0.05).}

\item{x, ...}{A \code{bayesqm_fit} and arguments passed on.}
}
\value{
\code{fit}, invisibly.
}
\description{
One row per participant, one interval per factor on the bounded
loading scale, with selected flags marked. The dotted rules sit at
the classical descriptive cut-off \code{1.96 / sqrt(J)} (reference).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{tucker_congruence}
\alias{tucker_congruence}
\alias{procrustes_rotation}
\title{Tucker's congruence and orthogonal Procrustes rotation}
\usage{
tucker_congruence(x, y)

procrustes_rotation(X, Target)
}
\arguments{
\item{x, y}{Numeric vectors (for Tucker).}

\item{X, Target}{Matrices (for Procrustes).}
}
\value{
Tucker returns a scalar; Procrustes returns the rotated
matrix with a \code{"rotation"} attribute.
}
\description{
Linear-algebra utilities shared by MatchAlign and the recovery
assessments. \code{tucker_congruence(x, y)} computes \code{sum(x*y) / sqrt(sum(x^2) * sum(y^2))}. \code{procrustes_rotation(X, Target)} finds
the orthogonal \code{R} minimising \verb{||X R - Target||_F} and returns
\code{X \%*\% R} with \code{R} attached as attribute \code{"rotation"}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{grizzly_sorts}
\alias{grizzly_sorts}
\title{Grizzly bear reintroduction Q sorts}
\format{
A \link{qsort_data} object: the 41 x 67 integer matrix of printed
grid values with statement and participant ids, and the forced
distribution.
}
\source{
Easter, T. S., Santo, A. R., Sage, A. H., Carter, N. H.,
Chan, K. M. A., & Ransom, J. I. (2025). Divergent values and
perspectives drive three distinct viewpoints on grizzly bear
reintroduction in Washington, the United States. \emph{People and
Nature}, 7, 127--145. \doi{10.1002/pan3.10748}. Data from the
Dryad repository under CC0, \doi{10.5061/dryad.73n5tb369}.
}
\usage{
grizzly_sorts
}
\description{
The grizzly bear reintroduction Q dataset from Easter et al. (2025):
67 participants each force-sorted 41 statements about reintroducing
grizzly bears to the North Cascades onto an eleven-column grid
(quotas 1-2-3-5-6-7-6-5-3-2-1, printed values -5 to +5). A real,
published panel where the two-signal rule selects a two-factor
solution, used in the package documentation.
}
\examples{
grizzly_sorts
plot(grizzly_sorts, participants = 1:4)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gibbs.R
\name{delta_grid}
\alias{delta_grid}
\title{Grid width on the z scale}
\usage{
delta_grid(distribution)
}
\arguments{
\item{distribution}{Integer vector of forced-distribution counts (the
number of statements allowed in each grid column).}
}
\value{
A single numeric value.
}
\description{
The width of one grid column on the standardized score scale,
\verb{1 / sd(forced positions)} with the population standard deviation. This
is the equivalence-region half-width used for consensus verdicts: a
score contrast smaller than one grid column is not expressible in a
completed sort.
}
\examples{
delta_grid(c(2, 3, 4, 5, 7, 7, 5, 4, 3, 2))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rotation.R
\name{rotate_factors}
\alias{rotate_factors}
\title{Rotate every aligned draw toward a target}
\usage{
rotate_factors(fit, target, oblique = FALSE)
}
\arguments{
\item{fit}{A \code{bayesqm_fit}.}

\item{target}{A \verb{J x K} numeric matrix of target scores (for example, a
hand-edited copy of the posterior-mean scores).}

\item{oblique}{Allow an oblique (non-orthogonal) rotation
(default \code{FALSE}).}
}
\value{
The fit with rotated draws; \code{fit$align$rotated} records the
call.
}
\description{
Judgmental rotation: each aligned draw's statement scores are rotated
toward an analyst-specified target by Procrustes, and the loadings
receive the same rotation (its inverse transpose when \code{oblique = TRUE}),
so the latent utilities every draw implies are unchanged. Summaries
computed from the returned fit are summaries of the rotated solution,
with full posterior uncertainty.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R
\name{posterior_interval}
\alias{posterior_interval}
\title{Posterior interval generic}
\usage{
posterior_interval(object, ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{...}{Passed to methods.}
}
\value{
See the method documentation.
}
\description{
Generic for posterior credible intervals, defined here so the package
needs no Stan-ecosystem dependency. Compatible with the generic of the
same name in \pkg{rstantools}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{import-aliases}
\alias{import-aliases}
\alias{import.pqmethod}
\alias{import.htmlq}
\alias{import.kenq}
\alias{import.easyhtmlq}
\title{qmethod-style import aliases}
\usage{
import.pqmethod(file, ...)

import.htmlq(file, ...)

import.kenq(file, ...)

import.easyhtmlq(file, ...)
}
\arguments{
\item{file}{Path to the data file.}

\item{...}{Passed to the underlying reader.}
}
\value{
A \code{qsort_data} object.
}
\description{
Thin aliases that forward to \code{\link[=read_pqmethod]{read_pqmethod()}}, \code{\link[=read_qsort]{read_qsort()}} (HTMLQ
auto-detection), \code{\link[=read_kenq]{read_kenq()}}, and \code{\link[=read_easyhtml_firebase]{read_easyhtml_firebase()}}. These
exist only so scripts written against the \code{qmethod} package continue
to work; new code should call the \verb{read_*} functions directly.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qsort_data.R
\name{qsort_data}
\alias{qsort_data}
\alias{validate_qsort}
\alias{check_distribution}
\alias{infer_distribution}
\alias{parse_distribution}
\title{Construct a validated qsort_data object}
\usage{
qsort_data(
  Y,
  statements = NULL,
  participants = NULL,
  distribution = NULL,
  metadata = list(),
  source = "manual",
  validate = TRUE
)

validate_qsort(qdata, distribution = NULL)

check_distribution(Y, distribution)

infer_distribution(Y)

parse_distribution(x)
}
\arguments{
\item{Y}{A \verb{J x N} numeric matrix (statements as rows, participants
as columns) or a data frame.}

\item{statements, participants}{Optional character vectors of IDs;
default to \code{S1..SJ} and \code{P1..PN}.}

\item{distribution}{Optional integer vector of forced-distribution
counts. Inferred from \code{Y[, 1]} when \code{NULL}.}

\item{metadata}{Optional named list of study-level info.}

\item{source}{Provenance string stored on the object.}

\item{validate}{If \code{TRUE} (default), run \code{\link[=validate_qsort]{validate_qsort()}} and emit
warnings / messages for any issues found.}

\item{qdata}{A \code{qsort_data} object or bare matrix, passed to
\code{validate_qsort()}.}

\item{x}{Numeric vector, character string, or path to a file
containing the forced distribution, passed to
\code{parse_distribution()}.}
}
\value{
\code{qsort_data()} returns a \code{qsort_data} S3 list with fields
\code{Y}, \code{statements}, \code{participants}, \code{distribution}, \code{metadata}, and
\code{source}. \code{validate_qsort()} returns a list with \code{valid}, \code{issues},
\code{warnings}, and \code{summary}. \code{check_distribution()} returns a list
with \code{ok}, \code{non_conforming}, and \code{grid_values}.
\code{infer_distribution()} and \code{parse_distribution()} return integer
vectors.
}
\description{
\code{qsort_data()} is the canonical constructor for a Q-sort dataset.
\code{validate_qsort()}, \code{check_distribution()}, and \code{infer_distribution()}
are the validation helpers used internally by the constructor and
by the file readers. \code{parse_distribution()} accepts a numeric vector,
a comma-/semicolon-/space-separated string, or a text file containing
one of those.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaries.R
\name{crib_sheet}
\alias{crib_sheet}
\title{Extreme-placement probabilities per statement and factor}
\usage{
crib_sheet(fit)
}
\arguments{
\item{fit}{A \code{bayesqm_fit}.}
}
\value{
A long data frame: \code{statement}, \code{factor}, \code{p_top}, \code{p_bottom},
\code{p_highest}, \code{p_lowest}.
}
\description{
The crib sheet: for each statement and factor, the posterior
probability of landing in the top or bottom grid column of that
factor's array, and of carrying the highest or lowest score across
factors.
}
\examples{
head(crib_sheet(demo_fit()))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R
\name{fitted.bayesqm_fit}
\alias{fitted.bayesqm_fit}
\title{Posterior-mean reconstruction on the utility scale}
\usage{
\method{fitted}{bayesqm_fit}(object, ...)
}
\arguments{
\item{object}{A \code{bayesqm_fit}.}

\item{...}{Unused.}
}
\value{
A \verb{J x N} matrix of posterior-mean latent utilities
\verb{F Lambda'}.
}
\description{
Posterior-mean reconstruction on the utility scale
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R
\name{posterior_interval.bayesqm_fit}
\alias{posterior_interval.bayesqm_fit}
\title{Credible intervals for bayesqm_fit parameters}
\usage{
\method{posterior_interval}{bayesqm_fit}(object, prob = 0.95, pars = NULL, regex_pars = NULL, ...)
}
\arguments{
\item{object}{A \code{bayesqm_fit}.}

\item{prob}{Coverage probability (default 0.95).}

\item{pars}{Optional character vector of exact parameter names.}

\item{regex_pars}{Optional regex; matching parameter names are included
in addition to those named in \code{pars}.}

\item{...}{Unused.}
}
\value{
A matrix with one row per parameter and two columns for the
lower and upper interval bounds.
}
\description{
Posterior credible intervals for any subset of parameters in a
\code{bayesqm_fit}. Method for \code{\link[=posterior_interval]{posterior_interval()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/caption.R, R/plots.R
\name{plot_tucker}
\alias{plot_tucker}
\alias{plot_convergence}
\title{Convergence and alignment view}
\usage{
plot_tucker(...)

plot_convergence(fit, n_series = 3)
}
\arguments{
\item{...}{Passed on.}

\item{fit}{A \code{bayesqm_fit}.}

\item{n_series}{How many worst-ESS persons to trace (default 3).}
}
\value{
\code{fit}, invisibly.
}
\description{
Traces of the invariant person spreads \code{s_i} for the persons with the
lowest effective sample size — the series the convergence gate reads —
plus the per-draw congruence of the aligned draws to the pivot, with
the gate report in the margin. A congruence histogram hugging 1 says
the alignment found one common orientation; a long left tail says some
draws sit far from it.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot_person_check}
\alias{plot_person_check}
\title{The person check against the mixed bands}
\usage{
plot_person_check(x)
}
\arguments{
\item{x}{A \code{bayesqm_persons} from \code{\link[=check_persons]{check_persons()}}.}
}
\value{
\code{x}, invisibly.
}
\description{
The \code{m} (model agreement) by \code{w} (person agreement) scatter with the
mixed-replication bands; verdict colours, and arrows from unspanned
persons to their nearest partners.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot_choice_k}
\alias{plot_choice_k}
\title{The two-signal choice-of-K display}
\usage{
plot_choice_k(x)
}
\arguments{
\item{x}{A \code{bayesqm_selection} from \code{\link[=select_k]{select_k()}}.}
}
\value{
\code{x}, invisibly.
}
\description{
The whole decision in one strip, one row per ladder rung. The left
span shows the adequacy signal, the extra-factor percentile inside
its shaded band, with a warning ring where the person check found a
mutual unspanned cluster. The right span shows one chip per factor,
coloured when the factor is supported and grey when not, with the
reason inside: \code{f} means fewer than two selected flags, \code{d} means no
selected distinguishing statement. Each row ends with its own verdict
in words, the selected row is boxed, and when no K passes both checks
the conclusion is written across the bottom of the plot.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qsort_data.R
\name{qsort_data-methods}
\alias{qsort_data-methods}
\alias{print.qsort_data}
\alias{summary.qsort_data}
\alias{as.matrix.qsort_data}
\title{Print, summary, and matrix conversion for qsort_data}
\usage{
\method{print}{qsort_data}(x, ...)

\method{summary}{qsort_data}(object, ...)

\method{as.matrix}{qsort_data}(x, ...)
}
\arguments{
\item{x, object}{A \code{qsort_data} object.}

\item{...}{Unused.}
}
\value{
\code{print()} and \code{summary()} return the input invisibly;
\code{as.matrix()} returns the \verb{J x N} Q-sort matrix.
}
\description{
Print, summary, and matrix conversion for qsort_data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{generate_data}
\alias{generate_data}
\alias{generate_loadings}
\alias{generate_noise}
\alias{discretize_to_grid}
\alias{get_distribution}
\title{Simulate Q-sort data}
\usage{
generate_data(
  N,
  J,
  K,
  noise_sd = 1,
  error_type = "normal",
  nu = 5,
  contam_prop = 0.1,
  contam_scale = 4,
  loading_type = "simple",
  primary_range = c(0.55, 0.85),
  cross_range = c(-0.15, 0.15),
  seed = NULL
)

generate_loadings(
  N,
  K,
  primary_range = c(0.55, 0.85),
  cross_range = c(-0.15, 0.15),
  type = "simple"
)

generate_noise(
  J,
  N,
  type = "normal",
  sd = 1,
  nu = 5,
  contam_prop = 0.1,
  contam_scale = 4
)

discretize_to_grid(Y_cont, distr)

get_distribution(J)
}
\arguments{
\item{N, J, K}{Numbers of participants, statements, and factors.}

\item{noise_sd}{Residual SD.}

\item{error_type}{One of \code{"normal"}, \code{"t"}, \code{"contaminated"}.}

\item{nu}{Degrees of freedom for \code{error_type = "t"}.}

\item{contam_prop, contam_scale}{Contamination rate and scale.}

\item{loading_type}{\code{"simple"} or \code{"complex"}.}

\item{primary_range, cross_range}{Uniform ranges for primary and
cross-loadings.}

\item{seed}{Optional RNG seed; restored on exit.}

\item{type}{For \code{generate_noise()}, one of \code{"normal"}, \code{"t"},
\code{"contaminated"}. For \code{generate_loadings()}, \code{"simple"} or
\code{"complex"}.}

\item{sd}{Residual SD for \code{generate_noise()}.}

\item{Y_cont}{Continuous scores (for \code{discretize_to_grid()}).}

\item{distr}{Integer forced-distribution counts.}
}
\value{
\code{generate_data()} returns a list with \code{Y}, \code{Lambda_true},
\code{F_true}, \code{distribution}, \code{N}, \code{J}, \code{K}. The component helpers
return their respective raw objects.
}
\description{
\code{generate_data()} is the top-level data-generating function used
by the package's simulation studies and tests. It builds a
loading matrix (\code{generate_loadings()}), factor scores, noise of the
chosen type (\code{generate_noise()}), and discretises the continuous
signal onto a forced Q-sort grid (\code{discretize_to_grid()}). See also
\code{get_distribution()} for the standard forced-distribution lookup.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{read_qsort}
\alias{read_qsort}
\alias{read_qsort_csv}
\alias{read_qsort_excel}
\alias{read_pqmethod}
\alias{read_kenq}
\alias{read_kenq_excel}
\alias{read_kade_zip}
\alias{read_easyhtml_firebase}
\alias{read_statements}
\title{Read Q-sort data from file}
\usage{
read_qsort(file, format = "auto", ...)

read_qsort_csv(
  file,
  orientation = c("auto", "statements_rows", "participants_rows"),
  id_col = c("auto", "first", "none"),
  statements = NULL,
  distribution = NULL,
  ...
)

read_qsort_excel(
  file,
  sheet = 1,
  orientation = c("auto", "statements_rows", "participants_rows"),
  id_col = c("auto", "first", "none"),
  statements = NULL,
  distribution = NULL,
  ...
)

read_pqmethod(file, statements_file = NULL)

read_kenq(file, format = c("auto", "json", "csv"))

read_kenq_excel(file)

read_kade_zip(file)

read_easyhtml_firebase(file)

read_statements(file, column = 1, id_column = NULL)
}
\arguments{
\item{file}{Path to the data file.}

\item{format}{For \code{read_qsort()}, \code{"auto"} (default) or one of
\code{"csv"}, \code{"excel"}, \code{"pqmethod"}, \code{"kenq"}, \code{"kenq_excel"},
\code{"kade"}, \code{"easyhtml_firebase"}. For \code{read_kenq()}, one of
\code{"auto"}, \code{"json"}, \code{"csv"}.}

\item{...}{Passed to the underlying reader.}

\item{orientation}{For generic CSV/Excel: \code{"auto"},
\code{"statements_rows"}, or \code{"participants_rows"}.}

\item{id_col}{For generic CSV/Excel: \code{"auto"}, \code{"first"}, or \code{"none"}.}

\item{statements, distribution}{Optional overrides passed to
\code{\link[=qsort_data]{qsort_data()}}.}

\item{sheet}{Excel sheet name or index (default \code{1}).}

\item{statements_file}{For PQMethod, optional separate statements
file.}

\item{column, id_column}{For \code{read_statements()}: column index or name.}
}
\value{
A \code{qsort_data} object, except \code{read_statements()} which
returns a named character vector.
}
\description{
\code{read_qsort()} auto-detects the file format from extension and content
and dispatches to a specialised reader. The specialised readers are
also exported for explicit use:
\itemize{
\item \code{read_qsort_csv()}, \code{read_qsort_excel()} for generic CSV / Excel
(with HTMLQ / FlashQ / Ken-Q auto-detection baked in)
\item \code{read_pqmethod()} for PQMethod \code{.DAT} files
\item \code{read_kenq()} for Ken-Q JSON or CSV
\item \code{read_kenq_excel()} for multi-sheet Ken-Q Excel (Type 1 and Type 2,
both old and Ver2 sub-formats)
\item \code{read_kade_zip()} for KADE ZIP archives
\item \code{read_easyhtml_firebase()} for Easy-HTMLQ Firebase JSON
\item \code{read_statements()} for a standalone statement-text file
}

All readers return a \code{qsort_data} object in \verb{J x N} orientation
(statements as rows, participants as columns).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R
\name{coef.bayesqm_fit}
\alias{coef.bayesqm_fit}
\title{Posterior-mean bounded loadings}
\usage{
\method{coef}{bayesqm_fit}(object, ...)
}
\arguments{
\item{object}{A \code{bayesqm_fit}.}

\item{...}{Unused.}
}
\value{
An \verb{N x K} matrix of posterior-mean bounded loadings (the
correlation-scale loading of the accompanying paper).
}
\description{
Posterior-mean bounded loadings
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colors.R
\name{bayesqm-colors}
\alias{bayesqm-colors}
\alias{bayesqm_colors}
\alias{bayesqm_set_colors}
\title{Get or set the bayesqm colour scheme}
\usage{
bayesqm_colors()

bayesqm_set_colors(scheme)
}
\arguments{
\item{scheme}{Character name of a built-in scheme, or a named list
of colours with the slot names listed in the description.}
}
\value{
\code{bayesqm_colors()} returns the active palette as a named
list. \code{bayesqm_set_colors()} returns the previous scheme name,
invisibly.
}
\description{
Every plot in the package reads its palette through
\code{bayesqm_colors()}. Call \code{bayesqm_set_colors()} to switch the active
scheme for every subsequent plot. The available built-in schemes are
\code{"sorts"} (default, the \code{\link[=plot_sorts]{plot_sorts()}} colour family), \code{"blue"},
\code{"teal"}, \code{"red"}, \code{"purple"}, and \code{"grey"}.
For full control, pass a named list with slots \code{dark}, \code{accent},
\code{grey}, \code{gridgrey}, and \code{fill}.
}
\examples{
bayesqm_colors()
old <- bayesqm_set_colors("teal")
bayesqm_colors()[["fill"]]
bayesqm_set_colors(old)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{assess_recovery}
\alias{assess_recovery}
\alias{assess_classification}
\title{Simulation-study assessment helpers}
\usage{
assess_recovery(Lambda_hat, Lambda_true, Lambda_draws = NULL, prob = 0.95)

assess_classification(flags, Lambda_true)
}
\arguments{
\item{Lambda_hat}{Estimated loading matrix (N x K).}

\item{Lambda_true}{True loading matrix.}

\item{Lambda_draws}{Optional array of shape \verb{[T, N, K]} of posterior
draws, used for coverage / interval metrics.}

\item{prob}{Credible-interval probability.}

\item{flags}{Logical matrix of factor assignments.}
}
\value{
\code{assess_recovery()} returns a list of metrics;
\code{assess_classification()} returns a list with \code{accuracy} and
\code{per_factor}.
}
\description{
\code{assess_recovery()} compares a point estimate and optional posterior
draws to a known loading truth, returning RMSE, per-factor RMSE,
bias, Tucker's congruence, credible-interval coverage, width, and
Gneiting-Raftery interval score. \code{assess_classification()} compares
a logical flag matrix to the true factor assignments using optimal
permutation matching.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_zscores.R
\name{plot_zscores}
\alias{plot_zscores}
\title{Statement scores across factors, whole panel}
\usage{
plot_zscores(
  fit,
  order_by = c("divergence", "score"),
  q = 0.05,
  cons_floor = 0.95
)
}
\arguments{
\item{fit}{A \code{bayesqm_fit}.}

\item{order_by}{\code{"divergence"} (default; by the largest median pairwise
contrast) or \code{"score"} (by the first factor's median score).}

\item{q, cons_floor}{As in \code{\link[=compute_qdc]{compute_qdc()}} (used for the verdict marks
when \code{K >= 2}).}
}
\value{
\code{fit}, invisibly.
}
\description{
Every statement's score under every factor, with nested 50\% and 95\%
credible intervals — the classic Q z-score chart carrying its
uncertainty. Statements are ordered by how strongly the factors
diverge on them, so the top of the chart is where the viewpoints
disagree and the bottom is where they concur; the left margin marks
each statement's verdict.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rotation.R
\name{flip_factor}
\alias{flip_factor}
\title{Flip the pole of one factor}
\usage{
flip_factor(fit, factor)
}
\arguments{
\item{fit}{A \code{bayesqm_fit}.}

\item{factor}{Factor index or \code{"f2"}-style name.}
}
\value{
The fit with the factor's pole reversed.
}
\description{
Reverses the sign of one factor's scores and loadings in every draw.
The polarity canon orients factors so defining sorts load positively;
this is the judgmental override.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rotation.R
\name{rename_factors}
\alias{rename_factors}
\title{Rename the factors}
\usage{
rename_factors(fit, new_names)
}
\arguments{
\item{fit}{A \code{bayesqm_fit}.}

\item{new_names}{Character vector of length \code{K}.}
}
\value{
The renamed fit.
}
\description{
Replaces the \verb{f1 ... fK} labels with substantive names everywhere the
fit carries them, so every downstream table and plot uses the new
names.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{bayesqm-defunct}
\alias{bayesqm-defunct}
\alias{run_bayes}
\alias{plot_hyper}
\alias{select_k_peak}
\alias{select_k_sivula}
\alias{compute_dominant_prob}
\alias{compute_dominant_sign}
\alias{compute_threshold_prob}
\alias{classify_membership}
\alias{compute_divergence}
\alias{critical_delta}
\alias{suggest_delta}
\alias{compute_posterior_scalars}
\alias{demo_run}
\alias{plot_elpd}
\alias{make_elpd_diff}
\alias{make_ppc_ridge}
\alias{make_dominant_panel}
\title{Defunct 0.1.0 functions}
\usage{
run_bayes(...)

plot_hyper(...)

select_k_peak(...)

select_k_sivula(...)

compute_dominant_prob(...)

compute_dominant_sign(...)

compute_threshold_prob(...)

classify_membership(...)

compute_divergence(...)

critical_delta(...)

suggest_delta(...)

compute_posterior_scalars(...)

demo_run(...)

plot_elpd(...)

make_elpd_diff(...)

make_ppc_ridge(...)

make_dominant_panel(...)
}
\arguments{
\item{...}{Ignored.}
}
\value{
Always signals an error.
}
\description{
These 0.1.0 entry points belonged to the removed Student-t score-scale
model and now signal an error naming their 0.2.0 replacement:
flag probabilities and signs come from \code{compute_flags()},
distinguishing/consensus verdicts from \code{compute_qdc()}, the critical
difference from the posterior itself, factor enumeration from
\code{fit_ladder()} and \code{select_k()}, and scalar summaries from
\code{as_draws_df()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checks.R
\name{check_persons}
\alias{check_persons}
\title{The person check against mixed-replication bands}
\usage{
check_persons(fit, draws = 60, mixes = 150)
}
\arguments{
\item{fit}{A \code{bayesqm_fit}.}

\item{draws}{Draw grid for the arrays (default 60).}

\item{mixes}{Mixed replicates for the bands (default 150).}
}
\value{
A \code{bayesqm_persons} data frame: \code{participant}, \code{m}, \code{w},
\code{partner}, \code{verdict}, with the band limits attached as attributes.
}
\description{
For each participant, the model-agreement statistic \code{m} (mean over
draws of the best absolute Spearman agreement with any factor array)
and the person-agreement statistic \code{w} (best absolute agreement with
any other sort), each located against bands from mixed replicates
(fresh persons drawn from the fitted model). Verdicts: \code{fits},
\code{no_shared} (below the model band, inside the person band),
\code{unspanned} (below the model band but above the person band, a shared
viewpoint the fitted factors do not span), and \code{atypical}. Unspanned
persons name their nearest partner.
}
\examples{
check_persons(demo_fit(), draws = 20, mixes = 40)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summaries.R
\name{compute_zscores}
\alias{compute_zscores}
\title{Statement scores with credible intervals}
\usage{
compute_zscores(fit, prob = NULL)
}
\arguments{
\item{fit}{A \code{bayesqm_fit}.}

\item{prob}{Credible-interval probability; defaults to the fit's.}
}
\value{
A data frame with one row per statement: \code{statement}, then
\verb{f\{k\}_zsc}, \verb{f\{k\}_lower}, \verb{f\{k\}_upper} per factor.
}
\description{
Statement scores with credible intervals
}
\examples{
compute_zscores(demo_fit())

}
